#EXEC OBJ LOAD FILE="MaricaCinematicCameraModels.ukx" Package=MaricaCinemaTools
//=============================================================================
// MaricaCameraVehicle.uc
// Author: Nick a.k.a Evil T-ModZ
// 2014.
//=============================================================================

class MaricaCameraVehicle extends HelicopterVehicle;

var P2Player CameraOwner;

var bool bFixed;

var float TryLockTime;

var byte GetOut;

var localized string HelpText1;
var localized string HelpText2;
var localized string HelpText3;
var localized string HelpText4;
var localized string HelpText5;
var localized string HelpText6;

simulated function Bump(Actor Other);
simulated function TakeDamage(int Damage, Pawn instigatedBy, vector hitlocation, vector momentum, class<DamageType> damageType);
simulated function PlayHit(float Damage, vector HitLocation, class<DamageType> damageType, vector Momentum);
simulated function PlayTakeHit(vector HitLoc, int Damage, class<DamageType> damageType);
function BloodHit(vector BloodHitLocation, vector Momentum);
simulated function SparksHit(vector SparkHitLocation, vector Momentum);
function AddDefaultInventory();
function EndClimbLadder(LadderVolume OldLadder);
function ClimbLadder(LadderVolume L);
simulated event ChangeAnimation();
simulated function DrawCrossHair(canvas Canvas);

simulated function VehicleHaveDriver()
{
	bHaveDriver = true;
	bImmortal = false;
}

simulated function VehicleHaveNoDriver()
{
	Driver = None;
	Controller = None;
}

simulated function DeleteOldCameras()
{
	local MaricaCameraVehicle testCamera;
	foreach AllActors(class'MaricaCameraVehicle', testCamera)
	{
		if( testCamera != None 
			&& testCamera != self 
			&& testCamera.Driver == None )
			testCamera.Destroy();
	}
}

simulated function FixVehicles(Pawn testDriver)
{
	local NKVehicle testVehicle;
	foreach AllActors(class'NKVehicle', testVehicle)
	{
		if( testVehicle != None 
			&& testVehicle.Driver != None 
			&& testVehicle.Driver == testDriver )
		{
			if( testVehicle.myDriver != None && testVehicle.myDriver.bHidden )
				testVehicle.myDriver.bHidden = false;
			if( testVehicle.DriverBase != None )
				testVehicle.DriverBase.SetFixedPosition(false);
			testVehicle.RotateSteeringWheel(true);
		}
	}
}

simulated function P2MoCapPawn SearchPlayerPawn()
{
	local P2MoCapPawn testPawn;
	foreach AllActors(class'P2MoCapPawn', testPawn)
	{
		if( testPawn != None 
			&& testPawn != self 
			&& testPawn.bPlayer 
			&& testPawn.Controller != None 
			&& P2Player(testPawn.Controller) != None )
			return testPawn;
	}
	return None;
}

simulated function DriverEnter(P2MocapPawn p)
{
	local PlayerController PC;
	local rotator ViewRotation;
	local Controller C;
	local vector RotX, RotY, RotZ;

	bImmortal = true; // hack, ignore any damage
	Driver = p;
	Driver.Velocity = vect(0,0,0);
	Driver.Acceleration = vect(0,0,0);
	//Driver.bHidden = false;

	C = P.Controller;
	PC = PlayerController(C);

	if(PC != None)
	{
		ViewRotation = PC.Rotation;
		ViewRotation.Yaw = Rotation.Yaw;
		//PC.ReceiveLocalizedMessage(class'NVMessage', 1);
		//P2Player(PC).SwitchToHands(true);
		PC.Unpossess();
		Driver.SetOwner(Self);
		PC.Possess(self);
		//PC.SetFOV(VehicleFOV);
		PC.MyHUD.bShowDebugInfo = true;
		//PC.bBehindView = true;
		//PC.ClientSetBehindView(true);
		PC.ClientSetViewTarget(self);
		PC.ClientSetRotation(ViewRotation);
		PC.GotoState(DriverState);
		PC.PlayerReplicationInfo.SetWaitingPlayer(false);
		bUpdateEyeHeight = true;
	}

	bHaveDriver = true;
	bMatineeStatusUpdate = true;

	VehicleHaveDriver();
	SetPhysics(PHYS_Flying);
	Driver.bHidden = false;
}

simulated function DriverLeave()
{
	local PlayerController PC;
	local Controller C;
	local rotator ExitRot;

	if(Driver == None)
		return;

	Driver.bHidden = false;

	C = Controller;
	PC = PlayerController(C);

	if(PC != None)
	{
		ExitRot = PC.Rotation;
		PC.Unpossess();
		Driver.SetOwner(C);
		PC.Possess(Driver);
		PC.MyHUD.bShowDebugInfo = False;
		PC.FixFOV();
		//PC.bBehindView = false;
		PC.ClientSetBehindView(false);
		PC.ClientSetRotation(ExitRot);
		PC.ClientSetViewTarget(Driver);
		PC.PlayerReplicationInfo.SetWaitingPlayer(false);
		bUpdateEyeHeight = true;
	}

	bHaveDriver = false;

	VehicleHaveNoDriver();
	SetPhysics(PHYS_None);
}

simulated function PostBeginPlay()
{
	Super(P2MoCapPawn).PostBeginPlay();

	DeleteOldCameras();
	DriverEnter(SearchPlayerPawn());

	if( Driver != None )
	{
		GetOut = 0;
		SetTimer(1.0, false);
		SetCollision(false,false,false);
	}
}

simulated function ContinuousUpdate(float DeltaTime, int State)
{
	local P2Player PC;

	if( CameraOwner.Pawn != None 
		&& CameraOwner.Pawn != self )
	{
		if( !CameraOwner.Pawn.bCollideWorld || CameraOwner.Pawn.IsA('NPVehicle') )
		{
			CameraOwner.ClientSetViewTarget(self);
			CameraOwner.ClientSetBehindView(!bFixed);
			FixVehicles(CameraOwner.Pawn);
		}
	}

	if( CameraOwner != None )
	{
		if( CameraOwner.ViewTarget != None && CameraOwner.ViewTarget == self )
			bHidden = true;
		else bHidden = false;
	}

	if( Controller == None )
	{
		if( CameraOwner.Pawn.bWantsToCrouch )
		{
			if( Level.TimeSeconds > TryLockTime  + 0.5 )
			{
				bFixed = !bFixed;
				TryLockTime = Level.TimeSeconds;
			}
		}
	}

	if( Controller == None )
	{
		if( !bFixed && CameraOwner != None )
			SetRotation(CameraOwner.Rotation);
	}
	else if( CameraOwner != None )
		SetRotation(CameraOwner.Rotation);

	if( Controller == None )
		return;

	PC = P2Player(Controller);

	CameraOwner = PC;

	if( Acceleration == vect(0,0,0) )
		Velocity = vect(0,0,0);

	if( Weapon != None )
	{
		Weapon.Destroy();
		Weapon = None;
	}

	if( PressingFire() )
	{
		if( Driver != None && GetOut == 1 )
		{
			GetOut = 2;
			SetTimer(1.0, false);
		}
	}
	else if( PressingAltFire() )
	{
		closeview();
	}

	mySpeed = VSize(Velocity);
}

function Timer()
{
	if( GetOut == 2 )
		DriverLeave();
	GetOut = 1;
}

simulated function DisplayDebug(Canvas Canvas, out float YL, out float YPos)
{
	local float X,Y;
	if( CameraOwner == None 
		|| Controller == None )
		return;

	if( CameraOwner.MyHud != None && CameraOwner.MyHud.bHideHUD )
		return;

	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.Font = Font'Engine.MediumFont';

	//help1
	X = 0.01*Canvas.SizeX;
	Y = 0.1*Canvas.SizeY;
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.DrawText(HelpText1);

	Canvas.SetPos(X, Y);
	Canvas.SetDrawColor(255,255,255,255);
	Canvas.DrawText(HelpText1);

	//help2
	X = 0.01*Canvas.SizeX;
	Y = 0.15*Canvas.SizeY;
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.DrawText(HelpText2);

	Canvas.SetDrawColor(255,0,0,255);
	Canvas.SetPos(X, Y);
	Canvas.DrawText(HelpText2);

	//help3
	X = 0.01*Canvas.SizeX;
	Y = 0.20*Canvas.SizeY;
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.DrawText(HelpText3);

	Canvas.SetDrawColor(255,0,0,255);
	Canvas.SetPos(X, Y);
	Canvas.DrawText(HelpText3);

	//help4
	X = 0.01*Canvas.SizeX;
	Y = 0.25*Canvas.SizeY;
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.DrawText(HelpText4);

	Canvas.SetDrawColor(255,255,255,255);
	Canvas.SetPos(X, Y);
	Canvas.DrawText(HelpText4);

	//help5
	X = 0.01*Canvas.SizeX;
	Y = 0.30*Canvas.SizeY;
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.DrawText(HelpText5);

	Canvas.SetDrawColor(255,255,255,255);
	Canvas.SetPos(X, Y);
	Canvas.DrawText(HelpText5);

	//help6
	X = 0.01*Canvas.SizeX;
	Y = 0.35*Canvas.SizeY;
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.DrawText(HelpText6);

	Canvas.SetDrawColor(255,255,255,255);
	Canvas.SetPos(X, Y);
	Canvas.DrawText(HelpText6);
}

exec function lockview()
{
	bFixed = !bFixed;
}

exec function closeview()
{
	if( Driver != None )
	{
		DriverLeave();
		LifeSpan = 0.1;
	}
	else Destroy();
}

function Destroyed()
{
	Super.Destroyed();
}

defaultproperties
{
	 HelpText1="ProjectMarica - CinematicCamera"
	 HelpText2="TO WORK WITH THAT CAMERA: spawn vehicle and ONLY AFTER spawn cinematic camera"
	 HelpText3="You are in replacing mode, you can fly and search good position, to deal with vehicles press fire"
	 HelpText4="Press fire (right mouse button) to return to player"
	 HelpText5="Press alt fire (left mouse button) to delete cinematic camera"
	 HelpText6="Press C (crouch) to lock/unlock camera rotation (effect appears when you leave replacing mode)"

	 bCollideWorld=false
	 bFixed=true
	 Tag="Vehicle Cinema Camera"

	 LandMovementState="PlayerHelicoptering"
     DriverState="PlayerHelicoptering"

     DriverPos=(X=0.0,Y=0.0,Z=0.0)

     HealthMax=9999.0
     AirSpeed=3000.0
     AccelRate=3000.0
     AirControl=0.5
     BaseEyeHeight=0.0
     EyeHeight=0.0
     Health=512.0
     Mesh=SkeletalMesh'MaricaCinemaTools.Camera_Model'
	 Skins(0)=Texture'Engine.LargeVertex'
     DrawScale=1.0
     CollisionRadius=1.0
     CollisionHeight=1.0
	 Mass=1.0
}
