//=============================================================================
// BlackDragon.uc
// Author: Nick Pilshikov "Evil"
// Email: Nick_Pv@mail.ru / EvilTheDark@gmail.com
// Site: Set-Games.ru / Revival-Games.ru
// Revival Games Studios.
// 2014.
//=============================================================================

class BlackDragon extends HelicopterVehicle;

const HIGH_SPEED_FLY = 2000;

var class<TimedMarker> DragonMarkerMade;

var DragonSpitFireCollision SpitFireCollision;
var ViewPoint FPViewPoint;

var(Dragons) config bool bAllowRagdoll;

var(Dragons) Texture DragonHealthHUD;

var(Dragons) class<Emitter> SpitFireClass;
var(Dragons) class<DragonSpitFireCollision> SpitFireCollisionClass;

var(Dragons) sound PainSounds[3];

var DragonsRagdollController RagdollController;
var Effects EyeLFlash, EyeRFlash;

var(Dragons) class<Effects> EyeFlashClass;

var(Dragons) vector EyeRPos;
var(Dragons) vector EyeLPos;
var(Dragons) vector FPViewPos;

var(Dragons) Name ActivateAnim;
var(Dragons) Name DeactivateAnim;
var(Dragons) Name FlyingAnim;
var(Dragons) Name LandedAnim;
var(Dragons) Name SpeedFlyAnim;
var(Dragons) Name DragHeadBone;
var(Dragons) Name SpitFireBone;

/*replication
{
	unreliable if( Role == ROLE_Authority )
		EyeLFlash, EyeRFlash;
}*/

simulated function SendEnterMessage(PlayerController PC)
{
	PC.ReceiveLocalizedMessage(class'NVMessage', 1);
	PC.ReceiveLocalizedMessage(class'NVMessage', 7);
}

simulated function Notify_TakeOffVelocity()
{
	Velocity = vect(0,0,1)*400;
}

simulated function PhysicsSimulation(int State)
{
	local vector TraceEndLoc;

	//if( Health <= 0 )
	//	return;

	if( State == 1 )
	{
		SetPhysics(PHYS_Flying);
		//Velocity = vect(0,0,0);
		Acceleration = vect(0,0,0);
	}
	else if( State == 2 )
	{
		SetPhysics(PHYS_Falling);
		Velocity = vect(0,0,0);
		Acceleration = vect(0,0,-24000);
	}
	else if( State == 3 )
	{
		TraceEndLoc = Location;
		TraceEndLoc.Z -= 200;
		if( FastTrace(TraceEndLoc, Location) )
		{
			//PhysicsSimulation(4);omg!&!&!&!
			SetPhysics(PHYS_Falling);
			Velocity.Z += 100;
			Acceleration = vect(0,0,-24000);
			return;
		}
		SetPhysics(PHYS_None);
		Velocity = vect(0,0,0);
		Acceleration = vect(0,0,0);
	}
	else if( State == 4 )
	{
		SetPhysics(PHYS_Falling);
		Velocity.Z += 100;
		Acceleration = vect(0,0,-24000);
	}
	else if( State == 5 )
	{
		//SetLocation(ServerPosition);
		//SetPhysics(PHYS_None);
		//Velocity = vect(0,0,0);
		//Velocity = vect(0,0,1)*800;
		SetPhysics(PHYS_Flying);
		//Acceleration = vect(0,0,0);
	}
	else if( State == 6 )
	{
		if( Controller != None && Physics != PHYS_Flying )
			SetPhysics(PHYS_Flying);
	}
	else
	{
		Velocity = vect(0,0,0);
		Acceleration = vect(0,0,0);
	}
}

simulated function VehicleActivateAnim()
{
	if( mySpeed < 800 )
		PlayAnim(ActivateAnim,,0.1);
}

simulated function VehicleDeactivateAnim()
{
	//if( mySpeed < 800 )
		PlayAnim(DeactivateAnim,,0.1);
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	if( EyeLFlash == None && EyeFlashClass != None && DragHeadBone != '' )
	{
		EyeLFlash = Spawn(EyeFlashClass,self,,Location);
		AttachToBone(EyeLFlash, DragHeadBone);
		EyeLFlash.SetRelativeLocation(EyeLPos);// >> Rotation);
	}

	if( EyeRFlash == None && EyeFlashClass != None && DragHeadBone != '' )
	{
		EyeRFlash = Spawn(EyeFlashClass,self,,Location);
		AttachToBone(EyeRFlash, DragHeadBone);
		EyeRFlash.SetRelativeLocation(EyeRPos);// >> Rotation);
	}

	if( FPViewPoint == None )
	{
		FPViewPoint = spawn(class'ViewPoint',self,,Location + (FPViewPos >> Rotation));
		//AttachToBone(FPViewPoint, 'dragonbip Spine');
		FPViewPoint.SetBase(self);
	}

	if( SpitFireCollision == None && SpitFireCollisionClass != None && SpitFireBone != '' )
	{
		SpitFireCollision = Spawn(SpitFireCollisionClass,self,,Location);
		//AttachToBone(SpitFireCollision, SpitFireBone);
	}
}

simulated function VehicleIdleAnim(bool State)
{
	local float ForwardSpeed;
	local vector worldForward;

	if( Health <= 0 )
	{
		PlayAnim(DeactivateAnim,,0.1);
		return;
	}

	if( State )
	{
		worldForward = vect(1,0,0) >> Rotation;
		ForwardSpeed = Velocity Dot worldForward;

		if( ForwardSpeed > HIGH_SPEED_FLY )
			LoopAnim(SpeedFlyAnim,,0.1);
		else LoopAnim(FlyingAnim,,0.1);

		//ClientMessage(ForwardSpeed);
	}
	else
	{
		LoopAnim(LandedAnim,,0.1);
	}
}

simulated function UpdateRotors(int Rot)
{
	//STUB
}

simulated function UpdateBlades(float Speed)
{
	//STUB
}

simulated function UpdateVehicleRotation(optional bool bFixed)
{
	local rotator VehicleRotation;
	local int RiseSpeed , StrafeSpeed , AccelRiseRot , AccelStrafeRot;
	local vector worldUp , worldStrafe;

	if( Controller != None )
	{
		ControllerRotation = Controller.Rotation;
	}

	if( bFixed )
	{
		if(RotatorActor != None)
		{
			RotatorActor.DesiredRotation = ServerRotation;
			RotatorActor.RotationRate = rot(0,0,0);
			RotatorActor.SetRotation(ServerRotation);
		}
		SetRotation(ServerRotation);
		SetBoneRotation(ChassisBone,rot(0,0,0),0,1);
		ClientSimulation(ServerRotation);
		return;
	}

	worldUp = vect(0,0,1) >> Rotation;
	RiseSpeed = Velocity Dot worldUp;
	RiseSpeed = -RiseSpeed;

	worldStrafe = vect(0,1,0) >> Rotation;
	StrafeSpeed = Velocity Dot worldStrafe;
	StrafeSpeed = -StrafeSpeed;

	AccelRiseRot = FClamp(RiseSpeed,AccelRotMin,AccelRotMax);
	AccelStrafeRot = FClamp(StrafeSpeed,AccelRotMin,AccelRotMax);

	if(RotatorActor != None)
	{
		RotatorActor.DesiredRotation = ControllerRotation;
		RotatorActor.RotationRate = Normalize((Normalize(ControllerRotation) - Normalize(Rotation))) * VehicleRotateSpeed;
		VehicleRotation.Yaw = RotatorActor.Rotation.Yaw;
		SetRotation(VehicleRotation);
	}
	ClientSimulation(VehicleRotation);

	VehicleRotation = rot(0,0,0);
	VehicleRotation.Yaw = AccelRiseRot;
	VehicleRotation.Roll = AccelStrafeRot;

	SetBoneRotation(ChassisBone,VehicleRotation,0,1);
}

simulated function ClientSimulation(rotator VehicleRotation)
{
	//STUB
}

simulated function VehicleFire(rotator Direction)
{
	local Emitter SpitFire;
	local vector Dir, Range, StartTrace, SpawnPos;
	local vector HitLoc, HitNorm;
	local rotator FireOrient;

	Dir = Vector(Direction);

	Super(NPVehicle).VehicleFire(Direction);

	//Range.X = FRand() * 0.06;
	//Range.Y = FRand() * 0.06;
	//Range.Z = FRand() * 0.06;

	SpawnPos = GetBoneCoords(SpitFireBone).Origin + (vect(5,0,0) >> Rotation);
	CalcWeaponFire( (vect(0,10,0) >> Rotation), HitLoc, HitNorm );
    FireOrient = Rotator( Normal(HitLoc - SpawnPos) );
	StartTrace = vector(FireOrient) + Range;

	if( SpitFireClass != None )
	{
		SpitFire = spawn(SpitFireClass,Instigator,,SpawnPos,FireOrient);
		if( Level.NetMode == NM_Client )
			SpitFire.SetRelativeRotation(FireOrient);
		SpitFire.SetBase(self);
	}

	if( SpitFireCollision != None )
	{
		SpitFireCollision.SetCollision(true, true, false);
		SpitFireCollision.SetRotation(FireOrient);
		SpitFireCollision.SetLocation(SpawnPos);
		SpitFireCollision.SetTimer(VehicleFireSpeed, false);
	}
}

simulated function VehicleAltFire(rotator Direction)
{
	local Pawn newTarget;
	local RocketProjectile Rocket;
	local vector Dir;
	local vector RotX, RotY, RotZ;

    GetAxes(Direction,RotX,RotY,RotZ);

	Dir = Vector(Direction);

	Super(NPVehicle).VehicleAltFire(Direction);

	if( AltFireState == 0 )
	{
		Rocket = spawn(RocketProjClass,Instigator,,Location + RocketLauncherOffset.X * RotX + 
				RocketLauncherOffset.Y * RotY + RocketLauncherOffset.Z * RotZ);
		Rocket.SetupShot(10);
		Rocket.AddRelativeVelocity(Velocity);

		if( CurrentTarget != None )
			Rocket.SetNewTarget(CurrentTarget);
		else Rocket.SetNewTarget(CheckOutTarget());

		AltFireState = 1;
	}
	else
	{
		Rocket = spawn(RocketProjClass,Instigator,,Location + RocketLauncherOffset.X * RotX - 
				RocketLauncherOffset.Y * RotY + RocketLauncherOffset.Z * RotZ);
		Rocket.SetupShot(10);
		Rocket.AddRelativeVelocity(Velocity);

		if( CurrentTarget != None )
			Rocket.SetNewTarget(CurrentTarget);
		else Rocket.SetNewTarget(CheckOutTarget());

		AltFireState = 0;
	}
}

simulated function ContinuousUpdate(float DeltaTime, int State)
{
	local vector HitLoc, HitNorm;

	mySpeed = VSize(Velocity);

	if(Driver != None && Controller != None)
		DriverController = PlayerController(Controller);

	//EyeLFlash.SetRelativeLocation(EyeLPos);// >> Rotation);
	//EyeRFlash.SetRelativeLocation(EyeRPos);// >> Rotation);

	UserInterfaceControl();

	if( DriverController != None && Level.TimeSeconds > LastCalcWeaponFire + 0.5 )
	{
		if( DragonMarkerMade != None )
		{
			DragonMarkerMade.static.NotifyControllersStatic(
				Level,
				DragonMarkerMade,
				self, 
				self, 
				DragonMarkerMade.default.CollisionRadius,
				Location);
		}
		LastCalcWeaponFire = Level.TimeSeconds;
	}

	GetVehicleData();

	//if( Health <= 0 )
	//	return;

	if( State == 1 )
	{
		ServerPhysicsSimulation(5);
		PhysicsSimulation(5);

		//ControlAccess(false);

		EngineModel(0);

		UpdateVehicleRotation(true);

		VehicleActivate();
	}
	else if( State == 2 )
	{
		EngineModel(0);

		//ControlAccess(true);

		UpdateVehicleRotation(true);

		VehicleDeactivate();
	}
	else
	{
		if( bHaveDriver )
			ServerPhysicsSimulation(6);

		EngineModel(mySpeed);

		//ControlAccess(true);

		if( !bHaveDriver )
		{
			if( mySpeed > 800 || bCrashWay )
			{
				bCrashWay = true;
			}
		}

		UpdateVehicleRotation(!bHaveDriver);

		ShadowRendering(bHaveDriver);

		VehicleGetFire();

		VehicleIdleAnim(bHaveDriver || bCrashWay);
	}
}

simulated function EngineModel(float Speed)
{
	//STUB
}

simulated function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, class<DamageType> damageType)
{
	local int AcceptDamage;
	local Controller Killer;

	AcceptDamage = Damage;

	if(ClassIsChildOf(damageType, class'PlasmaDamage'))
		AcceptDamage *= 0.3;
	else if(ClassIsChildOf(damageType, class'RocketExplosionDamage'))
		AcceptDamage += 110;

	if( Health > 0 )
	{
		PlayPainSounds();
		Super.TakeDamage(AcceptDamage, instigatedBy, hitlocation, momentum, damageType);
	}
	else
	{
		Super(P2MoCapPawn).TakeDamage(AcceptDamage, instigatedBy, hitlocation, momentum, damageType);
	}
}

simulated function Destroyed()
{
	Super.Destroyed();

	if( EyeLFlash != None )
	{
		DetachFromBone(EyeLFlash);
		EyeLFlash.Destroy();
		EyeLFlash = None;
	}

	if( EyeRFlash != None )
	{
		DetachFromBone(EyeRFlash);
		EyeRFlash.Destroy();
		EyeRFlash = None;
	}

	if( FPViewPoint != None )
	{
		FPViewPoint.Destroy();
		FPViewPoint = None;
	}

	if( SpitFireCollision != None )
	{
		SpitFireCollision.Destroy();
		SpitFireCollision = None;
	}

	UnhookRagdoll();
	GiveBackRagdoll();

	if( Health <= 0 )
	{
		spawn(class'DragonGoreExplosion',self,,Location);
	}
}

simulated function DrawCrossHair( canvas Canvas)
{
	//STUB
}

simulated function Explode()
{
	if( !IsAllowRagdoll() )
	{
		spawn(class'DragonGoreExplosion',self,,Location);
	}
}

simulated function ControlAccess(bool State)
{
	if( State )
	{
		if( AccelRate != default.AccelRate )
			AccelRate = default.AccelRate;
	}
	else
	{
		if( AccelRate != 0.0 )
			AccelRate = 0.0;
	}
}

simulated function SpawnMyShadow()
{
	if(bAllowDynamicShadows && Level.NetMode != NM_DedicatedServer)
	{
		if(myShadow == None)
		{
			myShadow = Spawn(class'DragonsShadowProjector',Self,'',Location);
			myShadow.ShadowActor = self;
			myShadow.ShadowZ = ShadowZ;
			myShadow.MyBase = None;
			myShadow.MyCar = None;
			myShadow.MyVeh = self;
			myShadow.LightDirection = Normal(vect(1,1,6));
			myShadow.InitShadow();
		}
	}
}

simulated function IsDying(Controller Killer, class<DamageType> damageType, vector HitLocation)
{
	local PlayerController PC;

	if( (Driver != None) && (Driver.Health > 0) )
	{
		if( Driver != None && Driver.myHead != None )
		{
			myDriver.DetachFromBone(Driver.myHead);
			Driver.AttachToBone(Driver.myHead, 'MALE01 head');
			Driver.myHead.SetOwner(Driver);
		}
		PC = PlayerController(Controller);
		if( PC != None )
		{
			bHideHUD = true;
			PC.MyHUD.bShowDebugInfo = False;
			PC.Unpossess();
			PC.Possess(Driver);
		}
		Driver.bCollideWorld = true;
		Driver.Died(Killer, damageType, HitLocation);
		if( Level.NetMode != NM_Client )
			Driver.bHidden = false;
		Driver = None;
	}
	if( IsAllowRagdoll() )
	{
		Health = 0;
		bHideHUD = true;
		bHidden = false;
		VehicleDied();
		GotoState('Destroying');
	}
	else
	{
		bHideHUD = true;
		GotoState('Destroying');
		// hack, anyway prepare it to destroy
		LifeSpan = 0.5;
	}
}

simulated function Died(Controller Killer, class<DamageType> damageType, vector HitLocation)
{
	bHideHUD = true;
	Super.Died(Killer, damageType, HitLocation);
	IsDying(Killer, damageType, HitLocation);
}

state Destroying
{
	ignores TakeDamage;
	Begin:
	if( IsAllowRagdoll() )
	{
		bHideHUD = true;
		Sleep(0.05);
		PlayDying(class'BulletDamage',vect(0,0,0));
	}
	else
	{
		bHideHUD = true;
		Sleep(0.2);
		Destroy();
	}
}

function GetKarmaSkeleton()
{
	local P2Player p2p, cont;

	if( !IsAllowRagdoll() )
		return;

	//if( RagdollController != None )
	//	RagdollController.UnhookOldDragonsCorpses();

	if(Level.NetMode != NM_DedicatedServer)
	{
		// Go through all the player controllers till you find the one on
		// your computer that has a valid viewport and has your ragdolls
		foreach DynamicActors(class'P2Player', Cont)
		{
			if (ViewPort(Cont.Player) != None)
			{
				p2p = Cont;
				break;
			}
		}
		if(p2p != None
			&& KParams == None)
		{
			KParams = p2p.GetNewRagdollSkel(self, 'NicksDragonRagdoll');
		}
	}

	// last chance leave me
	if( Driver != None )
	{
		Driver = None;
	}
	if( Controller != None )
	{
		ControllerLeave();
		Controller = None;
	}
	SetCollision(false,false,false);
	if( Driver == None )
	{
		if( KParams != None )
			LifeSpan = 10;
		else LifeSpan = 1.0;
	}
}

function bool AllowRagdoll(class<DamageType> DamageType)
{
	return bAllowRagdoll;
}

simulated function bool IsAllowRagdoll()
{
	return bAllowRagdoll && GameIsSingle() && Level.NetMode != NM_Client && Level.NetMode != NM_DedicatedServer;
}

event PreSaveGame()
{
	if( Health <= 0 && Driver == None )
	{
		//spawn(class'DragonGoreExplosion',self,,Location);
		LifeSpan = 0.1;
	}
	Super.PreSaveGame();
}

function VehicleDied()
{
	if( myDriver != None )
	{
		myDriver.Destroy();
		myDriver = None;
	}

	if( OurFactory != None )
		OurFactory.VehicleDestroyed();

	if( Controller != None )
		ControllerLeave();

	if( Driver != None )
	{
		//wtf
		Driver = None;
	}

	if( RightTrigger != None )
	{
		RightTrigger.Destroy();
		RightTrigger = None;
	}

	if( LeftTrigger != None )
	{
		LeftTrigger.Destroy();
		LeftTrigger = None;
	}

	if( ExitTrigger != None )
	{
		ExitTrigger.Destroy();
		ExitTrigger = None;
	}

	if( EyeLFlash != None )
	{
		DetachFromBone(EyeLFlash);
		EyeLFlash.Destroy();
		EyeLFlash = None;
	}

	if( EyeRFlash != None )
	{
		DetachFromBone(EyeRFlash);
		EyeRFlash.Destroy();
		EyeRFlash = None;
	}

	if( FPViewPoint != None )
	{
		FPViewPoint.Destroy();
		FPViewPoint = None;
	}

	if( SpitFireCollision != None )
	{
		SpitFireCollision.Destroy();
		SpitFireCollision = None;
	}
}

simulated function PlayHit(float Damage, vector HitLocation, class<DamageType> damageType, vector Momentum)
{
	Super(P2MoCapPawn).PlayHit(Damage, HitLocation, damageType, Momentum);
}

simulated function PlayTakeHit(vector HitLoc, int Damage, class<DamageType> damageType)
{
	Super(P2MoCapPawn).PlayTakeHit(HitLoc, Damage, damageType);
}

function BloodHit(vector BloodHitLocation, vector Momentum)
{
	Super(P2MoCapPawn).BloodHit(BloodHitLocation, Momentum);
}

function PlayPainSounds()
{
	PlaySound(PainSounds[Rand(ArrayCount(PainSounds))], SLOT_Misc, 0.7, false);
}

simulated function SwapToBurnVictim()
{
	// Always refix the ambient glow
	SetAmbientGlow(default.AmbientGlow);

	if(class'P2Player'.static.BloodMode())
	{
		// Set my body skin
		Skins[0] = BurnSkin;
		Skins[1] = BurnSkin;
	}
}

simulated function VehicleGetFire()
{
	if( PressingFire() )
	{
		if( VSize(Velocity) > HIGH_SPEED_FLY )
			return;

		if(Level.TimeSeconds > FireUpdate + VehicleFireSpeed)
		{
			VehicleFire(Rotation);
			if( MinigunFireSound != None )
				PlaySound(MinigunFireSound, SLOT_None, 1.0, true);
				//PlaySound(MinigunFireSound,,VehicleFireVolume);
				//PlaySound(MinigunFireSound,,TransientSoundVolume*2.5);
			FireUpdate = Level.TimeSeconds;
		}
	}
}

simulated function DisplayDebug(Canvas Canvas, out float YL, out float YPos)
{
	if( HealthTex != DragonHealthHUD )
		HealthTex = DragonHealthHUD;
	Super.DisplayDebug(Canvas, YL, YPos);
}

function DrawSpeedInfo(Canvas Canvas);

simulated function UserInterfaceControl()
{
	if( GameIsEnd() || Health <= 0 )
		bHideHUD = true;

	if( DriverController != None )
	{
		if( bHideHUD )
		{
			DriverController.MyHUD.bShowDebugInfo = false;
			bHideHUD = false;
		}

		if( Driver != None )
		{
			if( !DriverController.MyHUD.bShowScores && !bHideHUD && Health > 0 )
			{
				if(!DriverController.MyHUD.bShowDebugInfo)
					DriverController.MyHUD.bShowDebugInfo = true;
			}
			else
			{
				if(DriverController.MyHUD.bShowDebugInfo)
					DriverController.MyHUD.bShowDebugInfo = false;
			}

			if( DriverController.CameraDist != CameraDist && !GameIsEnd() )
				DriverController.CameraDist = CameraDist;

			//if( DriverController.ViewTarget != self && !GameIsEnd() )
			//	DriverController.ClientSetViewTarget(self);

			if( DriverController.bBehindView )
			{
				DriverController.ClientSetViewTarget(self);
			}
			else if( FPViewPoint != None )
			{
				FPViewPoint.SetRotation(DriverController.Rotation);
				DriverController.ClientSetViewTarget(FPViewPoint);
			}

			if( !DriverController.IsInState(DriverState) && Health > 0 )
				DriverController.GotoState(DriverState);

			DriverController.SetFOV(VehicleFOV);
		}
		else
		{
			DriverController.MyHUD.bShowDebugInfo = false;
			DriverController.CameraDist = DriverController.default.CameraDist;
			DriverController.FixFOV();
			DriverController.ClientSetViewTarget(Driver);
			DriverController.ClientSetBehindView(false);
			P2Player(DriverController).SwitchToHands(true);
			log("Current driver controller ("$DriverController$") leave this vehicle ("$self$")");
			DriverController = None;
		}
	}
	else DriverController = None;
}

defaultproperties
{
	 FPViewPos=(X=50.0,Z=200.0)
     DragonHealthHUD=Texture'MaricaDragonTex.Dragon_HUD_Health'
	 DragonMarkerMade=class'BulletHitMarker'
	 SpitFireBone="Bone_jaw_end"
	 SpitFireCollisionClass=class'DragonSpitFireCollision'
	 SpitFireClass=class'DragonSpitFire'
	 PainSounds(0)=Sound'MaricaDragonSounds.Dragon3'
	 PainSounds(1)=Sound'MaricaDragonSounds.Dragon1'
	 PainSounds(2)=Sound'MaricaDragonSounds.Dragon13'
	 bAllowRagdoll=true
	 EyeLPos=(X=32.0,Y=-28.0,Z=16.0)
	 EyeRPos=(X=32.0,Y=-28.0,Z=-15.0)
	 EyeFlashClass=Class'BlackDragonEyeFlash'
	 DragHeadBone="dragonbip Head"
	 EngStartUpVolume=0.3
	 EngStopVolume=0.3
	 Tag="Black Dragon"
	 VehicleFireVolume=0.5
	 SoundVolume=140.0
     SoundRadius=100.0
     TransientSoundVolume=0.0
     TransientSoundRadius=800.0

	 FireZ=132.0
	 AimAngle=2000
     ReticleTexture=None
	 ReticleTextureAim=None
     ReticleColor=(B=255,G=255,R=255,A=150)

	 CrashSounds(0)=Sound'MiscSounds.People.bodyhitground1'
	 CrashSounds(1)=Sound'MiscSounds.People.bodyhitground2'

	 VehicleRotateSpeed=2
	 MenuName="on Dragon"
	 SpeedFlyAnim="Fly"
	 FlyingAnim="Fly_Idle"
	 LandedAnim="idle2"
	 ActivateAnim="TakeOff"
	 DeactivateAnim="Land"
	 MinigunProjClass=None
	 RocketProjClass=None
	 MinigunMuzzleClass=None
	 CameraDist=4.0
	 AccelRotMin=-10000
	 AccelRotMax=10000
	 LandMovementState="PlayerHelicoptering"
     DriverState="PlayerHelicoptering"
	 VehicleState=3
	 StartTime=1.15
	 StopTime=1.46
	 MinigunRightBone=""
	 MinigunLeftBone=""
	 ChassisBone="dragonbip Pelvis"
	 MainRotorBone=""
	 TailRotorBone=""
	 MainRotorBoneTip=""
	 TailRotorBoneTip=""
	 VehicleFOV=100
	 bCanSeeDriver=false
     MinigunOffset=(X=200.0,Y=50.0,Z=-45.0)
     RocketLauncherOffset=(X=0.0,Y=155.0,Z=75.0)
	 StartupSound=Sound'MaricaDragonSounds.Dragon6'
	 StopSound=Sound'MaricaDragonSounds.Dragon5'
     DestroyedEffectClass=None
     DestroyedSound=None
     EngineAmbientSound=None
     MinigunFireSound=Sound'MaricaDragonSounds.Dragon_spit_fire'
	 RocketLaunchSound=None
     DriverPos=(X=20.0)
     EnterPos=(Y=220.0,Z=-10.0)
     ExitPos=(Y=220.0,Z=-10.0)
     VehicleFireSpeed=0.200000
     VehicleAltFireSpeed=2.0
     HealthMax=512.0
     AirSpeed=3000.0
     AccelRate=1000.0
     AirControl=0.5
     BaseEyeHeight=42.0
     EyeHeight=42.0
     Health=512.0
     Mesh=SkeletalMesh'MaricaDragonModels.Dragon_Model'
	 Skins(0)=Shader'MaricaDragonTex.dragon_black_skin01'
	 Skins(1)=Shader'MaricaDragonTex.dragon_black_skin02'
     DrawScale=1.0
     CollisionRadius=180.0
     CollisionHeight=85.0
	 Mass=400.0
}
