//=============================================================================
// RigCar.
//
// Base model: model from Carmageddon.
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2014.
//=============================================================================

class RigCar extends NKVCarNew;

var float FireUpdate, AltFireUpdate;
var(MaricaArmory) float VehicleFireSpeed, VehicleAltFireSpeed;

var(MaricaArmory) sound MinigunFireSound;
var(MaricaArmory) sound RocketLaunchSound;

var(MaricaArmory) class<MinigunProjectile> MinigunProjClass;
var(MaricaArmory) class<RocketProjectile> RocketProjClass;
var(MaricaArmory) class<MinigunMuzzleFlashEmitter> MinigunMuzzleClass;

var class<DamageType> BladesDamageClass;

simulated function SendEnterMessage(PlayerController PC)
{
	PC.ReceiveLocalizedMessage(class'NVMessage', 1);
	PC.ReceiveLocalizedMessage(class'NVMessage', 4);
	PC.ReceiveLocalizedMessage(class'NVMessage', 8);
}

simulated function PostBeginPlay()
{
	local class<DamageType> tempBladesDamageClass;
	Super.PostBeginPlay();
	tempBladesDamageClass = class<DamageType>(DynamicLoadObject("BaseFX.ScytheDamage", class'Class'));
	if( tempBladesDamageClass != None )
		BladesDamageClass = tempBladesDamageClass;
	else BladesDamageClass = class'BladesDamage';
}

simulated function ContinuousUpdate()
{
	Super.ContinuousUpdate();
	UpdateBlades();
}

simulated function VehicleGetFire(bool bFire, bool bAltFire)
{
	if( bFire )
	{
		if( Level.TimeSeconds > FireUpdate + VehicleFireSpeed )
		{
			VehicleFire(Rotation);
			if( MinigunFireSound != None )
				PlaySound(MinigunFireSound, SLOT_None, 1.0, true);
			FireUpdate = Level.TimeSeconds;
		}
	}
	else if( bAltFire )
	{
		if( Level.TimeSeconds > AltFireUpdate + VehicleAltFireSpeed )
		{
			VehicleAltFire(Rotation);
			if(RocketLaunchSound != None)
				PlaySound(RocketLaunchSound, SLOT_None, 1.0, true);
			AltFireUpdate = Level.TimeSeconds;
		}
	}
}

simulated function VehicleFire(rotator Direction)
{
	local Pawn Agressor;
	local MinigunProjectile Bullet;
	local MinigunMuzzleFlashEmitter Muzzle;
	local vector Dir, Range, StartTrace, SpawnPos;
	local vector RotX, RotY, RotZ, HitLoc, HitNorm;

    GetAxes(Direction,RotX,RotY,RotZ);

	Dir = Vector(Direction);

	Range.X = FRand() * 0.06;
	Range.Y = FRand() * 0.06;
	Range.Z = FRand() * 0.06;

	SpawnPos = GetBoneCoords('MuzzleRight').Origin + (vect(10,0,-10) >> Rotation);

	StartTrace = vector(Direction) + Range;

	if( MinigunProjClass != None )
	{
		Bullet = spawn(MinigunProjClass,Driver,,SpawnPos);
		Bullet.SetVelocity(StartTrace);
	}

	Range.X = FRand() * 0.06;
	Range.Y = FRand() * 0.06;
	Range.Z = FRand() * 0.06;

	SpawnPos = GetBoneCoords('MuzzleLeft').Origin + (vect(10,0,-10) >> Rotation);

	StartTrace = vector(Direction) + Range;

	if( MinigunProjClass != None )
	{
		Bullet = spawn(MinigunProjClass,Driver,,SpawnPos);
		Bullet.SetVelocity(StartTrace);
	}

	if( MinigunMuzzleClass != None )
	{
		Muzzle = spawn(MinigunMuzzleClass,Driver,,GetBoneCoords('MuzzleRight').Origin);
		Muzzle.SetBase(self);

		Muzzle = spawn(MinigunMuzzleClass,Driver,,GetBoneCoords('MuzzleLeft').Origin);
		Muzzle.SetBase(self);
	}
}

simulated function VehicleAltFire(rotator Direction)
{
	local RocketProjectile Rocket;

	Direction.Pitch -= 512;
	Rocket = spawn(RocketProjClass,Driver,,GetBoneCoords('Attachment').Origin, Direction);
	Rocket.SetupShot(10);
	Rocket.AddRelativeVelocity(Velocity);
}

state VehicleDriving
{
	simulated function MainSystem(float DesiredSpeed)
	{
		local bool bPSR;

		if(Passenger != None)
			bPSR = Passenger.bWantsToCrouch;

		if(Driver == None)
			return;

		RadioSystem(Driver.bWantsToCrouch, bPSR);

		//if( myVehicleHands == None )
		//	VehicleGetFire(Driver.PressingFire(), Driver.PressingAltFire());
		VehicleGetFire(Driver.PressingFire(), false);
	}

	simulated function HandsTryToFire()
	{
		if( Driver == None )
			return;
		VehicleGetFire(true, false);
	}

	simulated function HandsTryToAltFire()
	{
		if( Driver == None )
			return;
		VehicleGetFire(false, true);
	}

	function ScriptedDriving()
	{
		local float DistTopointer, FocusFactor, SignFactor;
		local bool bNeedToTurn;
		local vector PointerLocation, LeftPoint, RightPoint;
		local rotator RotToPointer;

		if(Enemy == None)
		{
			if(EnemyController == None)
			{
				Throttle = 0;
				Steering = 0;
				Gear = 0;
				OutputBrake = true;
				IsDriving = false;
				return;
			}
			else
			{
				Enemy = EnemyMonitor();
				return;
			}
		}

		PointerLocation = Enemy.Location;
		DistToPointer = VSize(PointerLocation - Location);
        RotToPointer = Rotator(PointerLocation - Location);
		FocusFactor = VSize(Normal(Vector(Rotation)) - Normal(Vector(RotToPointer)));

		LeftPoint = Location + (vect(0,4,0) >> Rotation);
		RightPoint = Location + (vect(0,-4,0) >> Rotation);
		if( VSize(PointerLocation - LeftPoint) < VSize(PointerLocation - RightPoint) )
			SignFactor = -1;
		else SignFactor = 1;

		if( FocusFactor > 1.2 && !bNeedToTurn)
			bNeedToTurn = true;
		else if( FocusFactor < 0.8 )
			bNeedToTurn = false;
		if( FocusFactor > 0.05 )
			FocusSteering = FClamp(FocusFactor * 2, 0.0, 1.0) * SignFactor;

		if( Level.TimeSeconds > AIThinkTimeDuration + AIThinkTime )
		{
			bAIBWGoing = bNeedToTurn;
			AIThinkTimeDuration = Level.TimeSeconds;
		}

		if( bAIBWGoing )
		{
			Steering = -FocusSteering;
			Gear = -1;
			OutputBrake = false;
			IsDriving = true;
			Throttle = -AIThrottle*0.8;
		}
		else if( !bNeedToTurn )
		{
			Steering = FocusSteering;

			if( DistToPointer > AIChasingDist )
			{
				Gear = 1;
				OutputBrake = false;
				IsDriving = true;
				Throttle = AIThrottle;
			}
			else
			{
				Gear = 0;
				OutputBrake = true;
				IsDriving = false;
				Throttle = 0;
			}
		}

		if( DistToPointer < 4096 && FocusFactor < 0.6 )
			VehicleGetFire(true, false);
	}
}

simulated function UpdateBlades()
{
	local Coords ArmBaseCoords, ArmTipCoords;
	local vector HitLocation, HitNormal;
	local Actor Victim;

	if( VSize(Velocity) < 50 )
		return;

	ArmBaseCoords = GetBoneCoords('BladeRight');
	ArmTipCoords = GetBoneCoords('BladeLeft');
	Victim = IntCollision.Trace(HitLocation, HitNormal, ArmTipCoords.Origin, ArmBaseCoords.Origin);

	if( Victim != None && Victim.bBlockActors )
	{
		if( Victim.IsA('Pawn') && !Victim.IsA('NPVehicle') && !Victim.IsA('PostalMultiplayerAirCraftVehicle') )
		{
			Pawn(Victim).TakeDamage(1000, Driver, HitLocation, Velocity * 100, BladesDamageClass);
			//log("left blade going blading");
		}
	}
}

defaultproperties
{
	 VehicleFireSpeed=0.09
	 VehicleAltFireSpeed=2.0

	 MinigunProjClass=class'CarmageddonBullets'
	 RocketProjClass=class'CarmageddonRocket'
	 MinigunMuzzleClass=class'MinigunMuzzleFlashEmitter'

     MinigunFireSound=Sound'ProjectMaricaCore.MinigunFireSound'
	 RocketLaunchSound=Sound'WeaponSounds.rocket_launch'

	 SlipVolume=20.0
	 Tag="Carmageddon Car"
	 ShadowZ=0.0
	 TorqueCurve=(Points=((InVal=0.0,OutVal=40.0),(InVal=650000.0,OutVal=120.0),(InVal=900000.0))

	 //SteerSpeed=31500.000000
	 MaxBrakeTorque=45
	 WheelRadius=37.0
	 ActionAngularDamp=0.0
	 RestAngularDamp=0.7

     EngStartUpSound=None
     EngStopSound=None
     //EngStartUpSound=Sound'MaricaSounds.ut_engine_start_02'
     //EngStopSound=Sound'MaricaSounds.ut_engine_stop_02'
	 ImpactSoundsMin(0)=Sound'MaricaSounds.ut_impact_03'
	 ImpactSoundsMed(0)=Sound'MaricaSounds.ut_impact_01'
	 ImpactSoundsMed(1)=Sound'MaricaSounds.ut_impact_02'
	 ImpactSoundsMax(0)=Sound'MaricaSounds.ut_impact_04'
	 ImpactSoundsMax(1)=Sound'MaricaSounds.ut_impact_05'
	 ImpactSoundsMax(2)=Sound'MaricaSounds.ut_impact_06'
	 ImpactSoundsMax(3)=Sound'MaricaSounds.ut_impact_07'

	 ShadowLightDistance=1200
	 ShadowMaxTraceDistance=300

	 CameraDist=15.0
	 Description="Carmageddon Car. || Model from Carmageddon. mod Originaly made by RigStudios"
	 DestroyBones(0)="Bonnet"
	 DestroyBones(1)="Windows"
	 DestroyBones(2)="Attachment"
	 bAllowBrokenWheels=True
	 bSpawnBrokenDetail=True

	 bFixedFPCamera=False
	 FrontLightMaterial=Texture'GenFX.LensFlar.6'
	 RearLightMaterial=Texture'GenFX.LensFlar.softlens2'
	 RearLightStopScale=0.200000
	 RearLightScale=0.100000
	 FrontLightScale=0.200000
     FrontLightClass=Class'NVFrontCarLight'
     RearLightClass=Class'NVRearCarLight'
     FrontLightPos(0)=(X=216.000000,Y=50.000000,Z=-2.000000)
     FrontLightPos(1)=(X=216.000000,Y=-50.000000,Z=-2.000000)
     FrontLightPos(2)=(X=216.000000,Y=68.000000,Z=-2.000000)
     FrontLightPos(3)=(X=216.000000,Y=-68.000000,Z=-2.000000)
     FrontLightPos(4)=(X=216.000000,Y=28.500000,Z=-27.500000)
     FrontLightPos(5)=(X=216.000000,Y=-28.500000,Z=-27.500000)
     RearLightPos(0)=(X=-220.000000,Y=55.000000,Z=10.000000)
     RearLightPos(1)=(X=-220.000000,Y=-55.000000,Z=10.000000)

     ExhaustClass=Class'CarExhaust'
	 ExhaustPos(0)=(X=-88.000000,Y=86.000000,Z=-45.000000)
	 ExhaustPos(1)=(X=-88.000000,Y=-86.000000,Z=-45.000000)
	 ExhaustPos(2)=(X=-65.000000,Y=86.000000,Z=-45.000000)
	 ExhaustPos(3)=(X=-65.000000,Y=-86.000000,Z=-45.000000)

	 DecoSystem=DS_Bones
	 ExhaustBones(0)="Exhaust00"
	 ExhaustBones(1)="Exhaust01"
	 SteerWheelBone="SteerWheel"

	 FrontLightBones(0)="FrontLight00"
	 FrontLightBones(1)="FrontLight01"

	 RearLightBones(0)="RearLight00"
	 RearLightBones(1)="RearLight01"

	 bUpdateSusp=True
	 RSuspHighLimit=17.000000
	 RSuspLowLimit=-17.000000
     ExitPos=(Y=-140.000000,Z=15.000000)
     ExitPosPassenger=(Y=140.000000,Z=15.000000)
     EnterPos=(Y=145.000000,Z=-5.000000)
	 WheelFrontRight="RightFrontTIRE"
	 WheelFrontLeft="LeftFrontTIRE"
	 WheelRearRight="RightRearTIRE"
	 WheelRearLeft="LeftRearTIRE"
	 StrutFrontRight=""
	 StrutFrontLeft=""
	 StrutRearRight=""
	 StrutRearLeft=""
	 IntCollisionSize=(X=1,Y=1,Z=1)
	 IntCollisionModel=StaticMesh'MaricaRigCarCollision.RigCar_IntColl'
	 IntCollisionClass=Class'IntCollisionActor'

	 LightProjectorClass=class'NVLightProjector'
	 ProjLightPos=(X=180.000000,Y=0.000000,Z=20.000000)
	 ProjectedLightRot=(Pitch=-100)
	 ProjLightMaterial=Texture'MaricaTextures.Effects.LightProj_01'
	 ProjLightFOV=40
	 ProjLightScale=0.400000

	 bCarIsFTWD=False
     bCarIsRTWD=True
     MaxSpeedAccel=30.000000
	 VehicleMaxSpeed=405325.000000
     MaxSteerAngle=3500.000000
     ChassisMass=2.7500000

	 bCanSeeDriver=True
	 myDriverSize=0.85
	 DriverPos=(X=-10.000000,Y=-36.000000,Z=-20.000000)
	 FPDriverViewPos=(X=-9.000000,Y=-39.000000,Z=40.000000)
	 FPPassengerViewPos=(X=-9.000000,Y=39.000000,Z=40.000000)
	 PassengerPos=(X=-10.000000,Y=36.000000,Z=-20.000000)

	 RadiatorPos=(X=125.000000,Y=0.000000,Z=25.000000)
     VehicleFireClass=Class'VehicleFireEmitter'

	 BrokenSkins(0)=Shader'MaricaRigCarTex.SCrashExp'

	 bDamagableWheels=True
	 WheelsHealth=50
	 WheelsMaxHealth=100
	 UnloadWheelHeight=0.013200
	 bSparkWheels=True
     EngTireClass=Class'RigCarEngTire'
     SteerTireClass=Class'RigCarSteerTire'
     WheelSteerAlong=142.000000
     WheelSteerAcross=70.000000
     WheelEngAlong=-130.500000
     WheelEngAcross=73.000000
     WheelVert=-32.0
     SuspStiffness=200.000000
     SuspDamping=5.000000
     SuspHighLimit=0.400000
     SuspLowLimit=-0.400000
     TireLateralFriction=2.000000
	 TireRollSlip=0.060000
	 TireLateralSlip=0.040000
	 TireSlipRate=0.007000

	 EngineVolume=55.0
	 EnginePitch=35.0
     EngineSound=Sound'MaricaSounds.UnWheel_idle10'
	 //Sound'MaricaSounds.UnWheel_idle10'
     ImpactSounds(0)=Sound'MaricaSounds.ut_impact_01'
     ImpactSounds(1)=Sound'MaricaSounds.ut_impact_02'

     StaticMesh=StaticMesh'MaricaRigCarCollision.RigCar_KColl'
	 Mesh=SkeletalMesh'MaricaRigCarModels.CarmageddonCar_Mesh'

	 bRandomSkin=false
	 SkinIndex=0
	 Skins(0)=Shader'MaricaRigCarTex.SCrash'

     Begin Object Class=KarmaParamsRBFull Name=CarmageddonCarPhysics
         KInertiaTensor(0)=4.000000
         KInertiaTensor(3)=25.000000
         KInertiaTensor(5)=45.000000
         KCOMOffset=(X=-0.3,Z=-1.5)
		 KLinearDamping=0.050000
         KAngularDamping=0.700000
         KStartEnabled=True
         bKNonSphericalInertia=True
         KFriction=0.200000
         KImpactThreshold=250.000000
         Name="CarmageddonCarPhysics"
     End Object
     KParams=KarmaParamsRBFull'CarmageddonCarPhysics'
}