//=============================================================================
// Segway.
//
// Base model: model from Postal III
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2014.
//=============================================================================

class Segway extends NKVCarNew;

var() Name RootBone;
var MotionRotatorActor RootRotator;
var float NETThrottle;

replication
{
	unreliable if( Role == ROLE_Authority )
		RootRotator , NETThrottle;
}

static final function int sign(float data)
{
	if( data < 0 )
		return -1;
	return 1;
}

simulated function GetVehicleData(optional bool bOnlyPositon)
{
	if( Role == ROLE_Authority )
		NETThrottle = Throttle;
	Super.GetVehicleData(bOnlyPositon);
}

function SimulateUpright()
{
	local int TempRot;
	if( Driver != None )
	{
		TempRot = 8000;
		if( (Rotation.Roll > TempRot || Rotation.Roll < -TempRot) 
			|| (Rotation.Pitch > TempRot || Rotation.Pitch < -TempRot) )
		{
			KSetStayUpright(true, true);
		}
		else KSetStayUpright(false, false);
	}
}

simulated function Destroyed()
{
	Super.Destroyed();

	if( RootRotator != None )
		RootRotator.Destroy();
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	//if(Level.NetMode != NM_Client)
		RootRotator = spawn(class'MotionRotatorActor');
}

simulated function ContinuousUpdate()
{
	Super.ContinuousUpdate();
	//SimulateUpright();
	if( RootBone != '')
	{
		if( DriverController != None )
			RotateRoot(DriverController.bBehindView);
		else if( bAIControlled )
			RotateRoot(true);
		else RotateRoot(false);
	}
}

simulated function RotateRoot(bool bFixed)
{
	local rotator TempRotation;

	if( !bFixed )
	{
		if( RootRotator != None )
		{
			if( NETThrottle < -0.1 )//if( Throttle < -0.1 )
			{
				RootRotator.DesiredRotation.Pitch = 1700;
				RootRotator.RotationRate.Pitch = 15000;
			}
			else if( NETThrottle > 0.1 )//else if( Throttle > 0.1 )
			{
				RootRotator.DesiredRotation.Pitch = -1700;
				RootRotator.RotationRate.Pitch = -15000;
			}
			else
			{
				if( Normalize(RootRotator.Rotation).Pitch > 0 )
					RootRotator.RotationRate.Pitch = -15000;
				else if( Normalize(RootRotator.Rotation).Pitch < 0 )
					RootRotator.RotationRate.Pitch = 15000;
				RootRotator.DesiredRotation.Pitch = 0;
			}
			TempRotation.Pitch = RootRotator.Rotation.Pitch;
			SetBoneRotation(RootBone,TempRotation,0,1);
		}
	}
	else
	{
		SetBoneRotation(RootBone,rot(0,0,0),0,1);
	}
}

state VehicleDriving
{
	function ScriptedDriving()
	{
		local float DistTopointer, FocusFactor, SignFactor;
		local bool bNeedToTurn;
		local vector PointerLocation, LeftPoint, RightPoint;
		local rotator RotToPointer;

		if(Enemy == None)
		{
			if(EnemyController == None)
			{
				Throttle = 0;
				Steering = 0;
				Gear = 0;
				OutputBrake = true;
				IsDriving = false;
				return;
			}
			else
			{
				Enemy = EnemyMonitor();
				return;
			}
		}

		PointerLocation = Enemy.Location;
		DistToPointer = VSize(PointerLocation - Location);
        RotToPointer = Rotator(PointerLocation - Location);
		FocusFactor = VSize(Normal(Vector(Rotation)) - Normal(Vector(RotToPointer)));

		LeftPoint = Location + (vect(0,4,0) >> Rotation);
		RightPoint = Location + (vect(0,-4,0) >> Rotation);
		if( VSize(PointerLocation - LeftPoint) < VSize(PointerLocation - RightPoint) )
			SignFactor = -1;
		else SignFactor = 1;

		if( FocusFactor > 1.2 && !bNeedToTurn)
			bNeedToTurn = true;
		else if( FocusFactor < 0.8 )
			bNeedToTurn = false;
		if( FocusFactor > 0.05 )
			FocusSteering = FClamp(FocusFactor * 2, 0.0, 1.0) * SignFactor;

		if( Level.TimeSeconds > AIThinkTimeDuration + AIThinkTime )
		{
			bAIBWGoing = bNeedToTurn;
			AIThinkTimeDuration = Level.TimeSeconds;
		}

		if( bAIBWGoing )
		{
			Steering = -FocusSteering;
			Gear = -1;
			OutputBrake = false;
			IsDriving = true;
			Throttle = -AIThrottle*0.8;
		}
		else if( !bNeedToTurn )
		{
			Steering = FocusSteering;

			if( DistToPointer > AIChasingDist )
			{
				Gear = 1;
				OutputBrake = false;
				IsDriving = true;
				Throttle = AIThrottle;
			}
			else
			{
				Gear = 0;
				OutputBrake = true;
				IsDriving = false;
				Throttle = 0;
			}
		}
	}

	function DrivingSimulation(float DesiredSpeed)
	{
		local float ForwardSpeed , RotSteer , RotAng , Torque , BrakeFactor;
		local vector worldForward;
		local int FrictionFactor;

		worldForward = vect(1,0,0) >> Rotation;
		ForwardSpeed = Velocity Dot worldForward;
		WheelSpinSpeed = (EngTireRight.SpinSpeed + EngTireLeft.SpinSpeed)/2;

		if(Driver == None && !bAIControlled)
			return;

		if(DesiredSpeed > 1100)
			RotAng = 1.0;
		else RotAng = 1.5;

		BrakeFactor = 1;

		if( ForwardSpeed < -1.0 )
		{
			EngTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed * 0.6;
			EngTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed * 0.6;
			SteerTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed * 0.6;
			SteerTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed * 0.6;
		}
		else
		{
			if( EngTireRight.WheelJoint.KMaxSpeed != VehicleMaxSpeed )
			{
				EngTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				EngTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				SteerTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				SteerTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
			}
		}

		Torque = EngineSimulation(Throttle) * 0.5;
		EngineModel(ForwardSpeed, Torque);

		EngTireRight.WheelJoint.KMotorTorque = Torque;
		EngTireLeft.WheelJoint.KMotorTorque = Torque;

		RotSteer = Steering * MaxSteerAngle * RotAng;
		SteerTireRight.WheelJoint.KSteerAngle = RotSteer;
		SteerTireLeft.WheelJoint.KSteerAngle = RotSteer;

		//EngTireRight.WheelJoint.KBraking = MaxBrakeTorque * Steering;
		//EngTireLeft.WheelJoint.KBraking = MaxBrakeTorque * Steering;

		if(Throttle > 0.01)
		{
			if(!bAIControlled)
			{
				if(ForwardSpeed < -StopThreshold && Gear != 1)
				{
					Gear = 0;
					OutputBrake = true;
					IsDriving = false;
				}
				else
				{
					Gear = 1;
					OutputBrake = false;
					IsDriving = true;
				}
			}

			if(bCarIsRTWD)
			{
				EngTireRight.WheelJoint.KWake();
				EngTireLeft.WheelJoint.KWake();
				EngTireRight.KWake();
				EngTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
			if(bCarIsFTWD)
			{
				SteerTireRight.WheelJoint.KWake();
				SteerTireLeft.WheelJoint.KWake();
				SteerTireRight.KWake();
				SteerTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
			FrictionFactor = 1;
		}
		else if(Throttle < -0.01)
		{
			if(!bAIControlled)
			{
				if(ForwardSpeed < StopThreshold && IsDriving == false)
				{
					Gear = -1;
					OutputBrake = false;
					IsDriving = false;
				}
				else
				{
					Gear = 0;
					OutputBrake = true;
					IsDriving = true;
				}
			}

			if(bCarIsRTWD)
			{
				EngTireRight.WheelJoint.KWake();
				EngTireLeft.WheelJoint.KWake();
				EngTireRight.KWake();
				EngTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
			if(bCarIsFTWD)
			{
				SteerTireRight.WheelJoint.KWake();
				SteerTireLeft.WheelJoint.KWake();
				SteerTireRight.KWake();
				SteerTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
			FrictionFactor = 1;
		}
		else
		{
			if( abs(Steering) > 0 )
			{
				Gear = 1;
				OutputBrake = false;
				IsDriving = true;
				Torque = EngineSimulation(Steering) * 0.5;
				EngineModel(ForwardSpeed, Torque);

				BrakeFactor = 2;

				EngTireRight.WheelJoint.KMotorTorque = sign(Steering)*Torque;
				EngTireLeft.WheelJoint.KMotorTorque = -sign(Steering)*Torque;
				FrictionFactor = 0;
			}
			else if( !bAIControlled )
			{
				if(Abs(ForwardSpeed) < StopThreshold)
				{
					Gear = 0;
					OutputBrake = true;
					IsDriving = false;
					OutputHandbrakeOn = false;
				}
				else
				{
					Gear = 0;
					OutputBrake = false;
					IsDriving = false;
				}
				FrictionFactor = 1;
			}
		}

		EngTireRight.RollFriction = TireRollFriction * BrakeFactor;
		EngTireLeft.RollFriction = TireRollFriction * BrakeFactor;

		if((ForwardSpeed > HandbrakeThresh || OutputHandbrakeOn == true) 
				&& Abs(Steering) > 0.01 
				&& OutputBrake == true)
			OutputHandbrakeOn = true;
		else
			OutputHandbrakeOn = false;

		if(OutputBrake)
		{
			EngTireRight.WheelJoint.KBraking = MaxBrakeTorque;
			EngTireLeft.WheelJoint.KBraking = MaxBrakeTorque;
			SteerTireRight.WheelJoint.KBraking = MaxBrakeTorque;
			SteerTireLeft.WheelJoint.KBraking = MaxBrakeTorque;
		}
		else
		{
			EngTireRight.WheelJoint.KBraking = 0.0;
			EngTireLeft.WheelJoint.KBraking = 0.0;
			SteerTireRight.WheelJoint.KBraking = 0.0;
			SteerTireLeft.WheelJoint.KBraking = 0.0;
		}

		if(OutputHandbrakeOn == true)
		{
			SteerTireRight.LateralFriction = TireLateralFriction + (TireHandbrakeFriction/2);
			SteerTireRight.LateralSlip = TireLateralSlip + TireHandbrakeSlip;

			SteerTireLeft.LateralFriction = TireLateralFriction + (TireHandbrakeFriction/2);
			SteerTireLeft.LateralSlip = TireLateralSlip + TireHandbrakeSlip;

			EngTireRight.LateralFriction = TireLateralFriction + TireHandbrakeFriction;
			EngTireRight.LateralSlip = TireLateralSlip + TireHandbrakeSlip;

			EngTireLeft.LateralFriction = TireLateralFriction + TireHandbrakeFriction;
			EngTireLeft.LateralSlip = TireLateralSlip + TireHandbrakeSlip;
		}
		else
		{
			SteerTireRight.LateralFriction = TireLateralFriction * FrictionFactor;
			SteerTireRight.LateralSlip = TireLateralSlip;

			SteerTireLeft.LateralFriction = TireLateralFriction * FrictionFactor;
			SteerTireLeft.LateralSlip = TireLateralSlip;

			EngTireRight.LateralFriction = TireLateralFriction;
			EngTireRight.LateralSlip = TireLateralSlip;

			EngTireLeft.LateralFriction = TireLateralFriction;
			EngTireLeft.LateralSlip = TireLateralSlip;
		}

		KWake();
		UpdateRearLight();
	}
}

state BeforeBeStart
{
Begin:
	KSetStayUpright(true, true);
    Sleep(EngineStartTime);
	KSetStayUpright(false, false);
    VehicleHaveDriver();
	GotoState('VehicleDriving');
}

simulated function MonitorBehindView(PlayerController PC, bool bDriver)
{
	local Actor myPawnActor, ViewPoint;
	local DriverBase PawnBase;
	local Pawn myPawn;
	if(PC == None)
		return;

	if(bDriver)
	{
		ViewPoint = FPDriverViewPoint;
		PawnBase = DriverBase;
		myPawnActor = myDriver;
		myPawn = Driver;
	}
	else
	{
		ViewPoint = FPPassengerViewPoint;
		PawnBase = PassengerBase;
		myPawnActor = myPassenger;
		myPawn = Passenger;
	}

	if(!bFixedFPCamera)
		ViewPoint.SetRotation(PC.Rotation);

	//if( SteerWheelBone != '')
	//	RotateSteeringWheel(PC.bBehindView);

	if(bAllowChangeView)
	{
		if( bNormalHUDView )
		{
			if( !PC.bBehindView )
				myPawnActor.bHidden = true;
			else myPawnActor.bHidden = false;
		}

		if(!PC.bBehindView)// && !ViewChanged)
		{
			PC.FixFOV();
			bHideDriverActor = true;
			if( myPawnActor != None && !bNormalHUDView )
				myPawnActor.bHidden = bHideDriverActor;
			//if( !bNormalHUDView )
				//PawnBase.SetHardAttach(true);
				PawnBase.SetFixedPosition(true);
			if(bFreeCamera)
				myPawn.SetBase(PawnBase);
			if( !bNormalHUDView )
				PC.ClientSetViewTarget(ViewPoint);
			else PC.ClientSetViewTarget(myPawn);
			PC.bFreeCamera = false;
			//ViewChanged = true;
		}
		else if(PC.bBehindView)// && ViewChanged)
		{
			bHideDriverActor = false;
			if( myPawnActor != None && !bNormalHUDView )
				myPawnActor.bHidden = bHideDriverActor;
			//if( !bNormalHUDView )
				//PawnBase.SetHardAttach(false);
				PawnBase.SetFixedPosition(false);
			if(bFreeCamera)
				myPawn.SetBase(PawnBase);
			PC.ClientSetViewTarget(self);
			//ViewChanged = false;
		}
	}
	else if(!PC.bBehindView)
		PC.ClientSetBehindView(true);
}

/*
	 SuspDamping=1.0
	 SuspStiffness=0.0
     SuspHighLimit=0.05
     SuspLowLimit=-4

	 SuspDamping=5.0
	 SuspStiffness=160.0
     SuspHighLimit=0.05
     SuspLowLimit=-0.05
*/

defaultproperties
{
	 bLittleVehicle=true
	 SuspDamping=5.0
	 SuspStiffness=160.0
     SuspHighLimit=0.05
     SuspLowLimit=-0.05

	 bDamagableWheels=false
	 bSparkWheels=false
     DrivingAnims=MeshAnimation'Characters.animAvg'
     DrivingAnimStatus="s_arcade"
     SittingAnimStatus="s_arcade"
	 EngStartUpSound=None
	 EngStopSound=None
	 EngineStartTime=0.1
	 RootBone="Root"
	 bUpdateSusp=false
	 AIChasingDist=100.0
	 bNormalHUDView=true
	 bAllowWeaponsInside=true
	 bDriverTakesDamage=true
	 Tag="Segway"
	 ShadowZ=0.0

	 TorqueCurve=(Points=((InVal=0.0,OutVal=50.0),(InVal=550000.0,OutVal=100.0),(InVal=800000.0))

	 WheelRadius=20.0
	 ActionAngularDamp=0.0
	 RestAngularDamp=5.01

	 ImpactSoundsMin(0)=Sound'MaricaSounds.ut_impact_03'
	 ImpactSoundsMed(0)=Sound'MaricaSounds.ut_impact_01'
	 ImpactSoundsMed(1)=Sound'MaricaSounds.ut_impact_02'
	 ImpactSoundsMax(0)=Sound'MaricaSounds.ut_impact_04'
	 ImpactSoundsMax(1)=Sound'MaricaSounds.ut_impact_05'
	 ImpactSoundsMax(2)=Sound'MaricaSounds.ut_impact_06'
	 ImpactSoundsMax(3)=Sound'MaricaSounds.ut_impact_07'

	 ShadowLightDistance=1200
	 ShadowMaxTraceDistance=230

	 CameraDist=10.0
	 bFixedFPCamera=False
	 bAllowPassenger=False
	 Description="Segway from Postal III. || Model by RWS / TrashMasters Studio."

	 DecoSystem=DS_Bones

	 RSuspHighLimit=24.000000
	 RSuspLowLimit=-24.000000
     ExitPos=(Y=-80.000000,Z=15.000000)
     ExitPosPassenger=(Y=80.000000,Z=15.000000)
     EnterPos=(Y=70.000000,Z=-5.000000)
	 WheelFrontRight="Wheel_RR"
	 WheelFrontLeft="Wheel_RL"
	 WheelRearRight="Wheel_FR"
	 WheelRearLeft="Wheel_FL"
	 IntCollisionSize=(X=1,Y=1,Z=1)
	 IntCollisionModel=StaticMesh'MaricaSegwayCollision.Segway_IntColl'
	 IntCollisionClass=Class'IntCollisionActor'
	 LightProjectorClass=None
	 ProjLightPos=(X=120.000000,Y=0.000000,Z=-17.000000)
	 ProjLightMaterial=None
	 ProjLightFOV=40
	 ProjLightScale=0.400000
	 ProjectedLightRot=(Pitch=-100)
     FrontLightClass=Class'NVFrontCarLight'
     RearLightClass=Class'NVRearCarLight'
     bSpawnBrokenDetail=True
     bAllowBrokenWheels=True
     MaxSpeedAccel=26.000000
	 bCanSeeDriver=True
	 VehicleMaxSpeed=525350.000000
     MaxSteerAngle=3500.000000
	 //SteerSpeed=31500.000000
	 //SteerSpeed=51500.000000
     ChassisMass=3.000000
	 RadiatorPos=(X=5.000000,Y=0.000000,Z=-17.000000)
	 DriverPos=(X=-10.000000,Y=0.000000,Z=70.000000)
	 FPDriverViewPos=(X=-15.000000,Y=0.000000,Z=30.000000)
	 FPPassengerViewPos=(X=-10.000000,Y=0.000000,Z=30.000000)
	 PassengerPos=(X=-50.000000,Y=0.000000,Z=70.000000)
     ExhaustClass=Class'CarExhaust'
     VehicleFireClass=Class'VehicleFireEmitter'
	 WheelsHealth=150
	 WheelsMaxHealth=200
	 UnloadWheelHeight=0.013200
     EngTireClass=Class'SegwayTire'
     SteerTireClass=Class'SegwayTire'
     WheelSteerAlong=30.0
     WheelSteerAcross=32.0
     WheelEngAlong=-30.0
     WheelEngAcross=32.0
     WheelVert=0.0
     TireRollFriction=1.500000
     TireLateralFriction=2.000000
	 TireRollSlip=0.060000
	 TireLateralSlip=0.040000
	 TireSlipRate=0.007000
	 EngineVolume=100
	 EnginePitch=32
     EngineSound=Sound'MaricaSegwaySounds.segway_engine_snd'

     StaticMesh=StaticMesh'MaricaSegwayCollision.Segway_KColl'
	 Mesh=SkeletalMesh'MaricaSegwayModels.Segway_Model'
	 DrawScale=1.0

	 bRandomSkin=false
	 SkinIndex=0
	 Skins(0)=Texture'MaricaSegwayTex.Segway_00_detail_01'
	 OtherSkins(0)=Texture'MaricaSegwayTex.Segway_00_detail_01'

	 //SteerWheelBone="SteerWheel"

	 BrokenSkins(0)=Texture'MaricaSegwayTex.Segway_00_detail_01'

     Begin Object Class=KarmaParamsRBFull Name=SegwayPhysics
         KInertiaTensor(0)=1.0
         KInertiaTensor(3)=4.0
         KInertiaTensor(5)=3.0
         KCOMOffset=(X=0.0,Z=-1.0)
		 //bKStayUpright=true
		 //bKAllowRotate=true
		 KLinearDamping=0.050000
         KAngularDamping=0.600000
         KStartEnabled=True
         bKNonSphericalInertia=True
         KFriction=0.200000
         KImpactThreshold=250.000000
         Name="SegwayPhysics"
     End Object
     KParams=KarmaParamsRBFull'SegwayPhysics'
}