//=============================================================================
// SuperMotoXLCar.
//
// Base model: model by SuperMotoXL, Kiop.
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2014.
//=============================================================================

class SuperMotoXLCar extends NKVCarNew;

var MaricaDecorActor DecorMeshActor;
var() SkeletalMesh DecorMesh;
var() float TorqueSplit;

replication
{
	unreliable if( Role == ROLE_Authority )
		DecorMeshActor;
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	if( Level.NetMode != NM_Client )
	{
		if( DecorMesh != None )
		{
			DecorMeshActor = Spawn(class'MaricaDecorActor',self,,Location, Rotation);
			DecorMeshActor.LinkMesh(DecorMesh);
			DecorMeshActor.SetBase(self);
		}
	}
}

simulated event ClientSimulationPack(optional bool LowRender)
{
	Super.ClientSimulationPack(LowRender);
	UpdateDecorActors();
}

simulated function UpdateDecorActors()
{
	if( DecorMeshActor != None )
	{
		DecorMeshActor.SetLocation(Location);
		DecorMeshActor.SetBase(self);
		DecorMeshActor.SetRotation(Rotation);
	}
}

simulated function Destroyed()
{
	Super.Destroyed();

	if( DecorMeshActor != None )
		DecorMeshActor.Destroy();
}


state VehicleDriving
{
	function EngineModel(float ForwardSpeed, float Torque)
	{
		local float RPM;

		if( IntCollision != None && IntCollision.AmbientSound != None )
		{
			RPM = abs(EnginePitch + (abs(ForwardSpeed) / (EnginePitch + 45)) + 0.1*abs(Torque));
			IntCollision.SoundPitch = FClamp(RPM,0,200);
		}
	}

	function DrivingSimulation(float DesiredSpeed)
	{
		local float ForwardSpeed , RotSteer , RotAng , Torque ;
		local vector worldForward;

		worldForward = vect(1,0,0) >> Rotation;
		ForwardSpeed = Velocity Dot worldForward;
		WheelSpinSpeed = (EngTireRight.SpinSpeed + EngTireLeft.SpinSpeed)/2;

		if(Driver == None && !bAIControlled)
			return;

		/*if(DesiredSpeed > 1100)
			RotAng = 1.0;
		else */RotAng = 1.5;

		if( ForwardSpeed < -1.0 )
		{
			EngTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed * 0.6;
			EngTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed * 0.6;
			SteerTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed * 0.6;
			SteerTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed * 0.6;
		}
		else
		{
			if( EngTireRight.WheelJoint.KMaxSpeed != VehicleMaxSpeed )
			{
				EngTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				EngTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				SteerTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				SteerTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
			}
		}

		Torque = EngineSimulation(Throttle) * 0.5;
		EngineModel(ForwardSpeed, Torque);

		if(bCarIsRTWD)
		{
			EngTireRight.WheelJoint.KMotorTorque = Torque * TorqueSplit;
			EngTireLeft.WheelJoint.KMotorTorque = Torque * TorqueSplit;
			if( CrossTireClass != None )
			{
				CrossTireRight.WheelJoint.KMotorTorque = Torque;
				CrossTireLeft.WheelJoint.KMotorTorque = Torque;
			}
		}
		if(bCarIsFTWD)
		{
			SteerTireRight.WheelJoint.KMotorTorque = Torque * (1-TorqueSplit);
			SteerTireLeft.WheelJoint.KMotorTorque = Torque * (1-TorqueSplit);
			if( CrossTireClass != None )
			{
				CrossTireRight.WheelJoint.KMotorTorque = Torque;
				CrossTireLeft.WheelJoint.KMotorTorque = Torque;
			}
		}

		RotSteer = Steering * MaxSteerAngle * RotAng;
		SteerTireRight.WheelJoint.KSteerAngle = RotSteer;
		SteerTireLeft.WheelJoint.KSteerAngle = RotSteer;

		if(Throttle > 0.01)
		{
			if(!bAIControlled)
			{
				if(ForwardSpeed < -StopThreshold && Gear != 1)
				{
					Gear = 0;
					OutputBrake = true;
					IsDriving = false;
				}
				else
				{
					Gear = 1;
					OutputBrake = false;
					IsDriving = true;
				}
			}

			if(bCarIsRTWD)
			{
				EngTireRight.WheelJoint.KWake();
				EngTireLeft.WheelJoint.KWake();
				EngTireRight.KWake();
				EngTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
			if(bCarIsFTWD)
			{
				SteerTireRight.WheelJoint.KWake();
				SteerTireLeft.WheelJoint.KWake();
				SteerTireRight.KWake();
				SteerTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
		}
		else if(Throttle < -0.01)
		{
			if(!bAIControlled)
			{
				if(ForwardSpeed < StopThreshold && IsDriving == false)
				{
					Gear = -1;
					OutputBrake = false;
					IsDriving = false;
				}
				else
				{
					Gear = 0;
					OutputBrake = true;
					IsDriving = true;
				}
			}

			if(bCarIsRTWD)
			{
				EngTireRight.WheelJoint.KWake();
				EngTireLeft.WheelJoint.KWake();
				EngTireRight.KWake();
				EngTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
			if(bCarIsFTWD)
			{
				SteerTireRight.WheelJoint.KWake();
				SteerTireLeft.WheelJoint.KWake();
				SteerTireRight.KWake();
				SteerTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
		}
		else
		{
			if(!bAIControlled)
			{
				if(Abs(ForwardSpeed) < StopThreshold)
				{
					Gear = 0;
					OutputBrake = true;
					IsDriving = false;
					OutputHandbrakeOn = false;
				}
				else
				{
					Gear = 0;
					OutputBrake = false;
					IsDriving = false;
				}
			}
		}

		if( DriverController != None && DriverController.bWantsToSkip > 0 )
		{
			Gear = 0;
			OutputBrake = true;
			IsDriving = true;
		}

		if((ForwardSpeed > HandbrakeThresh || OutputHandbrakeOn == true) 
				&& Abs(Steering) > 0.01 
				&& OutputBrake == true)
			OutputHandbrakeOn = true;
		else
			OutputHandbrakeOn = false;

		if(OutputBrake)
		{
			EngTireRight.WheelJoint.KBraking = MaxBrakeTorque;
			EngTireLeft.WheelJoint.KBraking = MaxBrakeTorque;
			SteerTireRight.WheelJoint.KBraking = MaxBrakeTorque;
			SteerTireLeft.WheelJoint.KBraking = MaxBrakeTorque;
		}
		else
		{
			EngTireRight.WheelJoint.KBraking = 0.0;
			EngTireLeft.WheelJoint.KBraking = 0.0;
			SteerTireRight.WheelJoint.KBraking = 0.0;
			SteerTireLeft.WheelJoint.KBraking = 0.0;
		}

		if(OutputHandbrakeOn == true)
		{
			SteerTireRight.LateralFriction = TireLateralFriction + (TireHandbrakeFriction/2);
			SteerTireRight.LateralSlip = TireLateralSlip + TireHandbrakeSlip;

			SteerTireLeft.LateralFriction = TireLateralFriction + (TireHandbrakeFriction/2);
			SteerTireLeft.LateralSlip = TireLateralSlip + TireHandbrakeSlip;

			EngTireRight.LateralFriction = TireLateralFriction + TireHandbrakeFriction;
			EngTireRight.LateralSlip = TireLateralSlip + TireHandbrakeSlip;

			EngTireLeft.LateralFriction = TireLateralFriction + TireHandbrakeFriction;
			EngTireLeft.LateralSlip = TireLateralSlip + TireHandbrakeSlip;
		}
		else
		{
			SteerTireRight.LateralFriction = TireLateralFriction;
			SteerTireRight.LateralSlip = TireLateralSlip;

			SteerTireLeft.LateralFriction = TireLateralFriction;
			SteerTireLeft.LateralSlip = TireLateralSlip;

			EngTireRight.LateralFriction = TireLateralFriction;
			EngTireRight.LateralSlip = TireLateralSlip;

			EngTireLeft.LateralFriction = TireLateralFriction;
			EngTireLeft.LateralSlip = TireLateralSlip;
		}

		KWake();
		UpdateRearLight();
	}
}


defaultproperties
{
	 TorqueSplit=0.2
	 Tag="SuperMotoXL"
	 DecorMesh=SkeletalMesh'MaricaSilverDragonModels.SuperMotoXL_Decor_Mesh'
	 ExhaustBones(0)="Exhaust00"
	 ExhaustBones(1)="Exhaust01"
	 TorqueCurve=(Points=((InVal=0.0,OutVal=200.0),(InVal=550000.0,OutVal=400.0),(InVal=800000.0))
	 ShadowZ=-20.0

	 SlipVolume=17.0
	 SteerSpeed=31500.000000
	 MaxSteerAngle=3400.000000
	 MaxBrakeTorque=70

     TireHandbrakeSlip=0.5
     TireHandbrakeFriction=-1.0
	 WheelRadius=66.0
	 CameraDist=16.5
	 bFixedFPCamera=False
     ExitPos=(Y=-170.000000,Z=5.000000)
     ExitPosPassenger=(Y=170.000000,Z=5.000000)
     DriverPos=(X=10.0,Y=-40.0,Z=26.000000)
     PassengerPos=(X=10.0,Y=40.0,Z=26.000000)
     FPDriverViewPos=(X=12.000000,Y=-40.000000,Z=100.000000)
	 FPPassengerViewPos=(X=12.000000,Y=40.000000,Z=100.000000)
     EnterPos=(Y=160.000000,Z=-2.000000)
     bCanSeeDriver=True
     ChassisMass=5.000000

     EnginePitch=32.000000
     EngineVolume=95.000000
     Description="SuperMotoXL. || Model by SuperMotoXL, Kiop."
     LightProjectorClass=class'NVLightProjector'
	 ProjLightPos=(X=80.000000,Y=0.000000,Z=20.000000)
	 ProjectedLightRot=(Pitch=-100)
	 ProjLightMaterial=Texture'MaricaTextures.Effects.LightProj_01'
	 ProjLightFOV=40
	 ProjLightScale=0.400000
     ExhaustPos(0)=(X=-165.000000,Y=33.000000,Z=-88.000000)
     RadiatorPos=(X=170.0,Z=70.0)
     FrontLightPos(0)=(X=185.000000,Y=50.000000,Z=10.000000)
     FrontLightPos(1)=(X=185.000000,Y=-50.000000,Z=10.000000)
     FrontLightPos(2)=(X=185.000000,Y=75.000000,Z=10.000000)
     FrontLightPos(3)=(X=185.000000,Y=-75.000000,Z=10.000000)
     RearLightPos(0)=(X=-218.000000,Y=98.000000,Z=35.000000)
     RearLightPos(1)=(X=-218.000000,Y=-98.000000,Z=35.000000)

     bRandomSkin=false
	 SkinIndex=0

     Health=4096.000000
     MaxHealth=4096.000000
     ImpactSoundsMin(0)=Sound'MaricaSounds.ut_impact_03'
     ImpactSoundsMed(0)=Sound'MaricaSounds.ut_impact_01'
     ImpactSoundsMed(1)=Sound'MaricaSounds.ut_impact_02'
     ImpactSoundsMax(0)=Sound'MaricaSounds.ut_impact_04'
     ImpactSoundsMax(1)=Sound'MaricaSounds.ut_impact_05'
     ImpactSoundsMax(2)=Sound'MaricaSounds.ut_impact_06'
     ImpactSoundsMax(3)=Sound'MaricaSounds.ut_impact_07'
     EngineSound=Sound'MaricaSounds.UnWheel_idle13'
     EngStartUpSound=None
     EngStopSound=None
     //EngStartUpSound=Sound'MaricaSounds.ut_engine_start_01'
     //EngStopSound=Sound'MaricaSounds.ut_engine_stop_01'
     MaxSpeedAccel=34.000000
     VehicleMaxSpeed=600300.000000
     MaxRearSpeed=800.000000
     TireMass=1.0
     WheelsHealth=200.000000
     WheelsMaxHealth=200.000000
     TireLateralFriction=1.250000
	 TireRollSlip=0.060000
	 TireLateralSlip=0.040000
	 TireSlipRate=0.007000
     EngTireClass=Class'SuperMotoXLCarTire'
     SteerTireClass=Class'SuperMotoXLCarTire'
     WheelSteerAlong=147.0
     WheelSteerAcross=140.0
     WheelEngAlong=-147.0
     WheelEngAcross=140.0
     WheelVert=-74.0

     StrutFrontRight=""
     StrutFrontLeft=""
     StrutRearRight=""
     StrutRearLeft=""
     DestroyBones(0)="RightFrontDOOR"
     DestroyBones(1)="RightRearDOOR"
     DestroyBones(2)="LeftFrontDOOR"
     DestroyBones(3)="LeftRearDOOR"
     DestroyBones(4)="Windows"
     DestroyBones(5)="Attachment"
	 DestroyBones(6)="SteerWheel"

	 SteerWheelBone="SteerWheel"
     RSuspHighLimit=24.000000
     RSuspLowLimit=-24.000000
	 SuspStiffness=200.0
	 SuspDamping=5.0

     IntCollisionModel=StaticMesh'MaricaSilverDragonCollision.SuperMotoXL_IntColl'
     FrontLightBones(0)="FrontLight00"
     FrontLightBones(1)="FrontLight01"
     RearLightBones(0)="RearLight00"
     RearLightBones(1)="RearLight01"

     ImpactSounds(0)=Sound'ProjectMaricaCore.ut_impact_01'
     ImpactSounds(1)=Sound'ProjectMaricaCore.ut_impact_01'

     Mesh=SkeletalMesh'MaricaSilverDragonModels.SuperMotoXL_Mesh'
     StaticMesh=StaticMesh'MaricaSilverDragonCollision.SuperMotoXL_KColl'

     Skins(0)=Texture'MaricaSilverDragonTex.SuperMotoXL.SupMotXL_remap'

	 BrokenSkins(0)=Texture'MaricaSilverDragonTex.SuperMotoXL.SupMotXL_remap'

     Mass=800.000000

     Begin Object Class=KarmaParamsRBFull Name=SuperMotoXLCarPhysics
         //KInertiaTensor(0)=20.000000
         //KInertiaTensor(3)=30.000000
         //KInertiaTensor(5)=58.000000
         KInertiaTensor(0)=6.000000
         KInertiaTensor(3)=25.000000
         KInertiaTensor(5)=45.000000
         KCOMOffset=(X=-0.3,Z=-1.5)
         KLinearDamping=0.050000
         KAngularDamping=0.700000
         KStartEnabled=True
         bKNonSphericalInertia=True
         KFriction=0.200000
         Name="SuperMotoXLCarPhysics"
     End Object
     KParams=KarmaParamsRBFull'SuperMotoXLCarPhysics'
}
