//=============================================================================
// T2000Car.uc
// Base model: model by SCS,Matheus.
// Author: Nick Pilshikov "Evil"
// Revival Games Studios.
// 2014.
//=============================================================================

class T2000Car extends NKVCarNew;

var MaricaDecorActor DecorMeshActor;
var TruckSoundIndicator TruckSoundIndicator;
var() SkeletalMesh DecorMesh;

replication
{
	unreliable if( Role == ROLE_Authority )
		DecorMeshActor;
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	if( Level.NetMode != NM_Client )
	{
		if( DecorMesh != None )
		{
			DecorMeshActor = Spawn(class'MaricaDecorActor',self,,Location, Rotation);
			DecorMeshActor.LinkMesh(DecorMesh);
			DecorMeshActor.SetBase(self);
		}
	}
	if( TruckSoundIndicator == None )
		TruckSoundIndicator = spawn(class'TruckSoundIndicator',self,,Location,Rotation);
	if( TruckSoundIndicator != None )
	{
		TruckSoundIndicator.SetLocation(Location);
		TruckSoundIndicator.SetBase(self);
	}
}

simulated event ClientSimulationPack(optional bool LowRender)
{
	Super.ClientSimulationPack(LowRender);
	UpdateDecorActors();
}

simulated function UpdateDecorActors()
{
	if( DecorMeshActor != None )
	{
		DecorMeshActor.SetLocation(Location);
		DecorMeshActor.SetBase(self);
		DecorMeshActor.SetRotation(Rotation);
	}
}

simulated function Destroyed()
{
	Super.Destroyed();

	if( DecorMeshActor != None )
		DecorMeshActor.Destroy();

	if( TruckSoundIndicator != None )
		TruckSoundIndicator.Destroy();
}

simulated function ContinuousUpdate()
{
	if( TruckSoundIndicator != None )
	{
		if( Driver != None )
		{
			if( Throttle < -0.1 && Gear == -1 )
				TruckSoundIndicator.SoundVolume = 150;
			else TruckSoundIndicator.SoundVolume = 0;
		}
		if( Driver == None || Health <= 0 )
			TruckSoundIndicator.SoundVolume = 0;
	}
	Super.ContinuousUpdate();
}

defaultproperties
{
	 Tag="KenWorth T2000 Truck"
	 ShadowZ=0.0
	 TorqueCurve=(Points=((InVal=0.0,OutVal=40.0),(InVal=650000.0,OutVal=110.0),(InVal=900000.0))

	 bTrailersVehicle=true
	 TrailerJointPos=(X=-250.0)
	 RearMoveAccel=0.5
	 SlipVolume=8.0
	 CameraDistMin=15.0
	 DecorMesh=SkeletalMesh'MaricaSilverDragonModels.T2000_Decor_Mesh'
	 //SteerSpeed=31500.000000
	 MaxBrakeTorque=60
	 WheelRadius=49.0
	 ActionAngularDamp=0.0
	 RestAngularDamp=0.7

	 ImpactSoundsMin(0)=Sound'MaricaSounds.ut_impact_03'
	 ImpactSoundsMed(0)=Sound'MaricaSounds.ut_impact_01'
	 ImpactSoundsMed(1)=Sound'MaricaSounds.ut_impact_02'
	 ImpactSoundsMax(0)=Sound'MaricaSounds.ut_impact_04'
	 ImpactSoundsMax(1)=Sound'MaricaSounds.ut_impact_05'
	 ImpactSoundsMax(2)=Sound'MaricaSounds.ut_impact_06'
	 ImpactSoundsMax(3)=Sound'MaricaSounds.ut_impact_07'

	 ShadowLightDistance=1200
	 ShadowMaxTraceDistance=300

	 CameraDist=20.0
	 Description="KenWorth T2000 Truck. || Model by SCS,Matheus."
	 DestroyBones(0)="Right_Door"
	 DestroyBones(1)="Left_Door"
	 DestroyBones(2)="Windows"
	 DestroyBones(3)="Attachment"
	 DestroyBones(4)="SteerWheel"
	 bAllowBrokenWheels=True
	 bSpawnBrokenDetail=True

	 bFixedFPCamera=False
	 FrontLightMaterial=Texture'GenFX.LensFlar.6'
	 RearLightMaterial=Texture'GenFX.LensFlar.softlens2'
	 RearLightStopScale=0.200000
	 RearLightScale=0.100000
	 FrontLightScale=0.200000
     FrontLightClass=Class'NVFrontCarLight'
     RearLightClass=Class'NVRearCarLight'
     FrontLightPos(0)=(X=216.000000,Y=50.000000,Z=-2.000000)
     FrontLightPos(1)=(X=216.000000,Y=-50.000000,Z=-2.000000)
     FrontLightPos(2)=(X=216.000000,Y=68.000000,Z=-2.000000)
     FrontLightPos(3)=(X=216.000000,Y=-68.000000,Z=-2.000000)
     FrontLightPos(4)=(X=216.000000,Y=28.500000,Z=-27.500000)
     FrontLightPos(5)=(X=216.000000,Y=-28.500000,Z=-27.500000)
     RearLightPos(0)=(X=-220.000000,Y=55.000000,Z=10.000000)
     RearLightPos(1)=(X=-220.000000,Y=-55.000000,Z=10.000000)

     ExhaustClass=Class'CarExhaust'
	 ExhaustPos(0)=(X=-88.000000,Y=86.000000,Z=-45.000000)
	 ExhaustPos(1)=(X=-88.000000,Y=-86.000000,Z=-45.000000)
	 ExhaustPos(2)=(X=-65.000000,Y=86.000000,Z=-45.000000)
	 ExhaustPos(3)=(X=-65.000000,Y=-86.000000,Z=-45.000000)

	 DecoSystem=DS_Bones
	 ExhaustBones(0)="Exhaust00"
	 ExhaustBones(1)="Exhaust01"
	 SteerWheelBone="SteerWheel"

	 FrontLightBones(0)="FrontLight00"
	 FrontLightBones(1)="FrontLight01"

	 RearLightBones(0)="RearLight00"
	 RearLightBones(1)="RearLight01"

	 bUpdateSusp=True
	 RSuspHighLimit=17.000000
	 RSuspLowLimit=-17.000000
     ExitPos=(X=100.0,Y=-150.000000,Z=15.000000)
     ExitPosPassenger=(X=100.0,Y=150.000000,Z=15.000000)
     EnterPos=(X=100.0,Y=165.000000,Z=-5.000000)
	 WheelFrontRight="RightFrontTIRE"
	 WheelFrontLeft="LeftFrontTIRE"
	 WheelRearRight="RightRearTIRE"
	 WheelRearLeft="LeftRearTIRE"
	 WheelCrossRight="RightCrossTIRE"
	 WheelCrossLeft="LeftCrossTIRE"
	 StrutFrontRight=""
	 StrutFrontLeft=""
	 StrutRearRight=""
	 StrutRearLeft=""
	 IntCollisionSize=(X=1,Y=1,Z=1)
	 IntCollisionModel=StaticMesh'MaricaSilverDragonCollision.t2000_IntColl'
	 IntCollisionClass=Class'IntCollisionActor'

	 LightProjectorClass=class'NVLightProjector'
	 ProjLightPos=(X=180.000000,Y=0.000000,Z=20.000000)
	 ProjectedLightRot=(Pitch=-100)
	 ProjLightMaterial=Texture'MaricaTextures.Effects.LightProj_01'
	 ProjLightFOV=40
	 ProjLightScale=0.400000

	 bCarIsFTWD=True
     bCarIsRTWD=True
     MaxSpeedAccel=30.000000
	 VehicleMaxSpeed=300300.000000
     MaxSteerAngle=3500.000000
     ChassisMass=5.0

     Health=6000.000000
     MaxHealth=6000.000000

	 bCanSeeDriver=True
	 myDriverSize=1.0
	 DriverPos=(X=73.000000,Y=-57.000000,Z=38.000000)
	 FPDriverViewPos=(X=72.000000,Y=-57.000000,Z=100.000000)
	 FPPassengerViewPos=(X=72.000000,Y=57.000000,Z=100.000000)
	 PassengerPos=(X=73.000000,Y=57.000000,Z=38.000000)

	 RadiatorPos=(X=225.000000,Y=0.000000,Z=45.000000)
     VehicleFireClass=Class'VehicleFireEmitter'

	 BrokenSkins(0)=Shader'MaricaSilverDragonTex.KenWorth.kw_Skin01'

	 bDamagableWheels=True
	 WheelsHealth=50
	 WheelsMaxHealth=100
	 UnloadWheelHeight=0.013200
	 bSparkWheels=True
     EngTireClass=Class'T2000CarTire'
     SteerTireClass=Class'T2000CarTire'
	 CrossTireClass=Class'T2000CarTire'
     WheelSteerAlong=182.000000
     WheelSteerAcross=88.000000
     WheelEngAlong=-367.500000
     WheelEngAcross=88.000000
     WheelCrossAlong=-255.000000
     WheelCrossAcross=88.000000
     WheelVert=-62.0
     SuspStiffness=200.000000
     SuspDamping=5.000000
     SuspHighLimit=0.400000
     SuspLowLimit=-0.400000
     TireLateralFriction=2.000000
	 TireRollSlip=0.060000
	 TireLateralSlip=0.040000
	 TireSlipRate=0.007000

	 EnginePitch=25.0
	 EngineVolume=60.0
     EngineSound=Sound'MaricaSounds.ut_prv_engine_idle'
     EngStartUpSound=None
     EngStopSound=None
     //EngStartUpSound=Sound'MaricaSounds.ut_engine_start_01'
     //EngStopSound=Sound'MaricaSounds.ut_engine_stop_01'
     ImpactSounds(0)=Sound'MaricaSounds.ut_impact_01'
     ImpactSounds(1)=Sound'MaricaSounds.ut_impact_02'

     StaticMesh=StaticMesh'MaricaSilverDragonCollision.t2000_KColl'
	 Mesh=SkeletalMesh'MaricaSilverDragonModels.T2000_Mesh'

	 bRandomSkin=false
	 SkinIndex=0
	 Skins(0)=Shader'MaricaSilverDragonTex.KenWorth.kw_Skin01'

     Begin Object Class=KarmaParamsRBFull Name=KenWorthTruckPhysics
         KInertiaTensor(0)=4.000000
         KInertiaTensor(3)=25.000000
         KInertiaTensor(5)=45.000000
         KCOMOffset=(X=-0.3,Z=-2.0)
		 KLinearDamping=0.050000
         KAngularDamping=0.700000
         KStartEnabled=True
         bKNonSphericalInertia=True
         KFriction=0.200000
         KImpactThreshold=250.000000
         Name="KenWorthTruckPhysics"
     End Object
     KParams=KarmaParamsRBFull'KenWorthTruckPhysics'
}