//=============================================================================
// SnowMobile.
// Special for ProjectMarica mod.
// Author: Evil T-ModZ
// T-ModZ. All Rights Reserved.
// 2012.
//=============================================================================

class SnowMobile extends NKVCarNew;

var(SnowMobile) Material BaseIdleTread;
var(SnowMobile) Material ForwardIdleTread;
var(SnowMobile) Material BackwardsIdleTread;
var(SnowMobile) int TreadIndex;
var int TreadMove;

var MotionRotatorActor SkiSteerRotator;

replication
{
	unreliable if( Role == ROLE_Authority )
		TreadMove;
}

simulated function PostBeginPlay()
{
	SkiSteerRotator = spawn(class'MotionRotatorActor');
	Super.PostBeginPlay();
}

simulated function Destroyed()
{
	if( SkiSteerRotator != None )
		SkiSteerRotator.Destroy();
	Super.Destroyed();
}

simulated event DriverLeave(optional bool bDestroying)
{
	Super.DriverLeave(bDestroying);
	TreadMove = 0;
}

simulated function UpdateTread()
{
	local float ForwardSpeed;
	local vector worldForward;

	worldForward = vect(1,0,0) >> Rotation;
	ForwardSpeed = Velocity Dot worldForward;

	if(GameIsEnd())
	{
		Skins[TreadIndex] = BaseIdleTread;
		return;
	}

	if( ForwardSpeed > 12 )
		Skins[TreadIndex] = ForwardIdleTread;
	else if( ForwardSpeed < -12 )
		Skins[TreadIndex] = BackwardsIdleTread;
	else Skins[TreadIndex] = BaseIdleTread;
}

simulated function GetVehicleData(optional bool bOnlyPositon)
{
	local vector Dir;
	local rotator ChassisRot;

	if(Role == ROLE_Authority)
		ChassisRot = Rotation;

	if(Level.NetMode != NM_Client)
	{
		ServerPosition = Location;
		ServerRotation = ChassisRot;
	}

	if(bOnlyPositon)
		return;

	if(Role == ROLE_Authority)
	{
		if(!bUpdateSusp)
		{
			Dir = vector(Rotation);
			Tire1Susp = Dir;
			Tire2Susp = Dir;
		}
		else
		{
			if( SteerTireRight != None )
				SteeringAngle = SteerTireRight.WheelJoint.KSteerAngle;
			else SteeringAngle = 0;

			if(SteerTireRight != None)
				Tire1Susp.Z = SteerTireRight.Location.Z - SteerWIRight.Location.Z;
			if(SteerTireLeft != None)
				Tire2Susp.Z = SteerTireLeft.Location.Z - SteerWILeft.Location.Z;

			if(Tire1Susp.Z > RSuspHighLimit)
				Tire1Susp.Z = RSuspHighLimit;
			else if(Tire1Susp.Z < RSuspLowLimit)
				Tire1Susp.Z = RSuspLowLimit;

			if(Tire2Susp.Z > RSuspHighLimit)
				Tire2Susp.Z = RSuspHighLimit;
			else if(Tire2Susp.Z < RSuspLowLimit)
				Tire2Susp.Z = RSuspLowLimit;

			Strut1Roll.Pitch = Tire1Susp.Z * 175;
			Strut2Roll.Pitch = -Tire2Susp.Z * 175;
		}
	}
}

simulated function ApplyVehicleData()
{
	local int i;
	local vector Dir;
	local rotator WheelRot;

	if( bBeep )
		PlayingBeepSound();

	if(!GameIsEnd())
		UpdateTread();

	if(Level.NetMode == NM_DedicatedServer)
		return;

	if( SteerWheelBone != '')
	{
		if( DriverController != None )
			RotateSteeringWheel(DriverController.bBehindView);
		else if( bAIControlled )
			RotateSteeringWheel(true);
		else RotateSteeringWheel(false);
	}

	Dir = vector(Rotation);

	WheelRot = ServerRotation;

	if( SkiSteerRotator != None )
	{
		if( Steering > 0.3 )
		{
			SkiSteerRotator.DesiredRotation.Yaw = 6000;
			SkiSteerRotator.RotationRate.Yaw = 45000;
		}
		else if( Steering < -0.3 )
		{
			SkiSteerRotator.DesiredRotation.Yaw = -6000;
			SkiSteerRotator.RotationRate.Yaw = -45000;
		}
		else
		{
			if( Normalize(SkiSteerRotator.Rotation).Yaw > 0 )
				SkiSteerRotator.RotationRate.Yaw = -45000;
			else if( Normalize(SkiSteerRotator.Rotation).Yaw < 0 )
				SkiSteerRotator.RotationRate.Yaw = 45000;
			SkiSteerRotator.DesiredRotation.Yaw = 0;
		}

		WheelRot.Yaw -= SkiSteerRotator.Rotation.Yaw;
	}

	// WHEELS
	// 1
	if(!bSteerRightDetached)
	{
		if(WheelFrontRight != '')
		{
			SetBoneDirection(WheelFrontRight,WheelRot,Tire1Susp,1);
			//SetBoneRotation(WheelFrontRight,WheelRot,0,1);
			//SetBoneLocation(WheelFrontRight,Tire1Susp,1);
		}
	}
	else
	{
		SetBoneDirection( WheelFrontRight , rot(0,0,0) , Dir , 0 );
		SetBoneScale(0,0,WheelFrontRight);
	}
	// 2
	if(!bSteerLeftDetached)
	{
		if(WheelFrontLeft != '')
		{
			SetBoneDirection(WheelFrontLeft,WheelRot,Tire2Susp,1);
			//SetBoneRotation(WheelFrontLeft,WheelRot,0,1);
			//SetBoneLocation(WheelFrontLeft,Tire2Susp,1);
		}
	}
	else
	{
		SetBoneDirection( WheelFrontLeft , rot(0,0,0) , Dir , 0 );
		SetBoneScale(1,0,WheelFrontLeft);
	}

	if(VehicleDestroyed() && !bBonesDestroyed)
	{
		for(i = 0; i < DestroyBones.Length; i++)
			SetVehicleBoneScale(DestroyBones[i],0);
		SetSkin();
		bBonesDestroyed = true;
	}

	if(!bUpdateSusp)
		return;

	//STRUTS
	// 1
	if(!bSteerRightDetached)
		if(StrutFrontRight != '')
			SetBoneRotation(StrutFrontRight,Strut1Roll,0,1);
	// 2
	if(!bSteerLeftDetached)
		if(StrutFrontLeft != '')
			SetBoneRotation(StrutFrontLeft,Strut2Roll,0,1);
}

defaultproperties
{
	 bLittleVehicle=true
	 AIChasingDist=100.0
	 bAllowPassenger=false
	 WheelIndicatorClass=class'SnowWheelIndicator'
	 RestFriction=0.1
	 bNormalHUDView=true
	 bAllowWeaponsInside=true
	 TorqueCurve=(Points=((InVal=0.0,OutVal=50.0),(InVal=550000.0,OutVal=100.0),(InVal=800000.0))
	 bDriverTakesDamage=true
	 Tag="SnowMobile"
	 BaseIdleTread=Shader'MaricaSnowMobileTex.SMTreadShadIdle'
	 ForwardIdleTread=Shader'MaricaSnowMobileTex.SMTreadShadFW'
	 BackwardsIdleTread=Shader'MaricaSnowMobileTex.SMTreadShadBW'

     DrivingAnimStatus="EvilBikeDriver"
     SittingAnimStatus="EvilBikeDriver"

	 WheelRadius=10.0
	 ActionAngularDamp=0.0
	 RestAngularDamp=0.7

	 ImpactSoundsMin(0)=Sound'MaricaSounds.ut_impact_03'
	 ImpactSoundsMed(0)=Sound'MaricaSounds.ut_impact_01'
	 ImpactSoundsMed(1)=Sound'MaricaSounds.ut_impact_02'
	 ImpactSoundsMax(0)=Sound'MaricaSounds.ut_impact_04'
	 ImpactSoundsMax(1)=Sound'MaricaSounds.ut_impact_05'
	 ImpactSoundsMax(2)=Sound'MaricaSounds.ut_impact_06'
	 ImpactSoundsMax(3)=Sound'MaricaSounds.ut_impact_07'

	 ShadowLightDistance=1200
	 ShadowMaxTraceDistance=300

	 CameraDist=10.0
	 Description="SnowMobile. || Model from COD Modern Warfare 2."
	 bAllowBrokenWheels=True
	 bSpawnBrokenDetail=True

	 bFixedFPCamera=False
	 FrontLightMaterial=Texture'GenFX.LensFlar.6'
	 RearLightMaterial=Texture'GenFX.LensFlar.softlens2'
	 RearLightStopScale=0.200000
	 RearLightScale=0.100000
	 FrontLightScale=0.200000
     FrontLightClass=Class'NVFrontCarLight'
     RearLightClass=Class'NVRearCarLight'
     FrontLightPos(0)=(X=216.000000,Y=50.000000,Z=-2.000000)
     FrontLightPos(1)=(X=216.000000,Y=-50.000000,Z=-2.000000)
     FrontLightPos(2)=(X=216.000000,Y=68.000000,Z=-2.000000)
     FrontLightPos(3)=(X=216.000000,Y=-68.000000,Z=-2.000000)
     FrontLightPos(4)=(X=216.000000,Y=28.500000,Z=-27.500000)
     FrontLightPos(5)=(X=216.000000,Y=-28.500000,Z=-27.500000)
     RearLightPos(0)=(X=-220.000000,Y=55.000000,Z=10.000000)
     RearLightPos(1)=(X=-220.000000,Y=-55.000000,Z=10.000000)

     ExhaustClass=Class'CarExhaust'
	 ExhaustPos(0)=(X=-88.000000,Y=86.000000,Z=-45.000000)
	 ExhaustPos(1)=(X=-88.000000,Y=-86.000000,Z=-45.000000)
	 ExhaustPos(2)=(X=-65.000000,Y=86.000000,Z=-45.000000)
	 ExhaustPos(3)=(X=-65.000000,Y=-86.000000,Z=-45.000000)

	 DecoSystem=DS_Bones
	 SteerWheelBone="SteerWheel"

	 FrontLightBones(0)="FrontLight00"
	 FrontLightBones(1)="FrontLight01"

	 RearLightBones(0)="RearLight00"

	 bUpdateSusp=True
	 RSuspHighLimit=17.000000
	 RSuspLowLimit=-17.000000
     ExitPos=(Y=-110.000000,Z=15.000000)
     ExitPosPassenger=(Y=110.000000,Z=15.000000)
     EnterPos=(Y=100.000000,Z=-5.000000)
     WheelFrontRight="RightFrontTIRE"
     WheelFrontLeft="LeftFrontTIRE"
	 StrutFrontRight="RightFrontSTRUT"
	 StrutFrontLeft="LeftFrontSTRUT"
	 StrutRearRight=""
	 StrutRearLeft=""
	 IntCollisionSize=(X=1,Y=1,Z=1)
	 IntCollisionModel=StaticMesh'MaricaSnowMobileCollision.SM_IntColl'
	 IntCollisionClass=Class'SnowMobileIntColl'

	 LightProjectorClass=class'NVLightProjector'
	 ProjLightPos=(X=220.000000,Y=0.000000,Z=20.000000)
	 ProjectedLightRot=(Pitch=-100)
	 ProjLightMaterial=Texture'MaricaTextures.Effects.LightProj_01'
	 ProjLightFOV=40
	 ProjLightScale=0.2

     MaxBrakeTorque=10.000000
	 bCarIsFTWD=False
     bCarIsRTWD=True
     MaxSpeedAccel=30.000000
	 VehicleMaxSpeed=675325.000000
     MaxSteerAngle=3500.000000
     ChassisMass=2.5000000

	 myDriverSize=1.0
	 bCanSeeDriver=True
	 DriverPos=(X=-50.000000,Y=0.000000,Z=13.000000)
	 FPDriverViewPos=(X=-42.000000,Y=0.000000,Z=50.000000)
	 FPPassengerViewPos=(X=-57.000000,Y=0.000000,Z=50.000000)
	 PassengerPos=(X=-57.000000,Y=0.000000,Z=13.000000)

	 RadiatorPos=(X=50.000000,Y=0.000000,Z=25.000000)
     VehicleFireClass=Class'VehicleFireEmitter'

	 BrokenSkins(0)=Shader'MaricaTextures.FrtTex.FrtSkin_s_destroyed'
	 BrokenSkins(1)=Texture'ben_skins.Misc.skyGlass_broken_glass'
	 BrokenSkins(2)=Texture'MaricaTextures.FrtTex.TailLightBroken'
	 BrokenSkins(3)=Shader'MaricaTextures.FrtTex.HortonVehicleAcc_s'
	 BrokenSkins(4)=Shader'MaricaTextures.FrtTex.FrtTire_s_Skin_01'

	 bDamagableWheels=False
	 WheelsHealth=50
	 WheelsMaxHealth=100
	 UnloadWheelHeight=0.013200
	 bSparkWheels=True
     EngTireClass=Class'SnowMobileTire'
     SteerTireClass=Class'SnowMobileTire'
     WheelSteerAlong=88.000000
     WheelSteerAcross=45.000000
     WheelEngAlong=-75.000000
     WheelEngAcross=35.000000
     WheelVert=-32.000000
     SuspStiffness=320.000000
     SuspDamping=8.000000
     SuspHighLimit=0.400000
     SuspLowLimit=-0.400000
     //TireRollFriction=1.500000
     //TireLateralFriction=1.250000

     EngStartUpSound=None
     EngStopSound=None
     //EngineStartTime=1.000000
     //EngStartUpSound=Sound'MaricaSnowMobileSounds.sm_start'
     //EngStopSound=Sound'MaricaSnowMobileSounds.sm_stop'

	 EnginePitch=20
	 EngineVolume=250
     EngineSound=Sound'MaricaSnowMobileSounds.sm_eng'
     ImpactSounds(0)=Sound'MaricaSounds.ut_impact_01'
     ImpactSounds(1)=Sound'MaricaSounds.ut_impact_02'

     StaticMesh=StaticMesh'MaricaSnowMobileCollision.SM_KColl'
	 Mesh=SkeletalMesh'MaricaSnowMobileMesh.Model_EvilSnowMobile'
	 DrawScale=0.7

	 bRandomSkin=False
	 TreadIndex=1
	 SkinIndex=0
	 Skins(0)=Shader'MaricaSnowMobileTex.SMChassis'
	 OtherSkins(0)=Shader'MaricaSnowMobileTex.SMChassis'

     Begin Object Class=KarmaParamsRBFull Name=SnowMobilePhysics
         //KInertiaTensor(0)=10.000000
         //KInertiaTensor(3)=15.000000
         //KInertiaTensor(5)=20.000000
         //KCOMOffset=(X=-0.3,Z=-2.2)
         KInertiaTensor(0)=3.000000
         KInertiaTensor(3)=20.000000
         KInertiaTensor(5)=25.000000
         KCOMOffset=(X=-0.3,Z=-2.0)
		 KLinearDamping=0.050000
         KAngularDamping=0.700000
         KStartEnabled=True
         bKNonSphericalInertia=True
         KFriction=0.200000
         KImpactThreshold=250.000000
         Name="SnowMobilePhysics"
     End Object
     KParams=KarmaParamsRBFull'SnowMobilePhysics'
}
