//=============================================================================
// SpaceFighterAttackerVehicle.uc
// Author: Nick Pilshikov "Evil"
// Email: Nick_Pv@mail.ru / EvilTheDark@gmail.com
// Site: Set-Games.ru / Revival-Games.ru
// Revival Games Studios.
// 2014.
//=============================================================================

class SpaceFighterAttackerVehicle extends SpaceFighterDefenderVehicle;

simulated function VehicleHaveDriver()
{
	local rotator TrailOrient;
	Super(HelicopterVehicle).VehicleHaveDriver();

	if( Controller != None 
		&& Controller.PlayerReplicationInfo != None 
		&& Controller.PlayerReplicationInfo.Team != None )
	{
		if( Controller.PlayerReplicationInfo.Team.TeamIndex == 1 )
			bBlue = true;
		else bBlue = false;
	}

	if( Trail != None )
	{
		DetachFromBone(Trail);
		Trail.Destroy();
		Trail = None;
	}

	Trail = Spawn(TrailClass,self,,Location);
	AttachToBone(Trail, ChassisBone);
	TrailOrient.Yaw = 32768;
	Trail.SetRelativeLocation(TrailRelativePos);// >> Rotation);
	Trail.SetRelativeRotation(TrailOrient);

	if( Trail != None )
	{
		if( bBlue )
			Trail.SetBlueColor();
		else Trail.SetRedColor();
	}
}

simulated function UpdateVehicleRotation(optional bool bFixed)
{
	local rotator VehicleRotation;
	local int RiseSpeed , StrafeSpeed , AccelRiseRot , AccelStrafeRot;
	local vector worldUp , worldStrafe;

	if( Controller != None )
	{
		ControllerRotation = Controller.Rotation;
	}

	if( bFixed )
	{
		if(RotatorActor != None)
		{
			RotatorActor.DesiredRotation = ServerRotation;
			RotatorActor.RotationRate = rot(0,0,0);
			RotatorActor.SetRotation(ServerRotation);
		}
		SetRotation(ServerRotation);
		SetBoneRotation(ChassisBone,rot(0,0,0),0,1);
		ClientSimulation(ServerRotation);
		return;
	}

	worldUp = vect(0,0,-1) >> Rotation;
	RiseSpeed = Velocity Dot worldUp;
	//RiseSpeed = -RiseSpeed;

	worldStrafe = vect(0,-1,0) >> Rotation;
	StrafeSpeed = Velocity Dot worldStrafe;
	//StrafeSpeed = -StrafeSpeed;

	AccelRiseRot = FClamp(RiseSpeed,AccelRotMin,AccelRotMax);
	AccelStrafeRot = FClamp(StrafeSpeed,AccelRotMin,AccelRotMax);

	if(RotatorActor != None)
	{
		RotatorActor.DesiredRotation = ControllerRotation;
		RotatorActor.RotationRate = Normalize((Normalize(ControllerRotation) - Normalize(Rotation))) * VehicleRotateSpeed;
		VehicleRotation.Yaw = RotatorActor.Rotation.Yaw;
		SetRotation(VehicleRotation);
	}
	ClientSimulation(VehicleRotation);

	VehicleRotation = rot(0,0,0);
	VehicleRotation.Pitch = AccelRiseRot;
	VehicleRotation.Roll = AccelStrafeRot;
	SetBoneRotation(ChassisBone,VehicleRotation,0,1);
}

defaultproperties
{
	 EngStartUpVolume=0.3
	 EngStopVolume=0.3
	 Tag="Attacker Space Ship"
	 VehicleFireVolume=0.6
	 SoundVolume=120.0
     SoundRadius=100.0
     TransientSoundVolume=1.0
     TransientSoundRadius=800.0

	 FireZ=132.0
	 AimAngle=2000
	 bBlue=true
	 ImpactCrashSound=Sound'MaricaSounds.ut_impact_04'
	 TrailClass=class'SpaceShipEngineTrail'
	 TrailRelativePos=(X=130.0,Y=0.0,Z=0.0)
	 VehicleRotateSpeed=2
	 MenuName="in Skarj SpaceFighter"
	 FlyingAnim="Flying"
	 LandedAnim="Landed"
	 ActivateAnim="TakeOff"
	 DeactivateAnim="Landing"
	 ReactiveAccel=64.0
	 MinigunProjClass=class'PlasmaProjectile'
	 RocketProjClass=class'SpaceShipRocketProjectile'
	 MinigunMuzzleClass=None
	 CameraDist=4.0
	 AccelRotMin=-10000
	 AccelRotMax=10000
	 LandMovementState="PlayerHelicoptering"
     DriverState="PlayerHelicoptering"
	 VehicleState=3
	 StartTime=1.7
	 StopTime=2.0
	 MinigunRightBone=""
	 MinigunLeftBone=""
	 ChassisBone="root"
	 MainRotorBone=""
	 TailRotorBone=""
	 MainRotorBoneTip=""
	 TailRotorBoneTip=""
	 BrokenDetailPos=(X=0.0,Y=0.0,Z=0.0)
	 BrokenDetailClass=class'SpaceFighterAttackerBrokenDetail'
	 VehicleFOV=100
	 bCanSeeDriver=false
     MinigunOffset=(X=100.0,Y=160.0,Z=0.0)
     RocketLauncherOffset=(X=170.0,Y=40.0,Z=-10.0)
	 StartupSound=Sound'MaricaSpaceFightersSounds.SkShipAccel01'
	 StopSound=Sound'MaricaSpaceFightersSounds.SkShipDecel01'
     DestroyedEffectClass=Class'tVehicleExplosion'
     DestroyedSound=Sound'WeaponSounds.grenade_explodeair'
     EngineAmbientSound=Sound'MaricaSpaceFightersSounds.SkShipEng01'
     MinigunFireSound=Sound'MaricaSpaceFightersSounds.HnShipFire01'
	 RocketLaunchSound=Sound'MaricaSpaceFightersSounds.HnShipFire02'
     DriverPos=(X=20.0)
     EnterPos=(Y=200.0,Z=-10.0)
     ExitPos=(Y=200.0,Z=-10.0)
     VehicleFireSpeed=0.200000
     VehicleAltFireSpeed=2.0
     HealthMax=512.0
     AirSpeed=8000.0
     AccelRate=1000.0
     AirControl=0.5
     BaseEyeHeight=42.0
     EyeHeight=42.0
     Health=512.0
     Mesh=SkeletalMesh'MaricaSpaceFightersMeshes.SpaceFighter_Skaarj'
	 Skins(0)=Shader'MaricaSpaceFightersTextures.SpaceFighter_Skaarj2_shad02'
	 Skins(1)=Shader'MaricaSpaceFightersTextures.SpaceFighter_Skaarj1_shad01'
     DrawScale=1.800000
     CollisionRadius=180.0
     CollisionHeight=85.0
	 Mass=400.0
}
