//=============================================================================
// SpaceFighterDefenderVehicle.uc
// Author: Nick Pilshikov "Evil"
// Email: Nick_Pv@mail.ru / EvilTheDark@gmail.com
// Site: Set-Games.ru / Revival-Games.ru
// Revival Games Studios.
// 2014.
//=============================================================================

class SpaceFighterDefenderVehicle extends HelicopterVehicle;

var SpaceShipEngineTrail Trail;
var sound ImpactCrashSound;

var class<SpaceShipEngineTrail> TrailClass;

var vector ReactiveVelocity;
var() vector TrailRelativePos;

var bool bBlue;

var float ReactiveAccel;
var float LastImpactTime;

var() Name ActivateAnim;
var() Name DeactivateAnim;
var() Name FlyingAnim;
var() Name LandedAnim;

replication
{
	unreliable if( Role == ROLE_Authority )
		bBlue , Trail;
}

simulated function SendEnterMessage(PlayerController PC)
{
	PC.ReceiveLocalizedMessage(class'NVMessage', 1);
	PC.ReceiveLocalizedMessage(class'NVMessage', 8);
}

simulated function VehicleActivateAnim()
{
	PlayAnim(ActivateAnim);
}

simulated function VehicleDeactivateAnim()
{
	PlayAnim(DeactivateAnim);
}

simulated function VehicleIdleAnim(bool State)
{
	if( State )
		LoopAnim(FlyingAnim);
	else LoopAnim(LandedAnim);
}

simulated function VehicleHaveDriver()
{
	Super.VehicleHaveDriver();

	if( Controller != None 
		&& Controller.PlayerReplicationInfo != None 
		&& Controller.PlayerReplicationInfo.Team != None )
	{
		if( Controller.PlayerReplicationInfo.Team.TeamIndex == 1 )
			bBlue = true;
		else bBlue = false;
	}

	if( Trail != None )
	{
		DetachFromBone(Trail);
		Trail.Destroy();
		Trail = None;
	}

	Trail = Spawn(TrailClass,self,,Location);
	AttachToBone(Trail, ChassisBone);
	Trail.SetRelativeLocation(TrailRelativePos);// >> Rotation);

	if( Trail != None )
	{
		if( bBlue )
			Trail.SetBlueColor();
		else Trail.SetRedColor();
	}
}

simulated function VehicleHaveNoDriver()
{
	Super.VehicleHaveNoDriver();
	if( Trail != None )
	{
		DetachFromBone(Trail);
		Trail.Destroy();
		Trail = None;
	}
}

simulated function UpdateRotors(int Rot)
{
	//STUB
}

simulated function UpdateBlades(float Speed)
{
	//STUB
}

simulated function UpdateVehicleRotation(optional bool bFixed)
{
	local rotator VehicleRotation;
	local int RiseSpeed , StrafeSpeed , AccelRiseRot , AccelStrafeRot;
	local vector worldUp , worldStrafe;

	if( Controller != None )
	{
		ControllerRotation = Controller.Rotation;
	}

	if( bFixed )
	{
		if(RotatorActor != None)
		{
			RotatorActor.DesiredRotation = ServerRotation;
			RotatorActor.RotationRate = rot(0,0,0);
			RotatorActor.SetRotation(ServerRotation);
		}
		SetRotation(ServerRotation);
		SetBoneRotation(ChassisBone,rot(0,0,0),0,1);
		ClientSimulation(ServerRotation);
		return;
	}

	worldUp = vect(0,0,1) >> Rotation;
	RiseSpeed = Velocity Dot worldUp;
	//RiseSpeed = -RiseSpeed;

	worldStrafe = vect(0,1,0) >> Rotation;
	StrafeSpeed = Velocity Dot worldStrafe;
	//StrafeSpeed = -StrafeSpeed;

	AccelRiseRot = FClamp(RiseSpeed,AccelRotMin,AccelRotMax);
	AccelStrafeRot = FClamp(StrafeSpeed,AccelRotMin,AccelRotMax);

	if(RotatorActor != None)
	{
		RotatorActor.DesiredRotation = ControllerRotation;
		RotatorActor.RotationRate = Normalize((Normalize(ControllerRotation) - Normalize(Rotation))) * VehicleRotateSpeed;
		VehicleRotation.Yaw = RotatorActor.Rotation.Yaw;
		SetRotation(VehicleRotation);
	}
	ClientSimulation(VehicleRotation);

	VehicleRotation = rot(0,0,0);
	VehicleRotation.Pitch = AccelRiseRot;
	VehicleRotation.Roll = AccelStrafeRot;
	SetBoneRotation(ChassisBone,VehicleRotation,0,1);
}

simulated function ClientSimulation(rotator VehicleRotation)
{
	//STUB
}

simulated function VehicleFire(rotator Direction)
{
	local PlasmaProjectileRed PlasmaProj;
	local vector Dir, Range, StartTrace, SpawnPos;
	local vector RotX, RotY, RotZ, HitLoc, HitNorm;
	local rotator FireOrient;

    GetAxes(Direction,RotX,RotY,RotZ);

	Dir = Vector(Direction);

	Super(NPVehicle).VehicleFire(Direction);

	/*Range.X = FRand() * 0.06;
	Range.Y = FRand() * 0.06;
	Range.Z = FRand() * 0.06;*/

	SpawnPos = Location + MinigunOffset.X * RotX +  MinigunOffset.Y * RotY + MinigunOffset.Z * RotZ;
	CalcWeaponFire( (vect(0,10,0) >> Rotation), HitLoc, HitNorm );
    FireOrient = Rotator( Normal(HitLoc - SpawnPos) );
	StartTrace = vector(FireOrient) + Range;

	if( bBlue )
		PlasmaProj = spawn(class'PlasmaProjectileBlue',Instigator,,SpawnPos);
	else PlasmaProj = spawn(class'PlasmaProjectileRed',Instigator,,SpawnPos);
	PlasmaProj.SetVelocity(StartTrace);

	/*Range.X = FRand() * 0.06;
	Range.Y = FRand() * 0.06;
	Range.Z = FRand() * 0.06;*/

	SpawnPos = Location + MinigunOffset.X * RotX -  MinigunOffset.Y * RotY + MinigunOffset.Z * RotZ;
	CalcWeaponFire( (vect(0,-10,0) >> Rotation), HitLoc, HitNorm );
    FireOrient = Rotator( Normal( HitLoc - SpawnPos ) );
	StartTrace = vector(FireOrient) + Range;

	if( bBlue )
		PlasmaProj = spawn(class'PlasmaProjectileBlue',Instigator,,SpawnPos);
	else PlasmaProj = spawn(class'PlasmaProjectileRed',Instigator,,SpawnPos);
	PlasmaProj.SetVelocity(StartTrace);
}

simulated function VehicleAltFire(rotator Direction)
{
	local Pawn newTarget;
	local RocketProjectile Rocket;
	local vector Dir;
	local vector RotX, RotY, RotZ;

    GetAxes(Direction,RotX,RotY,RotZ);

	Dir = Vector(Direction);

	Super(NPVehicle).VehicleAltFire(Direction);

	if( AltFireState == 0 )
	{
		Rocket = spawn(RocketProjClass,Instigator,,Location + RocketLauncherOffset.X * RotX + 
				RocketLauncherOffset.Y * RotY + RocketLauncherOffset.Z * RotZ);
		Rocket.SetupShot(10);
		Rocket.AddRelativeVelocity(Velocity);

		if( CurrentTarget != None )
			Rocket.SetNewTarget(CurrentTarget);
		else if( LaserDetector != None )
			Rocket.SetNewTarget(LaserDetector);
		else Rocket.SetNewTarget(CheckOutTarget());

		AltFireState = 1;
	}
	else
	{
		Rocket = spawn(RocketProjClass,Instigator,,Location + RocketLauncherOffset.X * RotX - 
				RocketLauncherOffset.Y * RotY + RocketLauncherOffset.Z * RotZ);
		Rocket.SetupShot(10);
		Rocket.AddRelativeVelocity(Velocity);

		if( CurrentTarget != None )
			Rocket.SetNewTarget(CurrentTarget);
		else if( LaserDetector != None )
			Rocket.SetNewTarget(LaserDetector);
		else Rocket.SetNewTarget(CheckOutTarget());

		AltFireState = 0;
	}
}

function CalcImpactDamages()
{
	local vector HitLocation, HitNormal, EndTrace;
	local Actor Other;

	if( VSize(Velocity) < 100 )
		return;

	EndTrace = 200 * vect(1,0,0) >> rotator(Velocity);
	EndTrace += Location;
	Other = Trace(HitLocation, HitNormal, EndTrace, Location, true);

	if( Other != None && Other.bBlockActors )
	{
		if( Other.IsA('Pawn') && !Other.IsA('NPVehicle') )
			Pawn(Other).TakeDamage(1000, Instigator, HitLocation, Velocity * 100, class'ExplodedDamage');
		else if( Other.IsA('NPVehicle') )
		{
			NPVehicle(Other).TakeDamage(VSize(Velocity)/30, Instigator, HitLocation, Velocity * 100, class'ExplodedDamage');
			if( ImpactCrashSound != None )
				PlaySound(ImpactCrashSound, SLOT_Misc, 0.8);
		}
		else
		{
			TakeDamage(VSize(Velocity)/90, Instigator, HitLocation, Velocity * 100, class'Crushed');
			if( ImpactCrashSound != None )
				PlaySound(ImpactCrashSound, SLOT_Misc, 0.8);
		}
	}
	else
	{
		if( !FastTrace(EndTrace, Location) )
		{
			TakeDamage(VSize(Velocity)/90, Instigator, HitLocation, Velocity * 100, class'Crushed');
			if( ImpactCrashSound != None )
				PlaySound(ImpactCrashSound, SLOT_Misc, 0.8);
		}
	}
}

simulated function ContinuousUpdate(float DeltaTime, int State)
{
	local Actor TestTarget;
	local vector HitLoc, HitNorm;

	mySpeed = VSize(Velocity);

	if(Driver != None && Controller != None)
		DriverController = PlayerController(Controller);

	UserInterfaceControl();

	if( Driver != None )
	{
		if( LaserDetector == None )
			LaserDetector = spawn(class'LaserDetectDot',self,,Location);
	}
	else
	{
		if( LaserDetector != None )
		{
			LaserDetector.Destroy();
			LaserDetector = None;
		}
	}

	TestTarget = CalcWeaponAltFire();
	if( DriverController != None && Level.TimeSeconds > LastCalcWeaponFire + 0.2 )
	{
		if( TestTarget != None && 
			( (Pawn(TestTarget) != None && Pawn(TestTarget).Health > 0) 
				|| (IntCollisionActor(TestTarget) != None && IntCollisionActor(TestTarget).myVehicle != None && IntCollisionActor(TestTarget).myVehicle.Health > 0 && IntCollisionActor(TestTarget).myVehicle.Driver != None) ) )
		{
			TargetThinkNum = 0;
			CurrentTarget = TestTarget;
			ClientToServerFire(TestTarget);
			bAimed = true;
			if( AimDetectionSound != None )
				PlaySound(AimDetectionSound, SLOT_Misc, 0.2);
		}
		else
		{
			if( CurrentTarget != None && TargetThinkNum < 3 )
				TargetThinkNum++;
			else
			{
				CurrentTarget = None;
				ClientToServerFire(None);
			}
			bAimed = false;
		}
		LastCalcWeaponFire = Level.TimeSeconds;
	}

	GetVehicleData();

	if( State == 1 )
	{
		ServerPhysicsSimulation(5);
		PhysicsSimulation(5);

		//ControlAccess(false);

		EngineModel(0);

		UpdateVehicleRotation(true);

		VehicleActivate();
	}
	else if( State == 2 )
	{
		EngineModel(0);

		//ControlAccess(true);

		UpdateVehicleRotation(true);

		VehicleDeactivate();
	}
	else
	{
		if( bHaveDriver )
			ServerPhysicsSimulation(6);

		EngineModel(mySpeed);

		//ControlAccess(true);

		UpdateVehicleRotation(!bHaveDriver);

		ShadowRendering(bHaveDriver);

		VehicleGetFire();

		VehicleIdleAnim(bHaveDriver);
	}
}

simulated function EngineModel(float Speed)
{
	local float DesiredEnginePitch, PitchMax, JetFireMag, JetFireSize, JetFireAmp;
	if( AmbientSound != None )
	{
		DesiredEnginePitch = Speed / (default.SoundPitch + 30);
		PitchMax = default.SoundPitch + 10;
		if(DesiredEnginePitch <= PitchMax)
			SoundPitch = default.SoundPitch + DesiredEnginePitch;
	}

	if( Trail != None )
	{
		if( Speed != 0 )
		{
			JetFireMag = -FClamp(Speed/20, 75, 250);
			JetFireSize = FClamp(Speed/15, 50, 200);
			JetFireAmp = FClamp(Speed/500, 2, 20);
			if( Trail.Emitters[0] != None )
			{
				Trail.Emitters[0].SizeScale[0].RelativeSize = JetFireAmp;
				Trail.Emitters[0].SizeScale[1].RelativeSize = JetFireAmp;
				Trail.Emitters[0].SizeScale[2].RelativeSize = JetFireAmp;
				Trail.Emitters[0].SizeScale[3].RelativeSize = JetFireAmp;
				Trail.Emitters[0].StartVelocityRange.X.Min = JetFireMag;
				Trail.Emitters[0].StartVelocityRange.X.Max = JetFireMag;
			}
			if( Trail.Emitters[1] != None )
			{
				Trail.Emitters[1].StartSizeRange.X.Min = JetFireSize;
				Trail.Emitters[1].StartSizeRange.X.Max = JetFireSize;
			}
		}
		else
		{
			if( Trail.Emitters[0] != None )
			{
				Trail.Emitters[0].SizeScale[0].RelativeSize = 1.0f;
				Trail.Emitters[0].SizeScale[1].RelativeSize = 2.0f;
				Trail.Emitters[0].SizeScale[2].RelativeSize = 2.0f;
				Trail.Emitters[0].SizeScale[3].RelativeSize = 2.0f;
				Trail.Emitters[0].StartVelocityRange.X.Min = -75;
				Trail.Emitters[0].StartVelocityRange.X.Max = -75;
			}
			if( Trail.Emitters[1] != None )
			{
				Trail.Emitters[1].StartSizeRange.X.Min = 30;
				Trail.Emitters[1].StartSizeRange.X.Max = 50;
			}
		}
	}
}

simulated function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, class<DamageType> damageType)
{
	local int AcceptDamage;

	AcceptDamage = Damage;

	if(ClassIsChildOf(damageType, class'PlasmaDamage'))
		AcceptDamage *= 0.3;
	else if(ClassIsChildOf(damageType, class'RocketExplosionDamage'))
		AcceptDamage += 110;

	Super.TakeDamage(AcceptDamage, instigatedBy, hitlocation, momentum, damageType);
}

simulated function Destroyed()
{
	Super.Destroyed();
	if( Trail != None )
	{
		DetachFromBone(Trail);
		Trail.Destroy();
		Trail = None;
	}
}

/*simulated function DrawCrossHair( canvas Canvas)
{
	local P2Player p2p;
	local float UseScale;
	local Texture CrossHair;
	local float AimErrorSize;
	local rotator DirYawPC, DirYawSelf, DirPitchFire, DirPitchPC;

	p2p = P2Player(Controller);

	if (ReticleTexture != None && p2p != None)
	{
		DirYawPC.Yaw = p2p.Rotation.Yaw;
		DirYawSelf.Yaw = Rotation.Yaw;
		DirPitchFire.Pitch = p2p.Rotation.Pitch + AimAngle;
		DirPitchPC.Pitch = DirPitchFire.Pitch;

		if( DirPitchFire.Pitch > 32000 && DirPitchFire.Pitch < 62000 )
			DirPitchFire.Pitch = 62000;
		else if( DirPitchFire.Pitch > 68000 || DirPitchFire.Pitch < 32000 )
			DirPitchFire.Pitch = 68000;

		AimErrorSize = (1 - FClamp((VSize(vector(DirPitchFire)-vector(DirPitchPC)))*1.5,0.f,1.0)) * (2 - VSize(vector(DirYawPC)-vector(DirYawSelf)))/2;
		//ClientMessage(AimErrorSize);

		ReticleColor.G = FClamp((255*AimErrorSize),50,255);
		ReticleColor.A = FClamp((150*AimErrorSize),50,150);

		// Draw reticle
		UseScale = P2Hud(p2p.MyHud).Scale;

		if( bAimed )
		{
			Canvas.Style = ERenderStyle.STY_Alpha;
			Canvas.DrawColor = default.ReticleColor;
			CrossHair = ReticleTextureAim;
			Canvas.SetPos(
				(Canvas.ClipX / 2) - UseScale*(CrossHair.USize/2),
				(Canvas.ClipY / 2.5) - UseScale*(CrossHair.VSize/2));
			Canvas.DrawIcon(CrossHair, UseScale);
		}

		Canvas.Style = ERenderStyle.STY_Alpha;
		Canvas.DrawColor = ReticleColor;
		CrossHair = ReticleTexture;
		Canvas.SetPos(
			(Canvas.ClipX / 2) - UseScale*(CrossHair.USize/2),
			(Canvas.ClipY / 2.5) - UseScale*(CrossHair.VSize/2));
		Canvas.DrawIcon(CrossHair, UseScale);
	}
}*/

defaultproperties
{
	 EngStartUpVolume=0.3
	 EngStopVolume=0.3
	 Tag="Defender Space Ship"
	 VehicleFireVolume=0.6
	 SoundVolume=120.0
     SoundRadius=100.0
     TransientSoundVolume=0.0
     TransientSoundRadius=800.0

	 FireZ=132.0
	 AimAngle=2000
     ReticleTexture=Texture'MaricaSpaceFightersTextures.SpaceFighterReticle01'
	 ReticleTextureAim=Texture'MaricaSpaceFightersTextures.SpaceFighterReticle02'
     ReticleColor=(B=255,G=255,R=255,A=150)
	 bBlue=false
	 ImpactCrashSound=Sound'MaricaSounds.ut_impact_04'
	 TrailClass=class'SpaceShipEngineTrail'
	 TrailRelativePos=(X=-95.0,Y=10.0,Z=0.0)
	 VehicleRotateSpeed=2
	 MenuName="in Human SpaceFighter"
	 FlyingAnim="Flying"
	 LandedAnim="Landed"
	 ActivateAnim="TakeOff"
	 DeactivateAnim="Landing"
	 ReactiveAccel=64.0
	 MinigunProjClass=class'PlasmaProjectile'
	 RocketProjClass=class'SpaceShipRocketProjectile'
	 MinigunMuzzleClass=None
	 CameraDist=4.0
	 AccelRotMin=-10000
	 AccelRotMax=10000
	 LandMovementState="PlayerHelicoptering"
     DriverState="PlayerHelicoptering"
	 VehicleState=3
	 StartTime=1.7
	 StopTime=2.0
	 MinigunRightBone=""
	 MinigunLeftBone=""
	 ChassisBone="root"
	 MainRotorBone=""
	 TailRotorBone=""
	 MainRotorBoneTip=""
	 TailRotorBoneTip=""
	 BrokenDetailPos=(X=0.0,Y=0.0,Z=0.0)
	 BrokenDetailClass=class'SpaceFighterDefenderBrokenDetail'
	 VehicleFOV=100
	 bCanSeeDriver=false
     MinigunOffset=(X=200.0,Y=50.0,Z=-45.0)
     RocketLauncherOffset=(X=0.0,Y=155.0,Z=75.0)
	 StartupSound=Sound'MaricaSpaceFightersSounds.HnShipAccel01'
	 StopSound=Sound'MaricaSpaceFightersSounds.HnShipDecel01'
     DestroyedEffectClass=Class'tVehicleExplosion'
     DestroyedSound=Sound'WeaponSounds.grenade_explodeair'
     EngineAmbientSound=Sound'MaricaSpaceFightersSounds.HnSpaceShipEng01'
     MinigunFireSound=Sound'MaricaSpaceFightersSounds.HnShipFire01'
	 RocketLaunchSound=Sound'MaricaSpaceFightersSounds.HnShipFire02'
     DriverPos=(X=20.0)
     EnterPos=(Y=200.0,Z=-10.0)
     ExitPos=(Y=200.0,Z=-10.0)
     VehicleFireSpeed=0.200000
     VehicleAltFireSpeed=2.0
     HealthMax=512.0
     AirSpeed=8000.0
     AccelRate=1000.0
     AirControl=0.5
     BaseEyeHeight=42.0
     EyeHeight=42.0
     Health=512.0
     Mesh=SkeletalMesh'MaricaSpaceFightersMeshes.SpaceFighter_Human'
	 Skins(0)=Shader'MaricaSpaceFightersTextures.SpaceFighterHum_shad01'
	 Skins(1)=Shader'MaricaSpaceFightersTextures.SpaceFighterHum_shad02'
     DrawScale=1.800000
     CollisionRadius=180.0
     CollisionHeight=85.0
	 Mass=400.0
}
