//=============================================================================
// HelicopterVehicle.
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2010 - 2012.
//=============================================================================

class HelicopterVehicle extends AircraftVehicle
	placeable;

var HelicopterRotatorActor RotatorActor, BladesRotatorActor;

var(MaricaArmory) class<MinigunProjectile> MinigunProjClass;
var(MaricaArmory) class<RocketProjectile> RocketProjClass;
var(MaricaArmory) class<MinigunMuzzleFlashEmitter> MinigunMuzzleClass;

var(MaricaArmory) sound MinigunFireSound;
var(MaricaArmory) sound RocketLaunchSound;
var(MaricaSounds) sound StartupSound;
var(MaricaSounds) sound StopSound;

var vector ServerPosition;
var(MaricaArmory) const vector MinigunOffset;
var(MaricaArmory) const vector RocketLauncherOffset;

var(MaricaSkeleton) Name MainRotorBone;
var(MaricaSkeleton) Name TailRotorBone;
var(MaricaSkeleton) Name MainRotorBoneTip;
var(MaricaSkeleton) Name TailRotorBoneTip;
var(MaricaSkeleton) Name ChassisBone;
var(MaricaSkeleton) Name MinigunRightBone;
var(MaricaSkeleton) Name MinigunLeftBone;

var rotator ServerRotation;
var rotator ControllerRotation;

var int VehicleState;
var int Rotor;
var int MinigunBarrel;
var int StartRotor;
var int StopRotor;
var int AltFireState;
var int velRotorRod;
var(MaricaArmory) int MinigunBarrelRate;
var(MaricaVehicle) int AccelRotMin;
var(MaricaVehicle) int AccelRotMax;

var float StartTime;
var float StopTime;
var(MaricaVehicle) int VehicleRotateSpeed;

var bool bBladesActive;
var bool bActivated;
var bool bCrashWay;

var Actor CurrentTarget;
var HelicopterDust CopterDust;
var LaserDetectDot LaserDetector;
var HelicopterKarmaBlock KarmaBlock;
var int TargetThinkNum;
var float LastCalcWeaponFire;
var bool bAimed;
var(MaricaArmory) int AimAngle;
var(MaricaArmory) float FireZ;
var(MaricaSounds) float VehicleFireVolume;
var(MaricaSounds) float VehicleAltFireVolume;
var(MaricaSounds) float AimDetectionVolume;
var(MaricaSounds) float EngStartUpVolume;
var(MaricaSounds) float EngStopVolume;
var(MaricaArmory) float DamageAmount;
var(MaricaVehicle) vector CopterDustOffset;
var(MaricaHUD) color ReticleColor;
var(MaricaHUD) Texture ReticleTexture;
var(MaricaHUD) Texture ReticleTextureAim;
var(MaricaArmory) sound AimDetectionSound;

var HelicopterVehicleMotorBlocker MotorBlocker;

replication
{
	unreliable if( Role == ROLE_Authority )
		RotatorActor , VehicleState , ServerPhysicsSimulation , bBladesActive , BladesRotatorActor , MotorBlocker , 
		MinigunBarrel , StartRotor , StopRotor , ServerRotation , ServerPosition , bActivated , CurrentTarget , LaserDetector;

	unreliable if (Role < ROLE_Authority)
		ClientToServerFire;
}

simulated function bool IsInDrivingState();

simulated function SendEnterMessage(PlayerController PC)
{
	PC.ReceiveLocalizedMessage(class'NVMessage', 1);
	PC.ReceiveLocalizedMessage(class'NVMessage', 8);
}

simulated function PossessedBy(Controller C)
{
	ControlAccess(false);
	if( MotorBlocker == None )
	{
		MotorBlocker = spawn(class'HelicopterVehicleMotorBlocker',self,,Location);
		MotorBlocker.SetTimer(StartTime,false);
	}
	else MotorBlocker.SetTimer(StartTime,false);
	Super.PossessedBy(C);
}

function ClientToServerFire(Actor myTarget)
{
	CurrentTarget = myTarget;
}

simulated state VehicleStart
{
	Begin:
	VehicleState = 1;
	bHaveDriver = true;
	ShadowRendering(true);
	Sleep(StartTime);
	VehicleState = 3;
	GotoState('VehicleIdle');
}

simulated state VehicleStop
{
	Begin:
	VehicleState = 2;
	Sleep(StopTime);
	ShadowRendering(false);
	bHaveDriver = false;
	VehicleState = 3;
	GotoState('VehicleIdle');
}

simulated state VehicleIdle
{
	Begin:
	VehicleState = 3;
}

simulated function VehicleActivate()
{
	if( !bActivated )
	{
		VehicleActivateAnim();
		if( StartupSound != None )
			PlaySound(StartupSound, SLOT_Misc, EngStartUpVolume);
			//PlaySound(StartupSound);
	}
	bActivated = true;
}

simulated function VehicleDeactivate()
{
	if( bActivated )
	{
		VehicleDeactivateAnim();
		if( StopSound != None )
			PlaySound(StopSound, SLOT_Misc, EngStopVolume);
			//PlaySound(StopSound);
	}
	bActivated = false;
}

simulated function VehicleActivateAnim()
{
	//STUB
}

simulated function VehicleDeactivateAnim()
{
	//STUB
}

simulated function DriverEnterVehicle()
{
	VehicleState = 1;
	bHaveDriver = true;
	ShadowRendering(true);
	GotoState('VehicleStart');
}

simulated function DriverLeaveVehicle()
{
	VehicleState = 2;
	bHaveDriver = false;
	GotoState('VehicleStop');
}

simulated function DriverEnter(P2MocapPawn p)
{
	if( VehicleState == 3 )
	{
		VehicleState = 1;
		ShadowRendering(true);
		Super.DriverEnter(p);
		DriverEnterVehicle();
	}
}

simulated function DriverLeave()
{
	if( VehicleState == 3 )
	{
		VehicleState = 2;
		Super.DriverLeave();
		DriverLeaveVehicle();
	}
}

simulated function TryToDrive(Pawn p)
{
	local Controller C;
	local P2MocapPawn P2MoPawn;
	C = p.Controller;
	P2MoPawn = P2MocapPawn(p);
    if ( (Driver == None) && (C != None) && C.bIsPlayer 
		&& C.IsInState('PlayerWalking') 
		&& p.IsHumanControlled() )
		DriverEnter(P2MoPawn);
}

simulated function UpdateRotors(int Rot)
{
	local rotator RotorRotation;

	if( BladesRotatorActor != None )
		BladesRotatorActor.RotationRate.Yaw = FClamp((Rot * 2048),0,200000);
	else BladesRotatorActor = Spawn(class'HelicopterRotatorActorBlades',self,,Location,Rotation);

	RotorRotation.Yaw = BladesRotatorActor.Rotation.Yaw;
	SetBoneRotation(MainRotorBone,RotorRotation,0,1);

	RotorRotation = rot(0,0,0);

	RotorRotation.Roll = BladesRotatorActor.Rotation.Yaw * 2;
	SetBoneRotation(TailRotorBone,RotorRotation,0,1);
}

simulated function UpdateBlades(float Speed)
{
	local Coords StartCoords, EndCoords;
	local vector HitLocation, HitNormal;
	local Actor Victim;

	if( Role == ROLE_Authority )
	{
		StartCoords = GetBoneCoords(MainRotorBone);
		EndCoords = GetBoneCoords(MainRotorBoneTip);
		Victim = Trace(HitLocation, HitNormal, EndCoords.Origin, StartCoords.Origin);

		if( Victim != None && Victim.bBlockActors )
		{
			if( Victim.IsA('Pawn') && !Victim.IsA('NPVehicle') && !Victim.IsA('PostalMultiplayerAirCraftVehicle') )
				Pawn(Victim).TakeDamage(1000, self, HitLocation, Velocity * 100, class'BladesDamage');
			/*else
			{
				//AddPhysImpulse(-Velocity);
				TakeDamage(Speed, Instigator, HitLocation, Velocity * 100, class'Crushed');
			}*/
		}
		/*else
		{
			if( !FastTrace(EndCoords.Origin, StartCoords.Origin) )
			{
				//AddPhysImpulse(-Velocity);
				TakeDamage(Speed, Instigator, HitLocation, Velocity * 100, class'Crushed');
			}
		}*/

		StartCoords = GetBoneCoords(TailRotorBone);
		EndCoords = GetBoneCoords(TailRotorBoneTip);
		Victim = Trace(HitLocation, HitNormal, EndCoords.Origin, StartCoords.Origin);

		if( Victim != None && Victim.bBlockActors )
		{
			if( Victim.IsA('Pawn') && !Victim.IsA('NPVehicle') && !Victim.IsA('PostalMultiplayerAirCraftVehicle') )
				Pawn(Victim).TakeDamage(1000, self, HitLocation, Velocity * 100, class'BladesDamage');
			/*else
			{
				//AddPhysImpulse(-Velocity);
				TakeDamage(Speed, Instigator, HitLocation, Velocity * 100, class'Crushed');
			}*/
		}
		/*else
		{
			if( !FastTrace(EndCoords.Origin, StartCoords.Origin) )
			{
				//AddPhysImpulse(-Velocity);
				TakeDamage(Speed, Instigator, HitLocation, Velocity * 100, class'Crushed');
			}
		}*/
	}
}

simulated function UpdateVehicleRotation(optional bool bFixed)
{
	local rotator VehicleRotation;
	local int ForwardSpeed , StrafeSpeed , AccelForwardRot , AccelStrafeRot;
	local vector worldForward , worldStrafe;

	if( Controller != None )
	{
		ControllerRotation = Controller.Rotation;
	}

	if( bFixed )
	{
		if(RotatorActor != None)
		{
			RotatorActor.DesiredRotation = ServerRotation;
			RotatorActor.RotationRate = rot(0,0,0);
			RotatorActor.SetRotation(ServerRotation);
		}
		SetRotation(ServerRotation);
		SetBoneRotation(ChassisBone,rot(0,0,0),0,1);
		ClientSimulation(ServerRotation);
		return;
	}

	worldForward = vect(1,0,0) >> Rotation;
	ForwardSpeed = Velocity Dot worldForward;
	ForwardSpeed = -ForwardSpeed;

	worldStrafe = vect(0,1,0) >> Rotation;
	StrafeSpeed = Velocity Dot worldStrafe;
	StrafeSpeed = -StrafeSpeed;

	AccelForwardRot = FClamp(ForwardSpeed,AccelRotMin,AccelRotMax);
	AccelStrafeRot = FClamp(StrafeSpeed,AccelRotMin,AccelRotMax);

	if(RotatorActor != None)
	{
		RotatorActor.DesiredRotation = ControllerRotation;
		RotatorActor.RotationRate = Normalize((Normalize(ControllerRotation) - Normalize(Rotation))) * VehicleRotateSpeed;
		VehicleRotation.Yaw = RotatorActor.Rotation.Yaw;
		SetRotation(VehicleRotation);
	}
	ClientSimulation(VehicleRotation);

	VehicleRotation = rot(0,0,0);
	VehicleRotation.Roll = AccelForwardRot;
	VehicleRotation.Pitch = AccelStrafeRot;
	SetBoneRotation(ChassisBone,VehicleRotation,0,1);
}

simulated function ClientSimulation(rotator VehicleRotation)
{
	local vector DriverLoc;

	if( bHaveDriver )
	{
		if( myDriver != None )
		{
			if( Level.NetMode == NM_Client )
			{
				DriverLoc = Location + (MyDriverPos >> Rotation);
				if( myDriver.Location != DriverLoc )
					myDriver.SetLocation(DriverLoc);
				if( myDriver.Rotation != Rotation )
					myDriver.SetRotation(Rotation);
			}
		}
	}
	else
	{
		if( myDriver != None )
		{
			myDriver.Destroy();
			myDriver = None;
		}
	}
}

simulated function VehicleHaveNoDriver()
{
	Super.VehicleHaveNoDriver();

	if(AmbientSound != None)
		AmbientSound = None;

	PhysicsSimulation(4);
}

simulated function VehicleHaveDriver()
{
	Super.VehicleHaveDriver();

	AmbientSound = EngineAmbientSound;

	PhysicsSimulation(1);
}

simulated function VehicleGetFire()
{
	if( PressingFire() )
	{
		if(Level.TimeSeconds > FireUpdate + VehicleFireSpeed)
		{
			VehicleFire(Rotation);
			if( MinigunFireSound != None )
				PlaySound(MinigunFireSound, SLOT_None, VehicleFireVolume, true);
				//PlaySound(MinigunFireSound,,VehicleFireVolume);
				//PlaySound(MinigunFireSound,,TransientSoundVolume*2.5);
			FireUpdate = Level.TimeSeconds;
		}
	}
	else if( PressingAltFire() )
	{
		if(Level.TimeSeconds > AltFireUpdate + VehicleAltFireSpeed)
		{
			VehicleAltFire(Rotation);
			if(RocketLaunchSound != None)
				PlaySound(RocketLaunchSound, SLOT_None, VehicleAltFireVolume, true);
				//PlaySound(RocketLaunchSound,,TransientSoundVolume*2.5);
			AltFireUpdate = Level.TimeSeconds;
		}
	}
}

simulated function VehicleFire(rotator Direction)
{
	local MinigunProjectile Bullet;
	local MinigunMuzzleFlashEmitter Muzzle;
	local rotator BarrelRotation;
	local vector Dir, Range, StartTrace, SpawnPos;
	local vector RotX, RotY, RotZ, HitLoc, HitNorm;
	local rotator FireOrient;

    GetAxes(Direction,RotX,RotY,RotZ);

	Dir = Vector(Direction);

	Super(NPVehicle).VehicleFire(Direction);

	Range.X = FRand() * 0.06;
	Range.Y = FRand() * 0.06;
	Range.Z = FRand() * 0.06;

	SpawnPos = Location + MinigunOffset.X * RotX +  MinigunOffset.Y * RotY + MinigunOffset.Z * RotZ;
	CalcWeaponFire( (vect(0,10,0) >> Rotation), HitLoc, HitNorm );
    FireOrient = Rotator( Normal(HitLoc - SpawnPos) );
	StartTrace = vector(FireOrient) + Range;

	if( MinigunProjClass != None )
	{
		Bullet = spawn(MinigunProjClass,Instigator,,SpawnPos);
		Bullet.SetVelocity(StartTrace);
	}

	Range.X = FRand() * 0.06;
	Range.Y = FRand() * 0.06;
	Range.Z = FRand() * 0.06;

	SpawnPos = Location + MinigunOffset.X * RotX -  MinigunOffset.Y * RotY + MinigunOffset.Z * RotZ;
	CalcWeaponFire( (vect(0,-10,0) >> Rotation), HitLoc, HitNorm );
    FireOrient = Rotator( Normal( HitLoc - SpawnPos ) );
	StartTrace = vector(FireOrient) + Range;

	if( MinigunProjClass != None )
	{
		Bullet = spawn(MinigunProjClass,Instigator,,SpawnPos);
		Bullet.SetVelocity(StartTrace);
	}

	if( MinigunMuzzleClass != None )
	{
		Muzzle = spawn(MinigunMuzzleClass,Instigator,,Location + MinigunOffset.X * RotX + 
			MinigunOffset.Y * RotY + MinigunOffset.Z * RotZ);
		Muzzle.SetBase(self);

		Muzzle = spawn(MinigunMuzzleClass,Instigator,,Location + MinigunOffset.X * RotX - 
			MinigunOffset.Y * RotY + MinigunOffset.Z * RotZ);
		Muzzle.SetBase(self);
	}

	MinigunBarrel += MinigunBarrelRate;

	if( MinigunBarrel > 65535 )
		MinigunBarrel = 0;

	BarrelRotation.Pitch = MinigunBarrel;
	if( MinigunRightBone != '')
		SetBoneRotation(MinigunRightBone,BarrelRotation,0,1);

	BarrelRotation.Pitch = -MinigunBarrel;
	if( MinigunLeftBone != '')
		SetBoneRotation(MinigunLeftBone,BarrelRotation,0,1);
}

/*simulated function VehicleAltFire(rotator Direction)
{
	local Pawn newTarget;
	local RocketProjectile Rocket;
	local vector Dir;
	local vector RotX, RotY, RotZ;

    GetAxes(Direction,RotX,RotY,RotZ);

	Dir = Vector(Direction);

	Super.VehicleAltFire(Direction);

	if( AltFireState == 0 )
	{
		Rocket = spawn(RocketProjClass,Instigator,,Location + RocketLauncherOffset.X * RotX + 
				RocketLauncherOffset.Y * RotY + RocketLauncherOffset.Z * RotZ);
		Rocket.SetupShot(10);
		Rocket.AddRelativeVelocity(Velocity);

		newTarget = CheckOutTarget();
		if( newTarget != None )
			Rocket.SetNewTarget(newTarget);
		AltFireState = 1;
	}
	else
	{
		Rocket = spawn(RocketProjClass,Instigator,,Location + RocketLauncherOffset.X * RotX - 
				RocketLauncherOffset.Y * RotY + RocketLauncherOffset.Z * RotZ);
		Rocket.SetupShot(10);
		Rocket.AddRelativeVelocity(Velocity);

		newTarget = CheckOutTarget();
		if( newTarget != None )
			Rocket.SetNewTarget(newTarget);
		AltFireState = 0;
	}
}*/

simulated function VehicleAltFire(rotator Direction)
{
	local Pawn newTarget;
	local RocketProjectile Rocket;
	local vector Dir;
	local vector RotX, RotY, RotZ;

    GetAxes(Direction,RotX,RotY,RotZ);

	Dir = Vector(Direction);

	Super(NPVehicle).VehicleAltFire(Direction);

	if( AltFireState == 0 )
	{
		Rocket = spawn(RocketProjClass,Instigator,,Location + RocketLauncherOffset.X * RotX + 
				RocketLauncherOffset.Y * RotY + RocketLauncherOffset.Z * RotZ);
		Rocket.SetupShot(10);
		Rocket.AddRelativeVelocity(Velocity);

		if( CurrentTarget != None )
			Rocket.SetNewTarget(CurrentTarget);
		else if( LaserDetector != None )
			Rocket.SetNewTarget(LaserDetector);
		else Rocket.SetNewTarget(CheckOutTarget());

		AltFireState = 1;
	}
	else
	{
		Rocket = spawn(RocketProjClass,Instigator,,Location + RocketLauncherOffset.X * RotX - 
				RocketLauncherOffset.Y * RotY + RocketLauncherOffset.Z * RotZ);
		Rocket.SetupShot(10);
		Rocket.AddRelativeVelocity(Velocity);

		if( CurrentTarget != None )
			Rocket.SetNewTarget(CurrentTarget);
		else if( LaserDetector != None )
			Rocket.SetNewTarget(LaserDetector);
		else Rocket.SetNewTarget(CheckOutTarget());

		AltFireState = 0;
	}
}

simulated function Tick(float DeltaTime)
{
	ContinuousUpdate(DeltaTime, VehicleState);
}

simulated function float DeltaModifier(float DeltaTime)
{
	return (DeltaTime*100 + 1.0);
}

simulated function UpdateDust()
{
	local Actor HitActor;
	local float HitDist;
	local vector TraceStart, HitLocation, HitNormal, TraceEnd;

	if( Driver != None && CopterDust == None )
	{
		CopterDust = spawn(class'HelicopterDust',self,,Location);
		CopterDust.SetRotation(rot(0,0,0));
	}

	if( Driver == None && CopterDust != None )
	{
		CopterDust.Destroy();
		CopterDust = None;
	}

	if( CopterDust == None )
		return;

	TraceStart = Location + (CopterDustOffset >> Rotation);
	TraceEnd = TraceStart - ( 800 * vect(0,0,1) );

	HitActor = Trace(HitLocation, HitNormal, TraceEnd, TraceStart, true);

	if( HitActor == None )
	{
		CopterDust.UpdateHoverDust(false, 0);
	}
	else if( HitActor.bWorldGeometry )
	{
		HitDist = VSize(HitLocation - TraceStart);

		CopterDust.SetLocation( HitLocation + (10+FClamp(HitDist/20,0,70))*HitNormal);

		CopterDust.UpdateHoverDust(true, HitDist/200);
	}
}

simulated function ContinuousUpdate(float DeltaTime, int State)
{
	local Actor TestTarget;
	local int velRotor;

	mySpeed = VSize(Velocity);

	if(Driver != None && Controller != None)
		DriverController = PlayerController(Controller);

	UserInterfaceControl();

	UpdateDust();

	GetVehicleData();

	if( Driver != None )
	{
		if( LaserDetector == None )
			LaserDetector = spawn(class'LaserDetectDot',self,,Location);
	}
	else
	{
		if( LaserDetector != None )
		{
			LaserDetector.Destroy();
			LaserDetector = None;
		}
	}

	TestTarget = CalcWeaponAltFire();
	if( DriverController != None && Level.TimeSeconds > LastCalcWeaponFire + 0.2 )
	{
		if( TestTarget != None && 
			( (Pawn(TestTarget) != None && Pawn(TestTarget).Health > 0) 
				|| (IntCollisionActor(TestTarget) != None && IntCollisionActor(TestTarget).myVehicle != None && IntCollisionActor(TestTarget).myVehicle.Health > 0 && IntCollisionActor(TestTarget).myVehicle.Driver != None) ) )
		{
			TargetThinkNum = 0;
			CurrentTarget = TestTarget;
			ClientToServerFire(TestTarget);
			bAimed = true;
			if( AimDetectionSound != None )
				PlaySound(AimDetectionSound, SLOT_Misc, AimDetectionVolume);
		}
		else
		{
			if( CurrentTarget != None && TargetThinkNum < 3 )
				TargetThinkNum++;
			else
			{
				CurrentTarget = None;
				ClientToServerFire(None);
			}
			bAimed = false;
		}
		LastCalcWeaponFire = Level.TimeSeconds;
	}

	if( State == 1 )
	{
		ServerPhysicsSimulation(5);
		PhysicsSimulation(5);

		//ControlAccess(false);

		EngineModel(0);

		UpdateVehicleRotation(true);

		StartRotor += DeltaModifier(DeltaTime);

		UpdateRotors(StartRotor*0.5);

		VehicleActivate();

		if( StartRotor > velRotorRod )
			bBladesActive = true;
		else bBladesActive = false;
	}
	else if( State == 2 )
	{
		//ControlAccess(true);

		UpdateVehicleRotation(true);

		if( mySpeed < 800 )
		{
			if( StopRotor > 0 )
				StopRotor -= DeltaModifier(DeltaTime);
			EngineModel(0);
			if( AmbientSound != None )
				AmbientSound = None;
			UpdateRotors(StopRotor*0.5);
		}
		else
		{
			velRotor = velRotorRod + mySpeed;
			UpdateRotors(velRotor);
			if( AmbientSound != EngineAmbientSound ) 
				AmbientSound = EngineAmbientSound;
			EngineModel(mySpeed-10);
		}

		VehicleDeactivate();

		if( StopRotor > 400 )
			bBladesActive = true;
		else bBladesActive = false;
	}
	else
	{
		if( bHaveDriver )
			ServerPhysicsSimulation(6);

		EngineModel(mySpeed);

		//ControlAccess(true);

		UpdateVehicleRotation(!bHaveDriver);

		ShadowRendering(bHaveDriver);

		VehicleGetFire();

		if( bHaveDriver )
		{
			velRotor = velRotorRod + mySpeed;
			UpdateRotors(velRotor);
			StopRotor = velRotor;
			bBladesActive = true;
		}
		else
		{
			if( mySpeed > 800 || bCrashWay )
			{
				bCrashWay = true;
				velRotor = velRotorRod + mySpeed;
				UpdateRotors(velRotor);
				if( AmbientSound != EngineAmbientSound ) 
					AmbientSound = EngineAmbientSound;
				EngineModel(mySpeed-10);
			}
			else
			{
				if( AmbientSound != None ) 
					AmbientSound = None;
			}
			bBladesActive = false;
		}

		StartRotor = 0;
		StopRotor = default.StopRotor;
	}

	if( bBladesActive )
		UpdateBlades(mySpeed);
}

simulated function GetVehicleData()
{
	local rotator ChassisRot;

	if( Role == ROLE_Authority )
		ChassisRot = Rotation;

	if( Level.NetMode != NM_Client )
	{
		ServerPosition = Location;
		ServerRotation = ChassisRot;
	}
}

function ServerPhysicsSimulation(int State)
{
	if( Level.NetMode != NM_Client )
		PhysicsSimulation(State);
}

function EngineModel(float Speed)
{
	local float DesiredEnginePitch,PitchMax;
	if(AmbientSound != None)
	{
		DesiredEnginePitch = Speed / (default.SoundPitch + 30);
		PitchMax = FClamp((default.SoundPitch + 10),0,254);
		SoundPitch = FClamp((default.SoundPitch + DesiredEnginePitch),0,PitchMax);
	}
}

simulated function ControlAccess(bool State)
{
	if( State )
	{
		if( AirSpeed != default.AirSpeed )
			AirSpeed = default.AirSpeed;
		if( AccelRate != default.AccelRate )
			AccelRate = default.AccelRate;
	}
	else
	{
		if( AirSpeed != 0.0 )
			AirSpeed = 0.0;
		if( AccelRate != 0.0 )
			AccelRate = 0.0;
	}
}

simulated function PhysicsSimulation(int State)
{
	local vector TraceEndLoc;
	if( State == 1 )
	{
		SetPhysics(PHYS_Flying);
		Velocity = vect(0,0,0);
		Acceleration = vect(0,0,0);
	}
	else if( State == 2 )
	{
		SetPhysics(PHYS_Falling);
		Velocity = vect(0,0,0);
		Acceleration = vect(0,0,-24000);
	}
	else if( State == 3 )
	{
		TraceEndLoc = Location;
		TraceEndLoc.Z -= 200;
		if( FastTrace(TraceEndLoc, Location) )
		{
			//PhysicsSimulation(4);wtf&!&!!&
			SetPhysics(PHYS_Falling);
			Velocity.Z += 100;
			Acceleration = vect(0,0,-24000);
			return;
		}
		SetPhysics(PHYS_None);
		Velocity = vect(0,0,0);
		Acceleration = vect(0,0,0);
	}
	else if( State == 4 )
	{
		SetPhysics(PHYS_Falling);
		Velocity.Z += 100;
		Acceleration = vect(0,0,-24000);
	}
	else if( State == 5 )
	{
		SetLocation(ServerPosition);
		SetPhysics(PHYS_None);
		Velocity = vect(0,0,0);
		Acceleration = vect(0,0,0);
	}
	else if( State == 6 )
	{
		if( Controller != None && Physics != PHYS_Flying )
			SetPhysics(PHYS_Flying);
	}
	else
	{
		Velocity = vect(0,0,0);
		Acceleration = vect(0,0,0);
	}
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	if( Controller == None )
		PhysicsSimulation(2);
	else PhysicsSimulation(1);

	if( Level.NetMode != NM_Client )
	{
		RotatorActor = Spawn(class'HelicopterRotatorActor',self,,Location,Rotation);
		//BladesRotatorActor = Spawn(class'HelicopterRotatorActorBlades',self,,Location,Rotation);
		//BladesRotatorActor.bHidden=false;
		//BladesRotatorActor.SetDrawType(DT_Mesh);
		//BladesRotatorActor.LinkMesh(Mesh);
		//BladesRotatorActor.bRotateToDesired = false;
		//BladesRotatorActor.RotationRate.Yaw = 32768;
		//KarmaBlock = spawn(class'HelicopterKarmaBlock',self,,Location);
		//KarmaBlock.SetCollisionSize(CollisionRadius + 8, CollisionHeight);
		//KarmaBlock.SetBase(self);
	}
	RotatorActor.SetRotation(Rotation);
}

simulated function PostNetBeginPlay()
{
	Super.PostNetBeginPlay();

	if( Controller == None )
		PhysicsSimulation(2);
	else PhysicsSimulation(1);
}

simulated function Destroyed()
{
	if( RotatorActor != None )
		RotatorActor.Destroy();
	if( BladesRotatorActor != None )
		BladesRotatorActor.Destroy();
	if( CopterDust != None )
		CopterDust.Destroy();
	if( LaserDetector != None )
		LaserDetector.Destroy();

	Super.Destroyed();
}

simulated function SpawnMyShadow()
{
	if(bAllowDynamicShadows && Level.NetMode != NM_DedicatedServer)
	{
		if(myShadow == None)
		{
			myShadow = Spawn(class'NVShadowProjector',Self,'',Location);
			myShadow.ShadowActor = self;
			myShadow.LightDirection = Normal(vect(1,1,6));
			myShadow.LightDistance = 1200;
			myShadow.MaxTraceDistance = 512;
			myShadow.UpdateShadow();
		}
	}
}

simulated function Actor CalcWeaponAltFire()
{
	local vector Target, StartLoc, HitLocation, DetectDotPos, HitNormal;
	local Actor Other;
	local rotator CurrentOrient;

	if( PlayerController(Controller) != None )
		CurrentOrient = PlayerController(Controller).Rotation;
	else CurrentOrient = Rotation;

	StartLoc = Location + ((FireZ*vect(0,0,1)) >> CurrentOrient);
	CurrentOrient.Pitch += AimAngle;
	//spawn(class'PistolPuff',self,,StartLoc);

	Target = Location + (vect(65536,0,0) >> CurrentOrient);

	Other = Trace(HitLocation, HitNormal, Target, StartLoc, true);

	if( VSize(HitLocation) == 0 )
		HitLocation = Location + (vect(65536,0,0) >> CurrentOrient);

	if( VSize(HitLocation - Location) < 800 )
		HitLocation = Location + (vect(800,0,0) >> CurrentOrient);

	if( LaserDetector != None )
	{
		DetectDotPos = (vect(40,0,0) >> rotator(Location-HitLocation)) + HitLocation;
		//DetectDotPos = HitLocation;
		LaserDetector.SetLocation(DetectDotPos);
	}

	return Other;
}

simulated function Actor CalcWeaponFire( vector AimSpace, out vector HitLocation, out Vector HitNormal )
{
	local vector Target, StartLoc;
	local Actor Other;
	local rotator CurrentOrient;

	if( PlayerController(Controller) != None )
	{
		CurrentOrient = PlayerController(Controller).Rotation;
		CurrentOrient.Yaw = Rotation.Yaw;
	}
	else CurrentOrient = Rotation;

	StartLoc = Location + ((FireZ*vect(0,0,1)) >> CurrentOrient);
	CurrentOrient.Pitch += AimAngle;

	if( CurrentOrient.Pitch > 32000 && CurrentOrient.Pitch < 62000 )
		CurrentOrient.Pitch = 62000;
	else if( CurrentOrient.Pitch > 68000 || CurrentOrient.Pitch < 32000 )
		CurrentOrient.Pitch = 68000;

	Target = Location + (vect(65536,0,0) >> CurrentOrient);

	Other = Trace(HitLocation, HitNormal, Target, StartLoc, true);

	if( VSize(HitLocation) == 0 )
		HitLocation = Location + (vect(65536,0,0) >> CurrentOrient);

	if( VSize(HitLocation - Location) < 800 )
		HitLocation = Location + (vect(800,0,0) >> CurrentOrient);

	//spawn(class'PistolPuff',self,,HitLocation);

	return Other;
}

simulated function DisplayDebug(Canvas Canvas, out float YL, out float YPos)
{
	if( !bDebugHUD )
		DrawCrossHair(Canvas);
	Super.DisplayDebug(Canvas, YL, YPos);
}

/********************************************************************************
* Converts a given directional vector to canvas coordinates.                    *
* X and Y values of the returned vector contain absolute coordinates.           *
* The function returns, whether the target direction is visible for the player. *
* Created by Wormbo                                                             *
********************************************************************************/
simulated function bool WormbosMapToHUD(out vector Result, rotator ViewRotation, float FOV, vector TargetDir, Canvas Canvas)
{
	local float TanFOVx, TanFOVy;
	local float TanX, TanY;
	local float dx, dy;
	local vector X, Y, Dir, XY;
 
	TanFOVx = Tan(FOV * Pi / 360);
	TanFOVy = (Canvas.ClipY / Canvas.ClipX) * TanFOVx;
	GetAxes(ViewRotation, Dir, X, Y);
 
	Dir *= TargetDir dot Dir;
	XY = TargetDir - Dir;
	dx = XY dot X;
	dy = XY dot Y;
 
	TanX = dx / VSize(dir);
	TanY = dy / VSize(dir);
 
	Result.X = Canvas.ClipX * 0.5 * (1 + TanX / TanFOVx);
	Result.Y = Canvas.ClipY * 0.5 * (1 - TanY / TanFOVy);
 
	return Dir dot vector(ViewRotation) > 0 && Result.X == FClamp(Result.X, Canvas.OrgX, Canvas.ClipX)
			&& Result.Y == FClamp(Result.Y, Canvas.OrgY, Canvas.ClipY);
}

simulated function DrawCrossHair(canvas Canvas)
{
	local P2Player p2p;
	local float UseScale;
	local Texture CrossHair;
	local float AimErrorSize;
	local rotator DirYawPC, DirYawSelf, DirPitchFire, DirPitchPC;
	local vector CamPos, DrawLoc;
	local rotator CamRot;

	p2p = P2Player(Controller);

	if( ReticleTexture != None && LaserDetector != None && p2p != None )
	{
		DirYawPC.Yaw = p2p.Rotation.Yaw;
		DirYawSelf.Yaw = Rotation.Yaw;
		DirPitchFire.Pitch = p2p.Rotation.Pitch + AimAngle;
		DirPitchPC.Pitch = DirPitchFire.Pitch;

		if( DirPitchFire.Pitch > 32000 && DirPitchFire.Pitch < 62000 )
			DirPitchFire.Pitch = 62000;
		else if( DirPitchFire.Pitch > 68000 || DirPitchFire.Pitch < 32000 )
			DirPitchFire.Pitch = 68000;

		AimErrorSize = (1 - FClamp((VSize(vector(DirPitchFire)-vector(DirPitchPC)))*1.5,0.f,1.0)) * (2 - VSize(vector(DirYawPC)-vector(DirYawSelf)))/2;
		//ClientMessage(AimErrorSize);

		ReticleColor.G = FClamp((255*AimErrorSize),50,255);
		ReticleColor.A = FClamp((150*AimErrorSize),50,150);

		// Draw reticle
		UseScale = P2Hud(p2p.MyHud).Scale;

		Canvas.GetCameraLocation(CamPos, CamRot);
		WormbosMapToHUD(DrawLoc, CamRot, p2p.FOVAngle, LaserDetector.Location-CamPos, Canvas);

		if( bAimed )
		{
			Canvas.Style = ERenderStyle.STY_Alpha;
			Canvas.DrawColor = default.ReticleColor;
			CrossHair = ReticleTextureAim;
			Canvas.SetPos((DrawLoc.X) - UseScale*(CrossHair.USize/2), (DrawLoc.Y) - UseScale*(CrossHair.VSize/2));
			Canvas.DrawIcon(CrossHair, UseScale);
		}

		Canvas.Style = ERenderStyle.STY_Alpha;
		Canvas.DrawColor = ReticleColor;
		CrossHair = ReticleTexture;
		Canvas.SetPos((DrawLoc.X) - UseScale*(CrossHair.USize/2), (DrawLoc.Y) - UseScale*(CrossHair.VSize/2));
		Canvas.DrawIcon(CrossHair, UseScale);
	}
}

simulated function Pawn CheckOutTarget()
{
	local vector HitNormal;
	local vector ProjectedHitLoc;
	local Pawn FindTarget, ReturnTarget;
	local Actor TestTarget;

	if(Level.NetMode == NM_Standalone
		|| Level.NetMode == NM_ListenServer)
	{
		foreach AllActors(class'Pawn',FindTarget)
		{
			if( FindTarget != None && FindTarget.Health > 0 )
			{
				TestTarget = Trace(ProjectedHitLoc, HitNormal, FindTarget.Location, Location, true);
				if( TestTarget != None 
					&& Pawn(TestTarget) != None 
					&& Pawn(TestTarget).Health > 0 
					&& Pawn(TestTarget).Controller != None )
					break;
			}
		}
	}
	if( TestTarget != None 
		&& Pawn(TestTarget) != None 
		&& Pawn(TestTarget).Health > 0 
		&& Pawn(TestTarget).Controller != None)
		ReturnTarget = Pawn(TestTarget);

	return ReturnTarget;
}

defaultproperties
{
	 Tag="Helicopter"
	 AimDetectionVolume=0.2
	 EngStartUpVolume=0.2
	 EngStopVolume=0.2
	 VehicleAltFireVolume=0.5
	 VehicleFireVolume=0.7
	 FireZ=197.0
	 AimAngle=2800
     ReticleTexture=Texture'MaricaTextures.HUD.Reticle01'
	 ReticleTextureAim=Texture'MaricaTextures.HUD.Reticle02'
     ReticleColor=(B=255,G=255,R=255,A=150)
	 AimDetectionSound=None//Sound'MiscSounds.Bleeps.Bleep'

	 CopterDustOffset=(Z=10.0)
	 CrashSounds(0)=Sound'MaricaSounds.ut_impact_03'
	 CrashSounds(1)=Sound'MaricaSounds.ut_impact_07'
	 bActivated=false
	 VehicleRotateSpeed=1
	 MenuName="in Helicopter"
	 MinigunProjClass=class'MinigunProjectile'
	 RocketProjClass=class'HelicopterRocketProjectile'
	 MinigunMuzzleClass=class'MinigunMuzzleFlashEmitter'
	 MinigunBarrelRate=2048
	 bCanFire=true
	 bCanFireAlt=true
	 AccelRotMin=-11000
	 AccelRotMax=11000
	 LandMovementState="PlayerHelicoptering"
     DriverState="PlayerHelicoptering"
	 VehicleState=3
	 StartTime=1.0
	 StopTime=1.0
	 StopRotor=50
	 velRotorRod=700
	 MinigunRightBone="MinigunR"
	 MinigunLeftBone="MinigunL"
	 ChassisBone="Chassis"
	 MainRotorBone="MainRotor"
	 TailRotorBone="TailRotor"
	 MainRotorBoneTip="MainRotor_tip"
	 TailRotorBoneTip="MainRotor_tip"
	 RanOverDamage=class'NVDamage'
	 BrokenDetailPos=(X=0.0,Y=0.0,Z=0.0)
	 BrokenDetailClass=class'HelicopterBrokenDetail'
	 VehicleFOV=90
	 bCanSeeDriver=true
	 MyDriverPos=(X=63.0,Y=-25.0,Z=-5.0)
     MinigunOffset=(X=96.0,Y=75.0,Z=-58.0)
     RocketLauncherOffset=(X=87.0,Y=113.0,Z=-61.0)
     DestroyedEffectClass=Class'tVehicleExplosion'
     DestroyedSound=Sound'WeaponSounds.grenade_explodeair'
     EngineAmbientSound=Sound'MaricaSounds.HelicopterIdle'
     MinigunFireSound=Sound'ProjectMaricaCore.MinigunFireSound'
	 RocketLaunchSound=Sound'WeaponSounds.rocket_launch'
     DriverPos=(X=20.0)
     EnterPos=(Y=185.0,Z=-10.0)
     ExitPos=(Y=185.0,Z=-20.0)
     bAllowDynamicShadows=true
     VehicleFireSpeed=0.09
     VehicleAltFireSpeed=2.0
     HealthMax=2048.0
     bCanBeBaseForPawns=true
     AirSpeed=2500.0
     AccelRate=1100.0
     AirControl=0.5
     BaseEyeHeight=42.0
     EyeHeight=42.0
     Health=2048.0
     Physics=PHYS_Flying
     Mesh=SkeletalMesh'MaricaModels.HelicopterModel'
	 Skins(0)=Shader'MaricaTextures.HelicopterTex.Helicopter_s_Skins02'
     DrawScale=1.300000
     AmbientGlow=32
     SoundRadius=100.0
	 SoundVolume=180.0
     TransientSoundVolume=1.0
     TransientSoundRadius=800.0
     CollisionRadius=128.0
     CollisionHeight=110.0
     bBlockKarma=false
	 RotationRate=(Pitch=0,Yaw=7800,Roll=0)
	 Mass=600.0
}
