//=============================================================================
// MinigunProjectile.
//
// Based on RifleProjectile.uc
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: RWS
// 2003.
//=============================================================================

class MinigunProjectile extends P2Projectile;

const FLESH_HIT_MAX	= 4;
const WAIT_TIME = 0.05;
const MIN_SIZE = 10.0;
const FORCE_RAD_CHECK =	50;

var class<TimedMarker> PawnHitMarkerMade;	// danger made when bullet hits a pawn 
var class<TimedMarker> BulletHitMarkerMade;	// danger made when bullet hits dirt
var Sound FleshHit[FLESH_HIT_MAX];
var Sound BulletSounds[FLESH_HIT_MAX];
var float FleshRad;
var vector Direction;
var float DamageAmmount;

var class<Actor> StaticImpactEffect;
var class<Actor> MetalImpactEffect;

function SetVelocity(vector Dir)
{
	Direction = Dir;
	Velocity = Speed*Direction;
}

auto state Moving
{
	ignores HitWall, Explode, Blowup, ProcessTouch, Touch, EncroachingOn;

	function TellNPCs(Actor Other, vector HitLocation)
	{
		if(Pawn(Other) != None)
		{
			if(P2Player(Instigator.Controller) != None
				&& FPSPawn(Other) != None)
			{
				P2Player(Instigator.Controller).Enemy = FPSPawn(Other);
			}

			PawnHitMarkerMade.static.NotifyControllersStatic(
				Level,
				PawnHitMarkerMade,
				FPSPawn(Owner), 
				FPSPawn(Other), 
				PawnHitMarkerMade.default.CollisionRadius,
				HitLocation);
		}
		else
		{
			BulletHitMarkerMade.static.NotifyControllersStatic(
				Level,
				BulletHitMarkerMade,
				FPSPawn(Owner), 
				None, 
				BulletHitMarkerMade.default.CollisionRadius,
				HitLocation);
		}
	}

	function bool ProcessTraceHit(Actor Other, out vector HitLocation, Vector HitNormal)
	{
		local Rotator NewRot;
		local float usesize;

		if(Other == None)
			return true;

		TellNPCs(Other, HitLocation);

		if(Other.bStatic)
		{
			spawn(StaticImpactEffect,Owner, ,HitLocation, Rotator(HitNormal));
			return false;
		}
		else 
		{
			Other.TakeDamage(DamageAmmount, Pawn(Owner),HitLocation,Velocity,MyDamageType);

			if(((P2Pawn(Other) != None && Other != Owner)
				|| PeoplePart(Other) != None
				|| CowheadProjectile(Other) != None) 
					&& NPVehicle(Other) == None )
			{
				Other.PlaySound(FleshHit[Rand(ArrayCount(FleshHit))],SLOT_Pain,,,FleshRad,GetRandPitch());
				spawn(class'BloodImpactHeadShot',Owner, ,HitLocation, Rotator(HitNormal));
			}
			else
			{
				spawn(MetalImpactEffect,Owner, ,HitLocation, Rotator(HitNormal));
			}

			if((NPVehicle(Other) != None 
					&& Other != Owner))
			{
				Other.PlaySound(BulletSounds[Rand(ArrayCount(BulletSounds))],SLOT_Pain,,,FleshRad,GetRandPitch());
				spawn(MetalImpactEffect,Owner, ,HitLocation, Rotator(HitNormal));
			}

			// Move to the other side of the thing you hit
			usesize = Other.CollisionRadius + Other.CollisionHeight;
			if(usesize < MIN_SIZE)
				usesize = MIN_SIZE;
			HitLocation += Direction*(usesize);
		}
		return true;
	}

	function TraceFire()
	{
		local vector HitLocation, HitNormal, StartTrace, EndTrace;
		local actor Other;

		StartTrace = Location;
		EndTrace = StartTrace + Velocity*WAIT_TIME;

		Other = Trace(HitLocation,HitNormal,EndTrace,StartTrace,true);

		if ( Other == None )
			HitLocation = EndTrace;

		if(ProcessTraceHit(Other, HitLocation, HitNormal))
		{
			if(!SetLocation(HitLocation))
				Destroy();
		}
		else Destroy();
	}

Begin:
	TraceFire();
	Sleep(WAIT_TIME);
	Goto('Begin');
}

defaultproperties
{
	 StaticImpactEffect=class'RifleBulletHitPack'
	 MetalImpactEffect=class'BulletSparkPack'
	 DamageAmmount=85.0
     FleshHit(0)=Sound'WeaponSounds.bullet_hitflesh1'
     FleshHit(1)=Sound'WeaponSounds.bullet_hitflesh2'
     FleshHit(2)=Sound'WeaponSounds.bullet_hitflesh3'
     FleshHit(3)=Sound'WeaponSounds.bullet_hitflesh4'
     BulletSounds(0)=Sound'MiscSounds.Doors.TryingLockedDoor'
     BulletSounds(1)=Sound'MiscSounds.Doors.TryingLockedDoor'
     BulletSounds(2)=Sound'MiscSounds.Doors.TryingLockedDoor'
     BulletSounds(3)=Sound'MiscSounds.Doors.TryingLockedDoor'
     PawnHitMarkerMade=Class'Postal2Game.PawnShotMarker'
     BulletHitMarkerMade=Class'Postal2Game.BulletHitMarker'
     FleshRad=350.000000
     speed=100000.000000
     MaxSpeed=100000.000000
     MomentumTransfer=50000.000000
     MyDamageType=Class'MinigunDamage'
     Physics=PHYS_None
     DrawType=DT_StaticMesh
     LifeSpan=10.000000
     TransientSoundRadius=400.000000
     bCollideActors=False
     bCollideWorld=False
     bBlockZeroExtentTraces=False
     bBlockNonZeroExtentTraces=False
}
