//=============================================================================
// PMPawn. (ProjectMarica's special multiplayer npc pawns)
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2011.
//=============================================================================

class PMPawn extends xMpPawnSS
	placeable;

var PMPawnRespawner PR;

function AttachBloodEffectsWhenDead();

simulated function Destroyed()
{
	if(PR != None)
		PR.PawnsCount--;
	Super.Destroyed();
}

simulated function Notify_StartPuking()
{
	if(MyHead != None)
	{
		//Head(MyHead).StartPuking(PukeType);

		// tell controller you started actually spewing chunks
		if(PersonController(Controller) != None)
			PersonController(Controller).ActuallyPuking();
	}
}

simulated function Tick(float DeltaTime)
{
	local name anim;
	local float frame,rate;
	GetAnimParams(0,Anim,frame,rate);
	if(Anim == 'None' || Anim == '')
	{
		SetupAnims();
	}
	Super.Tick(DeltaTime);
}

function AddDefaultInventory()
{
	local int i, randpick;
	local Inventory thisinv;
	local P2PowerupInv pinv;
	local P2Weapon p2weap;
	local class<P2Weapon> p2weapclass;
	local int usedam;
	local int Count;

	// Only let this be called once
	if (!bGotDefaultInventory)
	{
		//Log(self$" PersonPawn.AddDefaultInventory() is adding inventory");
		// Everyone gets hands, so add that explicitly here
		thisinv = CreateInventoryByClass(class'Inventory.HandsWeapon');

		// Handle money seperately
		if(MaxMoneyToStart > 0)
		{
			pinv = P2PowerupInv(CreateInventoryByClass(class'Inventory.MoneyInv'));
			pinv.SetAmount(MaxMoneyToStart);
		}

		// Add in the extra stuff now
		for ( i=0; i<BaseEquipment.Length; i++ )
		{
			if (BaseEquipment[i].weaponclass != None)
			{
				//log(self$" base equipment "$i$" weap "$BaseEquipment[i].weaponclass);
				p2weapclass = class<P2Weapon>(BaseEquipment[i].weaponclass);

				// If you're in Liebermode and this is a violent weapon, change
				// it to a weak one (but don't bother the player)
				if(p2weapclass != None
					&& P2GameInfo(Level.Game).InLiebermode()
					&& p2weapclass.default.ViolenceRank >= class'PistolWeapon'.default.ViolenceRank
					&& !bPlayer)
				{
					// If you're a cop, don't let them have their pistols/machine guns
					// (leave them with batons)
					if(class == class'Police')
						BaseEquipment[i].weaponclass = None;
					else // Other people get random melee weapons in their inventories
					{
						randpick = Rand(3);
						switch(randpick)
						{
							case 0:
								BaseEquipment[i].weaponclass = class'BatonWeapon';
								break;
							case 1:
								BaseEquipment[i].weaponclass = class'ShovelWeapon';
								break;
							case 2:
								BaseEquipment[i].weaponclass = class'ShockerWeapon';
								break;
						}
					}
				}
				
				// If you have anything at all
				if(BaseEquipment[i].weaponclass != None)
				{
					// Create the weapon from this base equipment index
					thisinv = CreateInventoryByClass(BaseEquipment[i].weaponclass);
					p2weap = P2Weapon(thisinv);
					p2weap.bCanThrow = False;

					// Special weapon handling:
					// Keep track of the foot now, because we can't do it below (like the urethra)
					// --it's not added to the inventory list
					// Link up your foot
					if(FootWeapon(thisinv) != None)
					{
						MyFoot = P2Weapon(thisinv);
						MyFoot.GotoState('Idle');
						MyFoot.bJustMade=false;
						if(Controller != None)
							Controller.NotifyAddInventory(MyFoot);
						ClientSetFoot(P2Weapon(thisinv));
					}
				}
			}
		}

		// If you're in heston mode, no matter what, always give them some more random guns
		// (unless you're the player)
		if(P2GameInfo(Level.Game).InHestonMode()
			&& !bPlayer)
		{
			randpick = Rand(3);
			switch(randpick)
			{
				case 0:
					CreateInventoryByClass(class'PistolWeapon');
					break;
				case 1:
					CreateInventoryByClass(class'ShotgunWeapon');
					break;
				case 2:
					CreateInventoryByClass(class'MachinegunWeapon');
					break;
			}
			// And bump them up to be ready to fight
			if(PainThreshold < 1.0)
				PainThreshold = 1.0;
			if(Cajones < 1.0)
				Cajones = 1.0;
		}

		// If you're in Insane-o mode, no matter what, always give them some more random guns
		// (unless you're the player) (but give them more than heston)
		if(P2GameInfo(Level.Game).InInsaneMode()
			&& !bPlayer)
		{
			randpick = Rand(6);
			switch(randpick)
			{
				case 0:
					CreateInventoryByClass(class'PistolWeapon');
					break;
				case 1:
					CreateInventoryByClass(class'ShotgunWeapon');
					break;
				case 2:
					CreateInventoryByClass(class'MachinegunWeapon');
					break;
				case 3:
					CreateInventoryByClass(class'GrenadeWeapon');
					break;
				case 4:
					CreateInventoryByClass(class'MolotovWeapon');
					break;
				case 5:
					CreateInventoryByClass(class'ScissorsWeapon');
					break;
				case 6:
					CreateInventoryByClass(class'LauncherWeapon');
					break;
				case 7:
					CreateInventoryByClass(class'NapalmWeapon');
					break;
			}
			// And bump them up to be ready to fight
			if(PainThreshold < 1.0)
				PainThreshold = 1.0;
			if(Cajones < 1.0)
				Cajones = 1.0;
		}

		// tell the weapons they've been made now
		thisinv = Inventory;
		while(thisinv != None)
		{
			if(P2Weapon(thisinv) != None)
			{
				// After the creation of the weapon, while  normal weapons link up their ammo, 
				// we wait and finally link up the urethra. Don't do it above, during the creation
				// of the weapon, it must wait till here.
				if(UrethraWeapon(thisinv) != None)
				{
					MyUrethra = Weapon(thisinv);
					MyUrethra.GiveAmmo(self);
					ClientSetUrethra(P2Weapon(thisinv));
				}

				P2Weapon(thisinv).bJustMade=false;
			}
			thisinv = thisinv.inventory;
			Count++;
			if (Count > 5000)
				break;
		}

		// Switch them as a default, to use their hands
		if(PersonController(Controller) != None)
			PersonController(Controller).SwitchToThisWeapon(StartWeapon_Group, StartWeapon_Offset);

		// If you're using distance-based weapon changing, then save the weapons to use
		if(WeapChangeDist > 0)
		{
			if(BaseEquipment.Length >= 2)
			{
				if(BaseEquipment[CloseWeaponIndex].weaponclass != None)
					CloseWeap = class<P2Weapon>(BaseEquipment[CloseWeaponIndex].weaponclass);
				if(CloseWeap == None)
					log(self$" ERROR: WeapChangeDist and Close weapon is invalid!");

				if(BaseEquipment[FarWeaponIndex].weaponclass != None)
					FarWeap = class<P2Weapon>(BaseEquipment[FarWeaponIndex].weaponclass);
				if(FarWeap == None)
					log(self$" ERROR: WeapChangeDist and Far weapon is invalid!");
			}
			else
			{
				log(self$" ERROR: WeapChangeDist set with only 1 weapon!");
				WeapChangeDist = 0;
			}
		}

		// Note that we got our default inventory
		bGotDefaultInventory = true;

		//log(self$" in they hate me: "$P2GameInfo(Level.Game).TheyHateMeMode()$" game diff "$P2GameInfo(Level.Game).GameDifficulty);
		// If you're in They Hate Me mode, make everyone hate the player.
		// Do this at the end, so we don't retry to do this whole function again in
		// the new Possess, should redo our controller (below)
		if(P2GameInfo(Level.Game).TheyHateMeMode()
			&& !bPlayer)
		{
			// Leave innocents and cashiers alone. Hostiles get
			// made more hostile.
			if(bHasViolentWeapon)
			{
				bPlayerIsEnemy=true;
				bPlayerIsFriend=false;
				// And bump them up to be ready to fight, if they have a weapon
				if(PainThreshold < 1.0)
					PainThreshold = 1.0;
				if(Cajones < 1.0)
					Cajones = 1.0;

				if(CashierController(Controller) == None)
				{
					// If you were a Cop/Military, then turn into a cop-like bystander
					// so you don't try arrests or anything like that.
					if(Controller.IsA('PoliceController')
						|| Controller.IsA('RWSController')
						|| Controller.IsA('RedneckController')
						|| Controller.IsA('KumquatController'))
					{
						Controller.Destroy();
						Controller = None;
						Controller = spawn(class'BystanderController');
						if(Controller != None )
						{
							Controller.Possess(self);
							AIController(Controller).Skill += SkillModifier;
							CheckForAIScript();
						}
					}
				}
			}
		}

		// Make sure *un-armed* bystanders take one pistol bullet if you hit them exactly in the middle
		if(!bHasViolentWeapon
			&& bInnocent)
		{
			// Calculate pistol damage for one hit kills.
			usedam = class'PistolBulletAmmoInv'.default.DamageAmount*class'Dude'.default.DamageMult - 1;
			//log(self$" new health "$usedam);
			HealthMax = usedam;
			Health = HealthMax;
		}
		//log(self$" initting for new level, game info diff "$P2GameInfoSingle(Level.Game).GameDifficulty$" game state diff "$P2GameInfoSingle(Level.Game).TheGameState.GameDifficulty$" GET game state diff "$P2GameInfoSingle(Level.Game).GetGameDifficulty());
	}
}

defaultproperties
{
	 bAllowStasis=false
	 //AmbientGlow=0
     //StartInterest="xMpPawn"
}
