//=============================================================================
// RVCar. (Nicks karma based vehicle car class)
//
// Base model: model from UT2004 by EpicGames.
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2011.
//=============================================================================

class PRVCar extends NKVCarNew;

var Actor MissilePackBase;
var NKVTurrelRotator MissilePackRotator;
var NKVMissilePack MissilePack;

var rotator TurrelRotation;
var rotator ControllerRotation;
var rotator OldTurrelRot;

var(MaricaArmory) vector MissilePackBasePos;
var(MaricaArmory) vector MissilePackPos;

var int EnemyDist;

var float FireTime;
var(MaricaArmory) float ScriptedFireTime;
var(MaricaArmory) float FireSpeed;

var LaserDetectDot LaserDetector;
var(MaricaArmory) Texture ReticleTexture;
var(MaricaArmory) Color ReticleColor;
var(MaricaArmory) float FireZ;
var(MaricaArmory) int AimAngle;
var(MaricaArmory) int TurrelAimError;

replication
{
	unreliable if( Role == ROLE_Authority )
		MissilePack , MissilePackRotator , MissilePackBase , TurrelRotation , ControllerRotation , OldTurrelRot , LaserDetector;
}

simulated function SendEnterMessage(PlayerController PC)
{
	PC.ReceiveLocalizedMessage(class'NVMessage', 1);
	PC.ReceiveLocalizedMessage(class'NVMessage', 4);
	PC.ReceiveLocalizedMessage(class'NVMessage', 8);
}

simulated function PostBeginPlay()
{
	Super.PostBeginPlay();

	if(Level.NetMode != NM_Client)
	{
		MissilePackBase = Spawn(class'NKVMissilePackBase',self,,Location + (MissilePackBasePos >> Rotation),Rotation);
		MissilePackBase.SetBase(self);

		MissilePack = Spawn(class'NKVMissilePack',self,,Location + (MissilePackPos >> Rotation),Rotation);
		MissilePack.SetBase(self);
		MissilePack.Chassis = self;
		MissilePack.SetRotation(Rotation);

		MissilePackRotator = Spawn(class'NKVTurrelRotator',self,,Location + (MissilePackPos >> Rotation));
		MissilePackRotator.SetRotation(Rotation);
	}
	if( IntCollision != None )
		IntCollision.SoundRadius += 120;

	OldTurrelRot = Rotation;
	ControllerRotation = Rotation;
}

state VehicleDriving
{
	simulated function MainSystem(float DesiredSpeed)
	{
		local bool bPSR;

		if(Passenger != None)
			bPSR = Passenger.bWantsToCrouch;

		if(Driver == None)
			return;

		RadioSystem(Driver.bWantsToCrouch, bPSR);

		if( myVehicleHands == None )
			FiringSystem(Driver.PressingFire(), Driver.PressingAltFire());
	}

	simulated function HandsTryToFire()
	{
		if( Driver == None )
			return;
		FiringSystem(true, false);
	}

	simulated function HandsTryToAltFire()
	{
		if( Driver == None )
			return;
		FiringSystem(false, true);
	}

	function FiringSystem(bool bFire, bool bAltFire)
	{
		if( MissilePack == None )
			return;

		if( Level.TimeSeconds > FireTime + FireSpeed )
		{
			if( bFire )
			{
				MissilePack.xDoFire(false, LaserDetector);
				FireTime = Level.TimeSeconds;
			}
			else if( bAltFire )
			{
				MissilePack.xDoFire(true);
				FireTime = Level.TimeSeconds;
			}
		}
	}

	simulated function ScriptedDriving()
	{
		local float DistTopointer, FocusFactor, SignFactor;
		local bool bNeedToTurn;
		local vector PointerLocation, LeftPoint, RightPoint;
		local rotator RotToPointer;

		if(Enemy == None)
		{
			if(EnemyController == None)
			{
				Throttle = 0;
				Steering = 0;
				Gear = 0;
				OutputBrake = true;
				IsDriving = false;
				return;
			}
			else
			{
				Enemy = EnemyMonitor();
				return;
			}
		}

		PointerLocation = Enemy.Location;
		DistToPointer = VSize(PointerLocation - Location);
        RotToPointer = Rotator(PointerLocation - Location);
		FocusFactor = VSize(Normal(Vector(Rotation)) - Normal(Vector(RotToPointer)));

		LeftPoint = Location + (vect(0,4,0) >> Rotation);
		RightPoint = Location + (vect(0,-4,0) >> Rotation);
		if( VSize(PointerLocation - LeftPoint) < VSize(PointerLocation - RightPoint) )
			SignFactor = -1;
		else SignFactor = 1;

		if( FocusFactor > 1.2 && !bNeedToTurn)
			bNeedToTurn = true;
		else if( FocusFactor < 0.8 )
			bNeedToTurn = false;
		if( FocusFactor > 0.05 )
			FocusSteering = FClamp(FocusFactor * 2, 0.0, 1.0) * SignFactor;

		if( Level.TimeSeconds > AIThinkTimeDuration + AIThinkTime )
		{
			bAIBWGoing = bNeedToTurn;
			AIThinkTimeDuration = Level.TimeSeconds;
		}

		if( bAIBWGoing )
		{
			Steering = -FocusSteering;
			Gear = -1;
			OutputBrake = false;
			IsDriving = true;
			Throttle = -AIThrottle*0.8;
		}
		else if( !bNeedToTurn )
		{
			Steering = FocusSteering;

			if( DistToPointer > AIChasingDist )
			{
				Gear = 1;
				OutputBrake = false;
				IsDriving = true;
				Throttle = AIThrottle;
			}
			else
			{
				Gear = 0;
				OutputBrake = true;
				IsDriving = false;
				Throttle = 0;
			}
		}

		EnemyDist = DistToPointer;

		if( MissilePack != None )
		{
			ControllerRotation = Rotator(PointerLocation - MissilePack.Location);

			if( DistToPointer < 8192 )
			{
				if( Level.TimeSeconds > FireTime + ScriptedFireTime )
				{
					MissilePack.DoBotFire(Enemy);
					FireTime = Level.TimeSeconds;
				}
			}
		}
	}
}

simulated event DriverEnter(Pawn p)
{
	local PlayerController PC;

	Super.DriverEnter(p);

	PC = PlayerController(Driver.Controller);
	//PC.ClientSetViewTarget(MissilePack);
	PC.ClientSetViewTarget(self);

	if(MissilePack != None)
		MissilePack.DriverInstigator = Driver;
}

simulated event DriverLeave(optional bool bDestroying)
{
	Super.DriverLeave(bDestroying);
	if(MissilePack != None)
		MissilePack.DriverInstigator = None;
}

simulated function VehicleHaveDriver()
{
	Super.VehicleHaveDriver();
	if(MissilePack != None)
		MissilePack.DriverInstigator = Driver;
}

function VehicleHaveNoDriver(optional bool bDestroying, optional bool bDriverFallOff)
{
	Super.VehicleHaveNoDriver(bDestroying, bDriverFallOff);
	if(MissilePack != None)
		MissilePack.DriverInstigator = None;
}

simulated function MonitorBehindView(PlayerController PC, bool bDriver)
{
	local Actor myPawnActor, ViewPoint;
	local DriverBase PawnBase;
	local Pawn myPawn;
	if(PC == None)
		return;

	if(bDriver)
	{
		ViewPoint = FPDriverViewPoint;
		PawnBase = DriverBase;
		myPawnActor = myDriver;
		myPawn = Driver;
	}
	else
	{
		ViewPoint = FPPassengerViewPoint;
		PawnBase = PassengerBase;
		myPawnActor = myPassenger;
		myPawn = Passenger;
	}

	if(!bFixedFPCamera)
		ViewPoint.SetRotation(PC.Rotation);

	if( SteerWheelBone != '')
		RotateSteeringWheel(PC.bBehindView);

	if(bAllowChangeView)
	{
		if(!PC.bBehindView && !ViewChanged)
		{
			PC.FixFOV();
			bHideDriverActor = true;
			if(myPawnActor != None)
				myPawnActor.bHidden = bHideDriverActor;
			PawnBase.SetFixedPosition(true);
			if(bFreeCamera)
				myPawn.SetBase(PawnBase);
			PC.ClientSetViewTarget(ViewPoint);
			PC.bFreeCamera = false;
			ViewChanged = true;
		}
		else if(PC.bBehindView && ViewChanged)
		{
			bHideDriverActor = false;
			if(myPawnActor != None)
				myPawnActor.bHidden = bHideDriverActor;
			PawnBase.SetFixedPosition(false);
			if(bFreeCamera)
				myPawn.SetBase(PawnBase);
			//PC.ClientSetViewTarget(MissilePack);
			PC.ClientSetViewTarget(self);
			ViewChanged = false;
		}
	}
	else if(!PC.bBehindView)
		PC.ClientSetBehindView(true);
}

simulated function ContinuousUpdate()
{
	Super.ContinuousUpdate();
	TurrelRotating();
}

simulated event ClientSimulationPack(optional bool LowRender)
{
	Super.ClientSimulationPack(LowRender);
	UpdateMissileActors();
}

simulated function UpdateMissileActors()
{
	local vector MissilePackLoc;
	local vector MissilePackBaseLoc;

	MissilePackLoc = Location + (MissilePackPos >> Rotation);
	MissilePackBaseLoc = Location + (MissilePackBasePos >> Rotation);

	/*if( MissilePack != None )
	{
		MissilePack.SetLocation(MissilePackLoc);
		MissilePack.SetBase(self);
	}*/

	if( MissilePackBase != None )
	{
		MissilePackBase.SetLocation(MissilePackBaseLoc);
		MissilePackBase.SetBase(self);
		MissilePackBase.SetRotation(Rotation);
	}
}

simulated function Actor CalcFireDirection()
{
	local vector Target, StartLoc, HitLocation, DetectDotPos, HitNormal;
	local Actor Other;
	local rotator CurrentOrient;

	if( Driver != None && Driver.Controller != None )
		CurrentOrient = Driver.Controller.Rotation;
	else CurrentOrient = Rotation;

	StartLoc = Location + ((FireZ * vect(0,0,1)) >> CurrentOrient);
	//spawn(class'PistolPuff',,,StartLoc);
	CurrentOrient.Pitch += AimAngle;

	Target = Location + (vect(65536,0,0) >> CurrentOrient);

	Other = Trace(HitLocation, HitNormal, Target, StartLoc, true);

	if( VSize(HitLocation) == 0 )
		HitLocation = Location + (vect(65536,0,0) >> CurrentOrient);

	if( VSize(HitLocation - Location) < 800 )
		HitLocation = Location + (vect(800,0,0) >> CurrentOrient);

	if( LaserDetector != None )
	{
		DetectDotPos = (vect(40,0,0) >> rotator(Location-HitLocation)) + HitLocation;
		//DetectDotPos = HitLocation;
		LaserDetector.SetLocation(DetectDotPos);
	}

	//LastCalcWeaponFire = Level.TimeSeconds;

	return Other;
}

simulated function TurrelRotating()
{
	local rotator FinalRotation, DirectRotation;

	if( MissilePack == None 
		|| MissilePackRotator == None )
		return;

	if( Driver != None )
	{
		if( LaserDetector == None )
			LaserDetector = spawn(class'LaserDetectDot',self,,Location);
		CalcFireDirection();
	}
	else
	{
		if( LaserDetector != None )
		{
			LaserDetector.Destroy();
			LaserDetector = None;
		}
	}

	if( !bAIControlled )
	{
		if( Driver != None && Driver.Controller != None )
		{
			ControllerRotation = Driver.Controller.Rotation;
			ControllerRotation.Pitch += 6200;
			ControllerRotation.Roll = 0;
		}
		else ControllerRotation = OldTurrelRot;
	}
	else
	{
		if( Enemy == None )
			ControllerRotation = OldTurrelRot;

		if( Enemy != None && EnemyDist >= 2048 )
			ControllerRotation.Pitch += 4096;

		MissilePack.SetRotation(ControllerRotation);
		OldTurrelRot = MissilePack.Rotation;
		return;
	}

	TurrelRotation = MissilePackRotator.Rotation;

	FinalRotation = Normalize((Normalize(ControllerRotation) - Normalize(TurrelRotation)));// * 1;

	MissilePackRotator.RotationRate = Normalize(FinalRotation);
	MissilePackRotator.DesiredRotation = ControllerRotation;

	DirectRotation.Yaw = MissilePackRotator.Rotation.Yaw - Rotation.Yaw;
	DirectRotation.Pitch = MissilePackRotator.Rotation.Pitch;

	MissilePack.SetRelativeRotation(DirectRotation);
	OldTurrelRot = TurrelRotation;
}

simulated function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, class<DamageType> damageType)
{
	if( ClassIsChildOf(damageType, class'RocketExplosionDamage') 
		&& instigatedBy == None && bAIControlled )
		return;
	else Super.TakeDamage(Damage, instigatedBy, hitlocation, momentum, damageType);
}

simulated function StartScriptedDriving(optional Actor myEnemy, optional Controller myEnemyContoller)
{
	local rotator RotToPointer;
	if( myEnemyContoller == None )
	{
		if( myEnemy != None )
		{
			if( Pawn(myEnemy) != None )
			{
				if( Pawn(myEnemy).Health > 0 )
					Enemy = myEnemy;
			}
			else Enemy = myEnemy;
		}
	}
	else EnemyController = myEnemyContoller;
	if( bAIInitialized )
		return;
	bAIInitialized = true;
	SpawnLights();
	Steering = 0.2;
	FocusSteering = 0.2;
	GotoState('BeforeBeStart');
}

simulated function Destroyed()
{
	Super.Destroyed();
	if( MissilePackBase != None )
		MissilePackBase.Destroy();
	if( MissilePack != None )
		MissilePack.Destroy();
	if( MissilePackRotator != None )
		MissilePackRotator.Destroy();
	if( LaserDetector != None )
		LaserDetector.Destroy();
}

state BeforeBeDestroyed
{
	ignores TryToDrive, DriverEnter, PassengerEnter;

    simulated function StartDestroying()
    {
		Super.StartDestroying();
		if( MissilePackBase != None )
		{
			MissilePackBase.Destroy();
			MissilePackBase = None;
		}
		if( MissilePack != None )
		{
			MissilePack.Destroy();
			MissilePack = None;
		}
		if( MissilePackRotator != None )
		{
			MissilePackRotator.Destroy();
			MissilePackRotator = None;
		}
    }

Begin:
    Sleep(0.1);
	Exploding();
    Sleep(0.1);
	StartDestroying();
    Sleep(DestroyedRespawnTime);
	if(!GameIsEnd())
		Destroy();
}

simulated function DisplayDebug(Canvas Canvas, out float YL, out float YPos)
{
	if( !bDebugHUD )
		DrawCrossHair(Canvas);
	Super.DisplayDebug(Canvas, YL, YPos);
}



/********************************************************************************
* Converts a given directional vector to canvas coordinates.                    *
* X and Y values of the returned vector contain absolute coordinates.           *
* The function returns, whether the target direction is visible for the player. *
* Created by Wormbo                                                             *
********************************************************************************/
simulated function bool WormbosMapToHUD(out vector Result, rotator ViewRotation, float FOV, vector TargetDir, Canvas Canvas)
{
	local float TanFOVx, TanFOVy;
	local float TanX, TanY;
	local float dx, dy;
	local vector X, Y, Dir, XY;
 
	TanFOVx = Tan(FOV * Pi / 360);
	TanFOVy = (Canvas.ClipY / Canvas.ClipX) * TanFOVx;
	GetAxes(ViewRotation, Dir, X, Y);
 
	Dir *= TargetDir dot Dir;
	XY = TargetDir - Dir;
	dx = XY dot X;
	dy = XY dot Y;
 
	TanX = dx / VSize(dir);
	TanY = dy / VSize(dir);
 
	Result.X = Canvas.ClipX * 0.5 * (1 + TanX / TanFOVx);
	Result.Y = Canvas.ClipY * 0.5 * (1 - TanY / TanFOVy);
 
	return Dir dot vector(ViewRotation) > 0 && Result.X == FClamp(Result.X, Canvas.OrgX, Canvas.ClipX)
			&& Result.Y == FClamp(Result.Y, Canvas.OrgY, Canvas.ClipY);
}

simulated function DrawCrossHair( canvas Canvas)
{
	local P2Player p2p;
	local float UseScale;
	local Texture CrossHair;
	local vector CamPos, DrawLoc;
	local rotator CamRot;

	if( Driver == None )
		return;

	p2p = P2Player(Driver.Controller);

	if( p2p == None )
		return;

	if( ReticleTexture != None && LaserDetector != None && p2p != None )
	{
		// Draw reticle
		UseScale = P2Hud(p2p.MyHud).Scale;

		Canvas.Style = ERenderStyle.STY_Alpha;
		Canvas.DrawColor = ReticleColor;
		CrossHair = ReticleTexture;

		Canvas.GetCameraLocation(CamPos, CamRot);
		WormbosMapToHUD(DrawLoc, CamRot, p2p.FOVAngle, LaserDetector.Location-CamPos, Canvas);

		Canvas.SetPos((DrawLoc.X) - UseScale*(CrossHair.USize/2), (DrawLoc.Y) - UseScale*(CrossHair.VSize/2));

		Canvas.DrawIcon(CrossHair, UseScale);
	}
}

defaultproperties
{
	 TurrelAimError=11000
	 Tag="WarHellbender"
	 ShadowZ=0.0
	 FireZ=190.0
	 AimAngle=3500
	 ReticleColor=(R=255,G=255,B=255,A=220)
	 ReticleTexture=Texture'MaricaTextures.HUD.Reticle02'

	 ExhaustBones(0)="Exhaust00"
	 ExhaustBones(1)="Exhaust01"
	 TorqueCurve=(Points=((InVal=0.0,OutVal=80.0),(InVal=550000.0,OutVal=180.0),(InVal=800000.0))

	 //SteerSpeed=31500.000000
	 MaxSteerAngle=3500.000000
	 MaxBrakeTorque=70
	 ScriptedFireTime=1.5
	 FireSpeed=1.5
	 MissilePackBasePos=(X=-160.0,Y=-5.0,Z=0.0)
	 MissilePackPos=(X=-160.0,Y=-5.0,Z=130.0)
     TireHandbrakeSlip=0.5
     TireHandbrakeFriction=-1.0
	 WheelRadius=49.0
	 CameraDist=15.0
	 bFixedFPCamera=False
     ExitPos=(Y=-170.000000,Z=5.000000)
     ExitPosPassenger=(Y=170.000000,Z=5.000000)
     DriverPos=(X=-20.800000,Y=-37.200000,Z=0.000000)
     PassengerPos=(X=-20.800000,Y=37.200000,Z=0.000000)
     FPDriverViewPos=(X=-20.000000,Y=-39.000000,Z=64.000000)
	 FPPassengerViewPos=(X=-20.000000,Y=39.000000,Z=64.000000)
     EnterPos=(Y=150.000000,Z=-2.000000)
     bCanSeeDriver=True
     ChassisMass=5.000000
     BrokenSkins(0)=Texture'MaricaTextures.PRVTex.PRVSkin_destroyed'
     BrokenSkins(1)=Shader'Timb.Misc.invisible_mask'
     BrokenSkins(2)=Texture'MaricaTextures.PRVTex.EnergyEffectMASKtex_exp'
	 BrokenSkins(3)=Texture'MaricaTextures.PRVTex.PRVSkin_destroyed'
     BrokenSkins(4)=Shader'Timb.Misc.invisible_mask'
	 BrokenSkins(5)=Texture'MaricaTextures.PRVTex.big_wheel_exp'
	 BrokenSkins(6)=Shader'Timb.Misc.invisible_mask'
     EnginePitch=32.000000
     EngineVolume=55.000000
	 SlipVolume=10.0
     Description="Hellbender from UT2004. || Model by EpicGames."
     LightProjectorClass=class'NVLightProjector'
	 ProjLightPos=(X=80.000000,Y=0.000000,Z=20.000000)
	 ProjectedLightRot=(Pitch=-100)
	 ProjLightMaterial=Texture'MaricaTextures.Effects.LightProj_01'
	 ProjLightFOV=40
	 ProjLightScale=0.400000
     ExhaustPos(0)=(X=-165.000000,Y=33.000000,Z=-88.000000)
     RadiatorPos=(Z=25.000000)
     FrontLightPos(0)=(X=185.000000,Y=50.000000,Z=10.000000)
     FrontLightPos(1)=(X=185.000000,Y=-50.000000,Z=10.000000)
     FrontLightPos(2)=(X=185.000000,Y=75.000000,Z=10.000000)
     FrontLightPos(3)=(X=185.000000,Y=-75.000000,Z=10.000000)
     RearLightPos(0)=(X=-218.000000,Y=98.000000,Z=35.000000)
     RearLightPos(1)=(X=-218.000000,Y=-98.000000,Z=35.000000)
     bRandomSkin=True
     OtherSkins(0)=Shader'MaricaTextures.PRVTex.PRVSkin01'
     OtherSkins(1)=Shader'MaricaTextures.PRVTex.PRVSkin02'
     OtherSkins(2)=Shader'MaricaTextures.PRVTex.PRVSkin03'
     Health=4096.000000
     MaxHealth=4096.000000
	 ImpactSoundsMin(0)=Sound'ProjectMaricaCore.ut_impact_03'
	 ImpactSoundsMed(0)=Sound'ProjectMaricaCore.ut_impact_01'
	 ImpactSoundsMed(1)=Sound'ProjectMaricaCore.ut_impact_02'
	 ImpactSoundsMax(0)=Sound'ProjectMaricaCore.ut_impact_04'
	 ImpactSoundsMax(1)=Sound'ProjectMaricaCore.ut_impact_05'
	 ImpactSoundsMax(2)=Sound'ProjectMaricaCore.ut_impact_06'
	 ImpactSoundsMax(3)=Sound'ProjectMaricaCore.ut_impact_07'
     EngineSound=Sound'MaricaSounds.ut_prv_engine_idle'
     //EngStartUpSound=Sound'MaricaSounds.ut_engine_start_01'
     //EngStopSound=Sound'MaricaSounds.ut_engine_stop_01'
     EngStartUpSound=None
     EngStopSound=None
     MaxSpeedAccel=34.000000
     VehicleMaxSpeed=300300.000000
     MaxRearSpeed=800.000000
     TireMass=0.700000
     WheelsHealth=200.000000
     WheelsMaxHealth=200.000000
     TireLateralFriction=1.250000
	 TireRollSlip=0.060000
	 TireLateralSlip=0.040000
	 TireSlipRate=0.007000
     EngTireClass=Class'PRVCarTire'
     SteerTireClass=Class'PRVCarTire'
     WheelSteerAlong=118.000000
     WheelSteerAcross=112.000000
     WheelEngAlong=-147.000000
     WheelEngAcross=112.000000
     WheelVert=-31.500000
     StrutFrontRight="RightFrontSTRUT"
     StrutFrontLeft="LeftFrontSTRUT"
     StrutRearRight="RightRearSTRUT"
     StrutRearLeft="LeftRearSTRUT"
     DestroyBones(0)="LeftDOOR"
     DestroyBones(1)="RightDOOR"
     DestroyBones(2)="WindowsGlass"
     DestroyBones(3)="Attachment"
	 DestroyBones(4)="SteerWheel"
	 SteerWheelBone="SteerWheel"
     RSuspHighLimit=24.000000
     RSuspLowLimit=-24.000000
	 SuspStiffness=200.0
	 SuspDamping=5.0
     IntCollisionModel=StaticMesh'MaricaCollision.PRVColl.PRVIntColl'
     FrontLightBones(0)="FrontLight00"
     FrontLightBones(1)="FrontLight01"
     FrontLightBones(2)="FrontLight02"
     FrontLightBones(3)="FrontLight03"
     RearLightBones(0)="RearLight00"
     RearLightBones(1)="RearLight01"
     ImpactSounds(0)=Sound'ProjectMaricaCore.ut_impact_01'
     ImpactSounds(1)=Sound'ProjectMaricaCore.ut_impact_01'
     Mesh=SkeletalMesh'MaricaModels.PRVModel'
     StaticMesh=StaticMesh'MaricaCollision.PRVColl.PRVKColl'
     Skins(0)=Shader'MaricaTextures.PRVTex.PRVSkin01'
     Skins(1)=Shader'ProjectMaricaCore.GlassTexShad'
     Mass=800.000000
     Begin Object Class=KarmaParamsRBFull Name=HellbenderPhysics
         //KInertiaTensor(0)=20.000000
         //KInertiaTensor(3)=30.000000
         //KInertiaTensor(5)=58.000000
         KInertiaTensor(0)=6.000000
         KInertiaTensor(3)=25.000000
         KInertiaTensor(5)=45.000000
         KCOMOffset=(X=-0.3,Z=-0.5)
         KLinearDamping=0.050000
         KAngularDamping=0.700000
         KStartEnabled=True
         bKNonSphericalInertia=True
         KFriction=0.200000
         Name="HellbenderPhysics"
     End Object
     KParams=KarmaParamsRBFull'HellbenderPhysics'
}
