//=============================================================================
// PositionRocketProjectile.uc
// Author: Nick Pilshikov "Evil"
// Email: Nick_Pv@mail.ru / EvilTheDark@gmail.com
// Site: Set-Games.ru / Revival-Games.ru
// Revival Games Studios.
// 2014.
//=============================================================================

class PositionRocketProjectile extends RocketProjectile;

var vector DirectionVector;

function DetermineTarget(vector FromHere);

function SetNewTarget(Actor NewTarget)
{
	SeekingTarget = NewTarget;
	DirectionTarget = SeekingTarget;
}

function SetNewTargetDirection(vector NewTargetDirection)
{
	DirectionVector = NewTargetDirection;
}

auto state Eject
{
	ignores TakeDamage;	 // so they won't blow up right outta the launcher.

	simulated function Tick(float DeltaTime)
	{
		// STUB, don't update your rotation here
	}

	simulated function ProcessTouch (Actor Other, Vector HitLocation)
	{
		// If two hit in the air, they both explode
		if(LauncherProjectile(Other) != None)
		{
			GenExplosion(HitLocation,Normal(HitLocation-Other.Location), Other);
			LauncherProjectile(Other).GenExplosion(HitLocation,Normal(Other.Location-HitLocation), self);
		}
		// Detonates on pawns
		else if ((Pawn(Other) != None
				&& Pawn(Other).Health > 0))
		{
			if(!RelatedToMe(Pawn(Other)))
			{
				GenExplosion(HitLocation,Normal(HitLocation-Other.Location), Other);
			}
		}
		// Bumps other projectiles (like grenades)
		else if(Projectile(Other) == None)
		{
			Other.Bump(self);
		}
	}

	simulated function Timer()
	{
		StartFlyTime=Level.TimeSeconds;
		GotoState('Flying');
	}

	simulated function BeginState()
	{
		//PlayAnim('Still', 0.1);
		SetTimer(0.05, false);
		// We jumped out, so make a smoke puff nearby
		EmitSmokePuff(0.3);

		PlaySound(RocketLaunch, SLOT_Misc, 1.0, false, 512.0, 1.0);
	}
}

simulated function RocketBounceRecoil(vector HitNormal, Actor SoundSource)
{
	local float usedamp;

	if(SeekingTarget != None)
		usedamp = VelDampen;
	else
		usedamp = 1;

	// Reflect and slow down
	Velocity = usedamp * (Velocity - 2 * HitNormal * (Velocity Dot HitNormal));

	// Decide if it's a wall
	if(HitNormal.z < FLOOR_Z)
		WallBounceCount++;

	// Bounce up wildly to try getting out of here.
	if(WallBounceCount >= WALL_HIT_MAX)
	{
		WallBounceCount=0;
		Velocity.z +=(FRand()*JumpStartMag);
	}

	// You need to seriously update
	if(!bNeedsToThink)
	{
		bNeedsToThink=true;
		LastBounceTime = Level.TimeSeconds;
		// Check to make sure you're target's not dead
		if( SeekingTarget == None )
		{
			ClearSeekingTarget();
		}
	}

	// Jump around a little crazily every time
	Velocity += (MinSpeedForBounce/2)*VRand();

	// Don't let it get too slow
	if(VSize(Velocity) < MinSpeedForBounce)
	{
		Velocity = MinSpeedForBounce*Normal(Velocity);
		// If we did go under, then make sure to bounce off the normal in a crazy manner.
		Velocity += (FRand()*MinSpeedForBounce*HitNormal);
	}

	// play a noise
	SoundSource.PlaySound(RocketBounce,,,,80,GetRandPitch());
}

simulated function DoThinking()
{
	bNeedsToThink=false;

	//Instigator.ClientMessage(SeekingTarget);

	// If you don't already have a target, try to find one again
	if(SeekingTarget != None)
	{
		// Drop you're speed significantly
		Velocity = VelDampenThink*Velocity;
		if(VSize(Velocity) < MinSpeedForBounce)
			Velocity = MinSpeedForBounce*Normal(Velocity);
		// Lose some seeking acceleration too
		SeekingAccelerationMag = RAND_RANGE*FRand()*default.SeekingAccelerationMag;
		ReadjustSpeed=READJUST_SLOW_MAG*default.ReadjustSpeed;
		// Decide when to seek again
		SeekingDot = FRand()*SEEKING_DOT_RANGE - 0.99;
	}
}

state Flying
{
	///////////////////////////////////////////////////////////////////////////////
	// Readjust your direction to find your target
	///////////////////////////////////////////////////////////////////////////////
	simulated function UpdateDirection()
	{
		local vector SeekingDir, VelDir;
		local float MagnitudeVel, dotcheck;
		local String num;
		local FPSPawn usepawn;

		VelDir = Normal(Velocity);

		// If you've bounced recently, consider slowing down
		if(bNeedsToThink
			&& Level.TimeSeconds > LastBounceTime + WAIT_TO_THINK_TIME)
			DoThinking();

		// Stop holding and go after him!
		if(bHolding
			&& Level.TimeSeconds > HoldTime + HOLD_AND_THINK_TIME)
		{
			// Go fast again after the target
			SeekingAccelerationMag = default.SeekingAccelerationMag;
			ReadjustSpeed = default.ReadjustSpeed;
			// Give it an instant, extra boost when starting off again
			MagnitudeVel += JumpstartMag;
			bHolding=false;
		}
		else
			MagnitudeVel=0;

		// Train onto your target
		if (DirectionTarget != None)
		{
			SeekingDir = DirectionTarget.Location - Location;
			// If we're not seeking our target, but seeking something nearby to help us navigate
			// then decide when we're close enough to it
			if(DirectionTarget != SeekingTarget)
			{
				//log(self$" dist to helper "$VSize(SeekingDir));
				if(VSize(SeekingDir) < HELPER_NEAR_RADIUS)
				{
					// Redirect ourself again at the seeking target again
					log(self$" close enough to "$DirectionTarget$" picking seeking again!");
					DirectionTarget = SeekingTarget;
					// Redo this direction, to help it readjust now to original target
					SeekingDir = DirectionTarget.Location - Location;
					// Boost the velocity back to a fast speed!
					Velocity = 2*Velocity;
				}
			}
			SeekingDir = Normal(SeekingDir);
			dotcheck = SeekingDir Dot VelDir;
			if(dotcheck != 0)
			{
				// Add to our mag, in case holding gave us a boost
				MagnitudeVel = MagnitudeVel + VSize(Velocity);
				SeekingDir = Normal(SeekingDir * ReadjustSpeed * MagnitudeVel + Velocity);

				if(!bHolding
					&& SeekingAccelerationMag < default.SeekingAccelerationMag)
				{
					//log(self$" dot "$SeekingDir dot Normal(Location - DirectionTarget.Location));
					if(SeekingDir dot Normal(Location - DirectionTarget.Location) < SeekingDot)
					{
						// Mark that you're tracking him
						HoldTime = Level.TimeSeconds;
						bHolding=true;
					}
					else
						SeekingAccelerationMag += ACC_REGAIN;
				}

				if(DirectionTarget == SeekingTarget)
					Velocity =  MagnitudeVel * SeekingDir;
				else
					Velocity =  MinSpeedForBounce * SeekingDir;

				if(SeekingAccelerationMag >= default.SeekingAccelerationMag)
				{
					Acceleration = SeekingAccelerationMag*SeekingDir;
					Acceleration.z += GravAccZ;
				}
				else
				{
					Acceleration.x=0;
					Acceleration.y=0;
					Acceleration.z=GravAccZ;
				}
			}
		}
		else
		{
			SeekingDir = DirectionVector - Location;
			SeekingDir = Normal(SeekingDir);
			dotcheck = SeekingDir Dot VelDir;
			if(dotcheck != 0)
			{
				// Add to our mag, in case holding gave us a boost
				MagnitudeVel = MagnitudeVel + VSize(Velocity);
				SeekingDir = Normal(SeekingDir * ReadjustSpeed * MagnitudeVel + Velocity);

				if(!bHolding
					&& SeekingAccelerationMag < default.SeekingAccelerationMag)
				{
					//log(self$" dot "$SeekingDir dot Normal(Location - DirectionTarget.Location));
					if(SeekingDir dot Normal(Location - DirectionVector) < SeekingDot)
					{
						// Mark that you're tracking him
						HoldTime = Level.TimeSeconds;
						bHolding=true;
					}
					else
						SeekingAccelerationMag += ACC_REGAIN;
				}

				/*if(DirectionTarget == SeekingTarget)
					Velocity =  MagnitudeVel * SeekingDir;
				else*/
					Velocity =  MinSpeedForBounce * SeekingDir;

				if(SeekingAccelerationMag >= default.SeekingAccelerationMag)
				{
					Acceleration = SeekingAccelerationMag*SeekingDir;
					Acceleration.z += GravAccZ;
				}
				else
				{
					Acceleration.x=0;
					Acceleration.y=0;
					Acceleration.z=GravAccZ;
				}
			}
		}

		// Check to run out of fuel
		// Looks here to find out if it's out fuel instead of
		// using the single timer like in the normal rocket to
		// tell it when it's out of fuel. A little slower, but
		// this doesn't get called constantly.
		if(LifeSpan < FallAfterNoFuelTime)
		{
			StartTumbling();
		}
		else
		{
			UpdateCount++;
			// Tell our seeking target we're after them
			if(UpdateCount == MAX_UPDATES)
			{
				usepawn = FPSPawn(SeekingTarget);
				if(usepawn != None
					&& LambController(usepawn.Controller) != None)
					LambController(usepawn.Controller).RocketIsAfterMe(FPSPawn(Instigator), self);
				UpdateCount = 0;
			}
		}
	}

Begin:
	Sleep(SeekReadjustFreq);
	UpdateDirection();
	Goto('Begin');
Bumped:
	Sleep(BumpReadjustFreq);
	UpdateDirection();
	Goto('Begin');
Controlled:
	Sleep(ControlledReadjustFreq);
	UpdateDirection();
	Goto('Begin');
}

defaultproperties
{
     TargetRadius=2000.0
     BounceMax=1
     SeekReadjustFreq=0.100000
     BumpReadjustFreq=0.400000
     ControlledReadjustFreq=1.0
     SeekingAccelerationMag=600.0
     GravAccZ=-20.0
     ReadjustSpeed=0.15
     VelDampenThink=0.3
     JumpstartMag=350.0
     SeekingDot=-0.950000
     ForwardAccelerationMag=600.0
     MinSpeedForBounce=200.0
     VelDampen=0.9
     Skins(0)=Texture'MaricaTextures.HelicopterTex.MissileSkin'
     Health=5
     speed=2000.0
     MaxSpeed=2500.0
     MyDamageType=Class'RocketExplosionDamage'
     bNetTemporary=False
     bUpdateSimulatedPosition=True
     DrawType=DT_StaticMesh
     StaticMesh=StaticMesh'MaricaCollision.HeliModel.Missile'
     DrawScale=1.0
     CollisionRadius=28.0
     CollisionHeight=28.0
}
