//=============================================================================
// ProjectMaricaMenu.uc
// Author: Nick Pilshikov "Evil"
// For Project Marica
// Revival Games Studios.
// 2014.
//=============================================================================

class ProjectMaricaMenu extends ShellMenuCW
	config(NicksVehicles);

var config string EnterVehicleKey;
var config string ChangeViewVehicleKey;
var config string LightsDetailConfig;

var int ShadowsDetail;
var int LightsDetail;
var bool bUpdate;
var bool bAllowChangedEvents;

var Texture LogoTexture;
var localized string ExitText;
var localized string ExitHelp;

var UWindowComboControl ShadowsQualityCombo;
var localized string ShadowsQualityText;
var localized string ShadowsQualityHelp;
const MaricaShadowsPath01 = "ProjectMaricaCore.NKVehicle bAllowDynamicShadows";
const MaricaShadowsPath02 = "ProjectMaricaCore.NPVehicle bAllowDynamicShadows";
const MaricaShadowsPath03 = "ProjectMaricaCore.NVShadowProjector ShadowDetail";
var string ShadowsQualityNames[6];

var UWindowComboControl LightsQualityCombo;
var localized string LightsQualityText;
var localized string LightsQualityHelp;
const MaricaLightsPath01 = "ProjectMaricaCore.NKVehicle bAllowLight";
const MaricaLightsPath02 = "ProjectMaricaCore.NKVehicle bAllowProjectionLight";
const MaricaLightsPath03 = "ProjectMarica.ProjectMaricaMenu LightsDetailConfig";
var string LightsQualityNames[3];

var UWindowCheckbox WildRideFOVCheckbox;
var localized string WildRideFOVText;
var localized string WildRideFOVHelp;
const MaricaWildRidePath01 = "ProjectMaricaCore.NKVehicle bWildRideFOV";

var UWindowCheckbox FixedCameraCheckbox;
var localized string FixedCameraText;
var localized string FixedCameraHelp;
const MaricaFixedCameraPath01 = "ProjectMaricaCore.NKVehicle bFreeCamera";

var UWindowCheckbox CorpseImpulseCheckbox;
var localized string CorpseImpulseText;
var localized string CorpseImpulseHelp;
const MaricaCorpseImpulsePath01 = "ProjectMaricaCore.IntCollisionActor bCorpseImpulse";

var UWindowCheckbox EnemyCarsCheckbox;
var localized string EnemyCarsText;
var localized string EnemyCarsHelp;
const MaricaEnemyCarsPath01 = "ProjectMarica.VehicleCreatorWeapon bEnemySystem";

var ShellMenuChoice ApplyChangesChoice;
var localized string ApplyChangesText;
var localized string ApplyChangesHelp;

var ShellMenuChoice DestroyVehiclesChoice;
var localized string DestroyVehiclesText;
var localized string DestroyVehiclesHelp;

var ProjectMaricaMenuInputControl EnterKeyEditBox;
var localized string EnterKeyText;
var localized string EnterKeyHelp;
const MaricaEnterKeyPath01 = "ProjectMarica.ProjectMaricaMenu EnterVehicleKey";
const MaricaEnterKeyPath02 = "ProjectMaricaCore.NVMessage ActionKeyString";

var ProjectMaricaMenuInputControl ViewKeyEditBox;
var localized string ViewKeyText;
var localized string ViewKeyHelp;
const MaricaViewKeyPath01 = "ProjectMarica.ProjectMaricaMenu ChangeViewVehicleKey";
const MaricaViewKeyPath02 = "Engine.PlayerController bBehindView";

///FIX2015
function ProjectMaricaMenuInputControl AddKeyWidget(String strText, String strHelp, int Font)
{
	local ProjectMaricaMenuInputControl ctl;
	
	ctl = ProjectMaricaMenuInputControl(CreateControl(class'ProjectMaricaMenuInputControl', GetNextItemPosX(), GetNextItemPosY(), ItemMaxWidth, ItemHeight));
	ctl.SetFont(Font);
	ctl.SetTextColor(ShellLookAndFeel(LookAndFeel).NormalTextColor);
	ctl.SetText(strText);
	ctl.SetHelpText(strHelp);
	ctl.EditBox.SetHelpText(strHelp);
	ctl.Align = TA_Left;
	// 02/01/03 JMI Well this explains why it wasn't working when I was setting fCommonCtlArea for the load/save
	//				menus.  I should've just done it instead of commenting about it below--duh.  Added.
	// 01/15/03 JMI Editbox is the only control that doesn't do a ctl.EditBoxWidth = ctl.WinWidth * fCommonCtlArea
	// which is likely only due to the relatively low use of this field.
	ctl.EditBoxWidth = ctl.WinWidth * fCommonCtlArea;
	AddItem(ctl, ItemHeight + ItemSpacingY);
	return ctl;
}
///FIX2015

function LoadShadowsConfig()
{
	local int i;

	ShadowsQualityCombo.Clear();

	for( i = 0 ; i < ArrayCount(ShadowsQualityNames) ; i++ )
	{
		ShadowsQualityCombo.AddItem(ShadowsQualityNames[i], string(i));
	}
}

function LoadLightsConfig()
{
	local int i;

	LightsQualityCombo.Clear();

	for( i = 0 ; i < ArrayCount(LightsQualityNames) ; i++ )
	{
		LightsQualityCombo.AddItem(LightsQualityNames[i], string(i));
	}
}

function ShadowsQualityChanged()
{
	ShadowsDetail = int(ShadowsQualityCombo.GetValue2());
}

function LightsQualityChanged()
{
	LightsDetail = int(LightsQualityCombo.GetValue2());
}

function EnterKeyEditChanged()
{
	local string N;

	bAllowChangedEvents = false;

	N = EnterKeyEditBox.GetValue();

	if ( Len(N) > 5 )
		N = left(N,5);

	ReplaceText(N, " ", "_");

	EnterKeyEditBox.SetValue(Caps(N));

	bAllowChangedEvents = true;
}

function ViewKeyEditChanged()
{
	local string N;

	bAllowChangedEvents = false;

	N = ViewKeyEditBox.GetValue();

	if ( Len(N) > 5 )
		N = left(N,5);

	ReplaceText(N, " ", "_");

	ViewKeyEditBox.SetValue(Caps(N));

	bAllowChangedEvents = true;
}

function CleanLevelFromVehicles()
{
	local Actor testActor;
	local Pawn myOwner;

	myOwner = GetPlayerOwner().Pawn;

	if( myOwner == None )
		return;

	foreach myOwner.AllActors(class'Actor', testActor)
	{
		if( testActor.IsA('NKVehicle') )
		{
			if( !NKVehicle(testActor).bSpecialVehicle )
				NKVehicle(testActor).Destroy();
		}
		else if( testActor.IsA('NPVehicle') )
		{
			if( !NPVehicle(testActor).bSpecialVehicle )
				NPVehicle(testActor).Destroy();
		}
	}
}

function WildRideFOVCheckboxChanged()
{
	if( bUpdate )
		GetPlayerOwner().ConsoleCommand("set" @ MaricaWildRidePath01 @ WildRideFOVCheckbox.bChecked);
}

function FixedCameraCheckboxChanged()
{
	if( bUpdate )
		GetPlayerOwner().ConsoleCommand("set" @ MaricaFixedCameraPath01 @ !FixedCameraCheckbox.bChecked);
}

function CorpseImpulseCheckboxChanged()
{
	if( bUpdate )
		GetPlayerOwner().ConsoleCommand("set" @ MaricaCorpseImpulsePath01 @ CorpseImpulseCheckbox.bChecked);
}

function EnemyCarsCheckboxChanged()
{
	if( bUpdate )
		GetPlayerOwner().ConsoleCommand("set" @ MaricaEnemyCarsPath01 @ EnemyCarsCheckbox.bChecked);
}

function UpdateMaricaConfigurations()
{
	local bool flag, tcamval;
	local int val;
	local string Str;
	local Actor testActor;
	local NKVehicle KarmaVehicle;
	local Pawn myOwner;

	if( !bUpdate )
		return;

	myOwner = GetPlayerOwner().Pawn;

	if( myOwner == None )
		return;

	Str = Caps(EnterKeyEditBox.GetValue());
	GetPlayerOwner().ConsoleCommand("set input" @ Str @ "use");
	GetPlayerOwner().ConsoleCommand("set" @ MaricaEnterKeyPath01 @ Str);
	//str = string(int(GetPlayerOwner().ConsoleCommand("KEYNO"@Str)));
	str = string(Asc(Str));
	GetPlayerOwner().ConsoleCommand("set" @ MaricaEnterKeyPath02 @ Str);

	Str = Caps(ViewKeyEditBox.GetValue());
	GetPlayerOwner().ConsoleCommand("set input" @ Str @ "NKVehCam");//tcam
	GetPlayerOwner().ConsoleCommand("set" @ MaricaViewKeyPath01 @ Str);

	val = int(GetPlayerOwner().ConsoleCommand("get" @ MaricaShadowsPath03));

	if( ShadowsDetail != val )
	{
		GetPlayerOwner().ConsoleCommand("set" @ MaricaShadowsPath03 @ string(ShadowsDetail));

		if( ShadowsDetail == 0 )
		{
			GetPlayerOwner().ConsoleCommand("set" @ MaricaShadowsPath01 @ false);
			GetPlayerOwner().ConsoleCommand("set" @ MaricaShadowsPath02 @ false);

			foreach myOwner.AllActors(class'Actor', testActor)
			{
				if( testActor.IsA('NKVehicle') )
				{
					NKVehicle(testActor).DestroyMyShadow();
				}
				else if( testActor.IsA('NPVehicle') )
				{
					NPVehicle(testActor).DestroyMyShadow();
				}
			}
		}
		else
		{
			GetPlayerOwner().ConsoleCommand("set" @ MaricaShadowsPath01 @ true);
			GetPlayerOwner().ConsoleCommand("set" @ MaricaShadowsPath02 @ true);

			foreach myOwner.AllActors(class'Actor', testActor)
			{
				if( testActor.IsA('VehicleCreatorWeapon') )
				{
					VehicleCreatorWeapon(testActor).ClearObjectPool();
				}
				else if( testActor.IsA('NKVehicle') )
				{
					NKVehicle(testActor).DestroyMyShadow(true);
					NKVehicle(testActor).SpawnMyShadow();
				}
				else if( testActor.IsA('NPVehicle') )
				{
					NPVehicle(testActor).DestroyMyShadow(true);
					NPVehicle(testActor).SpawnMyShadow();
				}	
			}
		}
	}

	val = int(GetPlayerOwner().ConsoleCommand("get" @ MaricaLightsPath03));

	if( LightsDetail != val )
	{
		if( LightsDetail == 0 )
		{
			GetPlayerOwner().ConsoleCommand("set" @ MaricaLightsPath01 @ false);
			GetPlayerOwner().ConsoleCommand("set" @ MaricaLightsPath02 @ false);
		}
		else if( LightsDetail == 1 )
		{
			GetPlayerOwner().ConsoleCommand("set" @ MaricaLightsPath01 @ true);
		}
		else
		{
			GetPlayerOwner().ConsoleCommand("set" @ MaricaLightsPath01 @ true);
			GetPlayerOwner().ConsoleCommand("set" @ MaricaLightsPath02 @ true);
		}
		GetPlayerOwner().ConsoleCommand("set" @ MaricaLightsPath03 @ LightsDetail);
	}
}

function CreateMenuContents()
{
	Super.CreateMenuContents();

	TitleFont = 1;
	ItemFont = 1;
	TitleAlign = TA_Center;
	ItemAlign = TA_Left;

	AddTitleBitmap(LogoTexture);

	AddTitle("", TitleFont, TitleAlign);

	ShadowsQualityCombo = AddComboBox(ShadowsQualityText, ShadowsQualityHelp, ItemFont);
	ShadowsQualityCombo.SetButtons(true);
	LoadShadowsConfig();

	LightsQualityCombo = AddComboBox(LightsQualityText, LightsQualityHelp, ItemFont);
	LightsQualityCombo.SetButtons(true);
	LoadLightsConfig();

	EnterKeyEditBox = AddKeyWidget(EnterKeyText, EnterKeyHelp, ItemFont);

	ViewKeyEditBox = AddKeyWidget(ViewKeyText, ViewKeyHelp, ItemFont);

	WildRideFOVCheckbox = AddCheckbox(WildRideFOVText, WildRideFOVHelp, ItemFont);

	FixedCameraCheckbox = AddCheckbox(FixedCameraText, FixedCameraHelp, ItemFont);

	CorpseImpulseCheckbox = AddCheckbox(CorpseImpulseText, CorpseImpulseHelp, ItemFont);

	EnemyCarsCheckbox = AddCheckbox(EnemyCarsText, EnemyCarsHelp, ItemFont);

	DestroyVehiclesChoice = AddChoice(DestroyVehiclesText, DestroyVehiclesHelp, ItemFont, ItemAlign);

	ApplyChangesChoice = AddChoice(ApplyChangesText, ApplyChangesHelp, ItemFont, ItemAlign);

	BackChoice = AddChoice(ExitText, "", ItemFont, ItemAlign, true);

	ResolutionChanged(Root.RealWidth, Root.RealHeight);

	GetPlayerOwner().SetPause(false);

	bAllowChangedEvents = true;

	LoadValues();
}

function LoadValues()
{
	local bool flag;
	local float val;
	local int iDef;

	aDefaultPaths[iDef++] = MaricaShadowsPath01;
	aDefaultPaths[iDef++] = MaricaShadowsPath02;
	aDefaultPaths[iDef++] = MaricaShadowsPath03;
	aDefaultPaths[iDef++] = MaricaWildRidePath01;

	StoreDefaultValues();

	bUpdate = False;


	// Value 0 to 5
	val = int(GetPlayerOwner().ConsoleCommand("get" @ MaricaShadowsPath03));
	ShadowsQualityCombo.SetSelectedIndex(val);
	ShadowsDetail = val;

	// Value 0 to 2
	val = int(GetPlayerOwner().ConsoleCommand("get" @ MaricaLightsPath03));
	LightsQualityCombo.SetSelectedIndex(val);
	LightsDetail = val;

	EnterKeyEditBox.SetValue(GetPlayerOwner().ConsoleCommand("get" @ MaricaEnterKeyPath01));

	ViewKeyEditBox.SetValue(GetPlayerOwner().ConsoleCommand("get" @ MaricaViewKeyPath01));

	// Value true or false
	flag = bool(GetPlayerOwner().ConsoleCommand("get" @ MaricaFixedCameraPath01));
	FixedCameraCheckbox.SetValue(!flag);

	// Value true or false
	flag = bool(GetPlayerOwner().ConsoleCommand("get" @ MaricaCorpseImpulsePath01));
	CorpseImpulseCheckbox.SetValue(flag);

	// Value true or false
	flag = bool(GetPlayerOwner().ConsoleCommand("get" @ MaricaEnemyCarsPath01));
	EnemyCarsCheckbox.SetValue(flag);

	// Value true or false
	flag = bool(GetPlayerOwner().ConsoleCommand("get" @ MaricaWildRidePath01));
	WildRideFOVCheckbox.SetValue(flag);


	bUpdate = True;
}

function Notify(UWindowDialogControl C, byte E)
{
	if(!bAllowChangedEvents)
		return;

	Super.Notify(C, E);
	switch(E)
	{
		case DE_Change:
			switch (C)
			{
				case ShadowsQualityCombo:
					ShadowsQualityChanged();
					break;
				case LightsQualityCombo:
					LightsQualityChanged();
					break;
			}
			break;
			
		case DE_Click:
			switch (C)
			{
				case EnterKeyEditBox:
					EnterKeyEditChanged();
					break;
				case ViewKeyEditBox:
					ViewKeyEditChanged();
					break;
				case FixedCameraCheckbox:
					FixedCameraCheckboxChanged();
					break;
				case CorpseImpulseCheckbox:
					CorpseImpulseCheckboxChanged();
					break;
				case EnemyCarsCheckbox:
					EnemyCarsCheckboxChanged();
				case WildRideFOVCheckbox:
					WildRideFOVCheckboxChanged();
					break;
				case DestroyVehiclesChoice:
					CleanLevelFromVehicles();
					break;
				case ApplyChangesChoice:
					UpdateMaricaConfigurations();
					break;
				case BackChoice:
					GoBack();
					break;
			}
			break;
	}
}

function GoBack()
{
	Super.GoBack();
	Close();
	HideMenu();
}

function Close(optional bool bByParent)
{
	Super.Close(bByParent);
}

defaultproperties
{
	 LightsDetailConfig="1"
	 LogoTexture=Texture'MaricaTextures.HUD.postal2underlined_PM'
     ShadowsQualityText="Shadows Quality"
     ShadowsQualityHelp="Click 'Apply changes' to save changes. For better performance, you should restart level. Lower values improve performance. 'Simple' saves FPS"
	 LightsQualityText="Lights Quality"
	 LightsQualityHelp="Click 'Apply changes' to save changes. 'Projector lights' can decrease performance."
	 ApplyChangesText="Apply changes"
	 ApplyChangesHelp="Click this button to update configurations."
	 DestroyVehiclesText="Destroy vehicles"
	 DestroyVehiclesHelp="Click this button to clean up level from vehicles."
	 WildRideFOVText="Wild Ride FOV"
	 WildRideFOVHelp="Check this box to turn on/off speed-fov."
	 FixedCameraText="Fixed Camera"
	 FixedCameraHelp="Check this box to turn on/off camera rotating fixation."
	 EnterKeyText="Enter vehicle key"
	 EnterKeyHelp="Type key, which will be enter key for vehicles, for example H or other keys on your keyboard. Click 'Apply changes' to save changes."
	 ViewKeyText="Change view vehicle key"
	 ViewKeyHelp="Type key, which will be key to change view for vehicles, for example J or other keys on your keyboard. Click 'Apply changes' to save changes."
	 CorpseImpulseText="Allow ragdoll impulse"
	 CorpseImpulseHelp="Check this box to turn on/off ragdoll impulse. Turn off this setting, if bystanders explodes."
	 EnemyCarsText="Enemy Cars"
	 EnemyCarsHelp="Check this box to turn on/off spawning AI bot controlled cars, shoot them to make you as enemy."
	 ExitText="Close"
	 ExitHelp="Click this button to close menu."
     MenuWidth=500.0
	 MenuHeight=600.0
     TitleHeight=100.000000
     TitleSpacingY=0.000000
     //ItemHeight=50.000000
     ItemSpacingY=2.000000
     //BorderLeft=15.000000
     //BorderRight=15.000000
     fCommonCtlArea=0.420000
	 LightsQualityNames(0)="No lights"
     LightsQualityNames(1)="Visible lights"
     LightsQualityNames(2)="Projector lights"
	 ShadowsQualityNames(0)="No shadows"
     ShadowsQualityNames(1)="Simple"
     ShadowsQualityNames(2)="Low"
     ShadowsQualityNames(3)="Medium"
     ShadowsQualityNames(4)="High"
     ShadowsQualityNames(5)="Ultra High"
}
