//=============================================================================
// RVCar. (Nicks karma based vehicle car class)
//
// Base model: model from UT2004 by EpicGames.
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2011.
//=============================================================================

class RVCar extends NKVCarNew;

var bool bNETBladesActivate;
var bool bBladesActivate;
var bool bBladesOpened;
var bool bLeftArmBroke;
var bool bRightArmBroke;
var bool bClientLeftArmBroke;
var bool bClientRightArmBroke;
var bool bAnimReplay;
var int AnimPlayed;

var(MaricaArmory) sound ArmExtendSound;
var(MaricaArmory) sound ArmRetractSound;
var(MaricaArmory) sound BladeBreakSound;

var class<DamageType> BladesDamageClass;

replication
{
	unreliable if( Role == ROLE_Authority )
		bNETBladesActivate;

	reliable if( Role == ROLE_Authority )
		bBladesActivate , bBladesOpened , AnimPlayed , bAnimReplay;

    reliable if( bNetDirty && Role == ROLE_Authority )
        bClientLeftArmBroke , bClientRightArmBroke;
}

simulated function SendEnterMessage(PlayerController PC)
{
	PC.ReceiveLocalizedMessage(class'NVMessage', 1);
	PC.ReceiveLocalizedMessage(class'NVMessage', 4);
	PC.ReceiveLocalizedMessage(class'NVMessage', 7);
}

simulated function PostBeginPlay()
{
	local class<DamageType> tempBladesDamageClass;
	Super.PostBeginPlay();
	tempBladesDamageClass = class<DamageType>(DynamicLoadObject("BaseFX.ScytheDamage", class'Class'));
	if( tempBladesDamageClass != None )
		BladesDamageClass = tempBladesDamageClass;
	else BladesDamageClass = class'BladesDamage';
}

simulated function ContinuousUpdate()
{
	Super.ContinuousUpdate();
	UpdateBlades();
}

state VehicleDriving
{
	simulated function MainSystem(float DesiredSpeed)
	{
		local bool bPSR;

		if(Passenger != None)
			bPSR = Passenger.bWantsToCrouch;

		if(Driver == None)
			return;

		RadioSystem(Driver.bWantsToCrouch, bPSR);

		bBladesActivate = Driver.PressingFire() || bNETBladesActivate;
	}

	simulated function HandsTryToAltFire()
	{
		if( Driver == None )
			return;
		bNETBladesActivate = !bNETBladesActivate;
	}

	function ScriptedDriving()
	{
		local float DistTopointer, FocusFactor, SignFactor;
		local bool bNeedToTurn;
		local vector PointerLocation, LeftPoint, RightPoint;
		local rotator RotToPointer;

		if(Enemy == None)
		{
			if(EnemyController == None)
			{
				Throttle = 0;
				Steering = 0;
				Gear = 0;
				OutputBrake = true;
				IsDriving = false;
				return;
			}
			else
			{
				Enemy = EnemyMonitor();
				return;
			}
		}

		PointerLocation = Enemy.Location;
		DistToPointer = VSize(PointerLocation - Location);
        RotToPointer = Rotator(PointerLocation - Location);
		FocusFactor = VSize(Normal(Vector(Rotation)) - Normal(Vector(RotToPointer)));

		LeftPoint = Location + (vect(0,4,0) >> Rotation);
		RightPoint = Location + (vect(0,-4,0) >> Rotation);
		if( VSize(PointerLocation - LeftPoint) < VSize(PointerLocation - RightPoint) )
			SignFactor = -1;
		else SignFactor = 1;

		if( FocusFactor > 1.2 && !bNeedToTurn)
			bNeedToTurn = true;
		else if( FocusFactor < 0.8 )
			bNeedToTurn = false;
		if( FocusFactor > 0.05 )
			FocusSteering = FClamp(FocusFactor * 2, 0.0, 1.0) * SignFactor;

		if( Level.TimeSeconds > AIThinkTimeDuration + AIThinkTime )
		{
			bAIBWGoing = bNeedToTurn;
			AIThinkTimeDuration = Level.TimeSeconds;
		}

		if( bAIBWGoing )
		{
			Steering = -FocusSteering;
			Gear = -1;
			OutputBrake = false;
			IsDriving = true;
			Throttle = -AIThrottle*0.8;
		}
		else if( !bNeedToTurn )
		{
			Steering = FocusSteering;

			if( DistToPointer > AIChasingDist )
			{
				Gear = 1;
				OutputBrake = false;
				IsDriving = true;
				Throttle = AIThrottle;
			}
			else
			{
				Gear = 0;
				OutputBrake = true;
				IsDriving = false;
				Throttle = 0;
			}
		}
		if( DistToPointer < 500 )
			bBladesActivate = true;
		else bBladesActivate = false;
	}
}

simulated function BladesOpen(bool bOnLoop)
{
	if( bOnLoop )
	{
		LoopAnim('RVarmOpenedIdle');
		return;
	}
	if( !bBladesOpened )
	{
        PlayAnim('RVArmExtend');
        if (!bLeftArmBroke || !bRightArmBroke)
            PlaySound(ArmExtendSound, SLOT_None, 0.5,,,, False);
    }
	bBladesOpened = true;
	AnimPlayed = 1;
	bAnimReplay = false;
}

simulated function BladesClose(bool bOnLoop)
{
	if( bOnLoop )
	{
		LoopAnim('RVarmClosedIdle');
		return;
	}
	if( bBladesOpened )
    {
        PlayAnim('RVArmRetract');
        if (!bLeftArmBroke || !bRightArmBroke)
            PlaySound(ArmRetractSound, SLOT_None, 0.5,,,, False);
	}
	bBladesOpened = false;
	AnimPlayed = 2;
	bAnimReplay = false;
}

simulated event AnimEnd( int Channel )
{
	bAnimReplay = true;
	if( myShadow != None )
		myShadow.bInstantUpdate = false;
	Super.AnimEnd(Channel);
}

simulated function UpdateBlades()
{
	local Coords ArmBaseCoords, ArmTipCoords;
	local vector HitLocation, HitNormal;
	local Actor Victim;

	if( myShadow != None && bBladesActivate )
		myShadow.bInstantUpdate = true;

	if( Driver != None )
		Instigator = Driver;
	else if( Passenger != None )
		Instigator = Passenger;
	else Instigator = None;

	if( bAnimReplay )
	{
		if( bBladesActivate )
		{
			if( AnimPlayed == 2 )
				BladesOpen(false);
		}
		else
		{
			if( AnimPlayed == 1 )
				BladesClose(false);
		}
	}

	// Left Blade Arm System
	if( Role == ROLE_Authority && bBladesActivate && !bLeftArmBroke )
	{
		ArmBaseCoords = GetBoneCoords('CarLShoulder');
		ArmTipCoords = GetBoneCoords('LeftBladeDummy');
		Victim = IntCollision.Trace(HitLocation, HitNormal, ArmTipCoords.Origin, ArmBaseCoords.Origin);

		if( Victim != None && Victim.bBlockActors )
		{
			if( Victim.IsA('Pawn') && !Victim.IsA('NPVehicle') && !Victim.IsA('PostalMultiplayerAirCraftVehicle') )
			{
				Pawn(Victim).TakeDamage(1000, Driver, HitLocation, Velocity * 100, BladesDamageClass);
				//log("left blade going blading");
			}
			else
			{
				//log("left blade going detach");
				bLeftArmBroke = True;
				bClientLeftArmBroke = True;
				BladeBreakOff('CarLSlider', class'RVCarLeftBladeBreakOffEffect');
			}
		}
		else if( !FastTrace(ArmTipCoords.Origin, ArmBaseCoords.Origin) )
		{
			bLeftArmBroke = True;
			bClientLeftArmBroke = True;
			BladeBreakOff('CarLSlider', class'RVCarLeftBladeBreakOffEffect');
		}
	}

	if( Role < ROLE_Authority && bClientLeftArmBroke )
	{
		bLeftArmBroke = True;
		bClientLeftArmBroke = False;
		BladeBreakOff('CarLSlider', class'RVCarLeftBladeBreakOffEffect');
	}

	// Right Blade Arm System
	if( Role == ROLE_Authority && bBladesActivate && !bRightArmBroke )
	{
		ArmBaseCoords = GetBoneCoords('CarRShoulder');
		ArmTipCoords = GetBoneCoords('RightBladeDummy');
		Victim = IntCollision.Trace(HitLocation, HitNormal, ArmTipCoords.Origin, ArmBaseCoords.Origin);

		if( Victim != None && Victim.bBlockActors )
		{
			if( Victim.IsA('Pawn') && !Victim.IsA('NPVehicle') && !Victim.IsA('PostalMultiplayerAirCraftVehicle') )
			{
				Pawn(Victim).TakeDamage(1000, Driver, HitLocation, Velocity * 100, BladesDamageClass);
				//log("left blade going blading");
			}
			else
			{
				//log("left blade going detach");
				bRightArmBroke = True;
				bClientRightArmBroke = True;
				BladeBreakOff('CarRSlider', class'RVCarRightBladeBreakOffEffect');
			}
		}
		else if( !FastTrace(ArmTipCoords.Origin, ArmBaseCoords.Origin) )
		{
			bRightArmBroke = True;
			bClientRightArmBroke = True;
			BladeBreakOff('CarRSlider', class'RVCarRightBladeBreakOffEffect');
		}
	}
	if( Role < ROLE_Authority && bClientRightArmBroke )
	{
		bRightArmBroke = True;
		bClientRightArmBroke = False;
		BladeBreakOff('CarRSlider', class'RVCarRightBladeBreakOffEffect');
	}
}

simulated function BladeBreakOff(name BoneName, class<Emitter> BreakEffect)
{
    PlaySound(BladeBreakSound, SLOT_None, 0.5,,,, False);
	SetVehicleBoneScale(BoneName,0.0);
    if (Level.NetMode != NM_DedicatedServer)
        spawn(BreakEffect);
}

defaultproperties
{
	 Tag="Scorpion"
	 ShadowZ=0.0
	 ExhaustBones(0)="Exhaust00"
	 ExhaustBones(1)="Exhaust01"
	 bAnimReplay=true
	 TorqueCurve=(Points=((InVal=0.0,OutVal=50.0),(InVal=550000.0,OutVal=90.0),(InVal=800000.0))

	 AnimPlayed=2
	 MaxBrakeTorque=60
     ArmExtendSound=Sound'MaricaSounds.RV.Shing1'
     ArmRetractSound=Sound'MaricaSounds.RV.Shing2'
     BladeBreakSound=Sound'MaricaSounds.RV.RVBladeBreakOff'

	 RearMoveAccel=0.55
	 SlipVolume=11.0
	 WheelRadius=43.0
	 ActionAngularDamp=0.0
	 RestAngularDamp=0.6

     //EngStartUpSound=Sound'MaricaSounds.ut_engine_start_02'
     //EngStopSound=Sound'MaricaSounds.ut_engine_stop_02'
     EngStartUpSound=None
     EngStopSound=None
	 ImpactSoundsMin(0)=Sound'MaricaSounds.ut_impact_03'
	 ImpactSoundsMed(0)=Sound'MaricaSounds.ut_impact_01'
	 ImpactSoundsMed(1)=Sound'MaricaSounds.ut_impact_02'
	 ImpactSoundsMax(0)=Sound'MaricaSounds.ut_impact_04'
	 ImpactSoundsMax(1)=Sound'MaricaSounds.ut_impact_05'
	 ImpactSoundsMax(2)=Sound'MaricaSounds.ut_impact_06'
	 ImpactSoundsMax(3)=Sound'MaricaSounds.ut_impact_07'

	 ShadowLightDistance=1200
	 ShadowMaxTraceDistance=230

	 CameraDist=13.0
	 bFixedFPCamera=False
	 bAllowPassenger=False
	 Description="Scorpion from UT2004. || Model by EpicGames."
	 DestroyBones(0)="CarLShoulder"
	 DestroyBones(1)="CarRShoulder"
	 DestroyBones(2)="Attachment"

	 DecoSystem=DS_Bones

	 FrontLightBones(0)="FrontLight00"
	 FrontLightBones(1)="FrontLight01"

	 RearLightBones(0)="RearLight00"
	 RearLightBones(1)="RearLight01"

	 RSuspHighLimit=24.000000
	 RSuspLowLimit=-24.000000
     ExitPos=(Y=-140.000000,Z=15.000000)
     ExitPosPassenger=(Y=140.000000,Z=15.000000)
     EnterPos=(Y=145.000000,Z=-5.000000)
	 WheelFrontRight="RightFrontTIRE"
	 WheelFrontLeft="LeftFrontTIRE"
	 WheelRearRight="RightRearTIRE"
	 WheelRearLeft="LeftRearTIRE"
	 StrutFrontRight="RfrontStrut"
	 StrutFrontLeft="LfrontStrut"
	 StrutRearRight="RrearStrut"
	 StrutRearLeft="LrearStrut"
	 IntCollisionSize=(X=1,Y=1,Z=1)
	 IntCollisionModel=StaticMesh'MaricaCollision.RVColl.RVIntColl'
	 IntCollisionClass=Class'IntCollisionActor'
     bCarIsRTWD=True
	 bCarIsFTWD=True
	 LightProjectorClass=class'NVLightProjector'
	 ProjLightPos=(X=120.000000,Y=0.000000,Z=-17.000000)
	 ProjLightMaterial=Texture'MaricaTextures.Effects.LightProj_01'
	 ProjLightFOV=40
	 ProjLightScale=0.400000
	 ProjectedLightRot=(Pitch=-100)
     FrontLightClass=Class'NVFrontCarLight'
     RearLightClass=Class'NVRearCarLight'
     FrontLightPos(0)=(X=148.000000,Y=45.000000,Z=-20.000000)
     RearLightPos(0)=(X=-152.000000,Y=35.000000,Z=-20.000000)
     FrontLightPos(1)=(X=148.000000,Y=-45.000000,Z=-20.000000)
     RearLightPos(1)=(X=-152.000000,Y=-35.000000,Z=-20.000000)
     bSpawnBrokenDetail=True
     bAllowBrokenWheels=True
     MaxSpeedAccel=26.000000
	 bCanSeeDriver=True
	 VehicleMaxSpeed=315350.000000
     MaxSteerAngle=3500.000000
	 //SteerSpeed=31500.000000
     ChassisMass=4.000000
	 RadiatorPos=(X=120.000000,Y=0.000000,Z=-17.000000)
	 DriverPos=(X=-16.000000,Y=0.000000,Z=-49.000000)
	 FPDriverViewPos=(X=-15.000000,Y=0.000000,Z=30.000000)
	 FPPassengerViewPos=(X=-15.000000,Y=0.000000,Z=30.000000)
	 PassengerPos=(X=16.000000,Y=0.000000,Z=-49.000000)
     ExhaustPos(0)=(X=-165.000000,Y=33.000000,Z=-88.000000)
     ExhaustClass=Class'CarExhaust'
     VehicleFireClass=Class'VehicleFireEmitter'
	 bDamagableWheels=True
	 WheelsHealth=150
	 WheelsMaxHealth=200
	 UnloadWheelHeight=0.013200
	 bSparkWheels=True
     EngTireClass=Class'RVCarTire'
     SteerTireClass=Class'RVCarTire'
     WheelSteerAlong=102.000000
     WheelSteerAcross=90.000000
     WheelEngAlong=-115.000000
     WheelEngAcross=90.000000
     WheelVert=-70.500000
	 SuspDamping=5.0
	 SuspStiffness=180.0
     SuspHighLimit=0.500000
     SuspLowLimit=-0.500000
     TireRollFriction=1.500000
     TireLateralFriction=2.000000
	 TireRollSlip=0.060000
	 TireLateralSlip=0.040000
	 TireSlipRate=0.007000
	 EnginePitch=32
     EngineSound=Sound'MaricaSounds.ut_rv_engine_idle'
     StaticMesh=StaticMesh'MaricaCollision.RVColl.RVKColl'
	 Mesh=SkeletalMesh'MaricaModels.RVModel'
	 Skins(0)=Shader'MaricaTextures.RVTex.RVSkin01'
	 SkinIndex=0
	 SteerWheelBone="SteerWheel"

	 bRandomSkin=True
	 OtherSkins(0)=Shader'MaricaTextures.RVTex.RVSkin01'
	 OtherSkins(1)=Shader'MaricaTextures.RVTex.RVSkin02'
	 OtherSkins(2)=Shader'MaricaTextures.RVTex.RVSkin03'

	 BrokenSkins(0)=Texture'MaricaTextures.RVTex.RVSkin_destroyed'
	 BrokenSkins(1)=Texture'MaricaTextures.PRVTex.PRVSkin_destroyed'
	 BrokenSkins(2)=Shader'Timb.Misc.invisible_mask'
	 BrokenSkins(3)=Texture'MaricaTextures.RVTex.RVSkin_destroyed'
	 BrokenSkins(4)=Texture'MaricaTextures.PRVTex.big_wheel_exp'
	 BrokenSkins(5)=Shader'Timb.Misc.invisible_mask'

     Begin Object Class=KarmaParamsRBFull Name=ScorpionPhysics
         //KInertiaTensor(0)=20.000000
         //KInertiaTensor(3)=30.000000
         //KInertiaTensor(5)=50.000000
         //KCOMOffset=(X=-0.25,Z=-1.5)
         KInertiaTensor(0)=4.000000
         KInertiaTensor(3)=8.000000
         KInertiaTensor(5)=12.000000
         KCOMOffset=(X=0.0,Z=-1.4)
		 KLinearDamping=0.050000
         KAngularDamping=0.600000
         KStartEnabled=True
         bKNonSphericalInertia=True
         KFriction=0.200000
         KImpactThreshold=250.000000
         Name="ScorpionPhysics"
     End Object
     KParams=KarmaParamsRBFull'ScorpionPhysics'
}