//=============================================================================
// VehicleCreatorWeapon.uc
// Author: Nick Pilshikov "Evil"
// Email: Nick_Pv@mail.ru / EvilTheDark@gmail.com
// Site: Set-Games.ru / Revival-Games.ru
// Revival Games Studios.
// 2014.
//=============================================================================

class VehicleCreatorWeapon extends VehicleCreatorBaseWeapon
	config(NicksVehicles);

var float VehicleNamePosY, VehicleNamePosX;
var float MenuHintPosY, MenuHintPosX;
var config array<class<Actor> > VehicleClasses;
var config bool bUseCustomVehicleClasses;
var array<string> defaultVehicleClasses;
var config int MaxVehiclesSpawn;
var travel int VehicleNum;
var VehicleCreatorMiniature Miniature;
var Texture EnemySystemSkin;
var bool bEnemySystem;
var bool bHoldListing;
var bool bHaveShellIcons;
var localized string OpenMenuText;
var localized string OpenMenuTextOrig;
var localized string HudHint1Orig;
var localized string NoVehicleText;
var localized string TooManyVehiclesText;

var float VehicleNamePosHold;
var float HoldDelay;

replication
{
	/*unreliable if( Role == ROLE_Authority )
		VehicleNum;*/

	reliable if( Role < ROLE_Authority )
		NETPreSpawnVehicle;
}

function ProjectileFire();
function MakeGlare();

simulated function NETPreSpawnVehicle(int newVehicleNum)
{
	VehicleNum = newVehicleNum;
}

simulated function CheckOutVehicleList()
{
	local int i, j;
	local class<Actor> newVehClass;

	bHaveShellIcons = ( DynamicLoadObject("FPSGame.FPSButtonInfo", class'Class') != None );

	if( !bHaveShellIcons )
	{
		OpenMenuText = OpenMenuTextOrig;
		HudHint1 = HudHint1Orig;
	}

	if( !bUseCustomVehicleClasses )
	{
		VehicleClasses.Length = 0;
		for( i = 0 ; i < defaultVehicleClasses.Length ; i++ )
		{
			newVehClass = class<Actor>(DynamicLoadObject(defaultVehicleClasses[i], class'Class'));
			if( newVehClass != None )
			{
				j = VehicleClasses.Length;
				VehicleClasses.Insert(VehicleClasses.Length, 1.0);
				VehicleClasses[j] = newVehClass;
			}
		}
		return;
	}

	for( i = 0 ; i < VehicleClasses.Length ; i++ )
	{
		if( VehicleClasses[i] == None )
			VehicleClasses.Remove(i, 1);
	}
}

event PostLoadGame()
{
	CheckOutVehicleList();
	Super.PostLoadGame();
}

simulated function PostBeginPlay()
{
	CheckOutVehicleList();
	Super.PostBeginPlay();
}

function int CountLevelVehicles()
{
	local int i;
	local NKVehicle NKVehicle;
	local NPVehicle NPVehicle;

	foreach AllActors(class'NKVehicle',NKVehicle)
	{
		if( NKVehicle != None && !NKVehicle.bSpecialVehicle )
			i++;
	}
	foreach AllActors(class'NPVehicle',NPVehicle)
	{
		if( NPVehicle != None && !NPVehicle.bSpecialVehicle )
			i++;
	}
	return i;
}

simulated function AltFire( float Value )
{
	bZoomed = !bZoomed;
	HoldDelay = Level.TimeSeconds + 5;
}

simulated function LocalFire()
{
	Super(P2Weapon).LocalFire();
	HoldDelay = Level.TimeSeconds + 5;
}

simulated function Fire( float Value )
{
	NETPreSpawnVehicle(VehicleNum);
	Super(P2Weapon).Fire(Value);
}

simulated function ZoomIn()
{
	if( Instigator != None && Instigator.bHidden )
		return;
	if( bZoomed )
	{
		VehicleNum = FClamp(VehicleNum-1,0,VehicleClasses.Length-1);
		HoldDelay = Level.TimeSeconds + 5;
	}
}

simulated function ZoomOut()
{
	if( Instigator != None && Instigator.bHidden )
		return;
	if( bZoomed )
	{
		VehicleNum = FClamp(VehicleNum+1,0,VehicleClasses.Length-1);
		HoldDelay = Level.TimeSeconds + 5;
	}
}

function Notify_ShootLauncher()
{
	local vector startloc, StartTrace, X,Y,Z, markerpos, HitNormal, HitLocation, Dir, UseVel;
	local actor HitActor;
	local Actor spawnA;
	local float giveback;
	local rotator rSpawn;
	local vector vSpawn;

	if( CountLevelVehicles() > MaxVehiclesSpawn )
	{
		if( Instigator != None )
			Instigator.ClientMessage(TooManyVehiclesText);
		return;
	}

	if( AmmoType != None )
	{
		GetAxes(Instigator.GetViewRotation(),X,Y,Z);
		StartTrace = GetFireStart(X,Y,Z); 
		AdjustedAim = Instigator.AdjustAim(AmmoType, StartTrace, 2*AimError);

		HitActor = Trace(HitLocation, HitNormal, Instigator.Location, StartTrace, true);

		if( VehicleClasses[VehicleNum] != None && HitActor == None
			|| (!HitActor.bStatic
				&& !HitActor.bWorldGeometry))
		{
			if( Miniature != None )
			{
				vSpawn = Miniature.Location;
				rSpawn = Miniature.Rotation;
			}
			else
			{
				vSpawn = Location+(vect(512,0,0) >> Instigator.GetViewRotation());
				rSpawn = rot(0,0,0);
			}
			//spawnA = spawn(VehicleClasses[VehicleNum],,,Location+(vect(512,0,0) >> Instigator.GetViewRotation()),rot(0,0,0));
			spawnA = spawn(VehicleClasses[VehicleNum],,,vSpawn, rSpawn);
		}
		if( spawnA != None && spawnA.Region.iLeaf < 0 )
			spawnA.Destroy();
		if( spawnA != None && NKVehicle(spawnA) != None && bEnemySystem )
		{
			NKVehicle(spawnA).bAIControlled = true;
			NKVehicle(spawnA).UpdateFactoryVariables();
		}
	}
}

state NormalFire
{
	simulated function Tick(float DeltaTime)
	{
		Global.Tick(DeltaTime);
	}
}

state ClientFiring
{
	simulated function Tick(float DeltaTime)
	{
		Global.Tick(DeltaTime);
	}
}

simulated function Tick(float DeltaTime)
{
	//Super(P2Weapon).Tick(DeltaTime);

	if( bZoomed && Level.TimeSeconds > HoldDelay )
	{
		bZoomed = false;
	}
}

simulated event RenderOverlays( canvas Canvas )
{
	local PlayerController PC;
	local string VehicleName;
	local float X,Y;
	local FontInfo myFontInfo;

	PC = PlayerController(Instigator.Controller);

	if( PC == None )
		return;

	if( PC.MyHud != None && PC.MyHud.bHideHUD )
		return;

	if( FPSHUD(PC.MyHud) != None )
		myFontInfo = FPSHUD(PC.MyHud).MyFont;

	if( myFontInfo == None )
		return;

	if( Miniature == None )
		Miniature = spawn(class'VehicleCreatorMiniature',self,,Location, rot(0,0,0));

	if( Miniature != None )
	{
		Miniature.SetLocation(Location+(vect(512,0,0) >> Instigator.GetViewRotation()));
		Miniature.SetRotation(rot(0,0,0));
		Miniature.bHidden = true;
	}

	UpdateEnemySkin();

	///FIX2015
	X = VehicleNamePosX*Canvas.SizeX;
	Y = VehicleNamePosY*Canvas.SizeY;

	if( VehicleClasses[VehicleNum] != None )
		VehicleName = ""$VehicleClasses[VehicleNum].default.Tag;
	else VehicleName = NoVehicleText;

	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.Font = Font'Engine.MediumFont';
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.DrawText(VehicleName);

	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetPos(X, Y);
	Canvas.Font = Font'Engine.MediumFont';
	if( bZoomed )
		Canvas.SetDrawColor(255,10,10,255);
	else Canvas.SetDrawColor(255,255,255,255);
	Canvas.DrawText(VehicleName);

	VehicleNamePosHold = default.VehicleNamePosHold * 1024/Canvas.SizeY;

	if( bZoomed )
	{
		if( VehicleNum+1 < VehicleClasses.Length )
		{
			X = VehicleNamePosX*Canvas.SizeX;
			Y = (VehicleNamePosY+VehicleNamePosHold)*Canvas.SizeY;

			if( VehicleClasses[VehicleNum+1] != None )
				VehicleName = ""$VehicleClasses[VehicleNum+1].default.Tag;
			else VehicleName = NoVehicleText;

			Canvas.bCenter = false;
			Canvas.Style = ERenderStyle.STY_Alpha;
			Canvas.SetPos(X+2.0, Y+2.0);
			Canvas.Font = Font'Engine.MediumFont';
			Canvas.SetDrawColor(25,25,25,90);
			Canvas.DrawText(VehicleName);

			Canvas.bCenter = false;
			Canvas.Style = ERenderStyle.STY_Alpha;
			Canvas.SetPos(X, Y);
			Canvas.Font = Font'Engine.MediumFont';
			Canvas.SetDrawColor(255,255,255,127);
			Canvas.DrawText(VehicleName);

			if( VehicleNum+2 < VehicleClasses.Length )
			{
				X = VehicleNamePosX*Canvas.SizeX;
				Y = (VehicleNamePosY+(VehicleNamePosHold*2))*Canvas.SizeY;

				if( VehicleClasses[VehicleNum+2] != None )
					VehicleName = ""$VehicleClasses[VehicleNum+2].default.Tag;
				else VehicleName = NoVehicleText;

				Canvas.bCenter = false;
				Canvas.Style = ERenderStyle.STY_Alpha;
				Canvas.SetPos(X+2.0, Y+2.0);
				Canvas.Font = Font'Engine.MediumFont';
				Canvas.SetDrawColor(25,25,25,45);
				Canvas.DrawText(VehicleName);

				Canvas.bCenter = false;
				Canvas.Style = ERenderStyle.STY_Alpha;
				Canvas.SetPos(X, Y);
				Canvas.Font = Font'Engine.MediumFont';
				Canvas.SetDrawColor(255,255,255,63);
				Canvas.DrawText(VehicleName);
			}
		}

		if( VehicleNum-1 >= 0 )
		{
			X = VehicleNamePosX*Canvas.SizeX;
			Y = (VehicleNamePosY-VehicleNamePosHold)*Canvas.SizeY;

			if( VehicleClasses[VehicleNum-1] != None )
				VehicleName = ""$VehicleClasses[VehicleNum-1].default.Tag;
			else VehicleName = NoVehicleText;

			Canvas.bCenter = false;
			Canvas.Style = ERenderStyle.STY_Alpha;
			Canvas.SetPos(X+2.0, Y+2.0);
			Canvas.Font = Font'Engine.MediumFont';
			Canvas.SetDrawColor(25,25,25,90);
			Canvas.DrawText(VehicleName);

			Canvas.bCenter = false;
			Canvas.Style = ERenderStyle.STY_Alpha;
			Canvas.SetPos(X, Y);
			Canvas.Font = Font'Engine.MediumFont';
			Canvas.SetDrawColor(255,255,255,127);
			Canvas.DrawText(VehicleName);

			if( VehicleNum-2 >= 0 )
			{
				X = VehicleNamePosX*Canvas.SizeX;
				Y = (VehicleNamePosY-(VehicleNamePosHold*2))*Canvas.SizeY;

				if( VehicleClasses[VehicleNum-2] != None )
					VehicleName = ""$VehicleClasses[VehicleNum-2].default.Tag;
				else VehicleName = NoVehicleText;

				Canvas.bCenter = false;
				Canvas.Style = ERenderStyle.STY_Alpha;
				Canvas.SetPos(X+2.0, Y+2.0);
				Canvas.Font = Font'Engine.MediumFont';
				Canvas.SetDrawColor(25,25,25,45);
				Canvas.DrawText(VehicleName);

				Canvas.bCenter = false;
				Canvas.Style = ERenderStyle.STY_Alpha;
				Canvas.SetPos(X, Y);
				Canvas.Font = Font'Engine.MediumFont';
				Canvas.SetDrawColor(255,255,255,63);
				Canvas.DrawText(VehicleName);
			}
		}
	}

	X = MenuHintPosX*Canvas.SizeX;
	Y = MenuHintPosY*Canvas.SizeY;

	/*Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.Font = Font'Engine.MediumFont';
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.DrawText(OpenMenuText);*/

	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetPos(X, Y);
	Canvas.Font = Font'Engine.MediumFont';
	Canvas.SetDrawColor(255,255,255,255);
	//Canvas.DrawText(OpenMenuText);
	myFontInfo.DrawText(Canvas, OpenMenuText, 1.0);
	///FIX2015

	if( Miniature != None )
	{
		if( VehicleClasses[VehicleNum] != None )
		{
			if( VehicleClasses[VehicleNum].default.Mesh != None )
			{
				if( Miniature.Mesh == None || Miniature.Mesh != VehicleClasses[VehicleNum].default.Mesh )
				{
					Miniature.SetDrawType(DT_Mesh);
					Miniature.SetDrawScale(VehicleClasses[VehicleNum].default.DrawScale);
					Miniature.LinkMesh(VehicleClasses[VehicleNum].default.Mesh);
				}
			}
			Canvas.DrawActor(Miniature, true);
		}
		else Miniature.SetDrawType(DT_None);
	}

	Super(P2Weapon).RenderOverlays(Canvas);
}

simulated function ClientIdleCheckFire()
{
	if( Instigator.PressingFire() ) 
		Fire(0.0);
	if( Instigator.Weapon == self )
		SetTimer(REPLICATE_TIME_CHECK_IDLE, false);
}

state Idle
{
	simulated function Tick(float DeltaTime)
	{
		Global.Tick(DeltaTime);
	}
Begin:
	bPointing=False;
	if ( NeedsToReload() && AmmoType.HasAmmo() )
		GotoState('Reloading');
	if ( !AmmoType.HasAmmo() )
		Instigator.Controller.SwitchToBestWeapon();
	if ( Instigator.PressingFire() ) 
		Fire(0.0);
	PlayIdleAnim();
}

simulated function UpdateEnemySkin()
{
	if( bEnemySystem && Skins[1] != EnemySystemSkin )
	{
		Skins[1] = EnemySystemSkin;
		Skins[2] = EnemySystemSkin;
		Skins[3] = EnemySystemSkin;
	}
	else if( !bEnemySystem && Skins[1] != default.Skins[1] )
	{
		Skins[1] = default.Skins[1];
		Skins[2] = default.Skins[2];
		Skins[3] = default.Skins[3];
	}
}

exec function EnemyCars()
{
	bEnemySystem = !bEnemySystem;
}

exec function BIGBANG()
{
	local NKVehicle NKVehicle;
	local NPVehicle NPVehicle;

	foreach AllActors(class'NKVehicle',NKVehicle)
	{
		if( NKVehicle != None && !NKVehicle.bSpecialVehicle )
			NKVehicle.TakeDamage(NKVehicle.Health, Instigator, NKVehicle.Location, vect(0,0,0), class'NVCarDamage');
	}
	foreach AllActors(class'NPVehicle',NPVehicle)
	{
		if( NPVehicle != None && !NPVehicle.bSpecialVehicle )
			NPVehicle.TakeDamage(NPVehicle.Health, Instigator, NPVehicle.Location, vect(0,0,0), class'NVCarDamage');
	}
}

function mMenu()
{
	local ShellRootWindow PlayerRootMenu;

	if( Instigator == None )
		return;

	PlayerRootMenu = ShellRootWindow(P2Player(Instigator.Controller).Player.InteractionMaster.BaseMenu);

	if( PlayerRootMenu == None )
		return;

	PlayerRootMenu.ShowMenu();
	PlayerRootMenu.CreateMenu(class'ProjectMaricaMenu');
	PlayerRootMenu.MyMenu.ShowWindow();
}

simulated function DropFrom(vector StartLocation)
{
	if( Pawn(Owner) != None && Pawn(Owner).Health > 0 )
		mMenu();
}

simulated function Destroyed()
{
	Super.Destroyed();
	if( Miniature != None )
		Miniature.Destroy();
}

/*
VehicleClasses="MaricaQuad.QuadBike"
VehicleClasses="MaricaSanchez.SanchezBike"
VehicleClasses="MaricaCinemaTools.MaricaCameraVehicle"
VehicleClasses="MaricaDragon.BlackDragon"
VehicleClasses="MaricaDragon.SilverDragon"
VehicleClasses="MaricaSpaceFighters.SpaceFighterDefenderVehicle"
VehicleClasses="MaricaSpaceFighters.SpaceFighterAttackerVehicle"
VehicleClasses="ProjectMarica.HelicopterVehicle"
VehicleClasses="ProjectMaricaCore.NKVCar"
VehicleClasses="ProjectMarica.FRTCar"
VehicleClasses="ProjectMarica.PRVCar"
VehicleClasses="ProjectMarica.RVCar"
VehicleClasses="ProjectMarica.NKVTank"
VehicleClasses="ProjectMarica.DudesCar"
VehicleClasses="Gallardo.GallardoCar"
VehicleClasses="MaricaSnowMobile.SnowMobile"
VehicleClasses="MaricaSpaceFighters.xPRVCar"
VehicleClasses="MaricaRigCar.RigCar"
VehicleClasses="MaricaSilverDragonCars.T2000Car"
VehicleClasses="MaricaSilverDragonCars.RenaultMagnumCar"
VehicleClasses="MaricaSilverDragonCars.SuperMotoXLCar"
VehicleClasses="MaricaSilverDragonCars.GmcC4500Car"
VehicleClasses="MaricaSegway.Segway"

	 ShakeOffsetMag=(X=15.0,Y=2.0,Z=2.0)
	 ShakeOffsetRate=(X=500.0,Y=500.0,Z=500.0)
	 ShakeOffsetTime=2.0
	 ShakeRotMag=(X=450.0,Y=50.0,Z=50.0)
	 ShakeRotRate=(X=8000.0,Y=8000.0,Z=8000.0)
	 ShakeRotTime=2.0
*/

defaultproperties
{
	 bUseCustomVehicleClasses=false
	 defaultVehicleClasses(0)="MaricaQuad.QuadBike"
	 defaultVehicleClasses(1)="MaricaSanchez.SanchezBike"
	 defaultVehicleClasses(2)="MaricaCinemaTools.MaricaCameraVehicle"
	 defaultVehicleClasses(3)="MaricaDragon.BlackDragon"
	 defaultVehicleClasses(4)="MaricaDragon.SilverDragon"
	 defaultVehicleClasses(5)="MaricaSpaceFighters.SpaceFighterDefenderVehicle"
	 defaultVehicleClasses(6)="MaricaSpaceFighters.SpaceFighterAttackerVehicle"
	 defaultVehicleClasses(7)="ProjectMarica.HelicopterVehicle"
	 defaultVehicleClasses(8)="ProjectMaricaCore.NKVCar"
	 defaultVehicleClasses(9)="ProjectMarica.FRTCar"
	 defaultVehicleClasses(10)="ProjectMarica.PRVCar"
	 defaultVehicleClasses(11)="ProjectMarica.RVCar"
	 defaultVehicleClasses(12)="ProjectMarica.NKVTank"
	 defaultVehicleClasses(13)="ProjectMarica.DudesCar"
	 defaultVehicleClasses(14)="Gallardo.GallardoCar"
	 defaultVehicleClasses(15)="MaricaSnowMobile.SnowMobile"
	 defaultVehicleClasses(16)="MaricaSpaceFighters.xPRVCar"
	 defaultVehicleClasses(17)="MaricaRigCar.RigCar"
	 defaultVehicleClasses(18)="MaricaSilverDragonCars.T2000Car"
	 defaultVehicleClasses(19)="MaricaSilverDragonCars.RenaultMagnumCar"
	 defaultVehicleClasses(20)="MaricaSilverDragonCars.SuperMotoXLCar"
	 defaultVehicleClasses(21)="MaricaSilverDragonCars.GmcC4500Car"
	 defaultVehicleClasses(22)="MaricaSegway.Segway"

	 bDrawMuzzleFlash=false
     ShakeRotMag=(X=0.000000,Y=0.000000,Z=0.000000)
     ShakeRotRate=(X=0.000000,Y=0.000000,Z=0.000000)
     ShakeRotTime=0.000000
     ShakeOffsetMag=(X=0.000000,Y=0.000000,Z=0.000000)
     ShakeOffsetRate=(X=0.000000,Y=0.000000,Z=0.000000)
	 ShakeOffsetTime=0.000000

	 bArrestableWeapon=true
	 DropWeaponHint1=""
	 DropWeaponHint2=""
	 ShotMarkerMade=None
	 BulletHitMarkerMade=None
	 ViolenceRank=0
	 bAllowHints=true

	 HudHint1="%KEY_AltFire% scrolls vehicles."
	 HudHint1Orig="Press AltFire to scrolls vehicles."
	 HudHint2=""
	 TooManyVehiclesText="Too many vehicles on level, try to destroy old one"
	 NoVehicleText="Can't find vehicle class"
	 VehicleNamePosHold=0.02
	 OpenMenuText="Press %KEY_ThrowWeapon% to open vehicles configuration menu"
	 OpenMenuTextOrig="Press [Z] to open vehicles configuration menu"
	 EnemySystemSkin=Shader'MaricaTextures.Effects.NicksCollision'
	 bUnlit=true
	 DrawScale=1.0
	 MaxVehiclesSpawn=30
     VehicleNamePosY=0.8
	 VehicleNamePosX=0.45
	 MenuHintPosY=0.1
	 MenuHintPosX=0.35
     bUsesAltFire=True
     ShotCountMaxForNotify=0
     holdstyle=WEAPONHOLDSTYLE_Both
     switchstyle=WEAPONHOLDSTYLE_Both
     firingstyle=WEAPONHOLDSTYLE_Both
     MinRange=1024.000000
     bBumpStartsFight=False
     FirstPersonMeshSuffix="Launcher"
     WeaponSpeedLoad=2.000000
     WeaponSpeedShoot1=1.500000
     WeaponSpeedShoot1Rand=0.200000
     WeaponSpeedIdle=0.300000
     AmmoName=Class'VehicleCreatorAmmoInv'
     AutoSwitchPriority=9
     FireOffset=(X=60.000000,Y=35.000000,Z=0.000000)
     aimerror=1000.000000
     MaxRange=3000.000000
     FireSound=Sound'WeaponSounds.napalm_fire'
     InventoryGroup=9
     GroupOffset=241817
     PickupClass=Class'VehicleCreatorPickup'
     PlayerViewOffset=(X=10.0,Y=2.0,Z=-12.0)
     AttachmentClass=Class'VehicleCreatorAttachment'
     ItemName="Vehicle Launcher"
     Mesh=SkeletalMesh'MP_Weapons.MP_LS_Launcher'
     Skins(0)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
     Skins(1)=Shader'MaricaTextures.Effects.VehicleCreatorSkin'
     Skins(2)=Shader'MaricaTextures.Effects.VehicleCreatorSkin'
     Skins(3)=Shader'MaricaTextures.Effects.VehicleCreatorSkin'
}
