//=============================================================================
// DriverActor. (Special actor that looks like a driver)
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2011.
//=============================================================================

class DriverActor extends Actor
	notplaceable;

const PAWN_HEAD_BONE = 'MALE01 head';
var MeshAnimation DrivingAnims;
var class<BodyPart> HeadClass;
var BodyPart Head;
var float HeadScale;
var Mesh BodyModel, HeadModel;
var Material BodySkin, HeadSkin;
var NKVehicle myVehicle;
var Rotator HeadRotation;
var Name CurrentAnim;
var bool bDriver;
var bool bHeadFixed;

replication
{
	unreliable if( Role == ROLE_Authority )
		myVehicle, CurrentAnim, BodyModel, HeadModel, BodySkin, HeadSkin;
}

simulated function UpdateStuff()
{
	local vector ScaleVector;
	if(BodyModel == None)
		return;

	if(bDriver)
	{
		if(!myVehicle.bAIControlled)
		{
			if(BodySkin == None)
				BodySkin = myVehicle.Driver.Skins[0];
		}
		else
		{
			if(BodySkin == None)
				BodySkin = myVehicle.AIDriverTex;
			if(HeadSkin == None)
				HeadSkin = myVehicle.AIDriverHeadTex;
		}
	}
	else
	{
		if(BodySkin == None)
			BodySkin = myVehicle.Passenger.Skins[0];
	}

	if(BodySkin != None)
		Skins[0] = BodySkin;

	// hack, we have head from driver
	if( bDriver && MpPawn(myVehicle.Driver) != None && MpPawn(myVehicle.Driver).myHead != None )
	{
		MpPawn(myVehicle.Driver).DetachFromBone(MpPawn(myVehicle.Driver).myHead);
		AttachToBone(MpPawn(myVehicle.Driver).myHead, PAWN_HEAD_BONE);
		MpPawn(myVehicle.Driver).myHead.SetOwner(self);
	}
	else if( MpPawn(myVehicle.Passenger) != None && MpPawn(myVehicle.Passenger).myHead != None )
	{
		MpPawn(myVehicle.Passenger).DetachFromBone(MpPawn(myVehicle.Passenger).myHead);
		AttachToBone(MpPawn(myVehicle.Passenger).myHead, PAWN_HEAD_BONE);
		MpPawn(myVehicle.Passenger).myHead.SetOwner(self);
	}
	else
	{
		if(Head == None && HeadClass != None)
			Head = Spawn(HeadClass,self,,Location,Rotation);

		if( Head != None )
		{
			ScaleVector.X = HeadScale;
			ScaleVector.Y = HeadScale;
			ScaleVector.Z = HeadScale;
			AttachToBone(Head, PAWN_HEAD_BONE);
			if( HeadModel == None )
				HeadModel = default.Head.Mesh;
			if( HeadSkin == None )
				HeadSkin = default.Head.Skins[0];
			Head(Head).Setup(HeadModel, HeadSkin, ScaleVector, AmbientGlow);
			Head.SetCollisionSize(0,0);
			Head.SetRelativeRotation(HeadRotation);
		}
	}

	if(DrivingAnims != None)
		LinkSkelAnim(DrivingAnims);

	if(bDriver)
		CurrentAnim = myVehicle.DrivingAnimStatus;
	else
		CurrentAnim = myVehicle.SittingAnimStatus;

	if(Mesh != None && CurrentAnim != '')
		LoopAnim(CurrentAnim,1.0);

	SetPhysics(PHYS_None);
	myVehicle.UpdateDriverActors();
}

simulated function Tick(float DeltaTime)
{
	local name anim;
	local float frame,rate;

	if( myVehicle == None || myVehicle.Health <= 0 )
		Destroy();

	if( Level.NetMode != NM_DedicatedServer )
		GetAnimParams(0,Anim,frame,rate);

	if( Mesh == None || Anim != CurrentAnim )
	{
		if(BodyModel == None)
		{
			if(bDriver)
				BodyModel = myVehicle.FindPawnModel(True);
			else BodyModel = myVehicle.FindPawnModel(False);
		}
		LinkMeshAndAnims(BodyModel, True);
		UpdateStuff();
	}
}

simulated function LinkMeshAndAnims(Mesh NewMesh, optional bool bKeepAnimState)
{
	LinkMesh(NewMesh, bKeepAnimState);
	LinkAnims();
}

simulated function LinkAnims()
{
	LinkSkelAnim(GetDefaultAnim(SkeletalMesh(Mesh)));
	LinkSkelAnim(DrivingAnims);
}

simulated function Destroyed()
{
	if( bDriver && MpPawn(myVehicle.Driver) != None && MpPawn(myVehicle.Driver).myHead != None )
	{
		DetachFromBone(MpPawn(myVehicle.Driver).myHead);
		myVehicle.Driver.AttachToBone(MpPawn(myVehicle.Driver).myHead, PAWN_HEAD_BONE);
	}
	else if( MpPawn(myVehicle.Passenger) != None && MpPawn(myVehicle.Passenger).myHead != None )
	{
		DetachFromBone(MpPawn(myVehicle.Passenger).myHead);
		myVehicle.Passenger.AttachToBone(MpPawn(myVehicle.Passenger).myHead, PAWN_HEAD_BONE);
	}
	Super.Destroyed();
	if(Head != None)
		Head.Destroy();
}

defaultproperties
{
	 bGameRelevant=True
	 //bAlwaysRelevant=True
     DrivingAnims=MeshAnimation'ProjectMaricaCore.anim_PMDriving'
     HeadRotation=(Yaw=-16384,Roll=16384)
     bDriver=True
     bOwnerNoSee=True
     bReplicateInstigator=True
     RemoteRole=ROLE_SimulatedProxy
     DrawType=DT_Mesh
     AmbientGlow=32
     CollisionRadius=0.000000
     CollisionHeight=0.000000
     bBlockZeroExtentTraces=False
     bBlockNonZeroExtentTraces=False
     RotationRate=(Pitch=7800,Yaw=7800,Roll=7800)
     bDirectional=True
	 Physics=PHYS_None
	 bBlockKarma=False
	 bProjTarget=False
	 bCollideActors=False
	 bCollideWorld=False
	 bBlockPlayers=False
	 bBlockActors=False
     bStasis=False
}
