//=============================================================================
// IntCollisionActor.
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2011.
//=============================================================================

class IntCollisionActor extends Actor
	notplaceable;

var globalconfig bool bCorpseImpulse;

var Pawn CarDriver;
var NKVehicle myVehicle;

var float MinROSpeed;
var float BumpUpdate;

var class<DamageType> RODamage;

var Sound ROSound;

replication
{
	unreliable if( bNetDirty && Role==ROLE_Authority )
		CarDriver;

	unreliable if (Role == ROLE_Authority)
		myVehicle;
}

function Timer()
{
	myVehicle.EvilHavePassenger();
}

function TakePhysImpulse(vector Impulse, vector Location)
{
	if(myVehicle != None)
		myVehicle.KAddImpulse(Impulse,Location);
}

function Touch(Actor Other)
{
	if( Other != None && KActor(Other) != None && Other.bStasis )
	{
		Other.bStasis = false;
		Other.KWake();
	}
	Super.Touch(Other);
}

function bool encroachingOn(actor Other)
{
	local Pawn P;
	local Pickup Item;
	local int Speed;
	local vector TransLocation, PreLocation;
	local bool bAllowTrans;

	if( Other != None && KActor(Other) != None && Other.bStasis )
	{
		Other.bStasis = false;
		Other.KWake();
	}

	if( Other != None )
	{
		if(Pawn(Other) != None)
			P = Pawn(Other);
		else if(Pickup(Other) != None)
			Item = Pickup(Other);
	}
	if( NPVehicle(Other) != None )
	{
		Speed = VSize(myVehicle.Velocity);
		if(Speed < MinROSpeed)
			bAllowTrans = true;
		else bAllowTrans = false;
	}
	else bAllowTrans = true;

	if( P != None && P != myVehicle.Driver && P != myVehicle.Passenger && bAllowTrans )
	{
		PreLocation = P.Location - Location;
		PreLocation *= 0.1;
		TransLocation.X = P.Location.X + PreLocation.X;
		TransLocation.Y = P.Location.Y + PreLocation.Y;
		TransLocation.Z = P.Location.Z + PreLocation.Z;
		P.SetLocation(TransLocation);
		///FIX2015
	}
	else if( Item != None )
	{
		///FIX2015
		if( P2PowerupPickup(Item) == None || (P2PowerupPickup(Item) != None && !P2PowerupPickup(Item).bUseForErrands) )
		{
			Item.SetBase(None);
			Item.Destroy();
			Item = None;
		}
		///FIX2015
	}
	return false;
}

singular simulated function Bump(Actor Other)
{
	local Pawn P;
	//local bool bOtherIsPlayer;
	local PlayerController PC;
	//local Controller C;
	local vector Momentum, RagdollVel;
	local float Speed;

	if(CarDriver != None)
	{
		PC = myVehicle.DriverController;
		Instigator = CarDriver;
	}
	else if( myVehicle.Passenger != None )
		Instigator = myVehicle.Passenger;
	else Instigator = None;

	if( Other != None && KActor(Other) != None && Other.bStasis )
	{
		Other.bStasis = false;
		Other.KWake();
	}

	if(Pawn(Other) == None || IntCollisionActor(Other) != None 
		|| KVehicle(Other) != None 
		|| Other == Instigator 
		|| Other.Role != ROLE_Authority 
		|| myVehicle == None)
		return;

	if( Pawn(Other).Health <= 0 )
		return;

	P = Pawn(Other);

	Speed = VSize(myVehicle.Velocity);
	if( Speed > MinROSpeed )
	{
		Momentum = myVehicle.Velocity;// * Other.Mass;
		if(Other.Base != self)
		{
			if(PC != None && PC.SameTeamAs(Pawn(Other).Controller))
			{
				Momentum += Speed * 0.25 * Other.Mass * Normal(myVehicle.Velocity cross vect(0,0,1));
				Speed = 0;
			}
			if(Level.TimeSeconds > BumpUpdate + 0.6)
			{
				if(ROSound != None)
				{
					if(NPVehicle(Other) == None)
						PlaySound(ROSound, SLOT_Misc, myVehicle.RanOverVolume);
					else PlaySound(myVehicle.ImpactSoundsMed[0], SLOT_Misc, myVehicle.RanOverVolume);
				}
				BumpUpdate = Level.TimeSeconds;
			}

			if(AnimalPawn(Other) != None)
				Other.TakeDamage(int(Speed * 50), CarDriver, Other.Location, Momentum, class'Gibbed');
			else
			{
				Momentum = FClamp(VSize(100*Momentum),0,200000) * Normal(Momentum);
				Other.TakeDamage(int(Speed * 0.075), CarDriver, Location, vect(0,0,0), RODamage);
				if( Other.Physics == PHYS_KarmaRagdoll )
				{
					RagdollVel = FClamp(Speed*0.9,0,500) * Normal(Momentum);
					Other.KSetSkelVel( RagdollVel );
					if( bCorpseImpulse )
						Other.KAddImpulse(Momentum, vect(0,0,0));
				}
				else if( P != None && P.Health <= 0 )
					P.TakeDamage(1000, CarDriver, Location, vect(0,0,0), class'Gibbed');
			}
		}
	}
}

function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, class<DamageType> damageType)
{
	if(myVehicle == None
		|| damageType == None
		|| myVehicle.bImmortal
		|| damageType == class'BaseFX.BurnedDamage'
		|| damageType == class'BaseFX.SmashDamage'
		|| damageType == class'BaseFX.OnFireDamage'
		|| damageType == class'BaseFX.KickingDamage'
		|| damageType == class'BaseFX.BludgeonDamage')
		return;
	if(myVehicle.bAIControlled)
	{
		if(myVehicle.Enemy == None)
			myVehicle.StartScriptedDriving(instigatedBy);
	}
	myVehicle.TakeDamage(Damage, instigatedBy, Location, Momentum, damageType);
}

simulated function Tick(float DeltaTime)
{
	if( myVehicle != None && myVehicle.IsGoingToUpdate() )
	{
		if( Location != myVehicle.Location )
			SetLocation(myVehicle.Location);
		if( Rotation != myVehicle.Rotation )
			SetRotation(myVehicle.Rotation);
	}
}

defaultproperties
{
	bAlwaysTick=true

	 bCorpseImpulse=true
     CollisionRadius=1.000000
     CollisionHeight=1.000000
     bBlockZeroExtentTraces=True
     bBlockNonZeroExtentTraces=True
	 bBlockPlayers=True
	 bBlockActors=True
	 bCollideActors=True
	 bCollideWorld=False
	 bProjTarget=True
     bHidden=True
     bReplicateInstigator=True
     bUpdateSimulatedPosition=True
     Physics=PHYS_KarmaRagDoll
     RemoteRole=ROLE_SimulatedProxy
	 //StaticMesh=StaticMesh'ProjectMaricaCore.BulldogIntColl'
     DrawType=DT_StaticMesh
     bUnlit=True
     bGameRelevant=True
     SoundRadius=100.000000
     TransientSoundRadius=200.000000
     NetPriority=2.000000
     bDirectional=True
}
