#EXEC OBJ LOAD FILE="PMTex.utx" Package=ProjectMaricaCore
#EXEC OBJ LOAD FILE="PMMesh.ukx" Package=ProjectMaricaCore
#EXEC OBJ LOAD FILE="PMSound.uax" Package=ProjectMaricaCore
#EXEC OBJ LOAD FILE="PMStatic.usx" Package=ProjectMaricaCore
#EXEC OBJ LOAD FILE="MaricaDriftTex.utx" Package=ProjectMaricaCore
#EXEC OBJ LOAD FILE="MaricaDriftSounds.uax" Package=ProjectMaricaCore
//=============================================================================
// NKActor. (Nicks karma based actor class)
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2011.
//=============================================================================

class NKActor extends KActor
	config(NicksVehicles)
	placeable;

var(MaricaGraphics) class<NVShadowProjector> ShadowClass;
var NVShadowProjector myShadow;
var(MaricaGraphics) config bool bAllowDynamicShadows;
var(MaricaGraphics) float ShadowLightDistance;
var(MaricaGraphics) float ShadowMaxTraceDistance;
var(MaricaGraphics) vector ShadowLightDirection;
var(MaricaGraphics) Rotator ShadowRotation;
var float mySpeed;
var(MaricaPhysics) bool DynamicKarma;

replication
{
	unreliable if(Role == ROLE_Authority)
		mySpeed;
	reliable if(Role < ROLE_Authority)
		bAllowDynamicShadows,myShadow;
}

simulated function PostBeginPlay()
{
	if(DynamicKarma)
		KWake();
	SpawnMyShadow();
}

simulated function Tick(float DeltaTime)
{
	local vector NormalImpulse;
	mySpeed = VSize(Velocity);
	if(IsInWater())
	{
		NormalImpulse.X = 0;
		NormalImpulse.Y = 0;
		NormalImpulse.Z = 2048 / Mass;
		KAddImpulse(NormalImpulse, Location);
	}
}

simulated function SpawnMyShadow()
{
	if(bAllowDynamicShadows)
	{
		if(myShadow == None)
		{
			myShadow = spawn(ShadowClass, self,, Location, ShadowRotation);
			myShadow.ShadowActor = self;
			myShadow.MyBase = self;
			myShadow.MyCar = None;
			myShadow.MyVeh = None;
			myShadow.SetCollision(true, false, false);
			myShadow.LightDirection = Normal(ShadowLightDirection);
			myShadow.LightDistance = ShadowLightDistance;
			myShadow.MaxTraceDistance = ShadowMaxTraceDistance;
			myShadow.UpdateShadow();
		}
	}
}

simulated function DestroyMyShadow(optional bool bNoShadowCirculation)
{
	if( myShadow != None )
	{
		myShadow.bShadowCirculation = !bNoShadowCirculation;
		myShadow.Destroy();
		myShadow = None;
	}
}

event PreSaveGame()
{
	Super.PreSaveGame();
	DestroyMyShadow();
}

event PostSaveGame()
{
	Super.PostSaveGame();
	SpawnMyShadow();
}

simulated function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, class<DamageType> damageType)
{
	local vector ShootMomentum, ExpMoment;
	local Rotator NullRot;
	local int N, nForce;

	if(VSize(momentum) < 0.001)
		return;

	if(bKTakeShot)
	{
		if(IsInWater())
			ShootMomentum = Momentum / 32;
		else
			ShootMomentum = Momentum / 16;
		KWake();
		KAddImpulse(ShootMomentum, hitlocation);

		if(ClassIsChildOf(damageType, class'TankProjectileDamage'))
		{
			N = Rand(1);
			if(N == 0)
				nForce = 128;
			else
				nForce = -128;
			ExpMoment.Z = 256;
			ExpMoment.X = FRand() * nForce;
			N = Rand(1);
			if(N == 0)
				nForce = 128;
			else
				nForce = -128;
			ExpMoment.Y = FRand() * nForce;
			ExpMoment = ExpMoment + (Velocity / 2);
			ExpMoment = ExpMoment * 800;
			ExpMoment = ExpMoment >> NullRot;
			KAddImpulse(ExpMoment,HitLocation);
		}
		if(ClassIsChildOf(damageType, class'RocketExplosionDamage')
			|| ClassIsChildOf(damageType, class'BaseFX.ExplodedDamage'))
		{
			N = Rand(1);
			if(N == 0)
				nForce = 128;
			else
				nForce = -128;
			ExpMoment.Z = 128;
			ExpMoment.X = FRand() * nForce;
			N = Rand(1);
			if(N == 0)
				nForce = 128;
			else
				nForce = -128;
			ExpMoment.Y = FRand() * nForce;
			ExpMoment = ExpMoment + (Velocity / 2);
			ExpMoment = ExpMoment * 800;
			ExpMoment = ExpMoment >> NullRot;
			KAddImpulse(ExpMoment,HitLocation);
		}
	}
}

simulated function Destroyed()
{
	DestroyMyShadow();
	Super.Destroyed();
}

function bool IsInWater()
{
	/*local PhysicsVolume V;
	ForEach TouchingActors(class'PhysicsVolume',V)
	if(V != None 
		&& (V.IsA('WaterVolume') 
			|| (V.DamageType != None && V.DamageType == class'WaterDamage')))
		return true;*/
	return false;
}

function bool CanSplash()
{
	return True;
}

function bool IsInverted()
{
	local vector worldUp;
	worldUp = vect(0, 0, 1) >> Rotation;
	if(worldUp.Z < 0.2)
		return true;
	return false;
}

function bool GameIsEnd()
{
	if(Level.Game.bGameEnded)
		return True;
	return false;
}

function bool GameIsSingle()
{
	if(Level.NetMode == NM_StandAlone || 
		(Level.Game != None 
			&& Level.Game.bIsSingleplayer))
		return True;
	return false;
}

event KImpact(actor other, vector pos, vector impactVel, vector impactNorm)
{
	local int numSounds, soundNum;

	if(Level.TimeSeconds > LastImpactTime + ImpactInterval)
	{
		numSounds = ImpactSounds.Length;
		if(numSounds > 0)
		{
			soundNum = Rand(numSounds);
			PlaySound(ImpactSounds[soundNum], , ImpactVolume, , , 0.96 + FRand()*0.08);
		}
		LastImpactTime = Level.TimeSeconds;
	}
}

function bool encroachingOn(actor Other)
{
	local Pawn P;
	local Pickup Item;
	local vector TransLocation, PreLocation;

	if(Other != None)
	{
		if(Pawn(Other) != None)
			P = Pawn(Other);
		else if(Pickup(Other) != None)
			Item = Pickup(Other);
	}
	else Other = None;
	if(P != None && NPVehicle(Other) == None)
	{
		PreLocation = P.Location - Location;
		PreLocation *= 0.1;
		TransLocation.X = P.Location.X + PreLocation.X;
		TransLocation.Y = P.Location.Y + PreLocation.Y;
		TransLocation.Z = P.Location.Z + PreLocation.Z;
		P.SetLocation(TransLocation);
	}
	else if(Item != None)
	{
		Item.Destroy();
		Item = None;
	}
	return false;
}

defaultproperties
{
     ShadowClass=Class'ProjectMaricaCore.NVShadowProjector'
     ShadowLightDistance=1200.000000
     ShadowMaxTraceDistance=512.000000
     ShadowLightDirection=(X=1.000000,Y=1.000000,Z=6.000000)
     ImpactSounds(0)=Sound'ProjectMaricaCore.ut_impact_01'
     ImpactSounds(1)=Sound'ProjectMaricaCore.ut_impact_01'
     ImpactVolume=1.000000
     ImpactInterval=2.000000
     bPawnMovesMe=False
     bCanTeleport=True
     bAlwaysRelevant=True
     bReplicateInstigator=True
     bUpdateSimulatedPosition=True
     StaticMesh=StaticMesh'Timb_mesh.KITCHEN.pot2_timb'
     AmbientGlow=32
     bGameRelevant=True
     bTravel=True
     bIgnoreOutOfWorld=True
     bFixedRotationDir=True
     Mass=100.000000
     Begin Object Class=KarmaParams Name=NKActorPhysics
         KFriction=0.400000
         KRestitution=0.200000
         KImpactThreshold=50000.000000
         Name="NKActorPhysics"
     End Object
     KParams=KarmaParams'NKActorPhysics'
     NetPriority=3.000000
     bDirectional=True
}
