//=============================================================================
// NKVHandsWeapon.uc
// Author: Nick Pilshikov "Evil"
// For Project Marica
// Revival Games Studios.
// 2014.
//=============================================================================

class NKVHandsWeapon extends RifleWeapon;//P2Weapon;

var NKVehicle myVehicle;
var Weapon myOldWeapon;

replication
{
	unreliable if( Role == ROLE_Authority )
		myVehicle , myOldWeapon;
}

function ProjectileFire();
function MakeGlare();

simulated event RenderOverlays(canvas Canvas)
{
	Super(P2Weapon).RenderOverlays(Canvas);
}

simulated function Fire( float Value )
{
	ServerFire();
}

function ServerFire()
{
	if( myVehicle != None )
		myVehicle.HandsTryToFire();
	//P2Pawn(Instigator).ClientMessage("kurwa");
}

simulated function AltFire( float Value )
{
	ServerAltFire();
}

function ServerAltFire()
{
	if( myVehicle != None )
		myVehicle.HandsTryToAltFire();
	//P2Pawn(Instigator).ClientMessage("kurwa");
}

simulated function bool IsHoldAllowed()
{
	if( Pawn(Owner) == None )
		return false;
	if( myVehicle == None 
		|| myVehicle.Driver == None 
		|| (myVehicle.Driver != None && myVehicle.Driver != Pawn(Owner)) )
		return false;
	return true;
}

exec function NKVehCam()
{
	local PlayerController PC;

	PC = PlayerController(Pawn(Owner).Controller);

	if( PC == None )
		return;

	PC.bFreeCamera = !PC.bFreeCamera;
	PC.bBehindView = !PC.bBehindView;
}

exec function BeepBeep()
{
	if( myVehicle != None )
		myVehicle.PlayingBeepSound();
}

auto simulated state BaseIdle
{
}

state Idle
{
	ignores ClientIdleCheckFire;

	simulated function Tick(float DeltaTime)
	{
		Global.Tick(DeltaTime);
	}

Begin:
	bPointing=False;
	if ( !AmmoType.HasAmmo() )
		Instigator.Controller.SwitchToBestWeapon();  //Goto Weapon that has Ammo
	if ( Instigator.PressingFire() ) Fire(0.0);
	//if ( Instigator.PressingAltFire() ) AltFire(0.0);
	PlayIdleAnim();
	Sleep(WAIT_IDLE_TIME);	// Don't let them switch during this time
}

state NormalFire
{
	simulated function Tick(float DeltaTime)
	{
		Global.Tick(DeltaTime);
	}
}

state ClientFiring
{
	simulated function Tick(float DeltaTime)
	{
		Global.Tick(DeltaTime);
	}
}

simulated function Tick(float DeltaTime)
{
	local P2MoCapPawn PawnOwner;

	PawnOwner = P2MoCapPawn(Owner);

	if( PawnOwner == None )
		return;

	if( myVehicle == None )
		myVehicle = None;

	if( !IsHoldAllowed() )
	{
		bZoomed = false;
		if( PawnOwner.Weapon != None && PawnOwner.Weapon == self )
		{
			if( PawnOwner.Controller != None 
				&& PlayerController(PawnOwner.Controller) != None 
				&& PlayerController(PawnOwner.Controller).MyHUD != None )
				PlayerController(PawnOwner.Controller).MyHUD.bShowDebugInfo = false;
			//bZoomed = false;
			NKVHandsAmmoInv(AmmoType).bHideMe = true;
			PawnOwner.Weapon = myOldWeapon;
			///FIX2015
			myOldWeapon.GotoState('Idle');
			///FIX2015
		}
	}
	else
	{
		bZoomed = true;
		NKVHandsAmmoInv(AmmoType).bHideMe = false;
	}
}

/*simulated function bool AllowNextWeapon()
{
	if( myVehicle != None )
		myVehicle.CameraDist = FClamp(myVehicle.CameraDist-0.5,myVehicle.CameraDistMin,myVehicle.CameraDistMax);
	return !IsHoldAllowed();
}

simulated function bool AllowPrevWeapon()
{
	if( myVehicle != None )
		myVehicle.CameraDist = FClamp(myVehicle.CameraDist+0.5,myVehicle.CameraDistMin,myVehicle.CameraDistMax);
	return !IsHoldAllowed();
}*/

simulated function ZoomIn()
{
	if( myVehicle != None )
		myVehicle.CameraDist = FClamp(myVehicle.CameraDist-0.5,myVehicle.CameraDistMin,myVehicle.CameraDistMax);
}

simulated function ZoomOut()
{
	if( myVehicle != None )
		myVehicle.CameraDist = FClamp(myVehicle.CameraDist+0.5,myVehicle.CameraDistMin,myVehicle.CameraDistMax);
}

defaultproperties
{
	 ShotMarkerMade=None
	 BulletHitMarkerMade=None
	 ViolenceRank=0
	 bAllowHints=false
	 bArrestableWeapon=false
	 DropWeaponHint1=""
	 DropWeaponHint2=""

	 bCanThrow=false
     WeaponSpeedLoad=20.0
     WeaponSpeedReload=20.0
     WeaponSpeedHolster=20.0
     WeaponSpeedShoot1=20.0
     AmmoName=Class'ProjectMaricaCore.NKVHandsAmmoInv'
     FireSound=None
     bDrawMuzzleFlash=false
     InventoryGroup=0
     ///GroupOffset=976
	GroupOffset=-1
     PlayerViewOffset=(X=-20.000000)
     ItemName="Hands"
     bUsesAltFire=false
     bArrestableWeapon=false
     bNoHudReticle=true
     DropWeaponHint1="They've seen your weapon!"
     DropWeaponHint2="Press ; to drop it.      "
     bBumpStartsFight=False
     CombatRating=0.500000
     FirstPersonMeshSuffix="Nothing"
     AutoSwitchPriority=0
     ShakeRotMag=(X=0.000000,Y=0.000000,Z=0.000000)
     ShakeRotRate=(X=0.000000,Y=0.000000,Z=0.000000)
     ShakeRotTime=0.000000
     ShakeOffsetMag=(X=0.000000,Y=0.000000,Z=0.000000)
     ShakeOffsetRate=(X=0.000000,Y=0.000000,Z=0.000000)
	 ShakeOffsetTime=0.000000
     aimerror=0.000000
     AIRating=0.040000
     SelectSound=None
     BobDamping=0.975000
     AttachmentClass=None
     Mesh=SkeletalMesh'MP_Weapons.MP_LS_Nothing'
     Skins(0)=Texture'MP_FPArms.LS_arms.LS_hands_dude'
}
