//=============================================================================
// NKVehicle. (Nicks karma based vehicle class)
//
// Story: 2015 - NKVehicle, last update
// Story: 2011 - KarmaCarVehicleMP	(extends KActor).
// Story: 2010 - KarmaCarVehicle	(extends KVehicle).
// Story: 2009 - EvilCat			(extends KVehicle).
//
// Spawning wheels and gear system are merged from UT2003 by Epic Games.
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// 14.05.12: AI system added. Evil
//
// Author: Evil T-ModZ (Nick)
// Help: EQ2, T-ModZ:(Mormon, POSTAL_Serega, Vxe1815, VitalityMods), ErikFOV, StevenHorton, BACEK, Fallout.
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2010 - 2015.
//=============================================================================

class NKVehicle extends NKActor
	hidecategories(KActor,Force,LightColor,Lighting)
	config(NicksVehicles)
	notplaceable;

/*
MaricaPhysics
MaricaWheels
MaricaSuspension
MaricaVehicle
MaricaAISystem
MaricaHUD
MaricaGraphics
MaricaInterior
MaricaSounds
MaricaSkeleton
MaricaWeather
MaricaArmory
MaricaFastSettings
*/

const RADIAN_CONST = 10430.2192;

struct ReplaceDriverModels
{
	var Mesh OldModel;
	var Mesh NewModel;
};

/*struct ImpactBumpBonesStruct
{
	var Name BoneName;
	var class<Emitter> BoneFX;
};*/

struct ImpactStruct
{
	var Actor Other;
	var float Speed;
	var float MyVelocity;
	var float OtherVelocity;
	var vector Pos;
	var vector ImpactNorm;
};

var bool bImpactTimer;
var ImpactStruct ImpactInfo;

var(MaricaGraphics) enum DecorationSystem
{
	DS_Vector,
	DS_Bones
} DecoSystem;

var(MaricaGraphics) globalconfig enum MeshRenderingLevel
{
	MRL_High,
	MRL_Low
} MeshRenderLevel;

///FIX2015
var float OtherKWakeTime;
var(MaricaSounds) array<sound> defaultRadioSong;
var(MaricaSounds) globalconfig bool bUseCustomRadioSongs;
var(MaricaAISystem) bool bEnemyTriggerInstigator;
var(MaricaVehicle) bool bLittleVehicle;
var bool bPlayCrash;
var bool bDisableShadows;
var(MaricaVehicle) bool bSpecialVehicle;
var(MaricaVehicle) bool bTrailersVehicle;
var(MaricaVehicle) vector TrailerJointPos;
var Actor TrailerActor;
//var(MaricaSkeleton) array<ImpactBumpBonesStruct> ImpactBumpBones;
///FIX2015

var bool bDriverControllerLeft;
var bool bPassengerControllerLeft;

var	Pawn Driver;
var	Pawn Passenger;
var PlayerController DriverController;
var PlayerController PassengerController;
var Controller EnemyController;
var DriverActor myDriver;
var DriverActor myPassenger;
var DriverBase DriverBase;
var DriverBase PassengerBase;
var NKVehicleFactory OurFactory;
var NKVehicleTrigger EnterRightTrigger;
var NKVehicleTrigger EnterLeftTrigger;
var NKVehicleTrigger ExitTriggerDriver;
var NKVehicleTrigger ExitTriggerPassenger;
var NVLightProjector LightProjector;
var Explosion DestroyedEffect;
var Emitter NVFire;
var Emitter NVEngFire;
var Emitter VehicleSmoke;
var Emitter FrontRightWheelFire;
var Emitter FrontLeftWheelFire;
var Emitter RearRightWheelFire;
var Emitter RearLeftWheelFire;
var Emitter CrossRightWheelFire;
var Emitter CrossLeftWheelFire;
var NKVehicleTire EngTireRight;
var NKVehicleTire EngTireLeft;
var NKVehicleTire SteerTireRight;
var NKVehicleTire SteerTireLeft;
var NKVehicleTire CrossTireRight;
var NKVehicleTire CrossTireLeft;
var WheelIndicator EngWIRight;
var WheelIndicator EngWILeft;
var WheelIndicator SteerWIRight;
var WheelIndicator SteerWILeft;
var WheelIndicator CrossWIRight;
var WheelIndicator CrossWILeft;
var IntCollisionActor IntCollision;
var ViewPoint FPDriverViewPoint;
var ViewPoint FPPassengerViewPoint;
var Actor FPViewPointOwner;
var(MaricaAISystem) Mesh AIDiverMesh;
var(MaricaAISystem) Mesh AIDriverHeadMesh;
var(MaricaVehicle) StaticMesh IntCollisionModel;
var(MaricaAISystem) edfindable actor Enemy;
var(MaricaInterior) MeshAnimation DrivingAnims;

var bool bBeep;
var bool bBonesDestroyed;
var bool bHideDriverActor;
var bool bAIInitialized;
var bool IsDriving;
var bool ViewChanged;
var bool OutputBrake;
var bool OutputHandbrakeOn;
var bool bSteerLeftDetached;
var bool bSteerRightDetached;
var bool bEngLeftDetached;
var bool bEngRightDetached;
var bool bCrossLeftDetached;
var bool bCrossRightDetached;
var bool bHideHUD;
var bool bLightsOn;
var bool bAIBWGoing;
var(MaricaGraphics) bool bRandomSkin;
var(MaricaVehicle) bool bLocked;
var(MaricaVehicle) bool bSpawnBrokenDetail;
var(MaricaInterior) bool bAllowChangeView;
var(MaricaInterior) bool bFixedFPCamera;
var(MaricaInterior) bool bAllowPassenger;
var(MaricaInterior) bool bCanSeeDriver;
var(MaricaVehicle) bool bImmortal;
var(MaricaWheels) bool bAllowBrokenWheels;
var(MaricaWheels) bool bDamagableWheels;
var(MaricaWheels) bool bSparkWheels;
var(MaricaSounds) bool bAllowRadio;
var(MaricaAISystem) bool bAIControlled;
var(MaricaPhysics) bool bCarIsRTWD;
var(MaricaPhysics) bool bCarIsFTWD;
var(MaricaSkeleton) bool bUpdateSusp;
var(MaricaHUD) config bool bDebugHUD;
var(MaricaInterior) globalconfig bool bFreeCamera;

var int RadioSongNumber;
var int NewScaleFact;
var int TempSteerWheelBoneRot;
var(MaricaPhysics) int Gear;
var(MaricaGraphics) int SkinIndex;
var(MaricaVehicle) int ArenaDamageFactor;
var(MaricaHUD) /*globalconfig*/ int HUDOpacity;
var(MaricaHUD) /*globalconfig*/ int SpeedFactor;

var float WildRideFOV;
var float RadioUpdateTime;
var float AIThinkTimeDuration;
var(MaricaInterior) float myDriverSize;
var(MaricaSounds) globalconfig float MusicVolume;
var(MaricaSounds) float MusicRadius;
var(MaricaVehicle) float Health;
var(MaricaVehicle) float MaxHealth;
var(MaricaSounds) float RadioSwitchSpeed;
var(MaricaPhysics) float ChassisMass;
var(MaricaSounds) float EnginePitch;
var(MaricaSounds) float EngineVolume;
var(MaricaVehicle) float MinRunOverSpeed;
var(MaricaVehicle) float ProjLightFOV;
var(MaricaVehicle) float ProjLightScale;
var(MaricaVehicle) float FrontLightScale;
var(MaricaVehicle) float RearLightScale;
var(MaricaVehicle) float RearLightStopScale;
var(MaricaVehicle) float EngineStartTime;
var(MaricaPhysics) float ActionFriction;
var(MaricaPhysics) float RestFriction;
var(MaricaVehicle) float DestroyedRespawnTime;
var(MaricaPhysics) float ActionAngularDamp;
var(MaricaPhysics) float RestAngularDamp;
var(MaricaPhysics) float ActionLinearDamp;
var(MaricaPhysics) float RestLinearDamp;
var/*(MaricaAISystem)*/ float FocusSteering;
var/*(MaricaAISystem)*/ float AIThrottle;
var(MaricaAISystem) float AIChasingDist;
var(MaricaAISystem) float AIThinkTime;
var(MaricaPhysics) float MaxSteerAngle;
var(MaricaPhysics) float MaxSpeedAccel;
var(MaricaPhysics) float RearMoveAccel;
var(MaricaPhysics) float StopThreshold;
var/*(MaricaPhysics)*/ float Steering;
var/*(MaricaPhysics)*/ float Throttle;
var(MaricaPhysics) float HandbrakeThresh;
var(MaricaPhysics) float SteerPropGap;
var(MaricaPhysics) float SteerTorque;
var(MaricaPhysics) float SteerSpeed;
var(MaricaPhysics) float VehicleMaxSpeed;
var(MaricaPhysics) float TireHandbrakeSlip;
var(MaricaPhysics) float TireHandbrakeFriction;
var(MaricaPhysics) float MaxRearSpeed;
var(MaricaWheels) float TireMass;
var(MaricaWheels) float UnloadWheelHeight;
var(MaricaWheels) float WheelsHealth;
var(MaricaWheels) float WheelsMaxHealth;
var(MaricaWheels) float TireRollFriction;
var(MaricaWheels) float TireLateralFriction;
var(MaricaWheels) float TireRollSlip;
var(MaricaWheels) float TireLateralSlip;
var(MaricaWheels) float TireMinSlip;
var(MaricaWheels) float TireSlipRate;
var(MaricaWheels) float TireSoftness;
var(MaricaWheels) float TireAdhesion;
var(MaricaWheels) float TireRestitution;
var(MaricaWheels) float WheelRadius;
var(MaricaPhysics) float MaxBrakeTorque;
var(MaricaSuspension) float SuspStiffness;
var(MaricaSuspension) float SuspDamping;
var(MaricaSuspension) float SuspHighLimit;
var(MaricaSuspension) float SuspLowLimit;
var(MaricaSkeleton) float RSuspHighLimit;
var(MaricaSkeleton) float RSuspLowLimit;
var(MaricaInterior) config float MaxDesiredFOV;
var(MaricaInterior) config float CameraDist;
var(MaricaInterior) float CameraDistMax, CameraDistMin;
var(MaricaHUD) /*config*/ bool bShowVehicleHUD;
var(MaricaHUD) /*globalconfig*/ float HUDLocX;
var(MaricaHUD) /*globalconfig*/ float HUDLocSpeedY;
var(MaricaHUD) /*globalconfig*/ float HUDLocHealthY;
var(MaricaHUD) /*globalconfig*/ float HUDLocCDY;

var(MaricaHUD) /*globalconfig*/ Texture HealthTex;
var(MaricaHUD) /*globalconfig*/ Texture SpeedTex;
var(MaricaHUD) /*globalconfig*/ Texture CDTex;

var(MaricaWheels) const float WheelSteerAlong;
var(MaricaWheels) const float WheelSteerAcross;
var(MaricaWheels) const float WheelEngAlong;
var(MaricaWheels) const float WheelEngAcross;
var(MaricaWheels) const float WheelCrossAlong;
var(MaricaWheels) const float WheelCrossAcross;
var(MaricaWheels) const float WheelVert;
var(MaricaWheels) float SlipVolume;

var vector Tire1Susp;
var vector Tire2Susp;
var vector Tire3Susp;
var vector Tire4Susp;
var vector Tire5Susp;
var vector Tire6Susp;
var vector ServerPosition;
var(MaricaInterior) vector FPDriverViewPos;
var(MaricaInterior) vector FPPassengerViewPos;
var(MaricaInterior) vector FPViewAltPos;
var(MaricaVehicle) const vector IntCollisionSize;
var(MaricaInterior) const vector EnterPos;
var(MaricaInterior) const vector ExitPos;
var(MaricaInterior) const vector ExitPosPassenger;
var(MaricaInterior) const vector DriverPos;
var(MaricaInterior) const vector PassengerPos;
var(MaricaVehicle) const vector RadiatorPos;
var(MaricaVehicle) const vector ProjLightPos;

var(MaricaHUD) string SpeedText;
var(MaricaHUD) string PlayerText1;
var(MaricaHUD) string PlayerText2;
var(MaricaInterior) Name DrivingAnimStatus;
var(MaricaInterior) Name SittingAnimStatus;
var(MaricaSkeleton) Name SteerWheelBone;
var(MaricaSkeleton) Name WheelFrontRight;
var(MaricaSkeleton) Name WheelFrontLeft;
var(MaricaSkeleton) Name StrutFrontRight;
var(MaricaSkeleton) Name StrutFrontLeft;
var(MaricaSkeleton) Name WheelRearRight;
var(MaricaSkeleton) Name WheelRearLeft;
var(MaricaSkeleton) Name StrutRearRight;
var(MaricaSkeleton) Name StrutRearLeft;
var(MaricaSkeleton) Name WheelCrossRight;
var(MaricaSkeleton) Name WheelCrossLeft;
var(MaricaSkeleton) Name StrutCrossRight;
var(MaricaSkeleton) Name StrutCrossLeft;
var(MaricaSkeleton) Name WheelPlusRight;
var(MaricaSkeleton) Name WheelPlusLeft;
var(MaricaSkeleton) Name StrutPlusRight;
var(MaricaSkeleton) Name StrutPlusLeft;

var array<NVCarLight> FrontLight;
var array<NVCarLight> RearLight;
var array<Emitter> CarExhaust;
var(MaricaSkeleton) array<Name> ExhaustBones;
var(MaricaSkeleton) array<Name> FrontLightBones;
var(MaricaSkeleton) array<Name> RearLightBones;
var(MaricaGraphics) array<Material> BrokenSkins;
var(MaricaGraphics) array<Material> OtherSkins;
var(MaricaVehicle) array<vector> ExhaustPos;
var(MaricaVehicle) array<vector> FrontLightPos;
var(MaricaVehicle) array<vector> RearLightPos;
var(MaricaSounds) array<sound> ImpactSoundsMin;
var(MaricaSounds) array<sound> ImpactSoundsMed;
var(MaricaSounds) array<sound> ImpactSoundsMax;
var(MaricaSkeleton) array<Name>	DestroyBones;
var(MaricaSounds) globalconfig array<Sound> RadioSong;
var(MaricaInterior) globalconfig array<ReplaceDriverModels> AltDriverModels;

var(MaricaVehicle) class<Emitter> ImpactEffectClass;
var(MaricaVehicle) class<NVCarLight> FrontLightClass;
var(MaricaVehicle) class<NVCarLight> RearLightClass;
var(MaricaVehicle) class<NVLightProjector> LightProjectorClass;
var(MaricaVehicle) class<DamageType> RanOverDamage;
var(MaricaWheels) class<NKVehicleTire> EngTireClass;
var(MaricaWheels) class<NKVehicleTire> SteerTireClass;
var(MaricaWheels) class<NKVehicleTire> CrossTireClass;
var(MaricaVehicle) class<Explosion> VehicleExplosionClass;
var(MaricaVehicle) class<Emitter> VehicleFireClass;
var(MaricaVehicle) class<Emitter> ExhaustClass;
var(MaricaVehicle) class<IntCollisionActor> IntCollisionClass;

var(MaricaVehicle) string Description;

var rotator Tire1RotT;
var rotator Tire2RotT;
var rotator Tire3RotT;
var rotator Tire4RotT;
var rotator Tire5RotT;
var rotator Tire6RotT;
var rotator Strut1Roll;
var rotator Strut2Roll;
var rotator Strut3Roll;
var rotator Strut4Roll;
var rotator Strut5Roll;
var rotator Strut6Roll;
var rotator ServerRotation;
var(MaricaVehicle) const rotator ProjectedLightRot;

var Material CurrentSkin;
var(MaricaAISystem) Material AIDriverTex;
var(MaricaAISystem) Material AIDriverHeadTex;
var(MaricaVehicle) Material ProjLightMaterial;
var(MaricaVehicle) Material FrontLightMaterial;
var(MaricaVehicle) Material RearLightMaterial;

var(MaricaSounds) sound RadioSwitchSound;
var(MaricaSounds) sound RanOverSound;
var(MaricaSounds) sound EngineSound;
var(MaricaSounds) sound EngStartUpSound;
var(MaricaSounds) sound EngStopSound;
var(MaricaSounds) sound DestroyedSound;
var(MaricaSounds) sound BeepSound;
var /*config*/ sound CurrentSong;

var MotionRotatorActor SteerWheelRotator;
var MaricaLevelTravelObject LevelTravelObject;
var float WheelSpinSpeed;
var float SteeringAngle;
var(MaricaPhysics) InterpCurve TorqueCurve;
var(MaricaPhysics) float ImpactThreshold;
var(MaricaInterior) globalconfig bool bWildRideFOV;
var(MaricaGraphics) globalconfig bool bAllowProjectionLight;
var(MaricaGraphics) globalconfig bool bAllowLight;
var(MaricaWeather) float DustPosZ;
var(MaricaWeather) float DustSlipRate;
var(MaricaWeather) float DustSlipThresh;
var(MaricaWeather) enum MapSurfaceTypeSystem
{
	MST_Default,
	MST_Grass,
	MST_Dirt,
	MST_Snow,
	MST_Sand,
	MST_Ice
} MapSurfaceType;

var NKVHandsWeapon myVehicleHands;
var Pawn oldDriverTemp;
var PlayerController oldDriverController;
var bool bDriverGodMode;
var(MaricaWheels) class<WheelIndicator> WheelIndicatorClass;
var class<P2Weapon> NKVHandsClass;
var(MaricaVehicle) bool bNormalHUDView;
var(MaricaVehicle) bool bAllowWeaponsInside;
var(MaricaVehicle) bool bDriverTakesDamage;
var(MaricaGraphics) float ShadowZ;
var(MaricaSounds) float RadioSwitchVolume;
var(MaricaSounds) float BeepVolume;
var(MaricaSounds) float DestroyedVolume;
var(MaricaSounds) float EngineStartUpVolume;
var(MaricaSounds) float EngineStopVolume;
var(MaricaSounds) float RanOverVolume;
var(MaricaSounds) float WheelUnloadVolume;
var(MaricaSounds) Sound WheelUnloadSound;

var(MaricaFastSettings) float CarSpeedScale;
var(MaricaFastSettings) float CarGripScale;
var(MaricaFastSettings) float CarSteeringScale;
var(MaricaFastSettings) float CarWeightScale;
var(MaricaFastSettings) float CarSuspensionScale;

replication
{
	unreliable if( Role == ROLE_Authority )
		Health , RadioSongNumber , CurrentSkin , IsGoingToUpdate , CurrentSong , bHideHUD , ArenaDamageFactor , 
		ServerPosition , ServerRotation , bBeep , VehicleDestroyed , bAIControlled , bLightsOn , bCanSeeDriver , 
		WheelSpinSpeed , SteeringAngle , SteerWheelRotator , ImpactInfo , bPlayCrash , myVehicleHands , MapSurfaceType;

	unreliable if( bNetDirty && Role == ROLE_Authority )
		Driver , Passenger , DriverController , PassengerController , Enemy , EnemyController , IntCollision , Steering;

	unreliable if( Role == ROLE_Authority && Driver != None )
		WildRideFOV;

	unreliable if( Role == ROLE_Authority && bAllowChangeView )
		FPDriverViewPoint, FPPassengerViewPoint , FPViewPointOwner;

	unreliable if( Role == ROLE_Authority && bCanSeeDriver )
		myDriver , myPassenger;

	unreliable if( Role == ROLE_Authority /*&& DrawType == DT_Mesh*/ )
		Tire1RotT , Tire2RotT , Tire3RotT , Tire4RotT , Tire5RotT , Tire6RotT ,
		bSteerLeftDetached , bSteerRightDetached , bEngLeftDetached , bEngRightDetached ,
		bCrossLeftDetached , bCrossRightDetached , bBonesDestroyed;

	unreliable if( Role == ROLE_Authority && MeshRenderLevel != MRL_Low )
		Tire1Susp , Tire2Susp , Tire3Susp , Tire4Susp , Tire5Susp , Tire6Susp ,
		Strut1Roll , Strut2Roll , Strut3Roll , Strut4Roll , Strut5Roll , Strut6Roll;

	//reliable if( Role == ROLE_Authority )
	//	ImpactInfo , Scrape;

	reliable if( Role < ROLE_Authority )
		bHideDriverActor;// , ViewChanged , TempSteerWheelBoneRot;
}

simulated function bool IsInAir()
{
	return ( !SteerTireRight.bTireOnGround && !SteerTireLeft.bTireOnGround && !EngTireRight.bTireOnGround && !EngTireLeft.bTireOnGround );
}

simulated function bool IsSomeWheelOnGround()
{
	return ( SteerTireRight.bTireOnGround || SteerTireLeft.bTireOnGround || EngTireRight.bTireOnGround || EngTireLeft.bTireOnGround );
}

simulated function bool IsSomeWheelNotOnGround()
{
	return ( !SteerTireRight.bTireOnGround || !SteerTireLeft.bTireOnGround || !EngTireRight.bTireOnGround || !EngTireLeft.bTireOnGround );
}

simulated function bool IsLeftOnGround()
{
	return (EngTireLeft.bTireOnGround && SteerTireLeft.bTireOnGround);
}

simulated function bool IsRightOnGround()
{
	return (EngTireRight.bTireOnGround && SteerTireRight.bTireOnGround);
}

simulated function bool IsOnlyRightOnGround()
{
	return (IsRightOnGround() && !IsLeftOnGround());
}

simulated function bool IsOnlyLeftOnGround()
{
	return (!IsRightOnGround() && IsLeftOnGround());
}

simulated function bool IsInverted()
{
	return ( (vect(0, 0, 1) >> Rotation).Z < 0.2 );
}

simulated function bool VehicleDestroyed()
{
	if( Health <= 0 )
		return true;
	return false;
}

simulated function bool IsGoingToUpdate()
{
	if( mySpeed > 0.1 )
		return true;
	return false;
}

simulated function ClientSimulationPack(optional bool LowRender)
{
	UpdateSimulatedPosition(LowRender);
	if( !LowRender )
		UpdateDriverActors();
	SetBlockKarma();
}

simulated function SetBlockKarma()
{
	KSetBlockKarma(false);
	KSetBlockKarma(true);
}

simulated function UpdateSimulatedPosition(bool bOnlyPosition)
{
	local vector DistD, FPPos;

	if( bOnlyPosition )
	{
		if( VSize(ServerPosition - Location) > 2 )
		{
			SetLocation(ServerPosition);
			Velocity = vect(0,0,0);
		}
		else if( ((ServerRotation.Yaw - Rotation.Yaw) > 4) 
			|| ((ServerRotation.Roll - Rotation.Roll) > 4) 
			|| ((ServerRotation.Pitch - Rotation.Pitch) > 4) )
		{
			SetRotation(ServerRotation);
			RotationRate = rot(0,0,0);
			DesiredRotation = rot(0,0,0);
		}
		return;
	}

	if( bNormalHUDView && Driver != None && DriverController != None && !DriverController.bBehindView && DriverController.ViewTarget != None )
	{
		RotationRate = Normalize((Normalize(ServerRotation) - Normalize(Rotation))) * 20;
		DesiredRotation = ServerRotation;
		///NETEyeHeight FIX2015
		FPPos = -(DriverPos >> ServerRotation);
		FPPos += DriverController.ViewTarget.Location;// - DriverController.ShakeOffset;
		SetLocation(FPPos);
		return;
	}

	RotationRate = Normalize((Normalize(ServerRotation) - Normalize(Rotation))) * 20;
	DesiredRotation = ServerRotation;
	DistD = Location - ServerPosition;
	Velocity = -DistD * 20;
}

simulated function xForceReturnView(PlayerController aPC)
{
	if( aPC == None )
		return;
	aPC.MyHUD.bShowDebugInfo = false;
	aPC.CameraDist = aPC.default.CameraDist;
	aPC.FixFOV();
	if( Health > 0 && aPC.Pawn != None )
		aPC.ClientSetViewTarget(aPC.Pawn);
}

simulated function xForceReturnViewDelayed(PlayerController aPC)
{
	local PMMiscTimer_FixView PMMiscTimer_FixView;
	PMMiscTimer_FixView = spawn(class'PMMiscTimer_FixView',aPC);
	PMMiscTimer_FixView.VehicleHealth = Health;
}

simulated function UserInterfaceControl()
{
	if( DriverController != None )
	{
		if( Driver != None )
		{
			if( !bNormalHUDView )
			{
				if( !DriverController.MyHUD.bShowScores && !bHideHUD )
				{
					if(!DriverController.MyHUD.bShowDebugInfo)
						DriverController.MyHUD.bShowDebugInfo = true;
				}
				else
				{
					if(DriverController.MyHUD.bShowDebugInfo)
						DriverController.MyHUD.bShowDebugInfo = false;
				}
			}

			if( DriverController.CameraDist != CameraDist )
				DriverController.CameraDist = CameraDist;
			if( bWildRideFOV )
				MonitorDisplayFOV(DriverController,mySpeed);
			MonitorBehindView(DriverController,true);
			BanWeapons(DriverController);

			if( !DriverController.bGodMode && !bNormalHUDView )
				DriverController.bGodMode = true;

			oldDriverController = DriverController;
			oldDriverTemp = Driver;
			bDriverControllerLeft = false;
		}
		else if( !bDriverControllerLeft )
		{
			xForceReturnViewDelayed(DriverController);
			xForceReturnView(DriverController);
			log("Current driver controller ("$DriverController$") leave this vehicle ("$self$")");
			///DriverController = None;
			bDriverControllerLeft = true;
		}
	}
	///else DriverController = None;

	if( PassengerController != None )
	{
		if( Passenger != None )
		{
			if( !bNormalHUDView )
			{
				if( !PassengerController.MyHUD.bShowScores && !bHideHUD )
				{
					if(!PassengerController.MyHUD.bShowDebugInfo)
						PassengerController.MyHUD.bShowDebugInfo = true;
				}
				else
				{
					if(PassengerController.MyHUD.bShowDebugInfo)
						PassengerController.MyHUD.bShowDebugInfo = false;
				}
			}

			if( PassengerController.CameraDist != CameraDist )
				PassengerController.CameraDist = CameraDist;
			MonitorDisplayFOV(PassengerController,mySpeed);
			MonitorBehindView(PassengerController,false);
			BanWeapons(PassengerController);
			bPassengerControllerLeft = false;
		}
		else if( !bPassengerControllerLeft )
		{
			xForceReturnViewDelayed(PassengerController);
			xForceReturnView(PassengerController);
			log("Current passenger controller ("$PassengerController$") leave this vehicle ("$self$")");
			///PassengerController = None;
			bPassengerControllerLeft = true;
		}
	}
	///else PassengerController = None;
}

simulated function bool HasVehicleHandsOut()
{
	local P2Weapon p2weap;

	p2weap = P2Weapon(Driver.Weapon);

	if( p2weap != None
		&& p2weap == myVehicleHands )
		return true;

	return false;
}

simulated function BanWeapons( PlayerController PC )
{
	local P2Player P2P;

	if( bAllowWeaponsInside )
		return;

	if( myVehicleHands != None && !HasVehicleHandsOut() )
	{
		Driver.Weapon = myVehicleHands;
		//Driver.ClientMessage(Driver.Weapon);
	}

	/*P2P = P2Player(PC);

	if(P2P != None && !P2P.HasHandsOut())
	{
		P2P.SwitchToHands(true);
		//log("weapon was switched");
	}*/
}

simulated function StartScriptedDriving(optional Actor myEnemy, optional Controller myEnemyContoller)
{
	if( myEnemyContoller != None )
		EnemyController = myEnemyContoller;
	else
	{
		if( myEnemy != None )
		{
			if( Pawn(myEnemy) != None )
			{
				if( Pawn(myEnemy).Health > 0 )
					Enemy = myEnemy;
			}
			else Enemy = myEnemy;
		}
	}
	if( bAIInitialized )
		return;
	bAIInitialized = true;
	SpawnLights();
	Steering = 0.2;
	FocusSteering = 0.2;
	GotoState('BeforeBeStart');
}

function Pawn EnemyMonitor()
{
	if(EnemyController != None 
		&& EnemyController.Pawn != None 
		&& EnemyController.Pawn.Health > 0)
		return EnemyController.Pawn;
	return None;
}

simulated function bool DetectGameIsSingle()
{
	if( Level != None && Level.Game != None && Level.Game.bIsSingleplayer )
		return Level.NetMode != NM_Client && Level.NetMode != NM_DedicatedServer;
	return false;
}

simulated function SwitchMessageType()
{
	if( class'NVMessage'.default.ActionUseGameSwitch != 1 )
	{
		if( DynamicLoadObject("ProjectMarica.ProjectMaricaMenu", class'Class') == None )
			class'NVMessage'.default.ActionUseGame = 1;
		else if( DynamicLoadObject("FPSGame.FPSButtonInfo", class'Class') != None )
			class'NVMessage'.default.ActionUseGame = 0;
		else if( !DetectGameIsSingle() )
			class'NVMessage'.default.ActionUseGame = 1;
		else class'NVMessage'.default.ActionUseGame = 2;
	}
}

simulated event PostNetBeginPlay()
{
	Super.PostNetBeginPlay();
	GetVehicleData();
	UpdateSimulatedPosition(true);
	ApplyVehicleData();
	if( bAIControlled )
		ShowDriver();
	if( IntCollision != None )
	{
		IntCollision.SetLocation(Location);
		IntCollision.SetRotation(Rotation);
	}
}

simulated function UpdateHeadLights()
{
	if( LightProjectorClass == None 
		|| !bAllowProjectionLight 
		|| !bAllowLight )
	{
		if( LightProjector != None )
		{
			LightProjector.Destroy();
			LightProjector = None;
		}
		return;
	}

	if( bLightsOn )
	{
		if( LightProjector == None && ProjLightMaterial != None
			&& Level.NetMode != NM_DedicatedServer )
		{
			LightProjector = spawn(LightProjectorClass, self,, Location + (ProjLightPos >> Rotation));
			LightProjector.SetBase(self);
			LightProjector.BaseActor = self;
			LightProjector.SetRelativeRotation(ProjectedLightRot);
			LightProjector.FOV = ProjLightFOV;
			LightProjector.ProjTexture = ProjLightMaterial;
			LightProjector.SetDrawScale(ProjLightScale);
			LightProjector.UpdateProjection();
		}
	}
	else
	{
		if( LightProjector != None )
		{
			LightProjector.Destroy();
			LightProjector = None;
		}
	}
}

function bool EncroachingOn( actor Other )
{
	return false;
}

event Trigger( Actor Other, Pawn EventInstigator )
{
	if( bAIControlled )
	{
		if( bEnemyTriggerInstigator )
		{
			Enemy = EventInstigator;
			StartScriptedDriving(EventInstigator);
		}
		else if( Enemy != None )
			StartScriptedDriving(Enemy);
	}
	else bLocked = !bLocked;
}

function TryToDrive(Pawn p)
{
	local Controller C;
	local P2Player P2P;
	C = p.Controller;
	P2P = P2Player(C);

	if( bLocked	|| C == None || !C.bIsPlayer )
		return;

    if( Driver == None && C.IsInState('PlayerWalking') )
		DriverEnter(p);
	else if( Driver != None && Passenger == None && C.IsInState('PlayerWalking') && bAllowPassenger )
		PassengerEnter(p);
}

function TryToExit(Pawn myUser)
{
	if( GameIsEnd() )
		return;
    if( Driver != None && myUser == Driver )
		DriverLeave();
	else if( Passenger != None && myUser == Passenger )
		PassengerLeave();
	else log("Someone "$myUser$" tries to exit");
}

simulated function SendEnterMessage(PlayerController PC)
{
	PC.ReceiveLocalizedMessage(class'NVMessage', 1);
	PC.ReceiveLocalizedMessage(class'NVMessage', 4);
}

simulated event DriverEnter(Pawn NewDriver)
{
	local PlayerController PC;

	if( NewDriver == None
		|| NewDriver.Health <= 0
		|| PlayerController(NewDriver.Controller) == None )
		return;

	//ViewChanged = false;

	PC = PlayerController(NewDriver.Controller);
	PC.SetCollision( PC.default.bCollideActors, PC.default.bBlockActors, PC.default.bBlockPlayers );
	//PC.ReceiveLocalizedMessage(class'NVMessage', 1);
	//PC.ReceiveLocalizedMessage(class'NVMessage', 4);
	SendEnterMessage(PC);
	PC.ClientSetBehindView(true);
	PC.bBehindView = true;
	PC.ClientSetViewTarget(self);
	if( bNormalHUDView )
	{
		PC.ClientSetBehindView(false);
		PC.bBehindView = false;
		PC.ClientSetViewTarget(NewDriver);
		//spawn(class'DriverFPSwitchScript', NewDriver);
	}
	PC.bFreeCamera = true;
	if( !bNormalHUDView )
		PC.MyHUD.bShowDebugInfo = true;
	PC.CameraDist = CameraDist;
	bDriverGodMode = PC.bGodMode;
	if( bDriverGodMode )
		bImmortal = true;
	if( !bDriverTakesDamage )
		PC.bGodMode = true;
	BanWeapons(PC);
	PC.GotoState('PlayerWalking');

	NewDriver.SetCollision(true, false, false);
	NewDriver.bCollideWorld = false;
	NewDriver.bHidden = true;
	NewDriver.bPhysicsAnimUpdate = false;
	NewDriver.Velocity = vect(0,0,0);
	NewDriver.SetLocation(Location + (DriverPos >> Rotation));
	NewDriver.SetPhysics(PHYS_None);
	Driver = NewDriver;

	if(DriverBase != None)
	{
		if( bNormalHUDView )
			DriverBase.SetFixedPosition(true);
		else DriverBase.SetFixedPosition(!bFreeCamera);
		Driver.SetBase(DriverBase);
	}
	else Driver.SetBase(self);

	GotoState('BeforeBeStart');

	if(myDriver != None)
	{
		myDriver.Destroy();
		myDriver = None;
	}

	ShowDriver();

	//SwitchSong();
	if( RadioSongNumber > 0)
		RadioSongNumber--;

	SpawnLights();
	CheckLevelTravelObject(NewDriver, true);
}

simulated event DriverEnterTravel(Pawn NewDriver, rotator DriverViewRotation)
{
	local PlayerController PC;

	if( NewDriver == None
		|| NewDriver.Health <= 0
		|| PlayerController(NewDriver.Controller) == None )
		return;

	//ViewChanged = false;

	PC = PlayerController(NewDriver.Controller);
	PC.SetCollision( PC.default.bCollideActors, PC.default.bBlockActors, PC.default.bBlockPlayers );
	//PC.ReceiveLocalizedMessage(class'NVMessage', 1);
	PC.ClientSetBehindView(true);
	PC.bBehindView = true;
	PC.ClientSetViewTarget(self);
	if( bNormalHUDView )
	{
		PC.ClientSetBehindView(false);
		PC.bBehindView = false;
		PC.ClientSetViewTarget(NewDriver);
		//spawn(class'DriverFPSwitchScript', NewDriver);
	}
	PC.SetRotation(DriverViewRotation);
	PC.bFreeCamera = true;
	if( !bNormalHUDView )
		PC.MyHUD.bShowDebugInfo = true;
	PC.CameraDist = CameraDist;
	bDriverGodMode = PC.bGodMode;
	if( bDriverGodMode )
		bImmortal = true;
	if( !bDriverTakesDamage )
		PC.bGodMode = true;
	BanWeapons(PC);
	PC.GotoState('PlayerWalking');

	NewDriver.SetCollision(true, false, false);
	NewDriver.bCollideWorld = false;
	NewDriver.bHidden = true;
	NewDriver.bPhysicsAnimUpdate = false;
	NewDriver.Velocity = vect(0,0,0);
	NewDriver.SetLocation(Location + (DriverPos >> Rotation));
	NewDriver.SetPhysics(PHYS_None);
	Driver = NewDriver;

	if(DriverBase != None)
	{
		if( bNormalHUDView )
			DriverBase.SetFixedPosition(true);
		else DriverBase.SetFixedPosition(!bFreeCamera);
		Driver.SetBase(DriverBase);
	}
	else Driver.SetBase(self);

	if(myDriver != None)
	{
		myDriver.Destroy();
		myDriver = None;
	}

	ShowDriver();

	//SwitchSong();
	if( RadioSongNumber > 0)
		RadioSongNumber--;

	SpawnLights();
	CheckLevelTravelObject(NewDriver, true);

    VehicleHaveDriver();
	GotoState('VehicleDriving');
}

simulated event PassengerEnter(Pawn NewPassenger)
{
	local PlayerController PC;

	if( NewPassenger == None
		|| NewPassenger.Health <= 0
		|| PlayerController(NewPassenger.Controller) == None )
		return;

	PC = PlayerController(NewPassenger.Controller);
	PC.ReceiveLocalizedMessage(class'NVMessage', 1);
	PC.ReceiveLocalizedMessage(class'NVMessage', 4);
	PC.ClientSetBehindView(true);
	PC.bBehindView = true;
	PC.ClientSetViewTarget(self);
	/*if( bNormalHUDView )
	{
		PC.ClientSetBehindView(false);
		PC.ClientSetViewTarget(NewPassenger);
	}*/
	PC.bFreeCamera = true;
	if( !bNormalHUDView )
		PC.MyHUD.bShowDebugInfo = true;
	PC.CameraDist = CameraDist;
	PC.bGodMode = true;
	BanWeapons(PC);
	PC.GotoState('PlayerWalking');

	NewPassenger.SetCollision(true, false, false);
	NewPassenger.bCollideWorld = false;
	NewPassenger.bHidden = true;
	NewPassenger.bPhysicsAnimUpdate = false;
	NewPassenger.Velocity = vect(0,0,0);
	NewPassenger.SetLocation(Location + (PassengerPos >> Rotation));
	NewPassenger.SetPhysics(PHYS_None);
	Passenger = NewPassenger;

	if(PassengerBase != None)
		Passenger.SetBase(PassengerBase);
	else Passenger.SetBase(self);

	if( IntCollision != None )
		IntCollision.SetTimer(1.0,false);

	if(myPassenger != None)
	{
		myPassenger.Destroy();
		myPassenger = None;
	}

	ShowPassenger();

	//if( Role < ROLE_Authority )
	//	ViewChanged = true;
}

simulated event DriverLeave(optional bool bDestroying)
{
	local int i;
	local Pawn ownOldDriver;
	local PlayerController PC;
	local vector AltExitPos;
	local Rotator ExitRot;

	if( Driver == None )
		return;

	ownOldDriver = Driver;

	AltExitPos = Location;
	AltExitPos.Z += 175;

	PC = PlayerController(Driver.Controller);

	if( PC != None )
	{
		ExitRot = PC.Rotation;
		PC.MyHUD.bShowDebugInfo = false;
		PC.ClientSetBehindView(false);
		PC.bBehindView = false;
		PC.bFreeCamera = false;
		PC.ClientSetViewTarget(Driver);
		PC.ClientSetRotation(ExitRot);
		PC.FixFOV();
		PC.CameraDist = PC.default.CameraDist;
		if( bDriverGodMode )
			bImmortal = default.bImmortal;
		PC.bGodMode = bDriverGodMode;
	}
	Driver.SetCollision(true, true, true);
	Driver.bCollideWorld = true;

	if( !IsInverted() )
		Driver.SetLocation(Location + (ExitPos >> Rotation));
	else Driver.SetLocation(AltExitPos);

	Driver.PlayWaiting();
	Driver.bPhysicsAnimUpdate = Driver.Default.bPhysicsAnimUpdate;
	Driver.Acceleration = vect(0, 0, 24000);
	Driver.SetPhysics(PHYS_Falling);
	Driver.SetBase(None);
	Driver.bHidden = false;

	if(EnterRightTrigger != None)
		EnterRightTrigger.SetPawnIgnore(Driver, 1.0);

	if(EnterLeftTrigger != None)
		EnterLeftTrigger.SetPawnIgnore(Driver, 1.0);

	VehicleHaveNoDriver(bDestroying);
	if( Passenger != None )
	{
		Passenger.SetLocation(Location + (PassengerPos >> Rotation));
		Passenger.SetPhysics(PHYS_None);
		Passenger.SetBase(PassengerBase);
	}
	DestroyLights();
	if( EngStopSound != None && !bDestroying )
		PlaySound(EngStopSound, SLOT_Misc, EngineStopVolume);
	CheckLevelTravelObject(ownOldDriver, false);
	SaveConfig();
}

simulated function DriverForceLeave(optional bool bDontPlaceDriver)
{
	local vector BodyPos;

	if( Driver == None )
		return;

	if( GameIsSingle() )
	{
		Driver.bHidden = false;
		Driver.SetCollision(true, true, true);
	}
	else
	{
		Driver.bHidden = true;
		Driver.SetCollision(false, false, false);
		Driver.LifeSpan = 2.0;
	}
	Driver.bCollideWorld = true;
	if( !bDontPlaceDriver )
	{
		BodyPos = Location + (ExitPos >> Rotation);
		BodyPos.Z = Location.Z;
		Driver.SetLocation(BodyPos);
	}

	Driver.PlayWaiting();
	Driver.bPhysicsAnimUpdate = Driver.Default.bPhysicsAnimUpdate;
	Driver.SetBase(None);

	if( GameIsSingle() && P2Pawn(Driver) != None )
	{
		P2Pawn(Driver).GotoState('');
		P2Pawn(Driver).GotoState('Dying');
	}
}

simulated event PassengerLeave()
{
	local rotator ExitRot;
	local PlayerController PC;
	local vector AltExitPos;

	if( Passenger == None )
		return;

	AltExitPos = Location;
	AltExitPos.Z += 175;
	AltExitPos.X -= 128;

	PC = PlayerController(Passenger.Controller);

	if( PC != None )
	{
		ExitRot = PC.Rotation;
		PC.ClientSetBehindView(false);
		PC.bBehindView = false;
		PC.bFreeCamera=false;
		PC.ClientSetViewTarget(Passenger);
		PC.ClientSetRotation(ExitRot);
		PC.CameraDist = PC.default.CameraDist;
		PC.bGodMode = false;
	}

	Passenger.SetCollision(true, true, true);
	Passenger.bCollideWorld = true;

	if( !IsInverted() )
		Passenger.SetLocation(Location + (ExitPosPassenger >> Rotation));
	else Passenger.SetLocation(AltExitPos);

	Passenger.PlayWaiting();
	Passenger.bPhysicsAnimUpdate = Passenger.Default.bPhysicsAnimUpdate;
	Passenger.Acceleration = vect(0, 0, 24000);
	Passenger.SetPhysics(PHYS_Falling);
	Passenger.SetBase(None);
	Passenger.bHidden = false;

	if( EnterRightTrigger != None )
		EnterRightTrigger.SetPawnIgnore(Passenger, 1.0);
	if( EnterLeftTrigger != None )
		EnterLeftTrigger.SetPawnIgnore(Passenger, 1.0);

	VehicleHaveNoPassenger();
}

simulated function PassengerForceLeave()
{
	local vector BodyPos;

	if( Passenger == None )
		return;

	if( GameIsSingle() )
	{
		Passenger.bHidden = false;
		Passenger.SetCollision(true, true, true);
	}
	else
	{
		Passenger.bHidden = true;
		Passenger.SetCollision(false, false, false);
		Passenger.LifeSpan = 2.0;
	}
	Passenger.bCollideWorld = true;
	BodyPos = Location + (ExitPosPassenger >> Rotation);
	BodyPos.Z = Location.Z;
	Passenger.SetLocation(BodyPos);
	Passenger.PlayWaiting();
	Passenger.bPhysicsAnimUpdate = Passenger.Default.bPhysicsAnimUpdate;
	Passenger.SetBase(None);

	if( GameIsSingle() && P2Pawn(Passenger) != None )
	{
		P2Pawn(Passenger).GotoState('');
		P2Pawn(Passenger).GotoState('Dying');
	}
}

function VehicleHaveNoDriver(optional bool bDestroying, optional bool bDriverFallOff)
{
	local int i,n;
	local Pawn OldDriver;
	local DriverFallOutScript tempFallOutScript;

	OldDriver = Driver;
	Driver = None;

	if( IntCollision != None )
		IntCollision.CarDriver = None;

    Throttle = 0;
    Steering = 0;

	//ViewChanged = false;

	if(bCarIsRTWD)
	{
		EngTireRight.WheelJoint.KMotorTorque = Throttle;
		EngTireLeft.WheelJoint.KMotorTorque = Throttle;
		if( CrossTireClass != None )
		{
			CrossTireRight.WheelJoint.KMotorTorque = Throttle;
			CrossTireLeft.WheelJoint.KMotorTorque = Throttle;
		}
	}
	if(bCarIsFTWD)
	{
		SteerTireRight.WheelJoint.KMotorTorque = Throttle;
		SteerTireLeft.WheelJoint.KMotorTorque = Throttle;
		if( CrossTireClass != None )
		{
			CrossTireRight.WheelJoint.KMotorTorque = Throttle;
			CrossTireLeft.WheelJoint.KMotorTorque = Throttle;
		}
	}

	if( IntCollision != None && !bDriverFallOff )
		IntCollision.AmbientSound = None;

	AmbientSound = None;
	DriverBase.AmbientSound = None;
	if( ExhaustClass != None && !bDriverFallOff )
	{
		if(DecoSystem == DS_Bones)
			n = ExhaustBones.Length;
		else n = ExhaustPos.Length;
		for( i = 0 ; i < n ; i++ )
		{
			CarExhaust.Length = n;
			if(CarExhaust[i] != None)
			{
				if(DecoSystem == DS_Bones)
					DetachFromBone(CarExhaust[i]);
				CarExhaust[i].Destroy();
				CarExhaust[i] = None;
			}
		}
	}
	if(myDriver != None)
	{
		if( MpPawn(OldDriver) != None && MpPawn(OldDriver).myHead != None )
		{
			myDriver.DetachFromBone(MpPawn(OldDriver).myHead);
			MpPawn(OldDriver).AttachToBone(MpPawn(OldDriver).myHead, 'MALE01 head');
			MpPawn(OldDriver).myHead.SetOwner(OldDriver);
		}
		myDriver.Destroy();
		myDriver = None;
	}
	if(ExitTriggerDriver != None)
	{
		ExitTriggerDriver.Destroy();
		ExitTriggerDriver = None;
	}

	if( !bDestroying )
		UpdateBrakes();
	if( bDriverFallOff )
	{
		if( oldDriverController != None )
		{
			oldDriverController.MyHUD.bShowDebugInfo = false;
			oldDriverController.CameraDist = DriverController.default.CameraDist;
			oldDriverController.FixFOV();
			tempFallOutScript = spawn(class'DriverFallOutScript');
			tempFallOutScript.DriverPC = oldDriverController;
			tempFallOutScript.DriverPawn = oldDriverTemp;
		}
	}
	GotoState('VehicleIdling');
}

simulated function VehicleHaveDriver()
{
	local int i;

	if( IntCollision != None )
	{
		IntCollision.AmbientSound = EngineSound;
		IntCollision.SoundPitch = EnginePitch;
		IntCollision.CarDriver = Driver;
	}

	UpdateBrakes();
	if( ExhaustClass != None )
	{
		if(DecoSystem == DS_Vector)
		{
			for(i = 0; i < ExhaustPos.Length; i++)
			{
				CarExhaust.Length = ExhaustPos.Length;
				if(VSize(ExhaustPos[i]) != 0 
					&& ExhaustClass != None 
					&& CarExhaust[i] == None)
				{
					CarExhaust[i] = spawn(ExhaustClass, self,, Location + (ExhaustPos[i] >> Rotation));
					CarExhaust[i].SetBase(self);
				}
			}
		}
		else if(DecoSystem == DS_Bones)
		{
			for(i = 0; i < ExhaustBones.Length; i++)
			{
				CarExhaust.Length = ExhaustBones.Length;
				if(ExhaustBones[i] != ''
					&& ExhaustClass != None 
					&& CarExhaust[i] == None)
				{
					CarExhaust[i] = spawn(ExhaustClass, self,, Location);
					AttachToBone(CarExhaust[i], ExhaustBones[i]);
				}
			}
		}
	}
	if(bAIControlled)
		return;
	if(ExitTriggerDriver == None)
		ExitTriggerDriver = spawn(class'NKVehicleTrigger', self,, Location + (DriverPos >> Rotation));
	if(ExitTriggerDriver != None)
	{
		ExitTriggerDriver.SetBase(self);
		ExitTriggerDriver.myUser = Driver;
		ExitTriggerDriver.bExitTrigger = true;
		ExitTriggerDriver.MyVehicle = self;
		ExitTriggerDriver.SetCollision(true, false, false);
		ExitTriggerDriver.SetCollisionSize(8.0,8.0);
	}
}

function UpdateBrakes()
{
	local float Speed;
	Speed = VSize(Velocity);

	if( IsInWater() )
		return;

	if( Driver == None && !bAIControlled )
	{
		if( Speed <= 800 )
		{
			EngTireRight.WheelJoint.KWake();
			EngTireLeft.WheelJoint.KWake();
			SteerTireRight.WheelJoint.KWake();
			SteerTireLeft.WheelJoint.KWake();
			EngTireRight.WheelJoint.KBraking = MaxBrakeTorque;
			EngTireLeft.WheelJoint.KBraking = MaxBrakeTorque;
			SteerTireRight.WheelJoint.KBraking = MaxBrakeTorque;
			SteerTireLeft.WheelJoint.KBraking = MaxBrakeTorque;
		}
	}
	else
	{
		EngTireRight.WheelJoint.KWake();
		EngTireLeft.WheelJoint.KWake();
		SteerTireRight.WheelJoint.KWake();
		SteerTireLeft.WheelJoint.KWake();
		EngTireRight.WheelJoint.KBraking = 0.0;
		EngTireLeft.WheelJoint.KBraking = 0.0;
		SteerTireRight.WheelJoint.KBraking = 0.0;
		SteerTireLeft.WheelJoint.KBraking = 0.0;
	}
}

simulated function Mesh CheckPawnModel(mesh Model)
{
	local int i;
	local Mesh ThisModel;
	if(AltDriverModels.Length > 0 && AltDriverModels[0].OldModel != None)
	{
		for(i = 0; i < AltDriverModels.Length; i++)
		{
			if(Model == AltDriverModels[i].OldModel)
				ThisModel = AltDriverModels[i].NewModel;
		}
	}
	if(ThisModel != None)
		return ThisModel;
	return None;
}

simulated function Mesh FindPawnModel(bool bDriver)
{
	local Mesh myModel, returnModel;
	if(bDriver)
		myModel = Driver.Mesh;
	else
		myModel = Passenger.Mesh;
	returnModel = CheckPawnModel(myModel);
	if(returnModel != None)
		return returnModel;
	else
		return myModel;
}

simulated function ShowDriver()
{
	local Material BodySkin;

	if((Driver == None 
		&& !bAIControlled 
		&& myDriver != None)
		|| !bCanSeeDriver)
		return;
	if(!bAIControlled)
	{
		BodySkin = Driver.Skins[0];
		myDriver = spawn(class'DriverActor', self,,Location + (DriverPos >> Rotation),Rotation,BodySkin);
		myDriver.SetCollision(false, false, false);
		myDriver.SetBase(self);
		myDriver.MyVehicle = self;
		myDriver.BodySkin = Driver.Skins[0];
		myDriver.DrivingAnims = DrivingAnims;
		myDriver.SetDrawScale(myDriverSize);
		myDriver.AmbientGlow = Driver.AmbientGlow;
	}
	else
	{
		BodySkin = AIDriverTex;
		myDriver = spawn(class'DriverActor', self,,Location + (DriverPos >> Rotation),Rotation,BodySkin);
		myDriver.SetCollision(false, false, false);
		myDriver.SetBase(self);
		myDriver.MyVehicle = self;
		myDriver.BodyModel = AIDiverMesh;
		myDriver.BodySkin = AIDriverTex;
		myDriver.HeadModel = AIDriverHeadMesh;
		myDriver.HeadSkin = AIDriverHeadTex;
		myDriver.HeadScale = 1.0;
		myDriver.HeadClass = class'BasePeople.Head';
		myDriver.DrivingAnims = DrivingAnims;
		myDriver.SetDrawScale(myDriverSize);
		myDriver.AmbientGlow = 0;
	}
}

simulated function UpdateDriverActors()
{
	local vector DriverLoc;
	local vector PassengerLoc;

	DriverLoc = Location + (DriverPos >> Rotation);
	PassengerLoc = Location + (PassengerPos >> Rotation);

	if(myDriver != None)
	{
		myDriver.SetLocation(DriverLoc);
		myDriver.SetBase(self);
		myDriver.SetRotation(Rotation);
	}

	if(myPassenger != None)
	{
		myPassenger.SetLocation(PassengerLoc);
		myPassenger.SetBase(self);
		myPassenger.SetRotation(Rotation);
	}
}

simulated function SpawnLights()
{
	local int i;

	if( !bAllowLight )
		return;

	if(Level.NetMode != NM_Client)
	{
		if(DecoSystem == DS_Vector)
		{
			if( FrontLightClass != None )
			{
				for(i = 0; i < FrontLightPos.Length; i++)
				{
					FrontLight.Length = FrontLightPos.Length;
					if(VSize(FrontLightPos[i]) != 0 
						&& FrontLightClass != None
						&& FrontLight[i] == None)
					{
						FrontLight[i] = spawn(FrontLightClass, self,, Location + (FrontLightPos[i] >> Rotation));
						FrontLight[i].Skins[0] = FrontLightMaterial;
						FrontLight[i].SetDrawScale(FrontLightScale);
						FrontLight[i].SetBase(self);
					}
				}
			}
			if( RearLightClass != None )
			{
				for(i = 0; i < RearLightPos.Length; i++)
				{
					RearLight.Length = RearLightPos.Length;
					if(VSize(RearLightPos[i]) != 0 
						&& RearLightClass != None
						&& RearLight[i] == None)
					{
						RearLight[i] = spawn(RearLightClass, self,, Location + (RearLightPos[i] >> Rotation));
						RearLight[i].Skins[0] = RearLightMaterial;
						RearLight[i].SetDrawScale(RearLightScale);
						RearLight[i].SetBase(self);
					}
				}
			}
		}
		else if(DecoSystem == DS_Bones)
		{
			if( FrontLightClass != None )
			{
				for(i = 0; i < FrontLightBones.Length; i++)
				{
					FrontLight.Length = FrontLightBones.Length;
					if(FrontLightBones[i] != '' 
						&& FrontLightClass != None
						&& FrontLight[i] == None)
					{
						FrontLight[i] = spawn(FrontLightClass, self,, Location);
						FrontLight[i].Skins[0] = FrontLightMaterial;
						FrontLight[i].SetDrawScale(FrontLightScale);
						AttachToBone(FrontLight[i], FrontLightBones[i]);
					}
				}
			}
			if( RearLightClass != None )
			{
				for(i = 0; i < RearLightBones.Length; i++)
				{
					RearLight.Length = RearLightBones.Length;
					if(RearLightBones[i] != '' 
						&& RearLightClass != None
						&& RearLight[i] == None)
					{
						RearLight[i] = spawn(RearLightClass, self,, Location);
						RearLight[i].Skins[0] = RearLightMaterial;
						RearLight[i].SetDrawScale(RearLightScale);
						AttachToBone(RearLight[i], RearLightBones[i]);
					}
				}
			}
		}
	}
	bLightsOn = true;
}

simulated function DestroyLights()
{
	local int i, n;

	if( FrontLightClass != None )
	{
		if(DecoSystem == DS_Bones)
			n = FrontLightBones.Length;
		else n = FrontLightPos.Length;
		for( i = 0 ; i < n ; i++ )
		{
			FrontLight.Length = n;
			if(FrontLight[i] != None)
			{
				if(DecoSystem == DS_Bones)
					DetachFromBone(FrontLight[i]);
				FrontLight[i].Destroy();
				FrontLight[i] = None;
			}
		}
		bLightsOn = false;
	}

	if( RearLightClass != None )
	{
		if(DecoSystem == DS_Bones)
			n = RearLightBones.Length;
		else n = RearLightPos.Length;
		for( i = 0 ; i < n ; i++ )
		{
			RearLight.Length = n;
			if(RearLight[i] != None)
			{
				if(DecoSystem == DS_Bones)
					DetachFromBone(RearLight[i]);
				RearLight[i].Destroy();
				RearLight[i] = None;
			}
		}
	}
}

simulated function SpawnMyShadow()
{
	if( bDisableShadows )
		return;

	if( bAllowDynamicShadows && Level.NetMode != NM_DedicatedServer )
	{
		if( myShadow == None && ShadowClass != None )
		{
			myShadow = spawn(ShadowClass, self,, Location, ShadowRotation);
			myShadow.ShadowActor = self;
			myShadow.ShadowZ = ShadowZ;
			myShadow.MyBase = None;
			myShadow.MyCar = self;
			myShadow.MyVeh = None;
			myShadow.SetCollision(true, false, false);
			myShadow.LightDirection = Normal(ShadowLightDirection);
			myShadow.LightDistance = ShadowLightDistance;
			myShadow.MaxTraceDistance = ShadowMaxTraceDistance;
			myShadow.InitShadow();
		}
	}
}

simulated function ShowPassenger()
{
	local Material BodySkin;
	local Mesh Body;
	if((Passenger == None 
		&& myPassenger != None)
		|| !bCanSeeDriver)
		return;
	BodySkin = Passenger.Skins[0];
	myPassenger = spawn(class'PassengerActor', self,,Location + (PassengerPos >> Rotation),Rotation,BodySkin);
	myPassenger.SetCollision(false, false, false);
	myPassenger.SetBase(self);
	myPassenger.MyVehicle = self;
	myPassenger.BodySkin = Passenger.Skins[0];
	myPassenger.DrivingAnims = DrivingAnims;
	myPassenger.SetDrawScale(myDriverSize);
	myPassenger.AmbientGlow = Passenger.AmbientGlow;
}

function VehicleHaveNoPassenger()
{
	local Pawn OldPassenger;
	OldPassenger = Passenger;
	Passenger = None;

	if( DriverBase != None )
		DriverBase.AmbientSound = None;

	if(myPassenger != None)
	{
		if( MpPawn(OldPassenger) != None && MpPawn(OldPassenger).myHead != None )
		{
			myPassenger.DetachFromBone(MpPawn(OldPassenger).myHead);
			MpPawn(OldPassenger).AttachToBone(MpPawn(OldPassenger).myHead, 'MALE01 head');
			MpPawn(OldPassenger).myHead.SetOwner(OldPassenger);
		}
		myPassenger.Destroy();
		myPassenger = None;
	}
	if(ExitTriggerPassenger != None)
	{
		ExitTriggerPassenger.Destroy();
		ExitTriggerPassenger = None;
	}

}

simulated function EvilHavePassenger()
{
	local vector RotX, RotY, RotZ;
    GetAxes(Rotation,RotX,RotY,RotZ);
	if(ExitTriggerPassenger == None)
		ExitTriggerPassenger = spawn(class'NKVehicleTrigger', self,, Location + PassengerPos.X * RotX + PassengerPos.Y * RotY + PassengerPos.Z * RotZ);
	ExitTriggerPassenger.SetBase(self);
	ExitTriggerPassenger.myUser = Passenger;
	ExitTriggerPassenger.bExitTrigger = true;
	ExitTriggerPassenger.MyVehicle = self;
	ExitTriggerPassenger.SetCollision(true, false, false);
	ExitTriggerPassenger.SetCollisionSize(8.0,8.0);
}

simulated function Destroyed()
{
	local int i,n;

	DestroyMyShadow();

	if( SteerWheelRotator != None )
		SteerWheelRotator.Destroy();

	if(Driver != None)
	{
		if(Driver.Controller != None)
			TryToExit(Driver);
		else Driver.Destroy();
	}
	if(Passenger != None)
	{
		if(Passenger.Controller != None)
			TryToExit(Passenger);
		else Passenger.Destroy();
	}
	if(DriverBase != None)
	{
		DriverBase.AmbientSound = None;
		DriverBase.Destroy();
	}
	if(PassengerBase != None)
		PassengerBase.Destroy();
	if(FPDriverViewPoint != None)
		FPDriverViewPoint.Destroy();
	if(FPPassengerViewPoint != None)
		FPPassengerViewPoint.Destroy();
	if(IntCollision != None)
	{
		IntCollision.AmbientSound = None;
		IntCollision.Destroy();
	}
	if(OurFactory != None)
		OurFactory.VehicleDestroyed();
	if(myDriver != None)
		myDriver.Destroy();
	if(myPassenger != None)
		myPassenger.Destroy();
	if(myShadow != None)
		myShadow.Destroy();
	if(EnterRightTrigger != None)
		EnterRightTrigger.Destroy();
	if(EnterLeftTrigger != None)
		EnterLeftTrigger.Destroy();
	if(ExitTriggerDriver != None)
		ExitTriggerDriver.Destroy();
	if(ExitTriggerPassenger != None)
		ExitTriggerPassenger.Destroy();
	if(EngTireRight != None)
	{
		EngTireRight.WheelJoint.Destroy();
		EngTireRight.Destroy();
	}
	if(EngTireLeft != None)
	{
		EngTireLeft.WheelJoint.Destroy();
		EngTireLeft.Destroy();
	}
	if(SteerTireRight != None)
	{
		SteerTireRight.WheelJoint.Destroy();
		SteerTireRight.Destroy();
	}
	if(SteerTireLeft != None)
	{
		SteerTireLeft.WheelJoint.Destroy();
		SteerTireLeft.Destroy();
	}
	if(CrossTireRight != None)
	{
		CrossTireRight.WheelJoint.Destroy();
		CrossTireRight.Destroy();
	}
	if(CrossTireLeft != None)
	{
		CrossTireLeft.WheelJoint.Destroy();
		CrossTireLeft.Destroy();
	}
	if(SteerWIRight != None)
		SteerWIRight.Destroy();
	if(SteerWILeft != None)
		SteerWILeft.Destroy();
	if(EngWIRight != None)
		EngWIRight.Destroy();
	if(EngWILeft != None)
		EngWILeft.Destroy();
	if(CrossWIRight != None)
		CrossWIRight.Destroy();
	if(CrossWILeft != None)
		CrossWILeft.Destroy();
	DestroyLights();
	if( ExhaustClass != None )
	{
		if(DecoSystem == DS_Bones)
			n = ExhaustBones.Length;
		else n = ExhaustPos.Length;
		for( i = 0 ; i < n ; i++ )
		{
			CarExhaust.Length = n;
			if(CarExhaust[i] != None)
			{
				if(DecoSystem == DS_Bones)
					DetachFromBone(CarExhaust[i]);
				CarExhaust[i].Destroy();
			}
		}
	}
	if(NVFire != None)
		NVFire.Destroy();
	if(VehicleSmoke != None)
		VehicleSmoke.Destroy();
	if(NVEngFire != None)
		NVEngFire.Destroy();
	AmbientSound = None;
	Super.Destroyed();
}

simulated function SetSkin()
{
	local int i, b;
	if(!VehicleDestroyed())
	{
		if(bRandomSkin)
		{
			b = OtherSkins.Length - ((FRand() * (OtherSkins.Length/2) + FRand()) * (FRand() + 1));
			//for(i = 0; i < b; i++){}
			i = b;
			if( i < 0 )
				i = -i;
			else if(i > OtherSkins.Length)
				i = OtherSkins.Length;
			if(OtherSkins[i] != None)
				CurrentSkin = OtherSkins[i];
			else CurrentSkin = None;
			Skins[SkinIndex] = CurrentSkin;
		}
		else if(Skins[SkinIndex] != None)
			CurrentSkin = Skins[SkinIndex];
	}
	else
	{
		for(i = 0; i < BrokenSkins.Length; i++)
		{
			Skins[i] = BrokenSkins[i];
		}
		CurrentSkin = BrokenSkins[SkinIndex];
	}
}

simulated function SearchWeatherController()
{
	local NKVehicleWeatherSurfaceController WeatherController;
	foreach AllActors(class'NKVehicleWeatherSurfaceController',WeatherController)
	{
		if( WeatherController != None )
		{
			WeatherController.TakeSurfaceType(self);
			break;
		}
	}
}

simulated function SetFactoryVariables()
{
	if( Owner != None && NKVehicleFactory(Owner) != None )
		OurFactory = NKVehicleFactory(Owner);

	if( OurFactory == None )
		return;

	if( OurFactory.MapSurfaceType == MST_Default )
		MapSurfaceType = MST_Default;
	else if( OurFactory.MapSurfaceType == MST_Grass )
		MapSurfaceType = MST_Grass;
	else if( OurFactory.MapSurfaceType == MST_Dirt )
		MapSurfaceType = MST_Dirt;
	else if( OurFactory.MapSurfaceType == MST_Snow )
		MapSurfaceType = MST_Snow;
	else if( OurFactory.MapSurfaceType == MST_Sand )
		MapSurfaceType = MST_Sand;
	else if( OurFactory.MapSurfaceType == MST_Ice )
		MapSurfaceType = MST_Ice;

	if( !OurFactory.bAllowProjLight || !bAllowProjectionLight )
		LightProjectorClass = None;
	else LightProjectorClass = OurFactory.LightProjectorClass;

	TorqueCurve = OurFactory.TorqueCurve;

	CarSpeedScale = OurFactory.CarSpeedScale;
	CarGripScale = OurFactory.CarGripScale;
	CarSteeringScale = OurFactory.CarSteeringScale;
	CarWeightScale = OurFactory.CarWeightScale;
	CarSuspensionScale = OurFactory.CarSuspensionScale;

	if(OurFactory.RanOverDamage != None)
		RanOverDamage = OurFactory.RanOverDamage;
}

simulated function UpdateFactoryVariables()
{
	if(GameIsSingle() && bAIControlled)
		ShowDriver();
}

function VehicleGetPhysics()
{
	if(!GameIsEnd())
	{
		if(Level.NetMode != NM_Client)
		{
			if(Physics != PHYS_Karma)
				SetPhysics(PHYS_Karma);
			KWake();
		}
	}
	else
	{
		if(Physics == PHYS_Karma)
			KDisableKarma();
		if(Physics != PHYS_None)
			SetPhysics(PHYS_None);
	}
}

simulated function CheckOutSongsList()
{
	local int i, j;

	if( !bUseCustomRadioSongs )
	{
		RadioSong.Length = 0;
		for( i = 0 ; i < defaultRadioSong.Length ; i++ )
		{
			if( defaultRadioSong[i] != None )
			{
				j = RadioSong.Length;
				RadioSong.Insert(RadioSong.Length, 1.0);
				RadioSong[j] = defaultRadioSong[i];
			}
		}
		return;
	}

	for( i = 0 ; i < RadioSong.Length ; i++ )
	{
		if( RadioSong[i] == None )
			RadioSong.Remove(i, 1);
	}
}

event PostLoadGame()
{
	Super.PostLoadGame();
	SpawnMyShadow();
	CheckOutSongsList();
	SwitchMessageType();
}

simulated function PostBeginPlay()
{
	local vector RotX, RotY, RotZ, lPos;

	SwitchMessageType();

	SetFactoryVariables();

	SearchWeatherController();

	VehicleGetPhysics();

	SpawnMyShadow();

	if(MeshRenderLevel == MRL_Low)
	{
		if(bUpdateSusp)
			bUpdateSusp = false;
	}

	CheckOutSongsList();

    GetAxes(Rotation,RotX,RotY,RotZ);

	SteerWheelRotator = spawn(class'MotionRotatorActor');

	if(Level.NetMode != NM_Client)
	{
		if(IntCollisionClass != None)
		{
			IntCollision = spawn(IntCollisionClass, self,, Location, Rotation);
			if( IntCollision != None )
			{
				IntCollision.SoundVolume = EngineVolume*0.8;
				IntCollision.SoundRadius = SoundRadius;
				IntCollision.TransientSoundVolume = TransientSoundVolume;
				IntCollision.TransientSoundRadius = TransientSoundRadius;
				IntCollision.SetRotation(Rotation);
				IntCollision.SetBase(self);
				IntCollision.SetStaticMesh(IntCollisionModel);
				IntCollision.SetDrawScale3D(IntCollisionSize);
				IntCollision.myVehicle = self;
				IntCollision.MinROSpeed = MinRunOverSpeed;
				IntCollision.RODamage = RanOverDamage;
				IntCollision.ROSound = RanOverSound;
			}
		}
		if(!bAIControlled)
		{
			DriverBase = spawn(class'DriverBase', self,, Location + (DriverPos >> Rotation));
			DriverBase.myVehicle = self;
			DriverBase.LocatedPos = DriverPos;
			DriverBase.SoundVolume = MusicVolume;
			DriverBase.SoundRadius = MusicRadius;
			PassengerBase = spawn(class'DriverBase', self,, Location + (PassengerPos >> Rotation));
			PassengerBase.myVehicle = self;
			PassengerBase.LocatedPos = PassengerPos;
		}
		//if(bAllowChangeView)//26-09-2015
		//{
			if(FPViewPointOwner == None)
				FPViewPointOwner = self;
			if(FPDriverViewPoint == None)
			{
				FPDriverViewPoint = spawn(class'ViewPoint', FPViewPointOwner,, Location + (FPDriverViewPos >> Rotation));
				FPDriverViewPoint.SetBase(FPViewPointOwner);
				FPDriverViewPoint.myVehicle = self;
			}
			if(FPPassengerViewPoint == None && bAllowPassenger)
			{
				FPPassengerViewPoint = spawn(class'ViewPoint', FPViewPointOwner,, Location + (FPPassengerViewPos >> Rotation));
				FPPassengerViewPoint.SetBase(FPViewPointOwner);
				FPPassengerViewPoint.myVehicle = self;
			}
		//}
		EnterRightTrigger = spawn(class'NKVehicleTrigger', self,, Location + (EnterPos >> Rotation));
		EnterRightTrigger.SetBase(self);
		EnterRightTrigger.SetCollision(true, false, false);
		EnterRightTrigger.MyVehicle = self;
		EnterLeftTrigger = spawn(class'NKVehicleTrigger', self,, Location + EnterPos.X * RotX - EnterPos.Y * RotY + EnterPos.Z * RotZ);
		EnterLeftTrigger.SetBase(self);
		EnterLeftTrigger.SetCollision(true, false, false);
		EnterLeftTrigger.MyVehicle = self;
		EngTireRight = spawn(EngTireClass, self,, Location + WheelEngAlong*RotX + WheelEngAcross*RotY + WheelVert*RotZ, Rotation);
		EngTireRight.Chassis = self;
		EngTireLeft = spawn(EngTireClass, self,, Location + WheelEngAlong*RotX - WheelEngAcross*RotY + WheelVert*RotZ, Rotation);
		EngTireLeft.Chassis = self;
		SteerTireRight = spawn(SteerTireClass, self,, Location + WheelSteerAlong*RotX + WheelSteerAcross*RotY + WheelVert*RotZ, Rotation);
		SteerTireRight.Chassis = self;
		SteerTireLeft = spawn(SteerTireClass, self,, Location + WheelSteerAlong*RotX - WheelSteerAcross*RotY + WheelVert*RotZ, Rotation);
		SteerTireLeft.Chassis = self;
		if( CrossTireClass != None )
		{
			CrossTireRight = spawn(CrossTireClass, self,, Location + WheelCrossAlong*RotX + WheelCrossAcross*RotY + WheelVert*RotZ, Rotation);
			CrossTireRight.Chassis = self;
			CrossTireLeft = spawn(CrossTireClass, self,, Location + WheelCrossAlong*RotX - WheelCrossAcross*RotY + WheelVert*RotZ, Rotation);
			CrossTireLeft.Chassis = self;
		}
		if( WheelIndicatorClass == None )
			WheelIndicatorClass = class'WheelIndicator';
		EngWIRight = spawn(WheelIndicatorClass, self,, Location + WheelEngAlong*RotX + WheelEngAcross*RotY + WheelVert*RotZ, Rotation);
		EngWIRight.SlipVolume = SlipVolume;
		EngWIRight.SetBase(self);
		EngWILeft = spawn(WheelIndicatorClass, self,, Location + WheelEngAlong*RotX - WheelEngAcross*RotY + WheelVert*RotZ, Rotation);
		EngWILeft.SlipVolume = SlipVolume;
		EngWILeft.SetBase(self);
		SteerWIRight = spawn(WheelIndicatorClass, self,, Location + WheelSteerAlong*RotX + WheelSteerAcross*RotY + WheelVert*RotZ, Rotation);
		SteerWIRight.SlipVolume = SlipVolume;
		SteerWIRight.SetBase(self);
		SteerWILeft = spawn(WheelIndicatorClass, self,, Location + WheelSteerAlong*RotX - WheelSteerAcross*RotY + WheelVert*RotZ, Rotation);
		SteerWILeft.SlipVolume = SlipVolume;
		SteerWILeft.SetBase(self);
		if( CrossTireClass != None )
		{
			CrossWIRight = spawn(WheelIndicatorClass, self,, Location + WheelCrossAlong*RotX + WheelCrossAcross*RotY + WheelVert*RotZ, Rotation);
			CrossWIRight.SlipVolume = SlipVolume;
			CrossWIRight.SetBase(self);
			CrossWILeft = spawn(WheelIndicatorClass, self,, Location + WheelCrossAlong*RotX - WheelCrossAcross*RotY + WheelVert*RotZ, Rotation);
			CrossWILeft.SlipVolume = SlipVolume;
			CrossWILeft.SetBase(self);
		}
	}
	else
	{
		if( IntCollision != None )
		{
			IntCollision.SetLocation(Location);
			IntCollision.SetRotation(Rotation);
		}
		KWake();
		SetPhysics(PHYS_Projectile);
		SetCollision(false,false,false);
		SetRotation(ServerRotation);
		bProjTarget = false;
		bCollideWorld = false;
		bBlockActors = false;
		Mass = 0.0;
	}

	lPos.X = WheelEngAlong;
	lPos.Y = WheelEngAcross;
	lPos.Z = WheelVert;
	if( EngTireRight != None )
	{
	EngTireRight.WheelJoint = spawn(class'NKVWheelJoint', self);
	EngTireRight.WheelJoint.KPos1 = lPos/50;
	EngTireRight.WheelJoint.KPriAxis1 = vect(0, 0, 1);
	EngTireRight.WheelJoint.KSecAxis1 = vect(0, 1, 0);
	EngTireRight.WheelJoint.KConstraintActor1 = self;
	EngTireRight.WheelJoint.KPos2 = vect(0, 0, 0);
	EngTireRight.WheelJoint.KPriAxis2 = vect(0, 0, 1);
	EngTireRight.WheelJoint.KSecAxis2 = vect(0, 1, 0);
	EngTireRight.WheelJoint.KConstraintActor2 = EngTireRight;
	EngTireRight.WheelJoint.SetPhysics(PHYS_Karma);
	}
	lPos.Y = -WheelEngAcross;
	if( EngTireLeft != None )
	{
    EngTireLeft.WheelJoint = spawn(class'NKVWheelJoint', self);
    EngTireLeft.WheelJoint.KPos1 = lPos/50;
    EngTireLeft.WheelJoint.KPriAxis1 = vect(0, 0, 1);
    EngTireLeft.WheelJoint.KSecAxis1 = vect(0, 1, 0);
    EngTireLeft.WheelJoint.KConstraintActor1 = self;
    EngTireLeft.WheelJoint.KPos2 = vect(0, 0, 0);
    EngTireLeft.WheelJoint.KPriAxis2 = vect(0, 0, 1);
    EngTireLeft.WheelJoint.KSecAxis2 = vect(0, 1, 0);
    EngTireLeft.WheelJoint.KConstraintActor2 = EngTireLeft;
    EngTireLeft.WheelJoint.SetPhysics(PHYS_Karma);
	}
	lPos.X = WheelSteerAlong;
	lPos.Y = WheelSteerAcross;
	lPos.Z = WheelVert;
	if( SteerTireRight != None )
	{
    SteerTireRight.WheelJoint = spawn(class'NKVWheelJoint', self);
    SteerTireRight.WheelJoint.KPos1 = lPos/50;
    SteerTireRight.WheelJoint.KPriAxis1 = vect(0, 0, 1);
    SteerTireRight.WheelJoint.KSecAxis1 = vect(0, 1, 0);
    SteerTireRight.WheelJoint.KConstraintActor1 = self;
    SteerTireRight.WheelJoint.KPos2 = vect(0, 0, 0);
    SteerTireRight.WheelJoint.KPriAxis2 = vect(0, 0, 1);
    SteerTireRight.WheelJoint.KSecAxis2 = vect(0, 1, 0);
    SteerTireRight.WheelJoint.KConstraintActor2 = SteerTireRight;
    SteerTireRight.WheelJoint.SetPhysics(PHYS_Karma);
	}
    lPos.Y = -WheelSteerAcross;
	if( SteerTireLeft != None )
	{
	SteerTireLeft.WheelJoint = spawn(class'NKVWheelJoint', self);
    SteerTireLeft.WheelJoint.KPos1 = lPos/50;
    SteerTireLeft.WheelJoint.KPriAxis1 = vect(0, 0, 1);
    SteerTireLeft.WheelJoint.KSecAxis1 = vect(0, 1, 0);
    SteerTireLeft.WheelJoint.KConstraintActor1 = self;
    SteerTireLeft.WheelJoint.KPos2 = vect(0, 0, 0);
    SteerTireLeft.WheelJoint.KPriAxis2 = vect(0, 0, 1);
    SteerTireLeft.WheelJoint.KSecAxis2 = vect(0, 1, 0);
    SteerTireLeft.WheelJoint.KConstraintActor2 = SteerTireLeft;
    SteerTireLeft.WheelJoint.SetPhysics(PHYS_Karma);
	}
	if( CrossTireClass != None )
	{
		lPos.X = WheelCrossAlong;
		lPos.Y = WheelCrossAcross;
		lPos.Z = WheelVert;
		if( CrossTireRight != None )
		{
		CrossTireRight.WheelJoint = spawn(class'NKVWheelJoint', self);
		CrossTireRight.WheelJoint.KPos1 = lPos/50;
		CrossTireRight.WheelJoint.KPriAxis1 = vect(0, 0, 1);
		CrossTireRight.WheelJoint.KSecAxis1 = vect(0, 1, 0);
		CrossTireRight.WheelJoint.KConstraintActor1 = self;
		CrossTireRight.WheelJoint.KPos2 = vect(0, 0, 0);
		CrossTireRight.WheelJoint.KPriAxis2 = vect(0, 0, 1);
		CrossTireRight.WheelJoint.KSecAxis2 = vect(0, 1, 0);
		CrossTireRight.WheelJoint.KConstraintActor2 = CrossTireRight;
		CrossTireRight.WheelJoint.SetPhysics(PHYS_Karma);
		}
		lPos.Y = -WheelCrossAcross;
		if( CrossTireLeft != None )
		{
		CrossTireLeft.WheelJoint = spawn(class'NKVWheelJoint', self);
		CrossTireLeft.WheelJoint.KPos1 = lPos/50;
		CrossTireLeft.WheelJoint.KPriAxis1 = vect(0, 0, 1);
		CrossTireLeft.WheelJoint.KSecAxis1 = vect(0, 1, 0);
		CrossTireLeft.WheelJoint.KConstraintActor1 = self;
		CrossTireLeft.WheelJoint.KPos2 = vect(0, 0, 0);
		CrossTireLeft.WheelJoint.KPriAxis2 = vect(0, 0, 1);
		CrossTireLeft.WheelJoint.KSecAxis2 = vect(0, 1, 0);
		CrossTireLeft.WheelJoint.KConstraintActor2 = CrossTireLeft;
		CrossTireLeft.WheelJoint.SetPhysics(PHYS_Karma);
		}
	}

	SetSkin();
	VehicleUpdateParams();
}

event VehicleUpdateParams()
{
	local int i;

	if( CarSpeedScale != 1.0 )
	{
		for( i = 0 ; i < TorqueCurve.Points.Length ; i++ )
		{
			TorqueCurve.Points[i].InVal = default.TorqueCurve.Points[i].InVal + CarSpeedScale;
			TorqueCurve.Points[i].OutVal = default.TorqueCurve.Points[i].OutVal + CarSpeedScale;
		}
	}
	if( CarGripScale != 1.0 )
	{
		TireLateralFriction = FClamp(default.TireLateralFriction * CarGripScale,default.TireLateralFriction*0.9,10);
		TireRollFriction = default.TireRollFriction * CarGripScale;
	}
	if( CarSteeringScale != 1.0 )
	{
		SteerSpeed = default.SteerSpeed * CarSteeringScale;
		SteerTorque = default.SteerTorque * CarSteeringScale;
	}
	if( CarWeightScale != 1.0 )
	{
		ChassisMass = default.ChassisMass * CarWeightScale;
	}
	if( CarSuspensionScale != 1.0 )
	{
		SuspStiffness = default.SuspStiffness * CarSuspensionScale;
		SuspDamping = default.SuspDamping * CarSuspensionScale;
	}

    EngTireRight.WheelJoint.bKSteeringLocked = true;
    EngTireLeft.WheelJoint.bKSteeringLocked = true;
	if( CrossTireClass != None )
	{
		CrossTireRight.WheelJoint.bKSteeringLocked = true;
		CrossTireLeft.WheelJoint.bKSteeringLocked = true;
		CrossTireRight.WheelJoint.KSuspHighLimit = SuspHighLimit;
		CrossTireRight.WheelJoint.KSuspLowLimit = SuspLowLimit;
		CrossTireRight.WheelJoint.KSuspStiffness = SuspStiffness;
		CrossTireRight.WheelJoint.KSuspDamping = SuspDamping;
		CrossTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
		CrossTireLeft.WheelJoint.KSuspHighLimit = SuspHighLimit;
		CrossTireLeft.WheelJoint.KSuspLowLimit = SuspLowLimit;
		CrossTireLeft.WheelJoint.KSuspStiffness = SuspStiffness;
		CrossTireLeft.WheelJoint.KSuspDamping = SuspDamping;
		CrossTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
		CrossTireRight.WheelJoint.KUpdateConstraintParams();
		CrossTireLeft.WheelJoint.KUpdateConstraintParams();
		CrossTireRight.KSetMass(TireMass);
		CrossTireLeft.KSetMass(TireMass);
		CrossTireRight.RollFriction = TireRollFriction;
		CrossTireRight.LateralFriction = TireLateralFriction;
		CrossTireRight.RollSlip = TireRollSlip;
		CrossTireRight.LateralSlip = TireLateralSlip;
		CrossTireRight.MinSlip = TireMinSlip;
		CrossTireRight.SlipRate = TireSlipRate;
		CrossTireRight.Softness = TireSoftness;
		CrossTireRight.Adhesion = TireAdhesion;
		CrossTireRight.Restitution = TireRestitution;
		CrossTireRight.UnloadWheelSoftness = UnloadWheelHeight;
		CrossTireRight.Health = WheelsHealth;
		CrossTireRight.MaxHealth = WheelsMaxHealth;
		CrossTireLeft.RollFriction = TireRollFriction;
		CrossTireLeft.LateralFriction = TireLateralFriction;
		CrossTireLeft.RollSlip = TireRollSlip;
		CrossTireLeft.LateralSlip = TireLateralSlip;
		CrossTireLeft.MinSlip = TireMinSlip;
		CrossTireLeft.SlipRate = TireSlipRate;
		CrossTireLeft.Softness = TireSoftness;
		CrossTireLeft.Adhesion = TireAdhesion;
		CrossTireLeft.Restitution = TireRestitution;
		CrossTireLeft.UnloadWheelSoftness = UnloadWheelHeight;
		CrossTireLeft.Health = WheelsHealth;
		CrossTireLeft.MaxHealth = WheelsMaxHealth;
	}
    SteerTireRight.WheelJoint.bKSteeringLocked = false;
    SteerTireRight.WheelJoint.KProportionalGap = SteerPropGap;
    SteerTireRight.WheelJoint.KMaxSteerTorque = SteerTorque;
	SteerTireRight.WheelJoint.KMaxSteerSpeed = SteerSpeed;
	SteerTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
    SteerTireLeft.WheelJoint.bKSteeringLocked = false;
    SteerTireLeft.WheelJoint.KProportionalGap = SteerPropGap;
    SteerTireLeft.WheelJoint.KMaxSteerTorque = SteerTorque;
	SteerTireLeft.WheelJoint.KMaxSteerSpeed = SteerSpeed;
	SteerTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
    SteerTireRight.WheelJoint.KSuspHighLimit = SuspHighLimit;
    SteerTireRight.WheelJoint.KSuspLowLimit = SuspLowLimit;
    SteerTireRight.WheelJoint.KSuspStiffness = SuspStiffness;
    SteerTireRight.WheelJoint.KSuspDamping = SuspDamping;
    SteerTireLeft.WheelJoint.KSuspHighLimit = SuspHighLimit;
    SteerTireLeft.WheelJoint.KSuspLowLimit = SuspLowLimit;
    SteerTireLeft.WheelJoint.KSuspStiffness = SuspStiffness;
    SteerTireLeft.WheelJoint.KSuspDamping = SuspDamping;
    EngTireRight.WheelJoint.KSuspHighLimit = SuspHighLimit;
    EngTireRight.WheelJoint.KSuspLowLimit = SuspLowLimit;
    EngTireRight.WheelJoint.KSuspStiffness = SuspStiffness;
    EngTireRight.WheelJoint.KSuspDamping = SuspDamping;
	EngTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
    EngTireLeft.WheelJoint.KSuspHighLimit = SuspHighLimit;
    EngTireLeft.WheelJoint.KSuspLowLimit = SuspLowLimit;
    EngTireLeft.WheelJoint.KSuspStiffness = SuspStiffness;
    EngTireLeft.WheelJoint.KSuspDamping = SuspDamping;
	EngTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
	SteerTireRight.WheelJoint.KUpdateConstraintParams();
	SteerTireLeft.WheelJoint.KUpdateConstraintParams();
	EngTireRight.WheelJoint.KUpdateConstraintParams();
	EngTireLeft.WheelJoint.KUpdateConstraintParams();
    SteerTireRight.KSetMass(TireMass);
    SteerTireLeft.KSetMass(TireMass);
    EngTireRight.KSetMass(TireMass);
    EngTireLeft.KSetMass(TireMass);
    SteerTireRight.RollFriction = TireRollFriction;
    SteerTireRight.LateralFriction = TireLateralFriction;
    SteerTireRight.RollSlip = TireRollSlip;
    SteerTireRight.LateralSlip = TireLateralSlip;
    SteerTireRight.MinSlip = TireMinSlip;
    SteerTireRight.SlipRate = TireSlipRate;
    SteerTireRight.Softness = TireSoftness;
    SteerTireRight.Adhesion = TireAdhesion;
    SteerTireRight.Restitution = TireRestitution;
	SteerTireRight.UnloadWheelSoftness = UnloadWheelHeight;
	SteerTireRight.Health = WheelsHealth;
	SteerTireRight.MaxHealth = WheelsMaxHealth;
    SteerTireLeft.RollFriction = TireRollFriction;
    SteerTireLeft.LateralFriction = TireLateralFriction;
    SteerTireLeft.RollSlip = TireRollSlip;
    SteerTireLeft.LateralSlip = TireLateralSlip;
    SteerTireLeft.MinSlip = TireMinSlip;
    SteerTireLeft.SlipRate = TireSlipRate;
    SteerTireLeft.Softness = TireSoftness;
    SteerTireLeft.Adhesion = TireAdhesion;
    SteerTireLeft.Restitution = TireRestitution;
	SteerTireLeft.UnloadWheelSoftness = UnloadWheelHeight;
	SteerTireLeft.Health = WheelsHealth;
	SteerTireLeft.MaxHealth = WheelsMaxHealth;
    EngTireRight.RollFriction = TireRollFriction;
    EngTireRight.LateralFriction = TireLateralFriction;
    EngTireRight.RollSlip = TireRollSlip;
    EngTireRight.LateralSlip = TireLateralSlip;
    EngTireRight.MinSlip = TireMinSlip;
    EngTireRight.SlipRate = TireSlipRate;
    EngTireRight.Softness = TireSoftness;
    EngTireRight.Adhesion = TireAdhesion;
    EngTireRight.Restitution = TireRestitution;
	EngTireRight.UnloadWheelSoftness = UnloadWheelHeight;
	EngTireRight.Health = WheelsHealth;
	EngTireRight.MaxHealth = WheelsMaxHealth;
    EngTireLeft.RollFriction = TireRollFriction;
    EngTireLeft.LateralFriction = TireLateralFriction;
    EngTireLeft.RollSlip = TireRollSlip;
    EngTireLeft.LateralSlip = TireLateralSlip;
    EngTireLeft.MinSlip = TireMinSlip;
    EngTireLeft.SlipRate = TireSlipRate;
    EngTireLeft.Softness = TireSoftness;
    EngTireLeft.Adhesion = TireAdhesion;
    EngTireLeft.Restitution = TireRestitution;
	EngTireLeft.UnloadWheelSoftness = UnloadWheelHeight;
	EngTireLeft.Health = WheelsHealth;
	EngTireLeft.MaxHealth = WheelsMaxHealth;

    KSetMass(ChassisMass);
	KSetImpactThreshold(ImpactThreshold);

	UpdateBrakes();

	if(!bAIControlled)
	{
		if(Driver == None)
			GotoState('VehicleIdling');
	}
	else
	{
		if(Enemy == None)
			GotoState('VehicleIdling');
		else StartScriptedDriving(Enemy);
	}
}

function Bump(Actor Other)
{
	//STUB
}

function DamagedCarEngine()
{
	local vector RotX, RotY, RotZ, ShootMomentum, ExpMoment;

    GetAxes(Rotation,RotX,RotY,RotZ);

	if( Health <= 600 )
	{
		if(VehicleSmoke == None)
		{
			VehicleSmoke = spawn(class'VehicleSmokeEmitter', self,, Location + RadiatorPos.X * RotX + RadiatorPos.Y * RotY + RadiatorPos.Z * RotZ);
			VehicleSmoke.SetBase(self);
			EnginePitch = EnginePitch - 5;
		}
	}
	if( Health <= 250 )
	{
		if(NVEngFire == None)
		{
			NVEngFire = spawn(class'VehicleEngineFireEmitter', self,, Location + RadiatorPos.X * RotX + RadiatorPos.Y * RotY + RadiatorPos.Z * RotZ);
			NVEngFire.SetBase(self);
		}
	}
}

function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, class<DamageType> damageType)
{
	local int actualDamage, N, nForce;
	local Controller Killer;
	local vector ShootMomentum, ExpMoment;

	//Driver.ClientMessage(self$" damaged "$Damage);

	if(Health > 0)
	{
		if(Driver != None && !bDriverTakesDamage)
		{
			if(Driver.Health <= 0)
			{
				DriverForceLeave();
				VehicleHaveNoDriver();
			}
		}
		if(Passenger != None)
		{
			if(Passenger.Health <= 0)
			{
				PassengerForceLeave();
				VehicleHaveNoPassenger();
			}
		}
	}

	if(bAIControlled && Enemy == None)
		StartScriptedDriving(instigatedBy);

	if( bImmortal || damageType == class'BaseFX.BurnedDamage'
		|| damageType == class'BaseFX.SmashDamage'
		|| damageType == class'BaseFX.OnFireDamage'
		|| damageType == class'BaseFX.KickingDamage'
		|| damageType == class'BaseFX.BludgeonDamage'
		|| damageType == class'NVExplodeCarDamage'
		|| VehicleDestroyed() )
		return;

	if(ClassIsChildOf(damageType, class'BaseFX.ExplodedDamage'))
	{
		//momentum = Velocity * 4 * Mass;
		KAddImpulse( momentum , hitlocation );
		Health -= damage * 5;
	}

	if(ClassIsChildOf(damageType, class'NVCarDamage') && !VehicleDestroyed())
		Health -= damage;

	if(ClassIsChildOf(damageType, class'Engine.Crushed'))
		Health -= damage;

	if(ClassIsChildOf(damageType, class'WaterDamage'))
		Health -= Health;

	if(ClassIsChildOf(damageType, class'BaseFX.ChemDamage'))
		Health -= damage / 2;

	if(ClassIsChildOf(damageType, class'BaseFX.BulletDamage'))
	{
		if(Driver != None && instigatedBy == Driver)
			return;
		else Health -= damage;
	}
	if(damageType == class'BaseFX.ElectricalDamage'
		|| damageType == class'NVRepairDamage')
	{
		if(Health < MaxHealth)
			Health += 25;
		if(Health >= 600)
		{
			if(VehicleSmoke != None)
			{
				VehicleSmoke.Destroy();
				VehicleSmoke = None;
				EnginePitch = EnginePitch + 5;
			}
		}
		if(Health >= 250)
		{
			if(NVEngFire != None)
			{
				NVEngFire.Destroy();
				NVEngFire = None;
			}
		}
	}
	if(ClassIsChildOf(damageType, class'TankProjectileDamage'))
	{
		N = Rand(1);
		if(N == 0)
			nForce = 128;
		else
			nForce = -128;
		Health -= damage;
		ExpMoment.Z = 256;
		ExpMoment.X = FRand() * nForce;
		N = Rand(1);
		if(N == 0)
			nForce = 128;
		else
			nForce = -128;
		ExpMoment.Y = FRand() * nForce;
		ExpMoment = ExpMoment + (Velocity / 2);
		ExpMoment = ExpMoment * 1024;
		ExpMoment = ExpMoment >> rot(0,0,0);
		KAddImpulse(ExpMoment,HitLocation);
	}
	if(ClassIsChildOf(damageType, class'RocketExplosionDamage'))
	{
		N = Rand(1);
		if(N == 0)
			nForce = 128;
		else
			nForce = -128;
		Health -= damage;
		ExpMoment.Z = 128;
		ExpMoment.X = FRand() * nForce;
		N = Rand(1);
		if(N == 0)
			nForce = 128;
		else
			nForce = -128;
		ExpMoment.Y = FRand() * nForce;
		ExpMoment = ExpMoment + (Velocity / 2);
		ExpMoment = ExpMoment * 512;
		ExpMoment = ExpMoment >> rot(0,0,0);
		KAddImpulse(ExpMoment,HitLocation);
	}

	if( LevelTravelObject != None )
		LevelTravelObject.TravelCarHealth = Health;

	DamagedCarEngine();

	if(Health <= 0)
	{
		if(instigatedBy != None)
			Killer = instigatedBy.GetKillerController();

		if(Driver != None && Driver.Health > 0)
		{
			if( !bDriverTakesDamage )
			{
				DriverForceLeave();
				Driver.Controller.bGodMode = false;
				if(GameIsSingle())
					Driver.Died( Killer, class'InVehicleDamage', HitLocation );
				else
					Driver.Died( Killer, class'InVehicleDamageMP', HitLocation );//Gibbed
				VehicleHaveNoDriver(true);
			}
			else
			{
				DriverLeave(true);
			}
		}
		if(Passenger != None && Passenger.Health > 0)
		{
			PassengerForceLeave();
			Passenger.Controller.bGodMode = false;
			if(GameIsSingle())
				Passenger.Died( Killer, class'InVehicleDamage', HitLocation );
			else
				Passenger.Died( Killer, class'InVehicleDamageMP', HitLocation );
			VehicleHaveNoPassenger();
		}
		if( ClassIsChildOf(damageType, class'NKVehicleKillZWaterDamage') )
		{
			VehicleExplosionClass = class'UnderWaterExplosion';
		}
		if( bSpawnBrokenDetail && !ClassIsChildOf(damageType, class'NKVehicleKillZDamage') )
			GotoState('BeforeBeDestroyed');
		else GotoState('ForceDestroy');
	}
	else
	{
		if(VSize(momentum) < 0.001)
			return;

		if(IsInWater())
			ShootMomentum = Momentum / 32;
		else
			ShootMomentum = Momentum / 16;
		KAddImpulse(ShootMomentum,HitLocation);
	}
}

function UpdateRearLight()
{
	local int i,n;

	/*if(DecoSystem == DS_Vector)
		n = RearLightPos.Length;
	else if(DecoSystem == DS_Bones)
		n = RearLightBones.Length;wtf*/

	n = RearLight.Length;

	if( n <= 0 
		|| FrontLightClass == None 
		|| RearLightClass == None )
		return;

	if(Gear == 0 && OutputBrake && IsDriving)
	{
		for(i = 0;i < n;i++)
		{
			if(RearLight[i] != None)
			{
				RearLight[i].LightSaturation = 0;
				RearLight[i].SetDrawScale(RearLightStopScale);
			}
		}
	}
	else if(Gear == -1)
	{
		for(i= 0 ;i < n;i++)
		{
			if(RearLight[i] != None)
			{
				RearLight[i].LightSaturation = 255.0;
				RearLight[i].SetDrawScale(RearLightStopScale);
			}
		}
	}
	else
	{
		for(i = 0;i < n;i++)
		{
			if(RearLight[i] != None)
			{
				RearLight[i].LightSaturation = 0;
				RearLight[i].SetDrawScale(RearLightScale);
			}
		}
	}

}

function Exploding()
{
	if(!IsInWater())
	{
		if(VehicleExplosionClass != None)
		{
			DestroyedEffect = spawn(VehicleExplosionClass, self, , Location );
			DestroyedEffect.ShakeCamera(2048);
		}
		if(DestroyedSound != None)
			PlaySound(DestroyedSound, SLOT_Misc, DestroyedVolume);
	}
	else
	{
		DestroyedEffect = spawn(class'UnderWaterExplosion', self, , Location );
		DestroyedEffect.ShakeCamera(2048);
	}
}

simulated function PlayingBeepSound()
{
	if( BeepSound != None && IntCollision != None )
		IntCollision.PlaySound(BeepSound, SLOT_Misc, BeepVolume);
	bBeep = false;
}

simulated function MatchEnd()
{
	if( !GameIsEnd() )
		return;

	VehicleGetPhysics();

	if( DriverBase != None )
		DriverBase.SetHardAttach(true);

	if( PassengerBase != None )
		PassengerBase.SetHardAttach(true);

	if( Driver != None )
		Driver.bCollideWorld = true;

	if( Passenger != None )
		Passenger.bCollideWorld = true;

	if(SteerTireLeft != None)
		SteerTireLeft.DisablePhysics();

	if(SteerTireRight != None)
		SteerTireRight.DisablePhysics();

	if(EngTireLeft != None)
		EngTireLeft.DisablePhysics();

	if(EngTireRight != None)
		EngTireRight.DisablePhysics();

	if(CrossTireLeft != None)
		CrossTireLeft.DisablePhysics();

	if(CrossTireRight != None)
		CrossTireRight.DisablePhysics();
}

function UpdatePhysParameters()
{
	local float ExistFriction, AngularDamping;

	ExistFriction = KGetFriction();

	if( IsInverted() )
	{
		if(ExistFriction != ActionFriction)
			KSetFriction(ActionFriction);
		AngularDamping = ActionAngularDamp;
	}
	else
	{
		if(ExistFriction != RestFriction)
			KSetFriction(RestFriction);
		AngularDamping = RestAngularDamp;
	}

	if( Throttle == 0 )
		KSetDampingProps(RestLinearDamp, AngularDamping);
	else KSetDampingProps(ActionLinearDamp, AngularDamping);
}

simulated function CheckPawns()
{
	local Pawn P;

	if( Driver != None )
	{
		if( Driver.Health <= 0 )
		{
			if( bDriverTakesDamage && Health > 0 && IsInState('VehicleDriving') )
			{
				DriverForceLeave(true);
				Driver.Died( None, class'InVehicleDamage', Driver.Location );
				VehicleHaveNoDriver(true, true);
				return;
			}
		}
		else Driver.SetRotation(rot(0,0,0));
	}
	else if( IsInState('VehicleDriving') && !bAIControlled )
	{
		GotoState('VehicleIdling');
		VehicleHaveNoDriver();
	}

	if( Passenger == None && bCanSeeDriver && myPassenger != None )
		VehicleHaveNoPassenger();

	if( bAIControlled )
	{
		if( Enemy != None )
		{
			if( Pawn(Enemy) != None )
			{
				P = Pawn(Enemy);
				if( P.Health <= 0 )
					Enemy = None;
			}
			else Enemy = None;
		}
		else Enemy = None;
		if( Driver != None )
			TryToExit(Driver);
		if( Passenger != None )
			TryToExit(Passenger);
	}
	else CheckPawnsLocation();
}

simulated function GetVehicleData(optional bool bOnlyPositon)
{
	local vector Dir;
	local rotator ChassisRot;

	if(Role == ROLE_Authority)
		ChassisRot = Rotation;

	if(Level.NetMode != NM_Client)
	{
		ServerPosition = Location;
		ServerRotation = ChassisRot;
	}

	if(bOnlyPositon)
		return;

	if(Role == ROLE_Authority)
	{
		if( SteerTireRight != None )
			SteeringAngle = SteerTireRight.WheelJoint.KSteerAngle;
		else SteeringAngle = 0;

		if(SteerTireRight != None)
			Tire1RotT = SteerTireRight.Rotation;
		if(SteerTireLeft != None)
			Tire2RotT = SteerTireLeft.Rotation;
		if(EngTireRight != None)
			Tire3RotT = EngTireRight.Rotation;
		if(EngTireLeft != None)
			Tire4RotT = EngTireLeft.Rotation;
		if(CrossTireRight != None)
			Tire5RotT = CrossTireRight.Rotation;
		if(CrossTireLeft != None)
			Tire6RotT = CrossTireLeft.Rotation;
		if(!bUpdateSusp)
		{
			Dir = vector(Rotation);
			Tire1Susp = Dir;
			Tire2Susp = Dir;
			Tire3Susp = Dir;
			Tire4Susp = Dir;
			Tire5Susp = Dir;
			Tire6Susp = Dir;
		}
		else
		{
			if(SteerTireRight != None)
				Tire1Susp.Z = SteerTireRight.Location.Z - SteerWIRight.Location.Z;
			if(SteerTireLeft != None)
				Tire2Susp.Z = SteerTireLeft.Location.Z - SteerWILeft.Location.Z;
			if(EngTireRight != None)
				Tire3Susp.Z = EngTireRight.Location.Z - EngWIRight.Location.Z;
			if(EngTireLeft != None)
				Tire4Susp.Z = EngTireLeft.Location.Z - EngWILeft.Location.Z;

			if(Tire1Susp.Z > RSuspHighLimit)
				Tire1Susp.Z = RSuspHighLimit;
			else if(Tire1Susp.Z < RSuspLowLimit)
				Tire1Susp.Z = RSuspLowLimit;

			if(Tire2Susp.Z > RSuspHighLimit)
				Tire2Susp.Z = RSuspHighLimit;
			else if(Tire2Susp.Z < RSuspLowLimit)
				Tire2Susp.Z = RSuspLowLimit;

			if(Tire3Susp.Z > RSuspHighLimit)
				Tire3Susp.Z = RSuspHighLimit;
			else if(Tire3Susp.Z < RSuspLowLimit)
				Tire3Susp.Z = RSuspLowLimit;

			if(Tire4Susp.Z > RSuspHighLimit)
				Tire4Susp.Z = RSuspHighLimit;
			else if(Tire4Susp.Z < RSuspLowLimit)
				Tire4Susp.Z = RSuspLowLimit;

			if( CrossTireClass != None )
			{
				if(CrossTireRight != None)
					Tire5Susp.Z = CrossTireRight.Location.Z - CrossWIRight.Location.Z;
				if(CrossTireLeft != None)
					Tire6Susp.Z = CrossTireLeft.Location.Z - CrossWILeft.Location.Z;

				if(Tire5Susp.Z > RSuspHighLimit)
					Tire5Susp.Z = RSuspHighLimit;
				else if(Tire5Susp.Z < RSuspLowLimit)
					Tire5Susp.Z = RSuspLowLimit;

				if(Tire6Susp.Z > RSuspHighLimit)
					Tire6Susp.Z = RSuspHighLimit;
				else if(Tire6Susp.Z < RSuspLowLimit)
					Tire6Susp.Z = RSuspLowLimit;

				Strut5Roll.Roll = Tire5Susp.Z * 175;
				Strut6Roll.Roll = -Tire6Susp.Z * 175;
			}
			else
			{
				Tire5Susp = Tire3Susp;
				Tire6Susp = Tire4Susp;
				Strut5Roll = Strut3Roll;
				Strut6Roll = Strut4Roll;
			}
			Strut1Roll.Roll = Tire1Susp.Z * 175;
			Strut2Roll.Roll = -Tire2Susp.Z * 175;
			Strut3Roll.Roll = Tire3Susp.Z * 175;
			Strut4Roll.Roll = -Tire4Susp.Z * 175;
		}
	}
}

simulated function ApplyVehicleData()
{
	local int i;
	local vector Dir;

	if( Level.NetMode == NM_DedicatedServer )
		return;

	if( SteerWheelBone != '' && Driver != None )
	{
		if( DriverController != None )
			RotateSteeringWheel(DriverController.bBehindView);
		else if( bAIControlled )
			RotateSteeringWheel(true);
		else RotateSteeringWheel(false);
	}

	Dir = vector(Rotation);

	// WHEELS
	// 1
	if(!bSteerRightDetached)
	{
		if(WheelFrontRight != '')
			SetBoneDirection(WheelFrontRight,Tire1RotT,Tire1Susp,1);
	}
	else
	{
		SetBoneDirection( WheelFrontRight , rot(0,0,0) , Dir , 0 );
		SetBoneScale(0,0,WheelFrontRight);
	}
	// 2
	if(!bSteerLeftDetached)
	{
		if(WheelFrontLeft != '')
			SetBoneDirection(WheelFrontLeft,Tire2RotT,Tire2Susp,1);
	}
	else
	{
		SetBoneDirection( WheelFrontLeft , rot(0,0,0) , Dir , 0 );
		SetBoneScale(1,0,WheelFrontLeft);
	}
	// 3
	if(!bEngRightDetached)
	{
		if(WheelRearRight != '')
			SetBoneDirection(WheelRearRight,Tire3RotT,Tire3Susp,1);
	}
	else
	{
		SetBoneDirection( WheelRearRight , rot(0,0,0) , Dir , 0 );
		SetBoneScale(2,0,WheelRearRight);
	}
	// 4
	if(!bEngLeftDetached)
	{
		if(WheelRearLeft != '')
			SetBoneDirection(WheelRearLeft,Tire4RotT,Tire4Susp,1);
	}
	else
	{
		SetBoneDirection( WheelRearLeft , rot(0,0,0) , Dir , 0 );
		SetBoneScale(3,0,WheelRearLeft);
	}
	if( CrossTireClass != None )
	{
		// 5
		if(!bCrossRightDetached)
		{
			if(WheelCrossRight != '')
				SetBoneDirection(WheelCrossRight,Tire5RotT,Tire5Susp,1);
		}
		else
		{
			SetBoneDirection( WheelCrossRight , rot(0,0,0) , Dir , 0 );
			SetBoneScale(4,0,WheelCrossRight);
		}
		// 6
		if(!bCrossLeftDetached)
		{
			if(WheelCrossLeft != '')
				SetBoneDirection(WheelCrossLeft,Tire6RotT,Tire6Susp,1);
		}
		else
		{
			SetBoneDirection( WheelCrossLeft , rot(0,0,0) , Dir , 0 );
			SetBoneScale(5,0,WheelCrossLeft);
		}
	}
	// 7
	if(!bEngRightDetached)
	{
		if(WheelPlusRight != '')
			SetBoneDirection(WheelPlusRight,Tire3RotT,Tire3Susp,1);
	}
	else
	{
		SetBoneDirection( WheelPlusRight , rot(0,0,0) , Dir , 0 );
		SetBoneScale(6,0,WheelPlusRight);
	}
	// 8
	if(!bEngLeftDetached)
	{
		if(WheelPlusLeft != '')
			SetBoneDirection(WheelPlusLeft,Tire4RotT,Tire4Susp,1);
	}
	else
	{
		SetBoneDirection( WheelPlusLeft , rot(0,0,0) , Dir , 0 );
		SetBoneScale(7,0,WheelPlusLeft);
	}

	if(VehicleDestroyed() && !bBonesDestroyed)
	{
		for(i = 0; i < DestroyBones.Length; i++)
			SetVehicleBoneScale(DestroyBones[i],0);
		SetSkin();
		bBonesDestroyed = true;
	}

	if(!bUpdateSusp)
		return;

	//STRUTS
	// 1
	if(!bSteerRightDetached)
		if(StrutFrontRight != '')
			SetBoneRotation(StrutFrontRight,Strut1Roll,0,1);
	// 2
	if(!bSteerLeftDetached)
		if(StrutFrontLeft != '')
			SetBoneRotation(StrutFrontLeft,Strut2Roll,0,1);
	// 3
	if(!bEngRightDetached)
		if(StrutRearRight != '')
			SetBoneRotation(StrutRearRight,Strut3Roll,0,1);
	// 4
	if(!bEngLeftDetached)
		if(StrutRearLeft != '')
			SetBoneRotation(StrutRearLeft,Strut4Roll,0,1);
	// 5
	if(!bCrossRightDetached)
		if(StrutCrossRight != '')
			SetBoneRotation(StrutCrossRight,Strut5Roll,0,1);
	// 6
	if(!bCrossLeftDetached)
		if(StrutCrossLeft != '')
			SetBoneRotation(StrutCrossLeft,Strut6Roll,0,1);
	// 7
	if(!bEngRightDetached)
		if(StrutPlusRight != '')
			SetBoneRotation(StrutPlusRight,Strut3Roll,0,1);
	// 8
	if(!bEngLeftDetached)
		if(StrutPlusLeft != '')
			SetBoneRotation(StrutPlusLeft,Strut4Roll,0,1);
}

simulated function RotateSteeringWheel(bool bFixed)
{
	local rotator TempRotation;
	local int MaxSteerSpeed;

	if( !bFixed )
	{
		MaxSteerSpeed = SteerSpeed + 20000;
		if( SteerWheelRotator != None )
		{
			if( SteeringAngle > 0.1 )
			{
				SteerWheelRotator.DesiredRotation.Yaw = 10000;
				SteerWheelRotator.RotationRate.Yaw = MaxSteerSpeed;//45000;
			}
			else if( SteeringAngle < -0.1 )
			{
				SteerWheelRotator.DesiredRotation.Yaw = -10000;
				SteerWheelRotator.RotationRate.Yaw = -MaxSteerSpeed;//-45000;
			}
			else
			{
				if( Normalize(SteerWheelRotator.Rotation).Yaw > 0 )
					SteerWheelRotator.RotationRate.Yaw = -MaxSteerSpeed;//-45000;
				else if( Normalize(SteerWheelRotator.Rotation).Yaw < 0 )
					SteerWheelRotator.RotationRate.Yaw = MaxSteerSpeed;//45000;
				SteerWheelRotator.DesiredRotation.Yaw = 0;
			}

			TempRotation.Roll = SteerWheelRotator.Rotation.Yaw;
			SetBoneRotation(SteerWheelBone,TempRotation,0,1);
		}
	}
	else
	{
		SetBoneRotation(SteerWheelBone,rot(0,0,0),0,1);
	}
	
}

simulated function SetVehicleBoneScale(name BoneName, float BoneScale)
{
	if(BoneName != '')
		SetBoneScale(NewScaleFact,BoneScale,BoneName);
	NewScaleFact++;
}

simulated function UpdateWheelsDust()
{
	if( EngWIRight != None && EngTireRight != None )
		EngWIRight.UpdateDust(EngTireRight, DustSlipRate, DustSlipThresh);
	if( EngWILeft != None && EngTireLeft != None )
		EngWILeft.UpdateDust(EngTireLeft, DustSlipRate, DustSlipThresh);
	if( SteerWIRight != None && SteerTireRight != None )
		SteerWIRight.UpdateDust(SteerTireRight, DustSlipRate, DustSlipThresh);
	if( SteerWILeft != None && SteerTireLeft != None )
		SteerWILeft.UpdateDust(SteerTireLeft, DustSlipRate, DustSlipThresh);
	if( CrossWIRight != None && CrossTireRight != None )
		CrossWIRight.UpdateDust(CrossTireRight, DustSlipRate, DustSlipThresh);
	if( CrossWILeft != None && CrossTireLeft != None )
		CrossWILeft.UpdateDust(CrossTireLeft, DustSlipRate, DustSlipThresh);
}

simulated function NETUpdatePriority()
{
	if( Driver != None )
	{
		if( NetPriority != default.NetPriority )
			NetPriority = default.NetPriority;
	}
	else
	{
		if( NetPriority != 1.0 )
			NetPriority = 1.0;
	}
}

simulated function ContinuousUpdate()
{
	local DriverSceneWatchScript SceneWatchScript;
	local Controller oldPC;

	if(GameIsEnd())
	{
		MatchEnd();
		bHideHUD = true;
	}

	NETUpdatePriority();

	if( bPlayCrash )
	{
		PlayCrash();
		bPlayCrash = false;
	}

	UpdateHeadLights();

	UpdateWheelsDust();

	CheckPawns();

	if(Driver != None && Driver.Controller != None)
		DriverController = PlayerController(Driver.Controller);

	if( DriverController == None )
		DriverController = None;

	if( DriverController != None && DriverController.bInterpolating && !bDriverControllerLeft )
	{
		SceneWatchScript = spawn(class'DriverSceneWatchScript');
		SceneWatchScript.DriverPC = DriverController;
		SceneWatchScript.bWasGodMode = bDriverGodMode;
		DriverLeave();
		DriverController = None;
		bDriverControllerLeft = true;
		return;
	}

	if(Passenger != None && Passenger.Controller != None)
		PassengerController = PlayerController(Passenger.Controller);

	if( PassengerController == None )
		PassengerController = None;

	if( !bAIControlled )
	{
		/*if(Level.NetMode == NM_Client)
			UserInterfaceControl();
		else */if(Level.NetMode != NM_DedicatedServer)
			UserInterfaceControl();
	}

	if(Level.NetMode != NM_Client)
	{
		mySpeed = VSize(Velocity);
		UpdatePhysParameters();
	}

	if(CurrentSkin != None && Skins[SkinIndex] != CurrentSkin)
		Skins[SkinIndex] = CurrentSkin;

	if( IsGoingToUpdate() 
		|| Driver != None 
		|| Passenger != None 
		|| Enemy != None )
	{
		GetVehicleData(false);
		ApplyVehicleData();
		if(Level.NetMode == NM_Client)
			ClientSimulationPack(false);
	}
	else
	{
		GetVehicleData(true);
		if(Level.NetMode == NM_Client)
			ClientSimulationPack(true);
	}
}

function CheckPawnsLocation()
{
	if(Driver != None)
	{
		if(abs(VSize(Driver.Location - DriverBase.Location)) > 0.5)
		{
			Driver.SetLocation(DriverBase.Location);
			Driver.SetPhysics(PHYS_None);
			Driver.SetBase(DriverBase);
		}
	}
	if(Passenger != None)
	{
		if(abs(VSize(Passenger.Location - PassengerBase.Location)) > 0.5)
		{
			Passenger.SetLocation(PassengerBase.Location);
			Passenger.SetPhysics(PHYS_None);
			Passenger.SetBase(PassengerBase);
		}
	}
}

simulated function SwitchSong()
{
	///FIX2015
	//if( CurrentSong != None )
	//	AmbientSound = CurrentSong;
	//else AmbientSound = None;
	if( CurrentSong != None )
		DriverBase.AmbientSound = CurrentSong;
	else DriverBase.AmbientSound = None;
	///FIX2015

	if( LevelTravelObject != None )
		LevelTravelObject.TravelCarSongNum = RadioSongNumber;
}

simulated function float EngineSimulation(float Throttle)
{
	local float torque;

	torque = Abs(Throttle) * Gear * InterpCurveEval(TorqueCurve, WheelSpinSpeed);

	return torque;
}

state VehicleDriving
{
	simulated function Tick(float DeltaTime)
	{
		ContinuousUpdate();

		if( Driver == None && !bAIControlled )
		{
			GotoState('VehicleIdling');
			return;
		}

		if( !bAIControlled )
		{
			PlayerInput();
			MainSystem(mySpeed);
		}
		else ScriptedDriving();

		DrivingSimulation(mySpeed);
	}

	function PlayerInput()
	{
		if(Driver.Acceleration.X > 2048)
			Throttle = 1;
		else if(Driver.Acceleration.X < -2048)
			Throttle = -1;
		else
			Throttle = 0;

		if(Driver.Acceleration.Y < -2048)
			Steering = 1;
		else if(Driver.Acceleration.Y > 2048)
			Steering = -1;
		else
			Steering = 0;
	}

	function ScriptedDriving()
	{
		local float DistTopointer, FocusFactor, SignFactor;
		local bool bNeedToTurn;
		local vector PointerLocation, LeftPoint, RightPoint;
		local rotator RotToPointer;

		if(Enemy == None)
		{
			if(EnemyController == None)
			{
				Throttle = 0;
				Steering = 0;
				Gear = 0;
				OutputBrake = true;
				IsDriving = false;
				return;
			}
			else
			{
				Enemy = EnemyMonitor();
				return;
			}
		}

		PointerLocation = Enemy.Location;
		DistToPointer = VSize(PointerLocation - Location);
        RotToPointer = Rotator(PointerLocation - Location);
		FocusFactor = VSize(Normal(Vector(Rotation)) - Normal(Vector(RotToPointer)));

		LeftPoint = Location + (vect(0,4,0) >> Rotation);
		RightPoint = Location + (vect(0,-4,0) >> Rotation);
		if( VSize(PointerLocation - LeftPoint) < VSize(PointerLocation - RightPoint) )
			SignFactor = -1;
		else SignFactor = 1;

		if( FocusFactor > 1.2 && !bNeedToTurn)
			bNeedToTurn = true;
		else if( FocusFactor < 0.8 )
			bNeedToTurn = false;
		if( FocusFactor > 0.05 )
			FocusSteering = FClamp(FocusFactor * 2, 0.0, 1.0) * SignFactor;

		if( Level.TimeSeconds > AIThinkTimeDuration + AIThinkTime )
		{
			bAIBWGoing = bNeedToTurn;
			AIThinkTimeDuration = Level.TimeSeconds;
		}

		if( bAIBWGoing )
		{
			Steering = -FocusSteering;
			Gear = -1;
			OutputBrake = false;
			IsDriving = true;
			Throttle = -AIThrottle*0.8;
		}
		else if( !bNeedToTurn )
		{
			Steering = FocusSteering;

			if( DistToPointer > AIChasingDist )
			{
				Gear = 1;
				OutputBrake = false;
				IsDriving = true;
				Throttle = AIThrottle;
			}
			else
			{
				Gear = 0;
				OutputBrake = true;
				IsDriving = false;
				Throttle = 0;
			}
		}
	}

	simulated function MainSystem(float DesiredSpeed)
	{
		local bool bDSR, bPSR;

		if(Passenger != None)
			bPSR = Passenger.bWantsToCrouch;

		if(Driver == None)
			return;

		bDSR = Driver.bWantsToCrouch;

		RadioSystem(bDSR, bPSR);

		///FIX2015
		//if( DriverController.bPressedJump )
		//	bBeep = true;
	}

	function EngineModel(float ForwardSpeed, float Torque)
	{
		local float RPM;

		if( IntCollision != None && IntCollision.AmbientSound != None )
		{
			RPM = abs(EnginePitch + (abs(ForwardSpeed) / (EnginePitch + 45)) + 0.1*abs(Torque));
			IntCollision.SoundPitch = FClamp(RPM,0,200);
		}
	}

	function RadioSystem(optional bool DriverInput, optional bool PassengerInput)
	{
		if(!bAllowRadio && RadioSong.Length == 0)
			return;
		if(DriverInput || PassengerInput)
		{
			if(Level.TimeSeconds > RadioUpdateTime + RadioSwitchSpeed)
			{
				if(RadioSongNumber < RadioSong.Length)
				{
					if( Level.NetMode != NM_Client )
					{
						CurrentSong = RadioSong[RadioSongNumber];
						SwitchSong();
					}
					RadioSongNumber++;
					if(RadioSwitchSound != None)
						PlaySound(RadioSwitchSound, SLOT_Misc, RadioSwitchVolume);
				}
				else //if(RadioSongNumber > RadioSong.Length)
				{
					CurrentSong = None;
					RadioSongNumber = 0;
					SwitchSong();
				}
				RadioUpdateTime = Level.TimeSeconds;
			}
		}
	}

	function DrivingSimulation(float DesiredSpeed)
	{
		local float ForwardSpeed , RotSteer , RotAng , Torque ;
		local vector worldForward;

		worldForward = vect(1,0,0) >> Rotation;
		ForwardSpeed = Velocity Dot worldForward;
		WheelSpinSpeed = (EngTireRight.SpinSpeed + EngTireLeft.SpinSpeed)/2;

		if(Driver == None && !bAIControlled)
			return;

		if(DesiredSpeed > 1100)
			RotAng = 1.0;
		else RotAng = 1.5;

		if( ForwardSpeed < -1.0 )
		{
			EngTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed * RearMoveAccel;
			EngTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed * RearMoveAccel;
			SteerTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed * RearMoveAccel;
			SteerTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed * RearMoveAccel;
		}
		else
		{
			if( EngTireRight.WheelJoint.KMaxSpeed != VehicleMaxSpeed )
			{
				EngTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				EngTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				SteerTireRight.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
				SteerTireLeft.WheelJoint.KMaxSpeed = VehicleMaxSpeed;
			}
		}

		Torque = EngineSimulation(Throttle) * 0.5;
		EngineModel(ForwardSpeed, Torque);

		if(bCarIsRTWD)
		{
			EngTireRight.WheelJoint.KMotorTorque = Torque;
			EngTireLeft.WheelJoint.KMotorTorque = Torque;
			if( CrossTireClass != None )
			{
				CrossTireRight.WheelJoint.KMotorTorque = Torque;
				CrossTireLeft.WheelJoint.KMotorTorque = Torque;
			}
		}
		if(bCarIsFTWD)
		{
			SteerTireRight.WheelJoint.KMotorTorque = Torque;
			SteerTireLeft.WheelJoint.KMotorTorque = Torque;
			if( CrossTireClass != None )
			{
				CrossTireRight.WheelJoint.KMotorTorque = Torque;
				CrossTireLeft.WheelJoint.KMotorTorque = Torque;
			}
		}

		RotSteer = Steering * MaxSteerAngle * RotAng;
		SteerTireRight.WheelJoint.KSteerAngle = RotSteer;
		SteerTireLeft.WheelJoint.KSteerAngle = RotSteer;

		if(Throttle > 0.01)
		{
			if(!bAIControlled)
			{
				if(ForwardSpeed < -StopThreshold && Gear != 1)
				{
					Gear = 0;
					OutputBrake = true;
					IsDriving = false;
				}
				else
				{
					Gear = 1;
					OutputBrake = false;
					IsDriving = true;
				}
			}

			if(bCarIsRTWD)
			{
				EngTireRight.WheelJoint.KWake();
				EngTireLeft.WheelJoint.KWake();
				EngTireRight.KWake();
				EngTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
			if(bCarIsFTWD)
			{
				SteerTireRight.WheelJoint.KWake();
				SteerTireLeft.WheelJoint.KWake();
				SteerTireRight.KWake();
				SteerTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
		}
		else if(Throttle < -0.01)
		{
			if(!bAIControlled)
			{
				if(ForwardSpeed < StopThreshold && IsDriving == false)
				{
					Gear = -1;
					OutputBrake = false;
					IsDriving = false;
				}
				else
				{
					Gear = 0;
					OutputBrake = true;
					IsDriving = true;
				}
			}

			if(bCarIsRTWD)
			{
				EngTireRight.WheelJoint.KWake();
				EngTireLeft.WheelJoint.KWake();
				EngTireRight.KWake();
				EngTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
			if(bCarIsFTWD)
			{
				SteerTireRight.WheelJoint.KWake();
				SteerTireLeft.WheelJoint.KWake();
				SteerTireRight.KWake();
				SteerTireLeft.KWake();
				if( CrossTireClass != None )
				{
					CrossTireRight.WheelJoint.KWake();
					CrossTireLeft.WheelJoint.KWake();
					CrossTireRight.KWake();
					CrossTireLeft.KWake();
				}
			}
		}
		else
		{
			if(!bAIControlled)
			{
				if(Abs(ForwardSpeed) < StopThreshold)
				{
					Gear = 0;
					OutputBrake = true;
					IsDriving = false;
					OutputHandbrakeOn = false;
				}
				else
				{
					Gear = 0;
					OutputBrake = false;
					IsDriving = false;
				}
			}
		}

		if( DriverController != None && DriverController.bWantsToSkip > 0 )
		{
			Gear = 0;
			OutputBrake = true;
			IsDriving = true;
		}

		if((ForwardSpeed > HandbrakeThresh || OutputHandbrakeOn == true) 
				&& Abs(Steering) > 0.01 
				&& OutputBrake == true)
			OutputHandbrakeOn = true;
		else
			OutputHandbrakeOn = false;

		if(OutputBrake)
		{
			EngTireRight.WheelJoint.KBraking = MaxBrakeTorque;
			EngTireLeft.WheelJoint.KBraking = MaxBrakeTorque;
			SteerTireRight.WheelJoint.KBraking = MaxBrakeTorque;
			SteerTireLeft.WheelJoint.KBraking = MaxBrakeTorque;
		}
		else
		{
			EngTireRight.WheelJoint.KBraking = 0.0;
			EngTireLeft.WheelJoint.KBraking = 0.0;
			SteerTireRight.WheelJoint.KBraking = 0.0;
			SteerTireLeft.WheelJoint.KBraking = 0.0;
		}

		if(OutputHandbrakeOn == true)
		{
			SteerTireRight.LateralFriction = TireLateralFriction + (TireHandbrakeFriction/2);
			SteerTireRight.LateralSlip = TireLateralSlip + TireHandbrakeSlip;

			SteerTireLeft.LateralFriction = TireLateralFriction + (TireHandbrakeFriction/2);
			SteerTireLeft.LateralSlip = TireLateralSlip + TireHandbrakeSlip;

			EngTireRight.LateralFriction = TireLateralFriction + TireHandbrakeFriction;
			EngTireRight.LateralSlip = TireLateralSlip + TireHandbrakeSlip;

			EngTireLeft.LateralFriction = TireLateralFriction + TireHandbrakeFriction;
			EngTireLeft.LateralSlip = TireLateralSlip + TireHandbrakeSlip;
		}
		else
		{
			SteerTireRight.LateralFriction = TireLateralFriction;
			SteerTireRight.LateralSlip = TireLateralSlip;

			SteerTireLeft.LateralFriction = TireLateralFriction;
			SteerTireLeft.LateralSlip = TireLateralSlip;

			EngTireRight.LateralFriction = TireLateralFriction;
			EngTireRight.LateralSlip = TireLateralSlip;

			EngTireLeft.LateralFriction = TireLateralFriction;
			EngTireLeft.LateralSlip = TireLateralSlip;
		}

		KWake();
		UpdateRearLight();
	}
}

state VehicleIdling
{
	simulated function Tick(float DeltaTime)
	{
		ContinuousUpdate();
	}
}

simulated function MonitorBehindView(PlayerController PC, bool bDriver)
{
	local Actor myPawnActor, ViewPoint;
	local DriverBase PawnBase;
	local Pawn myPawn;
	if(PC == None)
		return;

	if(bDriver)
	{
		ViewPoint = FPDriverViewPoint;
		PawnBase = DriverBase;
		myPawnActor = myDriver;
		myPawn = Driver;
	}
	else
	{
		ViewPoint = FPPassengerViewPoint;
		PawnBase = PassengerBase;
		myPawnActor = myPassenger;
		myPawn = Passenger;
	}

	if(!bFixedFPCamera)
	{
		if( ViewPoint != None )
			ViewPoint.SetRotation(PC.Rotation);
	}

	//if( SteerWheelBone != '')
	//	RotateSteeringWheel(PC.bBehindView);

	if(bAllowChangeView)
	{
		if( bNormalHUDView )
		{
			if( myPawnActor != None )
			{
				if( !PC.bBehindView )
				{
					if( !myPawnActor.bHidden )
						myPawnActor.bHidden = true;
				}
				else
				{
					if( myPawnActor.bHidden )
						myPawnActor.bHidden = false;
				}
			}
		}

		if(!PC.bBehindView)// && !ViewChanged)
		{
			PC.FixFOV();
			bHideDriverActor = true;
			if( myPawnActor != None && !bNormalHUDView )
				myPawnActor.bHidden = bHideDriverActor;
			//if( !bNormalHUDView )
			if( PawnBase != None )
				PawnBase.SetHardAttach(true);
			if(bFreeCamera)
			{
				if( myPawn != None && PawnBase != None )
					myPawn.SetBase(PawnBase);
			}
			if( !bNormalHUDView )
			{
				if( ViewPoint != None )
					PC.ClientSetViewTarget(ViewPoint);
			}
			else
			{
				if( myPawn != None )
					PC.ClientSetViewTarget(myPawn);
			}
			PC.bFreeCamera = false;
			//ViewChanged = true;
		}
		else if(PC.bBehindView)// && ViewChanged)
		{
			bHideDriverActor = false;
			if( myPawnActor != None && !bNormalHUDView )
				myPawnActor.bHidden = bHideDriverActor;
			//if( !bNormalHUDView )
			if( PawnBase != None )
				PawnBase.SetHardAttach(false);
			if(bFreeCamera)
			{
				if( myPawn != None && PawnBase != None )
					myPawn.SetBase(PawnBase);
			}
			PC.ClientSetViewTarget(self);
			//ViewChanged = false;
		}
	}
	else if(!PC.bBehindView)
	{
		PC.ClientSetBehindView(true);
		PC.bBehindView = true;
	}
}

simulated function MonitorDisplayFOV(PlayerController PC, float Speed)
{
	local float ExistFOV;

	if( PC == None || bNormalHUDView )
		return;

	if(PC.bBehindView)
	{
		ExistFOV = PC.DesiredFOV;
		if(Speed >= 500)
		{
			if(ExistFOV < MaxDesiredFOV)
			{
				WildRideFOV = PC.DesiredFOV + (Speed * 0.0001);
				PC.SetFOV(WildRideFOV);
			}
		}
		else
		{
			if(ExistFOV > PC.DefaultFOV)
			{
				if(Speed > 0)
				{
					WildRideFOV = PC.DesiredFOV - ((Speed+400) * 0.0008);
					PC.SetFOV(WildRideFOV);
				}
				else PC.FixFOV();
			}
		}
	}
	else WildRideFOV = PC.DesiredFOV;
}

simulated function Tick(float DeltaTime)
{
	ContinuousUpdate();
}

state BeforeBeStart
{
	simulated function Tick(float DeltaTime)
	{
		ContinuousUpdate();
	}

    function BeginState()
    {
		if(EngStartUpSound != None)
			PlaySound(EngStartUpSound, SLOT_Misc, EngineStartUpVolume);
    }

Begin:
    Sleep(EngineStartTime);
    VehicleHaveDriver();
	GotoState('VehicleDriving');
}

state ForceDestroy
{
	//ignores Tick, Trigger, TryToDrive, DriverEnter, PassengerEnter;loool wtf ???
	ignores TryToDrive, DriverEnter, PassengerEnter;

	simulated function Tick(float DeltaTime)
	{
		ContinuousUpdate();
	}

	function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation,
							Vector momentum, class<DamageType> damageType) 
	{
		local vector ShootMomentum;

		if(VSize(momentum) < 0.001)
			return;

		if(IsInWater())
			ShootMomentum = Momentum / 32;
		else
			ShootMomentum = Momentum / 16;
		KAddImpulse(ShootMomentum,HitLocation);
	}

    simulated function StartDestroying()
    {
		local int i, n;

		if( Throttle > 0 )
			Throttle = 0;

		if( Driver != None )
		{
			if( Driver.Health > 0 )
			{
				DriverForceLeave();
				Driver.Controller.bGodMode = false;
				if( GameIsSingle() )
					Driver.Died( None, class'InVehicleDamage', Location );
				else Driver.Died( None, class'InVehicleDamageMP', Location );
				VehicleHaveNoDriver();
			}
			else
			{
				//DriverForceLeave();
				DriverLeave(true);
			}
		}

		if( Passenger != None )
		{
			if( Passenger.Health > 0 )
			{
				PassengerForceLeave();
				Passenger.Controller.bGodMode = false;
				if(GameIsSingle())
					Passenger.Died( None, class'InVehicleDamage', Location );
				else Passenger.Died( None, class'InVehicleDamageMP', Location );
				VehicleHaveNoPassenger();
			}
			else
				PassengerForceLeave();
		}

		if( EnterRightTrigger != None )
		{
			EnterRightTrigger.Destroy();
			EnterRightTrigger = None;
		}

		if( EnterLeftTrigger != None )
		{
			EnterLeftTrigger.Destroy();
			EnterLeftTrigger = None;
		}

		if( ExitTriggerDriver != None )
		{
			ExitTriggerDriver.Destroy();
			ExitTriggerDriver = None;
		}

		if( ExitTriggerPassenger != None )
		{
			ExitTriggerPassenger.Destroy();
			ExitTriggerPassenger = None;
		}

		if( IntCollision != None )
			IntCollision.AmbientSound = None;

		if( DriverBase != None )
		{
			DriverBase.Destroy();
			DriverBase = None;
		}

		if( PassengerBase != None )
		{
			PassengerBase.Destroy();
			PassengerBase = None;
		}

		if( ExhaustClass != None )
		{
			if(DecoSystem == DS_Bones)
				n = ExhaustBones.Length;
			else n = ExhaustPos.Length;
			for( i = 0 ; i < n ; i++ )
			{
				CarExhaust.Length = n;
				if(CarExhaust[i] != None)
				{
					CarExhaust[i].Destroy();
					CarExhaust[i] = None;
				}
			}
		}
		DestroyLights();
		SetSkin();
    }

Begin:
	//bHidden = true;
    Sleep(0.01);
	Exploding();
	Sleep(0.01);
	StartDestroying();
	Sleep(0.1);
    Destroy();
}

simulated function BurnTires()
{
	if(IsInWater())
		return;
	if(SteerTireRight != None && SteerTireRight.bTireBurned)
	{
		if(FrontRightWheelFire == None && WheelFrontRight != '')
		{
			FrontRightWheelFire = Spawn(class'WheelFireEmitter',self,,Location);
			AttachToBone(FrontRightWheelFire, WheelFrontRight);
		}
	}
	else
	{
		if(FrontRightWheelFire != None)
		{
			DetachFromBone(FrontRightWheelFire);
			FrontRightWheelFire.Destroy();
			FrontRightWheelFire = None;
		}
	}
	if(SteerTireLeft != None && SteerTireLeft.bTireBurned)
	{
		if(FrontLeftWheelFire == None && WheelFrontLeft != '')
		{
			FrontLeftWheelFire = Spawn(class'WheelFireEmitter',self,,Location);
			AttachToBone(FrontLeftWheelFire, WheelFrontLeft);
		}
	}
	else
	{
		if(FrontLeftWheelFire != None)
		{
			DetachFromBone(FrontLeftWheelFire);
			FrontLeftWheelFire.Destroy();
			FrontLeftWheelFire = None;
		}
	}
	if(EngTireRight != None && EngTireRight.bTireBurned)
	{
		if(RearRightWheelFire == None && WheelRearRight != '')
		{
			RearRightWheelFire = Spawn(class'WheelFireEmitter',self,,Location);
			AttachToBone(RearRightWheelFire, WheelRearRight);
		}
	}
	else
	{
		if(RearRightWheelFire != None)
		{
			DetachFromBone(RearRightWheelFire);
			RearRightWheelFire.Destroy();
			RearRightWheelFire = None;
		}
	}
	if(EngTireLeft != None && EngTireLeft.bTireBurned)
	{
		if(RearLeftWheelFire == None && WheelRearLeft != '')
		{
			RearLeftWheelFire = Spawn(class'WheelFireEmitter',self,,Location);
			AttachToBone(RearLeftWheelFire, WheelRearLeft);
		}
	}
	else
	{
		if(RearLeftWheelFire != None)
		{
			DetachFromBone(RearLeftWheelFire);
			RearLeftWheelFire.Destroy();
			RearLeftWheelFire = None;
		}
	}
	if(CrossTireRight != None && CrossTireRight.bTireBurned)
	{
		if(CrossRightWheelFire == None && WheelCrossRight != '')
		{
			CrossRightWheelFire = Spawn(class'WheelFireEmitter',self,,Location);
			AttachToBone(CrossRightWheelFire, WheelCrossRight);
		}
	}
	else
	{
		if(CrossRightWheelFire != None)
		{
			DetachFromBone(CrossRightWheelFire);
			CrossRightWheelFire.Destroy();
			CrossRightWheelFire = None;
		}
	}
	if(CrossTireLeft != None && CrossTireLeft.bTireBurned)
	{
		if(CrossLeftWheelFire == None && WheelCrossLeft != '')
		{
			CrossLeftWheelFire = Spawn(class'WheelFireEmitter',self,,Location);
			AttachToBone(CrossLeftWheelFire, WheelCrossLeft);
		}
	}
	else
	{
		if(CrossLeftWheelFire != None)
		{
			DetachFromBone(CrossLeftWheelFire);
			CrossLeftWheelFire.Destroy();
			CrossLeftWheelFire = None;
		}
	}
}

state BeforeBeDestroyed
{
	ignores TryToDrive, DriverEnter, PassengerEnter;

	simulated function Tick(float DeltaTime)
	{
		ContinuousUpdate();
	}

	function TakeDamage( int Damage, Pawn instigatedBy, Vector hitlocation,
							Vector momentum, class<DamageType> damageType) 
	{
		local vector ShootMomentum, ExpMoment;
		local int N, nForce;

		if(VSize(momentum) < 0.001)
			return;

		if(IsInWater())
			ShootMomentum = Momentum / 32;
		else
			ShootMomentum = Momentum / 16;
		KAddImpulse(ShootMomentum,HitLocation);

		if(ClassIsChildOf(damageType, class'TankProjectileDamage'))
		{
			N = Rand(1);
			if(N == 0)
				nForce = 128;
			else
				nForce = -128;
			ExpMoment.Z = 256;
			ExpMoment.X = FRand() * nForce;
			N = Rand(1);
			if(N == 0)
				nForce = 128;
			else
				nForce = -128;
			ExpMoment.Y = FRand() * nForce;
			ExpMoment = ExpMoment + (Velocity / 2);
			ExpMoment = ExpMoment * 1024;
			ExpMoment = ExpMoment >> rot(0,0,0);
			KAddImpulse(ExpMoment,HitLocation);
		}
		if(ClassIsChildOf(damageType, class'RocketExplosionDamage')
			|| ClassIsChildOf(damageType, class'BaseFX.ExplodedDamage'))
		{
			N = Rand(1);
			if(N == 0)
				nForce = 128;
			else
				nForce = -128;
			ExpMoment.Z = 128;
			ExpMoment.X = FRand() * nForce;
			N = Rand(1);
			if(N == 0)
				nForce = 128;
			else
				nForce = -128;
			ExpMoment.Y = FRand() * nForce;
			ExpMoment = ExpMoment + (Velocity / 2);
			ExpMoment = ExpMoment * 512;
			ExpMoment = ExpMoment >> rot(0,0,0);
			KAddImpulse(ExpMoment,HitLocation);
		}
	}

    simulated function StartDestroying()
    {
		local int i;
		local int b;
		local int n;
		local vector ExploMomentum;
		local bool bBrokeWheel1;
		local bool bBrokeWheel2;
		local bool bBrokeWheel3;
		local bool bBrokeWheel4;
		local bool bBrokeWheel5;
		local bool bBrokeWheel6;

		if( Throttle > 0 )
			Throttle = 0;

		if( Driver != None )
		{
			if( Driver.Health > 0 )
			{
				DriverForceLeave();
				Driver.Controller.bGodMode = false;
				if( GameIsSingle() )
					Driver.Died( None, class'InVehicleDamage', Location );
				else Driver.Died( None, class'InVehicleDamageMP', Location );
				VehicleHaveNoDriver();
			}
			else
			{
				//DriverForceLeave();
				DriverLeave(true);
			}
		}

		if( Passenger != None )
		{
			if( Passenger.Health > 0 )
			{
				PassengerForceLeave();
				Passenger.Controller.bGodMode = false;
				if(GameIsSingle())
					Passenger.Died( None, class'InVehicleDamage', Location );
				else Passenger.Died( None, class'InVehicleDamageMP', Location );
				VehicleHaveNoPassenger();
			}
			else
				PassengerForceLeave();
		}

		if( EnterRightTrigger != None )
		{
			EnterRightTrigger.Destroy();
			EnterRightTrigger = None;
		}

		if( EnterLeftTrigger != None )
		{
			EnterLeftTrigger.Destroy();
			EnterLeftTrigger = None;
		}

		if( ExitTriggerDriver != None )
		{
			ExitTriggerDriver.Destroy();
			ExitTriggerDriver = None;
		}

		if( ExitTriggerPassenger != None )
		{
			ExitTriggerPassenger.Destroy();
			ExitTriggerPassenger = None;
		}

		if( IntCollision != None )
			IntCollision.AmbientSound = None;

		if( DriverBase != None )
		{
			DriverBase.Destroy();
			DriverBase = None;
		}

		if( PassengerBase != None )
		{
			PassengerBase.Destroy();
			PassengerBase = None;
		}

		if( EngTireRight != None )
		{
			EngTireRight.WheelJoint.KWake();
			EngTireRight.WheelJoint.KBraking = 5.0;
			EngTireRight.UnloadTire();
		}

		if( EngTireLeft != None )
		{
			EngTireLeft.WheelJoint.KWake();
			EngTireLeft.WheelJoint.KBraking = 5.0;
			EngTireLeft.UnloadTire();
		}

		if( SteerTireRight != None )
		{
			SteerTireRight.WheelJoint.KWake();
			SteerTireRight.WheelJoint.KBraking = 5.0;
			SteerTireRight.UnloadTire();
		}

		if( SteerTireLeft != None )
		{
			SteerTireLeft.WheelJoint.KWake();
			SteerTireLeft.WheelJoint.KBraking = 5.0;
			SteerTireLeft.UnloadTire();
		}

		ExploMomentum.Z = 512 / ChassisMass;
		ExploMomentum.X = FRand() * 128;
		ExploMomentum.Y = FRand() * 128;
		ExploMomentum = ExploMomentum + (Velocity / 2);
		ExploMomentum = ExploMomentum * 800;
		ExploMomentum = ExploMomentum >> rot(0,0,0);
		KAddImpulse(ExploMomentum,Location);

		if(SteerTireRight != None)
			SteerTireRight.bTireBurned = true;

		if(SteerTireLeft != None)
			SteerTireLeft.bTireBurned = true;

		if(EngTireRight != None)
			EngTireRight.bTireBurned = true;

		if(EngTireLeft != None)
			EngTireLeft.bTireBurned = true;

		BurnTires();

		if(bAllowBrokenWheels)
		{
			b = FRand() + 1 - FRand() - FRand();

			bBrokeWheel4 = bool(FClamp(b,0,1));

			b = -FRand() - 1 + FRand() + FRand();

			bBrokeWheel3 = bool(FClamp(b,0,1));

			b = FRand() + 1 - FRand() - FRand();

			bBrokeWheel2 = bool(FClamp(b,0,1));

			b = -FRand() - 1 + FRand() + FRand();

			bBrokeWheel1 = bool(FClamp(b,0,1));

			b = FRand() - 1 - FRand() - FRand();

			bBrokeWheel6 = bool(FClamp(b,0,1));

			b = FRand() + 1 - FRand() * FRand() - FRand();

			bBrokeWheel5 = bool(FClamp(b,0,1));

			if(bBrokeWheel4)
			{
				EngTireRight.WheelJoint.Destroy();
				EngTireRight.Destroy();
				EngTireRight = None;
				if(EngWIRight != None)
				{
				EngWIRight.Destroy();
				EngWIRight = None;
				}
				bEngRightDetached = true;
			}
			if(bBrokeWheel3)
			{
				EngTireLeft.WheelJoint.Destroy();
				EngTireLeft.Destroy();
				EngTireLeft = None;
				if(EngWILeft != None)
				{
				EngWILeft.Destroy();
				EngWILeft = None;
				}
				bEngLeftDetached = true;
			}
			if(bBrokeWheel2)
			{
				SteerTireRight.WheelJoint.Destroy();
				SteerTireRight.Destroy();
				SteerTireRight = None;
				if(SteerWIRight != None)
				{
				SteerWIRight.Destroy();
				SteerWIRight = None;
				}
				bSteerRightDetached = true;
			}
			if(bBrokeWheel1)
			{
				SteerTireLeft.WheelJoint.Destroy();
				SteerTireLeft.Destroy();
				SteerTireLeft = None;
				if(SteerWILeft != None)
				{
				SteerWILeft.Destroy();
				SteerWILeft = None;
				}
				bSteerLeftDetached = true;
			}
			if( CrossTireClass != None )
			{
				if(bBrokeWheel6)
				{
					CrossTireRight.WheelJoint.Destroy();
					CrossTireRight.Destroy();
					CrossTireRight = None;
					if(CrossWIRight != None)
					{
					CrossWIRight.Destroy();
					CrossWIRight = None;
					}
					bCrossRightDetached = true;
				}
				if(bBrokeWheel5)
				{
					CrossTireLeft.WheelJoint.Destroy();
					CrossTireLeft.Destroy();
					CrossTireLeft = None;
					if(CrossWILeft != None)
					{
					CrossWILeft.Destroy();
					CrossWILeft = None;
					}
					bCrossLeftDetached = true;
				}
			}
		}

		if( ExhaustClass != None )
		{
			if(DecoSystem == DS_Bones)
				n = ExhaustBones.Length;
			else n = ExhaustPos.Length;
			for( i = 0 ; i < n ; i++ )
			{
				CarExhaust.Length = n;
				if(CarExhaust[i] != None)
				{
					CarExhaust[i].Destroy();
					CarExhaust[i] = None;
				}
			}
		}
	
		DestroyLights();

		SetSkin();
		KWake();
		if(IsInWater())
		{
			if(NVEngFire != None)
			{
				NVEngFire.Destroy();
				NVEngFire = None;
			}
			if(VehicleSmoke != None)
			{
				VehicleSmoke.Destroy();
				VehicleSmoke = None;
			}
		}
		else
		{
			if(NVFire == None)
			{
				NVFire = spawn(VehicleFireClass, self,, Location);
				NVFire.SetBase(self);
			}
		}
    }

Begin:
    Sleep(0.1);
	Exploding();
    Sleep(0.1);
	StartDestroying();
    Sleep(DestroyedRespawnTime);
	if(!GameIsEnd())
		Destroy();
}

function float ImpactDamageModifier(vector ImpactNorm)
{
	local float Multiplier;
	local vector X, Y, Z;

	GetAxes(Rotation, X, Y, Z);
	if(ImpactNorm Dot Z > 0)
		Multiplier = 1-(ImpactNorm Dot Z);
	else Multiplier = 1.0;

    return Multiplier;
}

simulated function KImpactPlaySound(float CrashSpeed, optional float CrashVolume, optional float CrashRadius)
{
	local Sound CrashSound;

	if( CrashSpeed > 600 && CrashSpeed < 1200 || bLittleVehicle )
	{
		if( ImpactSoundsMin.Length > 0 )
			CrashSound = ImpactSoundsMin[Rand(ImpactSoundsMin.Length-1)];
	}
	else if( CrashSpeed > 1200 && CrashSpeed < 2400 )
	{
		if( ImpactSoundsMed.Length > 0 )
			CrashSound = ImpactSoundsMed[Rand(ImpactSoundsMed.Length-1)];
	}
	else if( CrashSpeed > 2400 )
	{
		if( ImpactSoundsMax.Length > 0 )
			CrashSound = ImpactSoundsMax[Rand(ImpactSoundsMax.Length-1)];
	}
	if( CrashSound != None )
	{
		if( CrashVolume != 0 && CrashRadius != 0 )
			PlaySound(CrashSound,,CrashVolume,,CrashRadius,0.96 + FRand()*0.08);
		else PlaySound(CrashSound,,ImpactVolume,,,0.96 + FRand()*0.08);
	}
}

simulated function KImpact(Actor Other, Vector Pos, Vector ImpactVel, Vector ImpactNorm)
{
	local bool bUpdateImpactTimer;

	local float ImpactFactorSpeed;
	local vector X, Y, Z;
	local float BumpZ;

	if( Level.TimeSeconds > OtherKWakeTime + 0.1 )
	{
		if( Other != None && KActor(Other) != None && Other.bStasis )
		{
			Other.bStasis = false;
			Other.KWake();
		}
		OtherKWakeTime = Level.TimeSeconds;
	}

	ImpactFactorSpeed = VSize(ImpactVel);

	if( ImpactFactorSpeed < 1.0 )
		return;

	GetAxes(Rotation, X, Y, Z);
	BumpZ = ImpactNorm Dot Z;
	//if( Driver != None )
	//	Driver.ClientMessage(BumpZ);
	if( BumpZ > -0.2 && BumpZ < 0.2 )
		ImpactFactorSpeed = VSize(ImpactVel);
	else return;

	if( Level.TimeSeconds > LastImpactTime + ImpactInterval )
	{
		if( !bImpactTimer )
		{
			bUpdateImpactTimer = true;

			if( Other != None && KActor(Other) != None && KActor(Other).Mass <= 5 )
				bUpdateImpactTimer = false;

			if( Other != None )
				ImpactInfo.Other = Other;
			else ImpactInfo.Other = None;
			ImpactInfo.Speed = ImpactFactorSpeed;
			ImpactInfo.Pos = Pos;
			ImpactInfo.MyVelocity = VSize(Velocity);
			if( Other != None )
				ImpactInfo.OtherVelocity = VSize(Other.Velocity);

			if( bUpdateImpactTimer )
			{
				SetTimer(0.1,false);
				bImpactTimer = true;
			}
		}
		LastImpactTime = Level.TimeSeconds;
	}
	if( mySpeed > 400 && FRand() > 0.8 )
		Scrape(Other, Pos);
}

simulated function Timer()
{
	bPlayCrash = true;
	bImpactTimer = false;
	TakeImpactDamage(ImpactInfo.Other, ImpactInfo.Pos, ImpactInfo.Speed, ImpactInfo.MyVelocity, ImpactInfo.OtherVelocity);
}

simulated function Scrape(Actor Other, vector Pos)
{
	if( Level.NetMode != NM_DedicatedServer 
		&& ImpactEffectClass != None 
		&& (Other == None || !Other.IsA('Pawn')) )
		Spawn(ImpactEffectClass,,,Pos);
}

simulated function PlayCrash()
{
	local NKVehicle CrashedCar;

	//26-09-2015
	TakeImpactBump(ImpactInfo.Other, ImpactInfo.Pos);

	if( ImpactInfo.Other != None )
	{
		if( NKVehicle(ImpactInfo.Other) != None )
			CrashedCar = NKVehicle(ImpactInfo.Other); 
		else if( NKVehicleTire(ImpactInfo.Other) != None )
			CrashedCar = NKVehicleTire(ImpactInfo.Other).Chassis;
	}

	if( ImpactInfo.Speed > 600 )
	{
		if( CrashedCar != None && ArenaDamageFactor > 1.0 && !bLittleVehicle && ImpactInfo.MyVelocity > ImpactInfo.OtherVelocity )
		{
			KImpactPlaySound(3000,1.3,800);
			return;
		}
		KImpactPlaySound(ImpactInfo.Speed);
	}
}

function TakeImpactDamage(Actor Other, vector Pos, float CrashSpeed, float MyVelocity, float OtherVelocity)
{
	local float OtherDamage;
	local float CrashDamage;
	local NKVehicle CrashedCar;

	if( Other != None )
	{
		if( NKVehicle(Other) != None )
			CrashedCar = NKVehicle(Other); 
		else if( NKVehicleTire(Other) != None )
		{
			Other = NKVehicleTire(Other).Chassis;
			CrashedCar = NKVehicleTire(Other).Chassis;
		}
	}

	if( bAIControlled && Enemy == None )
	{
		if( CrashedCar != None && CrashedCar.Driver != None )
			StartScriptedDriving(CrashedCar.Driver);
	}

	if( CrashSpeed > 600 )
	{
		if( Other != None )
			CrashDamage = ((CrashSpeed / 100) * Other.Mass) / 20;
		else CrashDamage = CrashSpeed / 20;

		if( Other != None && !bLittleVehicle && MyVelocity > OtherVelocity )
		{
			OtherDamage = (CrashSpeed / 8) * ArenaDamageFactor;
			Other.TakeDamage(OtherDamage, Driver, Location, Velocity, class'NVCarDamage');//Instigator wtf lool
			Other.KAddImpulse((CrashSpeed*ArenaDamageFactor*vect(100,0,0))>>ServerRotation, Pos);
		}
		if( CrashedCar == None )
			TakeDamage(CrashDamage, Driver, Location, Velocity, class'NVCarDamage');
	}
}

simulated function DisplayDebug(Canvas Canvas, out float YL, out float YPos)
{
	local String T;
	if(bDebugHUD)
		Super.DisplayDebug(Canvas, YL, YPos);
	else if( bShowVehicleHUD )
	{
		DrawHealthInfo(Canvas);
		DrawSpeedInfo(Canvas);
		DrawCDPlayerInfo(Canvas);
	}
}

simulated function DrawHealthInfo(Canvas Canvas)
{
	local Color TempColor;
	local rotator NewRot;
	local float ColorFact, XL, YL, HealthAverage, HealthLow;
	local int HealthStr;
	local float X,Y,tempTexSize;
	local Texture tempTex;

	if(Health > 0)
		HealthStr = Health;
	else
		HealthStr = 0;

	HealthAverage = MaxHealth / 3;
	HealthLow = MaxHealth / 4;
	if(HealthStr > HealthAverage)
	{
		TempColor.R = 255;
		TempColor.G = 255;
		TempColor.B = 255;
		TempColor.A = HUDOpacity;
		Canvas.DrawColor = TempColor;

		ColorFact = 1.0;
	}
	else if(HealthStr < HealthLow)
	{
		Canvas.DrawColor.R = 255;
		Canvas.DrawColor.G = 0;
		Canvas.DrawColor.B = 0;
		Canvas.DrawColor.A = HUDOpacity;

		TempColor.R = 255;
		TempColor.G = 0;
		TempColor.B = 0;
		TempColor.A = HUDOpacity;

		ColorFact = Level.TimeSeconds * 1.5 - int(Level.TimeSeconds * 1.5);
		Canvas.DrawColor = TempColor;
		Canvas.DrawColor = Canvas.DrawColor * ColorFact;
	}
	else
	{
		Canvas.DrawColor.R = 255;
		Canvas.DrawColor.G = 128;
		Canvas.DrawColor.B = 0;
		Canvas.DrawColor.G = 128;
		Canvas.DrawColor.A = HUDOpacity;

		TempColor.R = 255;
		TempColor.G = 128;
		TempColor.B = 0;
		TempColor.G = 128;
		TempColor.A = HUDOpacity;

		ColorFact = Level.TimeSeconds * 1.5 - int(Level.TimeSeconds * 1.5);
	}

	tempTexSize = 0.6 * Canvas.ClipY/1024;
	tempTex = HealthTex;
	X = HUDLocX * Canvas.ClipX - (tempTexSize*tempTex.USize/2);
	Y = HUDLocHealthY * Canvas.ClipY - (tempTexSize*tempTex.VSize/2);
	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetDrawColor(255,255,255,255);
	Canvas.SetPos(X, Y);
	Canvas.DrawIcon(tempTex, tempTexSize);

	X = HUDLocX * Canvas.ClipX + (tempTexSize*tempTex.USize/1.5);
	Y = HUDLocHealthY * Canvas.ClipY - (tempTexSize*tempTex.VSize/4);

	Canvas.Font = Canvas.MedFont;
	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.DrawText(HealthStr, false);

	Canvas.Font = Canvas.MedFont;
	Canvas.bCenter = false;
	Canvas.Style = 5;
	Canvas.DrawColor = TempColor * ColorFact;
	Canvas.SetPos(X, Y);
	Canvas.DrawText(HealthStr, false);
}

simulated function DrawSpeedInfo(Canvas Canvas)
{
	local Color TempColor;
	local string T;
	local vector CarVelocity;
	local rotator NewRot;
	local float ColorFact, XL, YL;
	local float VehicleSpeed;
	local int RealySpeed;
	local float X,Y,tempTexSize;
	local Texture tempTex;

	if( Driver != None || Passenger != None )
		VehicleSpeed = abs(mySpeed);
	else VehicleSpeed = 0;

	RealySpeed = VehicleSpeed / SpeedFactor;

	TempColor.R = 255;
	TempColor.G = 255;
	TempColor.B = 255;
	TempColor.A = HUDOpacity;
	Canvas.DrawColor = TempColor;

	tempTexSize = 0.6 * Canvas.ClipY/1024;
	tempTex = SpeedTex;
	X = HUDLocX * Canvas.ClipX - (tempTexSize*tempTex.USize/2);
	Y = HUDLocSpeedY  * Canvas.ClipY- (tempTexSize*tempTex.VSize/2);
	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetDrawColor(255,255,255,255);
	Canvas.SetPos(X, Y);
	Canvas.DrawIcon(tempTex, tempTexSize);

	T = RealySpeed$" "$SpeedText;

	X = HUDLocX * Canvas.ClipX + (tempTexSize*tempTex.USize/1.5);
	Y = HUDLocSpeedY * Canvas.ClipY - (tempTexSize*tempTex.VSize/4);

	Canvas.Font = Canvas.MedFont;
	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.DrawText(T, false);

	Canvas.Font = Canvas.MedFont;
	Canvas.bCenter = false;
	Canvas.Style = 5;
	Canvas.DrawColor = TempColor;
	Canvas.SetPos(X, Y);
	Canvas.DrawText(T, false);
}

simulated function DrawCDPlayerInfo(Canvas Canvas)
{
	local Color TempColor;
	local string SongName;
	local int n;
	local rotator NewRot;
	local float ColorFact, XL, YL;
	local float X,Y,tempTexSize;
	local Texture tempTex;

	if( !bAllowRadio )
		return;

	TempColor.R = 255;
	TempColor.G = 255;
	TempColor.B = 255;
	TempColor.A = HUDOpacity;
	Canvas.DrawColor = TempColor;

	tempTexSize = 0.6 * Canvas.ClipY/1024;
	tempTex = CDTex;
	X = HUDLocX * Canvas.ClipX - (tempTexSize*tempTex.USize/2);
	Y = HUDLocCDY * Canvas.ClipY - (tempTexSize*tempTex.VSize/2);
	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetDrawColor(255,255,255,255);
	Canvas.SetPos(X, Y);
	Canvas.DrawIcon(tempTex, tempTexSize);

	if( CurrentSong != None )//DriverBase != None && DriverBase.AmbientSound != None
	{
		SongName = ""$CurrentSong;
		n = InStr(""$CurrentSong,".")+1;
		SongName = Right(SongName, Len(SongName) - n);
		ReplaceText(SongName, "_", " ");
	}
	else SongName = PlayerText1;

	X = HUDLocX * Canvas.ClipX + (tempTexSize*tempTex.USize/1.5);
	Y = HUDLocCDY * Canvas.ClipY - (tempTexSize*tempTex.VSize/4);

	Canvas.Font = Canvas.MedFont;
	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.DrawText(SongName, false);

	Canvas.Font = Canvas.MedFont;
	Canvas.bCenter = false;
	Canvas.Style = 5;
	Canvas.DrawColor = TempColor;
	Canvas.Font = Canvas.MedFont;
	Canvas.SetPos(X, Y);
	Canvas.DrawText(SongName, false);
}

function CheckLevelTravelObject(Pawn ownDriver, bool bSetOwnVehicle)
{
	local MaricaLevelTravelObject tempTravelObject;
	local NKVHandsWeapon tempVehicleHands;

	if( ownDriver == None || P2Pawn(ownDriver) == None )
		return;

	if( LevelTravelObject == None )
	{
		tempTravelObject = MaricaLevelTravelObject(ownDriver.FindInventoryType(class'MaricaLevelTravelObject'));

		if( tempTravelObject != None )
			LevelTravelObject = tempTravelObject;
		else LevelTravelObject = MaricaLevelTravelObject(P2Pawn(ownDriver).CreateInventoryByClass(class'MaricaLevelTravelObject'));
	}

	if( !bAllowWeaponsInside && bSetOwnVehicle )
	{
		tempVehicleHands = NKVHandsWeapon(ownDriver.FindInventoryType(class'NKVHandsWeapon'));

		if( tempVehicleHands == None )
			myVehicleHands = NKVHandsWeapon(P2Pawn(ownDriver).CreateInventoryByClass(class'NKVHandsWeapon'));
		else myVehicleHands = tempVehicleHands;

		myVehicleHands.myOldWeapon = ownDriver.Weapon;
		myVehicleHands.myVehicle = self;
	}

	if( LevelTravelObject == None )
		return;

	if( bSetOwnVehicle )
	{
		LevelTravelObject.myVehicle = self;
		LevelTravelObject.TravelCarClass = Class;

		if( Skins[SkinIndex] != None )
			LevelTravelObject.TravelCarSkin = Skins[SkinIndex];

		LevelTravelObject.TravelCarHealth = Health;
		LevelTravelObject.DriverViewPitch = ownDriver.Controller.Rotation.Pitch;
	}
	else
	{
		LevelTravelObject.myVehicle = None;
		LevelTravelObject.TravelCarSkin = None;
		LevelTravelObject.TravelCarSongNum = 0;
		LevelTravelObject.TravelCarClass = None;
	}
}

function VehiclePostTravel()
{
	if( IntCollision == None && IntCollisionClass != None )
	{
		IntCollision = spawn(IntCollisionClass, self,, Location, Rotation);
		if( IntCollision != None )
		{
			IntCollision.SoundVolume = EngineVolume*0.8;
			IntCollision.SoundRadius = SoundRadius;
			IntCollision.TransientSoundVolume = TransientSoundVolume;
			IntCollision.TransientSoundRadius = TransientSoundRadius;
			IntCollision.SetRotation(Rotation);
			IntCollision.SetBase(self);
			IntCollision.SetStaticMesh(IntCollisionModel);
			IntCollision.SetDrawScale3D(IntCollisionSize);
			IntCollision.myVehicle = self;
			IntCollision.MinROSpeed = MinRunOverSpeed;
			IntCollision.RODamage = RanOverDamage;
			IntCollision.ROSound = RanOverSound;
		}
	}

	if( Driver != None && IntCollision != None )
	{
		IntCollision.AmbientSound = EngineSound;
		IntCollision.SoundPitch = EnginePitch;
		IntCollision.CarDriver = Driver;
	}

	DamagedCarEngine();
}

simulated function SwitchSongTravel()
{
	CurrentSong = RadioSong[RadioSongNumber];
	SwitchSong();
}

event PreSaveGame()
{
	if( Driver != None )
		DriverLeave();
	Super.PreSaveGame();
}

event PostSaveGame()
{
	Super.PostSaveGame();
}

simulated function HandsTryToFire()
{
	//STUB
}

simulated function HandsTryToAltFire()
{
	//STUB
}

simulated function TakeImpactBump(Actor Other, vector Pos)
{
	//local int i;
	//ImpactBumpBones
	//for(i = 0; i < DestroyBones.Length; i++)
	//	SetVehicleBoneScale(DestroyBones[i],0);
	//bBonesDestroyed = true;
}

/*
RadioSong=Sound'MaricaMusicPack.LAB_-_Beat_the_boys'
RadioSong=Sound'MaricaMusicPack.Scattle_-_Knock_Knock'
RadioSong=Sound'MaricaMusicPack.Megadeth_-_Symphony_Of_Destruction'
RadioSong=Sound'MaricaMusicPack.Onmyouza_-_Nemuri'
RadioSong=Sound'MaricaMusicPack.The_Ratt_-_Round_And_Round'

	 defaultRadioSong(0)=Sound'MaricaMusicPack.LAB_-_Beat_the_boys'
	 defaultRadioSong(1)=Sound'MaricaMusicPack.Scattle_-_Knock_Knock'
	 defaultRadioSong(2)=Sound'MaricaMusicPack.Megadeth_-_Symphony_Of_Destruction'
	 defaultRadioSong(3)=Sound'MaricaMusicPack.Onmyouza_-_Nemuri'
	 defaultRadioSong(4)=Sound'MaricaMusicPack.The_Ratt_-_Round_And_Round'
	 defaultRadioSong(5)=Sound'MaricaMusicPack.MegaDrive_-_Acid_Spit'
*/

defaultproperties
{
	 bUseCustomRadioSongs=false

	 RearMoveAccel=0.5
	 SlipVolume=40.0
	 CameraDistMin=8.0
	 CameraDistMax=40.0
	 bShowVehicleHUD=true
	 bAllowLight=true
	 WheelIndicatorClass=class'WheelIndicator'
	 NKVHandsClass=class'NKVHandsWeapon'
	 LightProjectorClass=class'NVLightProjector'
	 WheelUnloadSound=Sound'ProjectMaricaCore.pm_wheel_unload02'
	 WheelUnloadVolume=0.6
	 RanOverVolume=0.8
	 RadioSwitchVolume=0.5
	 BeepVolume=0.8
	 DestroyedVolume=0.9
	 EngineStartUpVolume=1.0
	 EngineStopVolume=0.7
	 ImpactVolume=1.1
	 CarSpeedScale=1.0
	 CarGripScale=1.0
	 CarSteeringScale=1.0
	 CarWeightScale=1.0
	 CarSuspensionScale=1.0
	 DustPosZ=-20.0
	 DustSlipRate=2.8
	 DustSlipThresh=0.2
	 WheelRadius=60.0
	 bWildRideFOV=false
	 ImpactThreshold=80.0
	 TorqueCurve=(Points=((InVal=0.0,OutVal=150.0),(InVal=550000.0,OutVal=400.0),(InVal=800000.0))
	 AIThinkTime=1.8
	 RestLinearDamp=1.0
	 ActionLinearDamp=0.0
	 myDriverSize=0.92
	 MaxDesiredFOV=105.0
     bAllowPassenger=true
     DrivingAnims=MeshAnimation'ProjectMaricaCore.anim_PMDriving'
     DrivingAnimStatus="EvilDriving"
     SittingAnimStatus="EvilSiting"
     bFreeCamera=true
     ChassisMass=2.000000
     ArenaDamageFactor=1
     EngineVolume=80.000000
     MinRunOverSpeed=350.000000
     ImpactEffectClass=Class'ProjectMaricaCore.SparkHitVehicle'
     Description="NKVehicle. Vehicle core class."
     ProjectedLightRot=(Pitch=-1500)
     ProjLightFOV=40.000000
     ProjLightScale=0.400000
     FrontLightMaterial=Texture'GenFX.LensFlar.6'
     RearLightMaterial=Texture'GenFX.LensFlar.softlens2'
     FrontLightScale=0.200000
     RearLightScale=0.100000
     RearLightStopScale=0.200000
     RadioSwitchSpeed=1.000000
     Health=2048.000000
     MaxHealth=2048.000000
     RanOverDamage=Class'ProjectMaricaCore.NVDamage'
	 ImpactSoundsMin(0)=Sound'ProjectMaricaCore.ut_impact_03'
	 ImpactSoundsMed(0)=Sound'ProjectMaricaCore.ut_impact_01'
	 ImpactSoundsMed(1)=Sound'ProjectMaricaCore.ut_impact_02'
	 ImpactSoundsMax(0)=Sound'ProjectMaricaCore.ut_impact_04'
	 ImpactSoundsMax(1)=Sound'ProjectMaricaCore.ut_impact_05'
	 ImpactSoundsMax(2)=Sound'ProjectMaricaCore.ut_impact_06'
	 ImpactSoundsMax(3)=Sound'ProjectMaricaCore.ut_impact_07'
     CameraDist=16.000000
     ActionFriction=0.2
     RestFriction=0.3
     DestroyedRespawnTime=60.000000
     RanOverSound=Sound'MiscSounds.People.bodyhitground1'//Sound'ProjectMaricaCore.ut_runover_01'
     //EngStartUpSound=Sound'ProjectMaricaCore.ut_engine_start_01'
     //EngStopSound=Sound'ProjectMaricaCore.ut_engine_stop_01'
     //EngineStartTime=1.800000
	 EngineStartTime=0.001
     EngStartUpSound=None
     EngStopSound=None
     DestroyedSound=Sound'MiscSounds.Props.CarExplode'
     BeepSound=Sound'ProjectMaricaCore.ut_beep_01'
     AIThrottle=0.500000
     AIChasingDist=256.000000
     AIDiverMesh=SkeletalMesh'MP_Characters.Avg_Dude'
     AIDriverTex=Texture'ChameleonSkins.Special.Dude'
     AIDriverHeadMesh=SkeletalMesh'heads.AvgDude'
     AIDriverHeadTex=Texture'ChamelHeadSkins.Special.Dude'
     MaxSteerAngle=3600.000000
     MaxSpeedAccel=45.000000
     StopThreshold=50.000000
     HandbrakeThresh=800.000000
     SteerPropGap=1000.000000
     SteerTorque=1400.000000
     SteerSpeed=34000.000000
     VehicleMaxSpeed=655350.000000
     TireHandbrakeSlip=1.500000
     TireHandbrakeFriction=-2.000000
     MaxRearSpeed=1000.000000
     TireMass=0.500000
     TireRollFriction=1.500000
     TireLateralFriction=1.250000
     TireRollSlip=0.085000
     TireLateralSlip=0.060000
     TireMinSlip=0.001000
     TireSlipRate=0.000500
     MaxBrakeTorque=35.000000
     NewScaleFact=8
     SuspStiffness=14.000000
     SuspDamping=220.000000
     SuspHighLimit=0.500000
     SuspLowLimit=-0.500000
     VehicleExplosionClass=Class'ProjectMaricaCore.tVehicleExplosion'
     IntCollisionClass=Class'ProjectMaricaCore.IntCollisionActor'
     IntCollisionSize=(X=1.000000,Y=1.000000,Z=1.000000)
     HUDLocX=0.100000
     HUDLocSpeedY=0.750000
     HUDLocHealthY=0.700000
     HUDLocCDY=0.800000
     HUDOpacity=255
     HealthTex=Texture'ProjectMaricaCore.HealthHUD'
     SpeedTex=Texture'ProjectMaricaCore.SpeedHUD'
     CDTex=Texture'ProjectMaricaCore.CDHUD'
     speedfactor=20
     SpeedText="km/h"
     PlayerText1="No track"
     PlayerText2=""
     bAllowDynamicShadows=true
     bKTakeShot=false
     ImpactSounds(0)=Sound'ProjectMaricaCore.ut_impact_01'
     ImpactSounds(1)=Sound'ProjectMaricaCore.ut_impact_01'
     ImpactInterval=0.2
     bAlwaysTick=false
     bReplicateMovement=false
	 bReplicateInstigator=true
     RemoteRole=ROLE_SimulatedProxy
     DrawType=DT_Mesh
	 MusicVolume=250.0
	 MusicRadius=300.0
	 SoundRadius=64.0
	 SoundPitch=64.0
	 SoundVolume=100.0
	 TransientSoundRadius=100.0
	 TransientSoundVolume=1.0
     CollisionRadius=40.000000
     CollisionHeight=40.000000
     bBlockZeroExtentTraces=false
     bBlockNonZeroExtentTraces=false
	 bBlockPlayers=false
	 bBlockActors=false
	 bCollideActors=true
	 bCollideWorld=false
     bRotateToDesired=true
	 bProjTarget=false
     Mass=600.000000
     Begin Object Class=KarmaParams Name=StandartNKVehiclePhysics
         KFriction=0.400000
         KRestitution=0.200000
         KImpactThreshold=50000.000000
         Name="StandartNKVehiclePhysics"
     End Object
     KParams=KarmaParams'StandartNKVehiclePhysics'
     ForceType=FT_DragAlong
     ForceRadius=100.000000
     ForceScale=5.000000
}
