//=============================================================================
// NKVehicleFactory. (Special auto repawn factory for karma vehicles)
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2011.
//=============================================================================

class NKVehicleFactory extends Vehicles
	notplaceable;

var(SpawnConfig) float CarSpeedScale;
var(SpawnConfig) float CarGripScale;
var(SpawnConfig) float CarSteeringScale;
var(SpawnConfig) float CarWeightScale;
var(SpawnConfig) float CarSuspensionScale;

var(SpawnConfig) class<NKVehicle> VehicleClass;
var NKVehicle SpawnedVehicle;
var int VehicleCount;
var(SpawnConfig) bool bSpecialVehicle;
var(SpawnConfig) bool bAllowLights;
var(SpawnConfig) bool bAllowProjLight;
var(SpawnConfig) bool bImmortal;
var(SpawnConfig) bool bSpawnBrokenDetail;
var(SpawnConfig) bool bCanSeeDriver;
var(SpawnConfig) class<DamageType> RanOverDamage;
var(SpawnConfig) bool bAllowRadio;
var(SpawnConfig) bool bLocked;
var(SpawnConfig) float RespawnTime;
var(SpawnConfig) float DestroyedRespawnTime;
var(SpawnConfig) int ArenaDamageFactor;
var(SpawnConfig) float Health;
var(SpawnConfig) float MaxHealth;
var(SpawnConfig) bool bAIControlled;
var(SpawnConfig) float AIThrottle, AIChasingDist;
var(SpawnConfig) InterpCurve TorqueCurve;
var(SpawnConfig) class<NVLightProjector> LightProjectorClass;
var(SpawnConfig) enum MapSurfaceTypeSystem
{
	MST_Default,
	MST_Grass,
	MST_Dirt,
	MST_Snow,
	MST_Sand,
	MST_Ice
} MapSurfaceType;

///FIX2015
var float xStasisSpawnTime;
var float xStasisCheckTime;
var(SpawnConfig) float StasisRemoveTime;
var(SpawnConfig) float StasisCollideRadius;
var(SpawnConfig) bool bStasisSystem;
var(SpawnConfig) bool bEnemyTriggerInstigator;
///FIX2015

event Tick(float DeltaTime)
{
	///FIX2015
	local P2MoCapPawn checkPawn;
	local int PawnChecked;
	if( bStasisSystem )
	{
		bHidden = false;
		if( SpawnedVehicle != None )
		{
			if( DrawType != DT_None )
				SetDrawType(DT_None);
		}
		else if( SpawnedVehicle == None )
		{
			SpawnedVehicle = None;
			if( DrawType != DT_Mesh )
				SetDrawType(DT_Mesh);
		}
		if( AmbientGlow != VehicleClass.default.AmbientGlow )
			AmbientGlow = VehicleClass.default.AmbientGlow;

		if( Level.NetMode != NM_Client )
		{
			if( Level.TimeSeconds > xStasisCheckTime + 1.0 )
			{
				if( SpawnedVehicle == None )
				{
					SpawnedVehicle = None;
					foreach CollidingActors(class'P2MoCapPawn', checkPawn, CollisionRadius*StasisCollideRadius, Location)
					{
						if( checkPawn != None && checkPawn.Controller != None && checkPawn.Health > 0 && checkPawn.bPlayer )
							SpawnVehicleNow();
					}
				}
				else
				{
					if( SpawnedVehicle.Driver == None )
					{
						if( Level.TimeSeconds > xStasisSpawnTime + 1.0 )
						{
							if( SpawnedVehicle.Health > 0 )
							{
								PawnChecked = 0;
								foreach CollidingActors(class'P2MoCapPawn', checkPawn, CollisionRadius*(StasisCollideRadius+0.5), SpawnedVehicle.Location)
								{
									if( checkPawn != None && checkPawn.Controller != None && checkPawn.Health > 0 && checkPawn.bPlayer )
										PawnChecked = 1;
								}
								if( PawnChecked == 0 )
									PawnChecked = 2;
								if( PawnChecked == 2 )
								{
									SpawnedVehicle.OurFactory = None;
									SpawnedVehicle.Destroy();
									SpawnedVehicle = None;
								}
							}

						}
					}
					else xStasisSpawnTime = Level.TimeSeconds + StasisRemoveTime;
				}
				xStasisCheckTime = Level.TimeSeconds;
			}
		}
		return;
	}
	///FIX2015

	if( VehicleCount != 0 ) // == 1
		return;

	if( Level.NetMode != NM_Client )
		SpawnVehicleNow();
}

event Trigger( Actor Other, Pawn EventInstigator )
{
	if( SpawnedVehicle != None )
	{
		if( bAIControlled && bEnemyTriggerInstigator )
		{
			if( EventInstigator != None )
			{
				SpawnedVehicle.Enemy = EventInstigator;
				SpawnedVehicle.StartScriptedDriving(EventInstigator);
			}
			else if( Other != None && Other.IsA('ADVehicle') )
			{
				SpawnedVehicle.Enemy = Other;
				SpawnedVehicle.StartScriptedDriving(Other);
			}
		}
		else if( SpawnedVehicle.bLocked )
		{
			SpawnedVehicle.bLocked = false;
			if( !bImmortal )
				SpawnedVehicle.bImmortal = false;
		}
	}
}

function VehicleDestroyed()
{
	SetTimer(RespawnTime, false);
}

function Timer()
{
	//VehicleCount--;
	VehicleCount = 0;
}

function SpawnVehicleNow()
{
	local Actor CollideActor;

	if(VehicleClass == None)
		return;

	foreach CollidingActors(class'Actor', CollideActor, CollisionRadius, Location)
	{
		if(CollideActor != None)
		{
			if((CollideActor.bBlockPlayers
				|| CollideActor.bBlockActors
				|| (CollideActor.bBlockKarma && !CollideActor.IsA('IntCollisionActor'))) && (!CollideActor.bStatic && !CollideActor.bCollideWorld))
			return;
		}
	}

	SpawnedVehicle = spawn(VehicleClass, self, , Location, Rotation);

	if( SpawnedVehicle == None )
		return;

	if(!bAllowLights)
	{
		SpawnedVehicle.FrontLightClass = None;
		SpawnedVehicle.RearLightClass = None;
	}
	SpawnedVehicle.bImmortal = bImmortal;
	SpawnedVehicle.bSpawnBrokenDetail = bSpawnBrokenDetail;
	SpawnedVehicle.bCanSeeDriver = bCanSeeDriver;
	SpawnedVehicle.bAllowRadio = bAllowRadio;
	SpawnedVehicle.RanOverDamage = RanOverDamage;
	SpawnedVehicle.ArenaDamageFactor = ArenaDamageFactor;
	SpawnedVehicle.Health = Health;
	SpawnedVehicle.DestroyedRespawnTime = DestroyedRespawnTime;
	SpawnedVehicle.MaxHealth = MaxHealth;
	SpawnedVehicle.bAIControlled = bAIControlled;
	SpawnedVehicle.AIThrottle = AIThrottle;
	SpawnedVehicle.AIChasingDist = AIChasingDist;
	SpawnedVehicle.bEnemyTriggerInstigator = bEnemyTriggerInstigator;
	SpawnedVehicle.bSpecialVehicle = bSpecialVehicle;

	if( bLocked )
	{
		SpawnedVehicle.bLocked = bLocked;
		SpawnedVehicle.bImmortal = true;
	}

	SpawnedVehicle.OurFactory = self;
	SpawnedVehicle.UpdateFactoryVariables();

	xStasisSpawnTime = Level.TimeSeconds;

	//VehicleCount++;
	VehicleCount = 1;
}

defaultproperties
{
	 StasisCollideRadius=2.0
	 StasisRemoveTime=360
	 LightProjectorClass=class'NVLightProjector'
	 CarSpeedScale=1.0
	 CarGripScale=1.0
	 CarSteeringScale=1.0
	 CarWeightScale=1.0
	 CarSuspensionScale=1.0
	 TorqueCurve=(Points=((InVal=0.0,OutVal=150.0),(InVal=550000.0,OutVal=400.0),(InVal=800000.0))
     RanOverDamage=Class'ProjectMaricaCore.NVDamage'
     RespawnTime=20.0
	 DestroyedRespawnTime=20.0
     ArenaDamageFactor=1
     Health=2048.0
     MaxHealth=3000.0
     AIThrottle=0.5
     AIChasingDist=256.0
     DrawType=DT_Mesh
     CollisionRadius=200.0
     CollisionHeight=30.0
}
