//=============================================================================
// NKVehicleTire. (Special karma actor for wheels)
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2010-2012.
//=============================================================================
class NKVehicleTire extends KTire;

var float Health;
var float MaxHealth;
var NKVehicle Chassis;
var bool bCarExploded;
var bool bWheelAlreadyUnloaded;
var bool bTireBurned;
var float UnloadWheelRollFriction;
var float UnloadWheelLateralFriction;
var float UnloadWheelSoftness;
var float UnloadWheelRollSlip;
var WheelUnloadEmitter UnloadEmitter;

var vector WheelIT1;
var vector WheelIT2;
var bool bTireOnGround;

var float OtherKWakeTime;

replication
{
	unreliable if( Role == ROLE_Authority )
		Chassis , bTireOnGround , bWheelAlreadyUnloaded;
}

simulated function KImpact(Actor Other, Vector Pos, Vector ImpactVel, Vector ImpactNorm)
{
	if( Level.TimeSeconds > OtherKWakeTime + 0.1 )
	{
		if( Other != None && KActor(Other) != None && Other.bStasis )
		{
			Other.bStasis = false;
			Other.KWake();
		}
		OtherKWakeTime = Level.TimeSeconds;
	}
	//Super.KImpact(Other, Pos, ImpactVel, ImpactNorm);
}

simulated function PostBeginPlay()
{
	KWake();
	KSetImpactThreshold(10.0);
}

function bool EncroachingOn(Actor Other)
{
	local Pickup pu;
	if(Chassis.Driver != None)
		Instigator = Chassis.Driver;
	if ( Other == None || Other == Instigator || Other.Role != ROLE_Authority || (!Other.bCollideActors && !Other.bBlockActors)
	     || VSize(Velocity) < 10 )
		return false;

	if(Pawn(Other) != None)
	{
		PlaySound(Chassis.RanOverSound, SLOT_Misc, 0.8);
		Other.TakeDamage(50, Instigator, Other.Location, Velocity, class'NVDamage');
		return false;
	}
}

simulated function Touch(Actor Other)
{
	local Pickup pu;
	if(Pickup(Other) == None
		|| IntCollisionActor(Other) != None
		|| KVehicle(Other) != None 
		|| Other == Instigator 
		|| Other.Role != ROLE_Authority)
		return;
	Super.Touch(Other);

	/*if(Other != None)
	{
		//if(Other == Pickup(Other))
		//{
			pu = Pickup(Other);
			if(pu != None)
			{
				if(pu.Base == self)
				{
					pu.SetDrawType(DT_None);
					pu.Destroy();
				}
			}
		//}
	}*/
}

function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, class<DamageType> damageType)
{
	if( Chassis == None )
		return;

	if(Chassis.bImmortal 
		|| !Chassis.bDamagableWheels
		|| Chassis.VehicleDestroyed()
		|| IsInWater())
		return;

	if(damageType == class'BaseFX.ExplodedDamage'
		|| damageType == class'BaseFX.SmashDamage'
		|| damageType == class'BaseFX.OnFireDamage'
		|| damageType == class'BaseFX.KickingDamage'
		|| damageType == class'BaseFX.BludgeonDamage'
		|| damageType == class'BaseFX.ChemDamage')
		return;

	if(ClassIsChildOf(damageType, class'BaseFX.BurnedDamage'))
		BurnTire();

	if(ClassIsChildOf(damageType, class'BaseFX.BulletDamage'))
		Health -= damage;
	if(ClassIsChildOf(damageType, class'BaseFX.RifleDamage'))
		Health -= damage * 4;
	if(damageType == class'BaseFX.ElectricalDamage'
		|| damageType == class'NVRepairDamage')
	{
		if(Health < MaxHealth)
		{
			RollFriction = Chassis.default.TireRollFriction;
			LateralFriction = Chassis.default.TireLateralFriction;
			Softness = Chassis.default.TireSoftness;
			RollSlip = Chassis.default.TireRollSlip;
			KWake();
			Health += 25;
			bWheelAlreadyUnloaded = False;
			bTireBurned = False;
			//BurnTire();
		}
	}

	if((Health <= 0) && (!bWheelAlreadyUnloaded) && (!Chassis.VehicleDestroyed()))
		UnloadTire();
	if(bTireBurned && !bWheelAlreadyUnloaded)
		Health -= 10;
}

function UnloadTire()
{
	if( bWheelAlreadyUnloaded )
		return;

	RollFriction = UnloadWheelRollFriction;
	LateralFriction = UnloadWheelLateralFriction;
	Softness = UnloadWheelSoftness;
	RollSlip = UnloadWheelRollSlip;
	Chassis.HandbrakeThresh = 10;
	KWake();
	if( Chassis.Health > 0 )
		PlaySound(Chassis.WheelUnloadSound, SLOT_Misc, Chassis.WheelUnloadVolume);
	bWheelAlreadyUnloaded = True;
	UnloadEmitter = spawn(class'WheelUnloadEmitter', self,, Location);
	UnloadEmitter.SetBase(self);
}

function BurnTire()
{
	if(IsInWater())
		return;
	bTireBurned = True;
	Chassis.BurnTires();
	SetTimer(15,false);
}

function Timer()
{
	bTireBurned = False;
	Chassis.BurnTires();
}

simulated function TouchingSurfaceIndicator()
{
	local vector EndTrace, WheelIT;

	EndTrace = Location - Chassis.WheelRadius*(vect(0,0,1) >> Owner.Rotation);
	bTireOnGround = !FastTrace(EndTrace, Location);

	if( bTireOnGround )
	{
		KSetInertiaTensor(WheelIT1, WheelIT2);
		//if( Chassis.Driver != None )
		//	Chassis.Driver.ClientMessage(self$" on ground");
	}
	else
	{
		KSetInertiaTensor(WheelIT, WheelIT);
		//if( Chassis.Driver != None )
		//	Chassis.Driver.ClientMessage(self$" NOT on ground");
	}
}

simulated function Tick(float DeltaTime)
{
	local vector NormalImpulse;

	if( Chassis == None )
		return;

	TouchingSurfaceIndicator();

	if(IsInWater())
	{
		NormalImpulse.X = 0;
		NormalImpulse.Y = 0;
		NormalImpulse.Z = 2048 / Chassis.ChassisMass;
		KAddImpulse(NormalImpulse, Location);
	}
}

simulated function Destroyed()
{
	Super.Destroyed();
	bTireBurned = False;
	Chassis.BurnTires();
}

function DisablePhysics()
{
	if(Physics == PHYS_Karma)
		KDisableKarma();
	if(Physics != PHYS_None)
		SetPhysics(PHYS_None);
}

function bool IsInWater()
{
	/*local WaterVolume V;

	ForEach TouchingActors(class'ProjectMaricaCore.WaterVolume',V)
			return true;*/
	return false;
}

defaultproperties
{
	 bBlockKarma=true
	 bAlwaysTick=true
	 WheelIT1=(X=0.01,Y=0.0,Z=0.01)
	 WheelIT2=(X=0.01,Y=0.0,Z=0.0)
     Health=50.000000
     MaxHealth=100.000000
     UnloadWheelRollFriction=0.200000
     UnloadWheelLateralFriction=0.200000
     UnloadWheelSoftness=0.013000
     UnloadWheelRollSlip=2.000000
     ImpactVolume=0.500000
     ImpactInterval=1.000000
     bPawnMovesMe=False
     bHidden=True
     bAlwaysRelevant=True
     RemoteRole=ROLE_None
     bUnlit=True
     bGameRelevant=True
     bBlockActors=False
     bBlockPlayers=False
	 SoundVolume=16.0
	 SoundRadius=32.0
     Mass=100.000000
}
