//=============================================================================
// NPVehicle. (Nicks pawn based vehicles class)
//
// Fully controlling vehicle for Unreal Engine 2 (Version 118.2).
// Base class for all non-karma Vehicles for STP
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2010.
//=============================================================================

class NPVehicle extends P2MoCapPawn
	hidecategories(Force,LightColor,Lighting,Character,AnimTweaks,Karma,PawnAttributes,Police)
	config(NicksVehicles)
    notplaceable;

/*
MaricaPhysics
MaricaVehicle
MaricaHUD
MaricaGraphics
MaricaInterior
MaricaSounds
MaricaSkeleton
MaricaArmory
*/

const BASE_TIME_TO_DESTROY = 40.0;

struct ReplaceDriverModels
{
	var Mesh OldModel;
	var Mesh NewModel;
};

var P2MocapPawn Driver;
var PlayerController DriverController;
var DriverActorNPV myDriver;
var NPVehicleFactory OurFactory;
var NPVehicleTrigger RightTrigger;
var NPVehicleTrigger LeftTrigger;
var NPVehicleTrigger ExitTrigger;
var NVShadowProjector myShadow;
var Explosion DestroyedEffect;
var BrokenVehicleDetail VehBrokenDetail;
var Pawn Attacker;
var(MaricaInterior) MeshAnimation	DrivingAnims;
var(MaricaHUD) /*globalconfig*/ Texture HealthTex;
var(MaricaHUD) /*globalconfig*/ Texture SpeedTex;

var float mySpeed;
var float FireUpdate;
var float BumpUpdate;
var float AltFireUpdate;
var(MaricaInterior) float myDriverSize;
var(MaricaVehicle) float CameraDist;
var(MaricaVehicle) float VehicleFOV;
var(MaricaVehicle) float MinRunOverSpeed;
var(MaricaArmory) float VehicleFireSpeed;
var(MaricaArmory) float VehicleAltFireSpeed;
var(MaricaHUD) /*globalconfig*/ float HUDLocX;
var(MaricaHUD) /*globalconfig*/ float HUDLocSpeedY;
var(MaricaHUD) /*globalconfig*/ float HUDLocHealthY;
var(MaricaHUD) /*globalconfig*/ float HUDLocCDY;

var(MaricaVehicle) class<Explosion> DestroyedEffectClass;
var(MaricaVehicle) class<BrokenVehicleDetail> BrokenDetailClass;
var(MaricaVehicle) class<DamageType> RanOverDamage;

var(MaricaVehicle) Name DriverState;
var(MaricaInterior) Name DrivingAnimStatus;
var(MaricaHUD) string SpeedText;

var(MaricaInterior) vector MyDriverPos;
var(MaricaInterior) vector DriverPos;
var(MaricaInterior) vector ExitPos;
var(MaricaVehicle) vector BrokenDetailPos;
var(MaricaInterior) const vector EnterPos;

var bool bHideHUD;
var bool bHaveDriver;
var bool bImmortal;
var(MaricaArmory) bool bCanFire;
var(MaricaArmory) bool bCanFireAlt;
var(MaricaInterior) bool bCanSeeDriver;
var(MaricaGraphics) config bool	bAllowDynamicShadows;
var(MaricaHUD) config bool bDebugHUD;
var(MaricaHUD) /*config*/ bool bShowVehicleHUD;

var(MaricaSounds) sound EngineAmbientSound;
var(MaricaSounds) sound DestroyedSound;
var(MaricaSounds) sound RanOverSound;

var(MaricaHUD) /*globalconfig*/ int HUDOpacity;
var(MaricaHUD) /*globalconfig*/ int SpeedFactor;

var(MaricaSounds) array<sound> CrashSounds;
var(MaricaInterior) globalconfig array<ReplaceDriverModels> AltDriverModels;

var(MaricaVehicle) bool bSpecialVehicle;
var(MaricaGraphics) float ShadowZ;
var bool bMatineeStatusUpdate;

replication
{
	unreliable if( Role == ROLE_Authority )
		myDriver , mySpeed , VehicleGetFire , GameIsEnd , bHaveDriver , bHideHUD;

	reliable if( Role < ROLE_Authority )
		ServerChangeView , ServerChangeZoom , VehicleFire , VehicleAltFire;

	unreliable if( bNetDirty && Role == ROLE_Authority )
		Driver , DriverController;

	reliable if( Role == ROLE_Authority )
		ClientSparks , ShadowRendering;
}

simulated function bool IsInWater()
{
	/*local PhysicsVolume V;

	ForEach TouchingActors(class'PhysicsVolume',V)

	if( V != None && ( V.IsA('WaterVolume') || (V.DamageType != None && V.DamageType == class'WaterDamage') ) )
		return true;*/
	return false;
}

simulated function bool GameIsSingle()
{
	if( Level != None && Level.Game != None && Level.Game.bIsSingleplayer )
		return true;
	return false;
}

simulated function bool GameIsEnd()
{
	if( Level != None && Level.Game != None && Level.Game.bGameEnded )
		return true;
	return false;
}

simulated function ShadowRendering(bool bRender)
{
	if( myShadow == None )
		return;
	myShadow.bAcceptRender = bRender;
}

simulated function UserInterfaceControl()
{
	if( GameIsEnd() || Health <= 0 )
		bHideHUD = true;

	if( DriverController != None )
	{
		if( bHideHUD )
		{
			DriverController.MyHUD.bShowDebugInfo = false;
			bHideHUD = false;
		}

		if( Driver != None )
		{
			if( !DriverController.MyHUD.bShowScores && !bHideHUD && Health > 0 )
			{
				if(!DriverController.MyHUD.bShowDebugInfo)
					DriverController.MyHUD.bShowDebugInfo = true;
			}
			else
			{
				if(DriverController.MyHUD.bShowDebugInfo)
					DriverController.MyHUD.bShowDebugInfo = false;
			}

			if( DriverController.CameraDist != CameraDist && !GameIsEnd() )
				DriverController.CameraDist = CameraDist;

			if( DriverController.ViewTarget != self && !GameIsEnd() )
				DriverController.ClientSetViewTarget(self);

			if( !DriverController.bBehindView )
			{
				DriverController.ClientSetBehindView(true);
				DriverController.bBehindView = true;
			}

			if( !DriverController.IsInState(DriverState) && Health > 0 )
				DriverController.GotoState(DriverState);

			DriverController.SetFOV(VehicleFOV);
		}
		else
		{
			DriverController.MyHUD.bShowDebugInfo = false;
			DriverController.CameraDist = DriverController.default.CameraDist;
			DriverController.FixFOV();
			DriverController.ClientSetViewTarget(Driver);
			DriverController.ClientSetBehindView(false);
			DriverController.bBehindView = false;
			P2Player(DriverController).SwitchToHands(true);
			log("Current driver controller ("$DriverController$") leave this vehicle ("$self$")");
			DriverController = None;
		}
	}
	else DriverController = None;
}

simulated function SendEnterMessage(PlayerController PC)
{
	PC.ReceiveLocalizedMessage(class'NVMessage', 1);
}

simulated function bool DetectGameIsSingle()
{
	if( Level != None && Level.Game != None && Level.Game.bIsSingleplayer )
		return Level.NetMode != NM_Client && Level.NetMode != NM_DedicatedServer;
	return false;
}

simulated function SwitchMessageType()
{
	if( class'NVMessage'.default.ActionUseGameSwitch != 1 )
	{
		if( DynamicLoadObject("ProjectMarica.ProjectMaricaMenu", class'Class') == None )
			class'NVMessage'.default.ActionUseGame = 1;
		else if( DynamicLoadObject("FPSGame.FPSButtonInfo", class'Class') != None )
			class'NVMessage'.default.ActionUseGame = 0;
		else if( !DetectGameIsSingle() )
			class'NVMessage'.default.ActionUseGame = 1;
		else class'NVMessage'.default.ActionUseGame = 2;
	}
}

/*simulated function SwitchMessageType()
{
	if( class'NVMessage'.default.ActionUseGameSwitch != 1 )
	{
		if( DynamicLoadObject("FPSGame.FPSButtonInfo", class'Class') != None )
			class'NVMessage'.default.ActionUseGame = 0;
		else if( Level.NetMode == NM_Client )
			class'NVMessage'.default.ActionUseGame = 1;
		else class'NVMessage'.default.ActionUseGame = 2;
	}
}*/

simulated function DriverEnter(P2MocapPawn p)
{
	local PlayerController PC;
	local rotator ViewRotation;
	local Controller C;
	local vector RotX, RotY, RotZ;

	bImmortal = true; // hack, ignore any damage
	Driver = p;
	Driver.SetCollision(false, false, false);
	Driver.SetLocation(Location + (DriverPos >> Rotation));
	Driver.bHidden = true;
	Driver.bPhysicsAnimUpdate = false;
	Driver.Velocity = vect(0,0,0);
	Driver.SetPhysics(PHYS_None);
	Driver.SetBase(self);

	C = P.Controller;
	PC = PlayerController(C);

	if(PC != None)
	{
		ViewRotation = PC.Rotation;
		ViewRotation.Yaw = Rotation.Yaw;
		//PC.ReceiveLocalizedMessage(class'NVMessage', 1);
		SendEnterMessage(PC);
		P2Player(PC).SwitchToHands(true);
		PC.Unpossess();
		Driver.SetOwner(Self);
		PC.Possess(self);
		PC.SetFOV(VehicleFOV);
		PC.MyHUD.bShowDebugInfo = true;
		//PC.bBehindView = true;
		PC.ClientSetBehindView(true);
		PC.bBehindView = true;
		PC.ClientSetViewTarget(self);
		PC.ClientSetRotation(ViewRotation);
		PC.GotoState(DriverState);
		PC.PlayerReplicationInfo.SetWaitingPlayer(false);
		bUpdateEyeHeight = true;
	}

	bHaveDriver = true;
	bMatineeStatusUpdate = true;

	VehicleHaveDriver();
}

simulated function DriverLeave()
{
	local PlayerController PC;
	local Controller C;
	local rotator ExitRot;

	if(Driver == None)
		return;

	C = Controller;
	PC = PlayerController(C);

	if(PC != None)
	{
		ExitRot = PC.Rotation;
		PC.Unpossess();
		Driver.SetOwner(C);
		PC.Possess(Driver);
		PC.MyHUD.bShowDebugInfo = False;
		PC.FixFOV();
		//PC.bBehindView = false;
		PC.ClientSetBehindView(false);
		PC.bBehindView = false;
		PC.ClientSetRotation(ExitRot);
		PC.ClientSetViewTarget(Driver);
		PC.PlayerReplicationInfo.SetWaitingPlayer(false);
		bUpdateEyeHeight = true;
	}

	Driver.SetCollision(true, true, true);
	Driver.SetLocation(Location + (ExitPos >> Rotation));
	Driver.PlayWaiting();
	Driver.bPhysicsAnimUpdate = Driver.Default.bPhysicsAnimUpdate;
	Driver.Acceleration = vect(0, 0, 24000);
	Driver.SetPhysics(PHYS_Falling);
	Driver.SetBase(None);
	Driver.bUpdateEyeHeight = true;

	if(Driver.bHidden)
		Driver.bHidden = false;

	if(RightTrigger != None)
		RightTrigger.SetPawnIgnore(Driver, 1.0);

	if(LeftTrigger != None)
		LeftTrigger.SetPawnIgnore(Driver, 1.0);

	bHaveDriver = false;

	VehicleHaveNoDriver();
}

simulated function PossessedBy(Controller C)
{
	Super.PossessedBy(C);
	NetPriority = 3;
	NetUpdateFrequency = 100;
}

simulated function UnPossessed()
{
	Super.UnPossessed();
	NetPriority = Default.NetPriority;
	NetUpdateFrequency = 8;
}

simulated function ControllerLeave()
{
	if( Controller != None )
	{
		if( Controller.bIsPlayer )
			Controller.PawnDied(self);
		else
		{
			Controller.Destroy();
			Controller = None;
		}
	}
}

simulated function VehicleHaveDriver()
{
	if(myDriver != None)
	{
		myDriver.Destroy();
		myDriver = None;
	}

	ShowDriver();

	if(ExitTrigger == None)
		ExitTrigger = spawn(class'NPVehicleTrigger', self,, Location);

	ExitTrigger.bExitTrigger = true;
	ExitTrigger.myVehicle = self;
	ExitTrigger.SetCollision(true, false, false);
	ExitTrigger.SetCollisionSize(8.0,8.0);

	bHaveDriver = true;
	bImmortal = false;
}

simulated function VehicleHaveNoDriver()
{
	local P2MoCapPawn OldDriver;
	OldDriver = Driver;
	Driver = None;
	Controller = None;

	if(myDriver != None)
	{
		if( OldDriver != None && OldDriver.myHead != None )
		{
			myDriver.DetachFromBone(OldDriver.myHead);
			OldDriver.AttachToBone(OldDriver.myHead, 'MALE01 head');
			OldDriver.myHead.SetOwner(OldDriver);
		}
		myDriver.Destroy();
		myDriver = None;
	}

	if(ExitTrigger != None)
	{
		ExitTrigger.Destroy();
		ExitTrigger = None;
	}

	bHaveDriver = false;
}

function Mesh CheckPawnModel(mesh Model)
{
	local int i;
	local Mesh ThisModel;

	if(AltDriverModels.Length > 0 && AltDriverModels[0].OldModel != None)
	{
		for(i = 0; i < AltDriverModels.Length; i++)
		{
			if(Model == AltDriverModels[i].OldModel)
				ThisModel = AltDriverModels[i].NewModel;
		}
	}

	if(ThisModel != None)
		return ThisModel;
	return None;
}

simulated function Mesh FindPawnModel()
{
	local Mesh myModel, returnModel;

	myModel = Driver.Mesh;

	returnModel = CheckPawnModel(myModel);

	if(returnModel != None)
		return returnModel;
	else return myModel;
}

simulated function ShowDriver()
{
	local Material BodySkin;

	if(Driver == None 
		|| myDriver != None 
		|| !bCanSeeDriver)
		return;

	BodySkin = Driver.Skins[0];
	myDriver = spawn(class'DriverActorNPV', self,,Location + (MyDriverPos >> Rotation),Rotation,BodySkin);
	myDriver.SetCollision(false, false, false);
	myDriver.SetBase(self);
	myDriver.myVehicle = self;
	myDriver.BodySkin = Driver.Skins[0];
	myDriver.DrivingAnims = DrivingAnims;
	myDriver.SetDrawScale(myDriverSize);
	myDriver.AmbientGlow = Driver.AmbientGlow;
}

function AddPhysImpulse(vector Impulse)
{
	local vector Force;
	Force = Impulse * 0.25 * Mass;
	AddVelocity(Force); 
}

simulated function PostBeginPlay()
{
	local vector RotX, RotY, RotZ;

    PhysicsSimulation(0);

	SpawnMyShadow();

    GetAxes(Rotation,RotX,RotY,RotZ);

	if(Level.NetMode != NM_Client)
	{
		RightTrigger = spawn(class'NPVehicleTrigger', self,, Location + EnterPos.X * RotX + EnterPos.Y * RotY + EnterPos.Z * RotZ);
		RightTrigger.SetBase(self);
		RightTrigger.SetCollision(true, false, false);
		RightTrigger.myVehicle = self;

		LeftTrigger = spawn(class'NPVehicleTrigger', self,, Location + EnterPos.X * RotX - EnterPos.Y * RotY + EnterPos.Z * RotZ);
		LeftTrigger.SetBase(self);
		LeftTrigger.SetCollision(true, false, false);
		LeftTrigger.myVehicle = self;
	}
	Super.PostBeginPlay();
	SwitchMessageType();
}

simulated function SpawnMyShadow()
{
	if(bAllowDynamicShadows && Level.NetMode != NM_DedicatedServer)
	{
		if(myShadow == None)
		{
			myShadow = Spawn(class'NVShadowProjector',Self,'',Location);
			myShadow.ShadowActor = self;
			myShadow.ShadowZ = ShadowZ;
			myShadow.MyBase = None;
			myShadow.MyCar = None;
			myShadow.MyVeh = self;
			myShadow.LightDirection = Normal(vect(1,1,6));
			myShadow.InitShadow();
		}
	}
}

simulated function DestroyMyShadow(optional bool bNoShadowCirculation)
{
	if( myShadow != None )
	{
		myShadow.bShadowCirculation = !bNoShadowCirculation;
		myShadow.Destroy();
		myShadow = None;
	}
}

simulated function PlayerChangedTeam()
{
	if( Controller != None )
		ControllerLeave();
	Destroy();
}

simulated function Destroyed()
{
	DestroyMyShadow();

	if( AmbientSound != None )
		AmbientSound = None;

	if( myDriver != None )
		myDriver.Destroy();

	if( OurFactory != None )
		OurFactory.VehicleDestroyed();

	if( Controller != None )
		ControllerLeave();

	if( Driver != None )
		Driver.Destroy();

	if( RightTrigger != None )
		RightTrigger.Destroy();

	if( LeftTrigger != None )
		LeftTrigger.Destroy();

	if( ExitTrigger != None)
		ExitTrigger.Destroy();

	if( Health <= 0 )
		Explode();

	Super.Destroyed();
}

function Landed(vector HitNormal)
{
	LandBob = FMin(50, 0.055 * Velocity.Z); 

	TakeFallingDamage();

	if ( Health > 0 )
		PlayLanded(Velocity.Z);
	if (Velocity.Z < -1.4 * JumpZ)
		MakeNoise(-0.5 * Velocity.Z/(FMax(JumpZ, 150.0)));

	bJustLanded = true;

	if( Controller == None )
		PhysicsSimulation(3);
}

function TakeFallingDamage()
{
	local int numSounds, soundNum;

	if(abs(Velocity.Z) > MaxFallSpeed)
	{
		if(Role == ROLE_Authority)
			TakeDamage(abs(Velocity.Z)*5, None, Location, vect(0,0,0), class'Crushed');

		numSounds = CrashSounds.Length;
		if(numSounds > 0)
		{
			soundNum = Rand(numSounds);
			PlaySound(CrashSounds[soundNum], SLOT_None, 0.8, true);
			//PlaySound(CrashSounds[soundNum], SLOT_Misc, 0.8);
		}
	}
}

exec function ChangeView(bool State)
{
	ServerChangeView(State);
}

exec function ChangeZoom(float Zoom)
{
	ServerChangeZoom(Zoom);
}

simulated function ServerChangeView(bool State)
{
	local PlayerController pc;

	pc = PlayerController(Controller);

	if(pc != None)
		pc.bBehindView = State;
}

simulated function ServerChangeZoom(float Zoom)
{
	local PlayerController pc;

	pc = PlayerController(Controller);

	if(pc != None)
		pc.CameraDist += Zoom;
}

simulated function Tick(float DeltaTime)
{
	ContinuousUpdate(DeltaTime, 0);
}

simulated function ContinuousUpdate(float DeltaTime, int State)
{
	local DriverSceneWatchScript SceneWatchScript;

	mySpeed = VSize(Velocity);

	VehicleGetFire();

	if(Driver != None && Controller != None)
		DriverController = PlayerController(Controller);

	if( DriverController != None && DriverController.bInterpolating )
	{
		if( bMatineeStatusUpdate )
		{
			SceneWatchScript = spawn(class'DriverSceneWatchScript');
			SceneWatchScript.DriverPC = DriverController;
			SceneWatchScript.bWasGodMode = DriverController.bGodMode;
			DriverController = None;
			bMatineeStatusUpdate = false;
		}
		return;
	}

	UserInterfaceControl();
}

simulated function VehicleGetFire()
{
	if( PressingFire() && bCanFire )
	{
		if(Level.TimeSeconds > FireUpdate + VehicleFireSpeed)
		{
			VehicleFire(Rotation);
			FireUpdate = Level.TimeSeconds;
		}
	}
	else if( PressingAltFire() && bCanFireAlt )
	{
		if(Level.TimeSeconds > AltFireUpdate + VehicleAltFireSpeed)
		{
			VehicleAltFire(Rotation);
			AltFireUpdate = Level.TimeSeconds;
		}
	}
}

simulated function Explode()
{
	local vector RotX, RotY, RotZ;
    GetAxes(Rotation,RotX,RotY,RotZ);

	if(Level.NetMode != NM_Client)
	{
		if(BrokenDetailClass != None)
		{
			VehBrokenDetail = Spawn(BrokenDetailClass,self,, Location + BrokenDetailPos.X * RotX + BrokenDetailPos.Y * RotY + BrokenDetailPos.Z * RotZ, Rotation);
			VehBrokenDetail.KWake();
		}
	}

	if(DestroyedEffectClass != None)
	{
		if(!IsInWater())
		{
			DestroyedEffect = spawn(DestroyedEffectClass, self, , Location);
			DestroyedEffect.ShakeCamera(1000);
		}
		else
		{
			DestroyedEffect = spawn(class'UnderWaterExplosion', self, , Location);
			DestroyedEffect.ShakeCamera(1000);
		}
	}

	if(DestroyedSound != None)
		PlaySound(DestroyedSound, SLOT_None, 0.9, true);
		//PlaySound(DestroyedSound, SLOT_Misc, 0.9);
}

simulated function DisplayDebug(Canvas Canvas, out float YL, out float YPos)
{
	if(bDebugHUD)
		Super.DisplayDebug(Canvas, YL, YPos);
	else if( bShowVehicleHUD )
	{
		DrawHealthInfo(Canvas);
		DrawSpeedInfo(Canvas);
	}
}

singular event BaseChange()
{
	local float decorMass;

	if( bInterpolating )
		return;

	if ( (base == None) && (Physics == PHYS_None) )
		SetPhysics(PHYS_Falling);
	else if( Pawn(Base) != None )
	{
		if( !Pawn(Base).bCanBeBaseForPawns )
			Base.TakeDamage(Pawn(Base).Health, Self, Location, 2 * Velocity , class'Gibbed');
		PhysicsSimulation(4);
	}
	else if ( (Decoration(Base) != None) && (Velocity.Z < -400) )
	{
		decorMass = FMax(Decoration(Base).Mass, 1);
		Base.TakeDamage((-2* Mass/decorMass * Velocity.Z/400), Self, Location, 0.5 * Velocity, class'Crushed');
	}
	else if( IntCollisionActor(Base) != None && IntCollisionActor(Base).myVehicle != None )
	{
		IntCollisionActor(Base).TakeDamage(500, Self, Location, 2 * Velocity , class'Crushed');
	}
}

function AddVelocity( vector NewVelocity)
{
	// STUB
}

simulated function DrawHealthInfo(Canvas Canvas)
{
	local Color TempColor;
	local rotator NewRot;
	local float ColorFact, XL, YL, HealthAverage, HealthLow;
	local int HealthStr;
	local float X,Y,tempTexSize;
	local Texture tempTex;

	if(Health > 0)
		HealthStr = Health;
	else
		HealthStr = 0;

	HealthAverage = HealthMax / 3;
	HealthLow = HealthMax / 4;
	if(HealthStr > HealthAverage)
	{
		TempColor.R = 255;
		TempColor.G = 255;
		TempColor.B = 255;
		TempColor.A = HUDOpacity;
		Canvas.DrawColor = TempColor;

		ColorFact = 1.0;
	}
	else if(HealthStr < HealthLow)
	{
		Canvas.DrawColor.R = 255;
		Canvas.DrawColor.G = 0;
		Canvas.DrawColor.B = 0;
		Canvas.DrawColor.A = HUDOpacity;

		TempColor.R = 255;
		TempColor.G = 0;
		TempColor.B = 0;
		TempColor.A = HUDOpacity;

		ColorFact = Level.TimeSeconds * 1.5 - int(Level.TimeSeconds * 1.5);
		Canvas.DrawColor = TempColor;
		Canvas.DrawColor = Canvas.DrawColor * ColorFact;
	}
	else
	{
		Canvas.DrawColor.R = 255;
		Canvas.DrawColor.G = 128;
		Canvas.DrawColor.B = 0;
		Canvas.DrawColor.G = 128;
		Canvas.DrawColor.A = HUDOpacity;

		TempColor.R = 255;
		TempColor.G = 128;
		TempColor.B = 0;
		TempColor.G = 128;
		TempColor.A = HUDOpacity;

		ColorFact = Level.TimeSeconds * 1.5 - int(Level.TimeSeconds * 1.5);
	}

	tempTexSize = 0.6 * Canvas.ClipY/1024;
	tempTex = HealthTex;
	X = HUDLocX * Canvas.ClipX - (tempTexSize*tempTex.USize/2);
	Y = HUDLocHealthY * Canvas.ClipY - (tempTexSize*tempTex.VSize/2);
	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetDrawColor(255,255,255,255);
	Canvas.SetPos(X, Y);
	Canvas.DrawIcon(tempTex, tempTexSize);

	X = HUDLocX * Canvas.ClipX + (tempTexSize*tempTex.USize/2);
	Y = HUDLocHealthY * Canvas.ClipY - (tempTexSize*tempTex.VSize/4);

	Canvas.Font = Canvas.MedFont;
	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.DrawText(HealthStr, false);

	Canvas.Font = Canvas.MedFont;
	Canvas.bCenter = false;
	Canvas.Style = 5;
	Canvas.DrawColor = TempColor * ColorFact;
	Canvas.SetPos(X, Y);
	Canvas.DrawText(HealthStr, false);
}

simulated function DrawSpeedInfo(Canvas Canvas)
{
	local Color TempColor;
	local string T;
	local vector CarVelocity;
	local rotator NewRot;
	local float ColorFact, XL, YL;
	local float VehicleSpeed;
	local int RealySpeed;
	local float X,Y,tempTexSize;
	local Texture tempTex;

	VehicleSpeed = VSize(Velocity);
	RealySpeed = VehicleSpeed / SpeedFactor;

	TempColor.R = 255;
	TempColor.G = 255;
	TempColor.B = 255;
	TempColor.A = HUDOpacity;
	Canvas.DrawColor = TempColor;

	tempTexSize = 0.6 * Canvas.ClipY/1024;
	tempTex = SpeedTex;
	X = HUDLocX * Canvas.ClipX - (tempTexSize*tempTex.USize/2);
	Y = HUDLocSpeedY  * Canvas.ClipY- (tempTexSize*tempTex.VSize/2);
	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetDrawColor(255,255,255,255);
	Canvas.SetPos(X, Y);
	Canvas.DrawIcon(tempTex, tempTexSize);

	T = RealySpeed$" "$SpeedText;

	X = HUDLocX * Canvas.ClipX + (tempTexSize*tempTex.USize/2);
	Y = HUDLocSpeedY * Canvas.ClipY - (tempTexSize*tempTex.VSize/4);

	Canvas.Font = Canvas.MedFont;
	Canvas.bCenter = false;
	Canvas.Style = ERenderStyle.STY_Alpha;
	Canvas.SetDrawColor(25,25,25,180);
	Canvas.SetPos(X+2.0, Y+2.0);
	Canvas.DrawText(T, false);

	Canvas.Font = Canvas.MedFont;
	Canvas.bCenter = false;
	Canvas.Style = 5;
	Canvas.DrawColor = TempColor;
	Canvas.SetPos(X, Y);
	Canvas.DrawText(T, false);
}

simulated function TakeDamage(int Damage, Pawn instigatedBy, Vector hitlocation, 
						Vector momentum, class<DamageType> damageType)
{
	local float actualDamage;
	local Controller Killer;

	if(damageType == class'BaseFX.BurnedDamage'
		|| damageType == class'BaseFX.SmashDamage'
		|| damageType == class'BaseFX.OnFireDamage'
		|| damageType == class'BaseFX.KickingDamage'
		|| damageType == class'BaseFX.BludgeonDamage'
		|| InGodMode()
		|| bImmortal)
		return;

	if( instigatedBy != None )
		Attacker = instigatedBy;

	if(ClassIsChildOf(damageType, class'ExplodedDamage'))
	{
		Health -= Damage;
		if( Controller != None )
			AddPhysImpulse(momentum);
	}

	if( ClassIsChildOf(damageType, class'WaterDamage') 
		|| ClassIsChildOf(damageType, class'TankProjectileDamage') )
		Health -= Health;

	if(damageType == class'BaseFX.ElectricalDamage'
		|| damageType == class'NVRepairDamage')
	{
		if(Health < HealthMax)
			Health += 25;
	}
	else Health -= Damage;

	if ( instigatedBy == self )
		momentum *= 0.6;
	momentum = momentum/Mass;

	actualDamage = Level.Game.ReduceDamage(Damage, self, instigatedBy, HitLocation, Momentum, DamageType);

	if ( HitLocation == vect(0,0,0) )
		HitLocation = Location;

	PlayHit(actualDamage, hitLocation, damageType, Momentum);

	if(Health <= 0)
	{
		if(instigatedBy != None)
			Killer = instigatedBy.GetKillerController();

		if(bPhysicsAnimUpdate)
			TearOffMomentum = momentum;

		IsDying(Killer, damageType, HitLocation);
	}
}

simulated function IsDying(Controller Killer, class<DamageType> damageType, vector HitLocation)
{
	local PlayerController PC;

	if( (Driver != None) && (Driver.Health > 0) )
	{
		if( Driver != None && Driver.myHead != None )
		{
			myDriver.DetachFromBone(Driver.myHead);
			Driver.AttachToBone(Driver.myHead, 'MALE01 head');
			Driver.myHead.SetOwner(Driver);
		}
		PC = PlayerController(Controller);
		if( PC != None )
		{
			bHideHUD = true;
			PC.MyHUD.bShowDebugInfo = False;
			PC.Unpossess();
			PC.Possess(Driver);
		}
		Driver.bCollideWorld = true;
		Driver.Died(Killer, damageType, HitLocation);
		if( Level.NetMode != NM_Client )
			Driver.bHidden = false;
		Driver = None;
	}
	bHideHUD = true;
	GotoState('Destroying');
	// hack, anyway prepare it to destroy
	LifeSpan = 0.5;
}

simulated function Died(Controller Killer, class<DamageType> damageType, vector HitLocation)
{
	bHideHUD = true;
	Super.Died(Killer, damageType, HitLocation);
	IsDying(Killer, damageType, HitLocation);
}

state Destroying
{
	ignores TakeDamage;
	Begin:
	bHideHUD = true;
	Sleep(0.2);
	Destroy();
}

simulated function ChunkUp(int Damage)
{
	bChunkedUp = true;
	//if( Controller != None )
	//	ControllerLeave();
	//GotoState('Destroying');
}

simulated function Bump(Actor Other)
{
	local vector Momentum;
	local float CurrentSpeed;;

	if (Pawn(Other) == None
		|| NKVehicle(Other) != None 
		|| KVehicle(Other) != None 
		|| NPVehicle(Other) != None 
		|| Other == Instigator 
		|| Other.Role != ROLE_Authority)
		return;

	if(Other.bStatic)
	{
		Momentum = Velocity * 0.5 * Mass;
		AddVelocity(Momentum);
	}

	if(IntCollisionActor(Other) != None)
		IntCollisionActor(Other).TakePhysImpulse(Velocity,Location);

	CurrentSpeed = VSize(Velocity);

	if(Driver != None)
	{
		if (CurrentSpeed > MinRunOverSpeed)
		{
			Momentum = Velocity * 0.25 * Other.Mass;
			if (Controller != None && Controller.SameTeamAs(Pawn(Other).Controller))
			{
				Momentum += CurrentSpeed * 0.25 * Other.Mass * Normal(Velocity cross vect(0,0,1));
				CurrentSpeed = 0;
			}
			if(Level.TimeSeconds > BumpUpdate + 1)
			{
				if(RanOverSound != None)
					PlaySound(RanOverSound, SLOT_Misc, 0.8);
				BumpUpdate = Level.TimeSeconds;
			}
			Other.TakeDamage(int(CurrentSpeed * 16), Instigator, Other.Location, Momentum, RanOverDamage);
		}
	}
}

simulated function PlayHit(float Damage, vector HitLocation, class<DamageType> damageType, vector Momentum)
{
	if ( Damage <= 0 
		&& Controller != None
		&& !Controller.bGodMode )
		return;

	ClientSparks(HitLocation, Momentum);

	if( Level.TimeSeconds - LastPainTime > 0.1 )
	{
		PlayTakeHit(HitLocation,Damage,damageType);
		LastPainTime = Level.TimeSeconds;
	}
	bReceivedHeadShot = false;
}

simulated function PlayTakeHit(vector HitLoc, int Damage, class<DamageType> damageType)
{
	if( Damage == 0 )
		return;
	PlayOwnedSound(Sound'WeaponSounds.bullet_ricochet1',SLOT_Misc,1.0);
}

function BloodHit(vector BloodHitLocation, vector Momentum)
{
	ClientSparks(BloodHitLocation, Momentum);
}

simulated function ClientSparks(vector SparkHitLocation, vector Momentum)
{
	local vector HitOffset;
	local vector dir;

	dir = SparkHitLocation - Location;
	dir = Normal(SparkHitLocation - Location);
	HitOffset = 0.2 * CollisionRadius * dir;
	HitOffset.Z = HitOffset.Z * 0.75;

	spawn(class'SparkHitMachineGun',self,,SparkHitLocation+HitOffset,Rotator(dir));
}

function GetKarmaSkeleton()
{
	KParams = None;
}

function bool AllowRagdoll(class<DamageType> DamageType)
{
	return false;
}

function JumpOffPawn()
{
	//STUB
}

simulated function PhysicsSimulation(int State)
{
	//STUB
}

simulated function VehicleFire(rotator Direction)
{
	//STUB
}

simulated function VehicleAltFire(rotator Direction)
{
	//STUB
}

event EncroachedBy(Actor Other) 
{
	//STUB
}

simulated function FaceRotation( rotator NewRotation, float DeltaTime ) 
{
	//STUB
}

function TryToDrive(Pawn p)
{
	//STUB
}

function TossWeapon(vector TossVel)
{
	//STUB
}

function ChangedWeapon()
{
	//STUB
}

event SetWalking(bool bNewIsWalking) 
{
	//STUB
}

function EndClimbLadder(LadderVolume OldLadder)
{
	//STUB
}

function ClimbLadder(LadderVolume L)
{
	//STUB
}

simulated event ChangeAnimation()
{
	//STUB
}

function ChangePhysicsAnimUpdate(bool bNewUpdate)
{
	//STUB
}

defaultproperties
{
	 bShowVehicleHUD=true
	 bAlwaysTick=false
	 CrashSounds(0)=Sound'ProjectMaricaCore.ut_impact_01'
	 CrashSounds(1)=Sound'ProjectMaricaCore.ut_impact_01'
	 bCanTeleport=false
	 ControllerClass=None
	 myDriverSize=0.92
	 CameraDist=6.0
     ExitPos=(Z=400.0)
     bAllowDynamicShadows=true
     VehicleFireSpeed=1.0
     VehicleAltFireSpeed=1.0
     DrivingAnims=MeshAnimation'ProjectMaricaCore.anim_PMDriving'
     DrivingAnimStatus="EvilDriving"
     MinRunOverSpeed=100.0
     RanOverSound=Sound'MiscSounds.People.bodyhitground1'//Sound'ProjectMaricaCore.ut_runover_01'
     HUDLocX=0.7
     HUDLocSpeedY=0.8
     HUDLocHealthY=0.75
     HUDLocCDY=0.85
     HUDOpacity=255
     HealthTex=Texture'ProjectMaricaCore.HealthHUD'
     SpeedTex=Texture'ProjectMaricaCore.SpeedHUD'
     speedfactor=20
     SpeedText="km/h"
     BodyHitSounds(0)=Sound'MiscSounds.People.bodyhitground1'
     BodyHitSounds(1)=Sound'MiscSounds.People.bodyhitground2'
     ChameleonMeshPkgs(0)="Characters"
     ChamelHeadMeshPkgs(0)="heads"
     HealthMax=2048.0
     bCanBeBaseForPawns=true
     bDontPossess=true
     bRollToDesired=true
     MaxFallSpeed=800.0
     bReplicateInstigator=true
     bGameRelevant=true
     CollisionRadius=1.0
     CollisionHeight=1.0
     Mass=600.0
     ForceType=FT_DragAlong
     ForceRadius=100.0
     ForceScale=5.0
     NetPriority=3.0
	 bCollideActors=True
	 bCollideWorld=True
}
