//=============================================================================
// NVShadowProjector. (Shadow projector for vehicles)
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2010.
//=============================================================================

class NVShadowProjector extends ShadowProjector
	config(NicksVehicles);

var config int ShadowDetail;
var bool bShadowCirculation;
var bool bUpdateShadow;
var bool bAcceptRender;
var bool bSteerUpdate;
var bool bBlobShadow;
var bool bInstantUpdate;
var NKActor MyBase;
var NKVehicle MyCar;
var NPVehicle MyVeh;
var float ActorSpeed;
var float UpdateAgain;
var float ShadowZ;

replication
{
	unreliable if (Role == ROLE_Authority)
		MyBase, MyCar, MyVeh, ActorSpeed, bInstantUpdate;
}

simulated event Touch( Actor Other );
simulated event Untouch( Actor Other );

function bool IsRenderingBan()
{
	return (ShadowDetail == 0);
}

event PreSaveGame()
{
	Super.PreSaveGame();
	bUpdateShadow = false;
}

event PostSaveGame()
{
	Super.PostSaveGame();
	bUpdateShadow = true;
}

event PostLoadGame()
{
	Super.PostLoadGame();
	bUpdateShadow = true;
}

event Destroyed()
{
	FreeShadowObject();
	Super.Destroyed();
}

simulated event PostBeginPlay()
{
	Super(Actor).PostBeginPlay();
}

function VehicleCreatorBaseWeapon SearchVehicleCreator()
{
	local VehicleCreatorBaseWeapon VehicleCreator;
	foreach AllActors(class'VehicleCreatorBaseWeapon',VehicleCreator)
	{
		if( VehicleCreator != None )
			return VehicleCreator;
	}
	return None;
}

function AllocateShadowObject()
{
	local ShadowBitmapMaterial tempShadow;
	local VehicleCreatorBaseWeapon VehicleCreator;

	VehicleCreator = SearchVehicleCreator();

	if( ShadowTexture == None )
	{
		if( VehicleCreator != None )
		{
			tempShadow = VehicleCreator.GetAllocatedObject();
		}

		if( tempShadow != None )
		{
			ShadowTexture = tempShadow;
			ProjTexture = ShadowTexture;
			return;
		}

		if( ShadowDetail == 1 )
			ShadowTexture = new(Level) class'NVShadowBitmapUltraLowRes';
		else if( ShadowDetail == 2 )
			ShadowTexture = new(Level) class'NVShadowBitmapLowRes';
		else if( ShadowDetail == 3 )
			ShadowTexture = new(Level) class'NVShadowBitmapMedRes';
		else if( ShadowDetail == 4 )
			ShadowTexture = new(Level) class'NVShadowBitmapHighRes';
		else if( ShadowDetail == 5 )
			ShadowTexture = new(Level) class'NVShadowBitmapUltraHighRes';

		ProjTexture = ShadowTexture;
	}
}

function FreeShadowObject()
{
	local VehicleCreatorBaseWeapon VehicleCreator;

	VehicleCreator = SearchVehicleCreator();

	if(ShadowTexture != None)
	{
		ShadowTexture.ShadowActor = None;

		if( !ShadowTexture.Invalid && bShadowCirculation )
		{
			if( VehicleCreator != None )
				VehicleCreator.AllocateObject(ShadowTexture);
		}

		ShadowTexture = None;
		ProjTexture = None;
	}
}

function ChangeShadowDetail()
{
	if( ShadowTexture != None )
	{
		ShadowTexture.ShadowActor = None;

		ShadowTexture = None;
		ProjTexture = None;
	}
	AllocateShadowObject();
}

function InitShadow()
{
	local Plane	BoundingSphere;
	local Actor tempShadowActor;

	if( IsRenderingBan() )
		return;

	tempShadowActor = ShadowActor;

	if( ShadowDetail == 1 
		&& ShadowActor != None 
		&& NKVehicle(ShadowActor) != None
		&& NKVehicle(ShadowActor).IntCollision != None )
	{
		tempShadowActor = NKVehicle(ShadowActor).IntCollision;
	}

	if( tempShadowActor != None )
	{
		BoundingSphere = tempShadowActor.GetRenderBoundingSphere();
		FOV = Atan(BoundingSphere.W * 2 / LightDistance) * 180 / PI + 5;

		AllocateShadowObject();

		if( ShadowTexture != None )
		{
			SetDrawScale(LightDistance * tan(0.5 * FOV * PI / 180) / (0.5 * ShadowTexture.USize));

			ShadowTexture.Invalid = false;
			ShadowTexture.bBlobShadow = false;
			ShadowTexture.ShadowActor = tempShadowActor;
			ShadowTexture.LightDirection = Normal(LightDirection);
			ShadowTexture.LightDistance = LightDistance;
			ShadowTexture.LightFOV = FOV;
    
			Enable('Tick');
			UpdateShadow();
		}
		else log(Name$".InitShadow: Failed to allocate texture");
	}
	else log(Name$".InitShadow: No actor");
}

function UpdateShadow()
{
	local vector ShadowLocation;
	local Actor tempShadowActor;

	if( ShadowTexture == None )
	{
		InitShadow();
		return;
	}

	DetachProjector(true);
	SetCollision(false,false,false);

	if( ShadowActor != None && !ShadowActor.bHidden && ShadowActor.DrawType != DT_None )
	{
		tempShadowActor = ShadowActor;
		if( ShadowDetail == 1 
			&& NKVehicle(ShadowActor) != None
			&& NKVehicle(ShadowActor).IntCollision != None )
		{
			tempShadowActor = NKVehicle(ShadowActor).IntCollision;
		}

		if( tempShadowActor != None )
		{
			ShadowLocation = ShadowActor.Location + ((ShadowZ*vect(0,0,1)) >> ShadowActor.Rotation);

			SetLocation(ShadowLocation);
			SetRotation(Rotator(Normal(-LightDirection)));

			ShadowTexture.bBlobShadow = bBlobShadow;
			ShadowTexture.ShadowActor = tempShadowActor;
			ShadowTexture.LightDirection = Normal(LightDirection);
			ShadowTexture.LightDistance = LightDistance;
			ShadowTexture.LightFOV = FOV;
			ShadowTexture.Dirty = true;

			AttachProjector();
			SetCollision(true,false,false);
		}
	}
}

simulated function Tick(float DeltaTime)
{
	if( ShadowActor == None 
		|| !bUpdateShadow
		|| IsRenderingBan() ) 
		return;

	if(MyBase != None)
	{
		if(MyBase.mySpeed > 1.0 || bAcceptRender)
		{
			UpdateShadow();
			//log('Shadow is going to update for nkactor '$ActorSpeed);
		}
	}
	else if(MyCar != None)
	{
		if(!bInstantUpdate)
		{
			if(VSize(MyCar.Velocity) > 1.0)
				bAcceptRender = True;
			else
				bAcceptRender = False;
		}
		else bAcceptRender = True;
		if(MyCar.mySpeed > 1.0 || bSteerUpdate || bAcceptRender)
		{
			UpdateShadow();
			//log('Shadow is going to update for nkvehicle '$ActorSpeed);
		}
		if(MyCar.Steering != 0)
		{
			bSteerUpdate = True;
			SetTimer(1.0,false);
		}
	}
	else if(MyVeh != None)
	{
		if(MyVeh.mySpeed > 1.0 || bAcceptRender)
		{
			UpdateShadow();
			//log('Shadow is going to update for npvehicle '$ActorSpeed);
		}
	}
	else
	{
		if (Role == ROLE_Authority)
			if(ShadowActor != None)
				ActorSpeed = VSize(ShadowActor.Velocity);
		if(ActorSpeed > 1.0 || bAcceptRender)
		{
			UpdateShadow();
			//log('Shadow is going to update '$ActorSpeed);
		}
	}
	if(Level.TimeSeconds > UpdateAgain + 5 + rand(5))
	{
		UpdateShadow();
		UpdateAgain = Level.TimeSeconds;
	}
}

function Timer()
{
	bSteerUpdate = False;
}

defaultproperties
{
	 ShadowTexture=1
	 bShadowCirculation=true
	 bUpdateShadow=true
     LightDistance=1024.000000
     FOV=40
     MaxTraceDistance=512
     bProjectOnUnlit=True
}
