//=============================================================================
// WheelIndicator.
//
// Official page of mod:
// http://www.set-games.ru/index/projectmarica/0-8
//
// Author: Evil T-ModZ (Nick)
// Email: Nick_Pv@mail.ru / TModZ.Evil@Yahoo.com
// Site: Set-Games.ru
// T-ModZ. All Rights Reserved.
// 2011.
//=============================================================================

class WheelIndicator extends Actor
	notplaceable;

var TireFXBase Effect;
var() class<TireFXBase> EffectDefaultClass;
var() class<TireFXBase> EffectGrassClass;
var() class<TireFXBase> EffectDirtClass;
var() class<TireFXBase> EffectSnowClass;
var() class<TireFXBase> EffectSandClass;
var() class<TireFXBase> EffectIceClass;
var() float MaxParticles;
var float SlipVolume;

replication
{
	unreliable if( Role == ROLE_Authority )
		Effect;
}

simulated function UpdateDustEffect(NKVehicle CarOwner)
{
	if( CarOwner == None )
		return;

	if( Effect != None )
	{
		//Effect.Destroy();
		Effect.ForceSelfDestroy();
		Effect = None;
	}

	if( CarOwner.MapSurfaceType == MST_Default )
		SpawnEffects(CarOwner, EffectDefaultClass);
	else if( CarOwner.MapSurfaceType == MST_Grass )
		SpawnEffects(CarOwner, EffectGrassClass);
	else if( CarOwner.MapSurfaceType == MST_Dirt )
		SpawnEffects(CarOwner, EffectDirtClass);
	else if( CarOwner.MapSurfaceType == MST_Snow )
		SpawnEffects(CarOwner, EffectSnowClass);
	else if( CarOwner.MapSurfaceType == MST_Sand )
		SpawnEffects(CarOwner, EffectSandClass);
	else if( CarOwner.MapSurfaceType == MST_Ice )
		SpawnEffects(CarOwner, EffectIceClass);
}

simulated function PostNetBeginPlay()
{
	local NKVehicle CarOwner;

	CarOwner = NKVehicle(Owner);

	if( CarOwner == None )
		return;

	if( CarOwner.MapSurfaceType == MST_Default )
		SpawnEffects(CarOwner, EffectDefaultClass);
	else if( CarOwner.MapSurfaceType == MST_Grass )
		SpawnEffects(CarOwner, EffectGrassClass);
	else if( CarOwner.MapSurfaceType == MST_Dirt )
		SpawnEffects(CarOwner, EffectDirtClass);
	else if( CarOwner.MapSurfaceType == MST_Snow )
		SpawnEffects(CarOwner, EffectSnowClass);
	else if( CarOwner.MapSurfaceType == MST_Sand )
		SpawnEffects(CarOwner, EffectSandClass);
	else if( CarOwner.MapSurfaceType == MST_Ice )
		SpawnEffects(CarOwner, EffectIceClass);

	Super.PostNetBeginPlay();
}

simulated function SpawnEffects(NKVehicle CarOwner, class<TireFXBase> EffectTypeClass)
{
	if( EffectTypeClass == None )
		return;
	Effect = Spawn(EffectTypeClass, self, , Location + (CarOwner.DustPosZ*vect(0,0,1) >> CarOwner.Rotation), Rotation);
	Effect.Emitters[0].ParticlesPerSecond = 0;
	Effect.SetBase(Self);
}

simulated function WheelScrape(NKVehicleTire Wheel)
{
	local UnloadWheelsFrictionEmitter UnloadFriciton;
	local NKVehicle CarOwner;

	CarOwner = NKVehicle(Owner);

	if( CarOwner == None )
		return;

	if( CarOwner.MapSurfaceType == MST_Default )
	{
		UnloadFriciton = spawn(class'UnloadWheelsFrictionEmitter', self,, Location + (Wheel.Chassis.DustPosZ*vect(0,0,1) >> Wheel.Chassis.Rotation), rotator(-vector(Rotation)));
		UnloadFriciton.SetBase(self);
	}
}

simulated function UpdateDust(NKVehicleTire Wheel, float DustSlipRate, float DustSlipThresh)
{
	local float SpritePPS;
	local int i;

	if( Wheel == None )
		return;

	if( Effect == None )
		return;

	Effect.Emitters[0].ParticlesPerSecond = 0;
	Effect.Emitters[0].InitialParticlesPerSecond = 0;
	Effect.SoundVolume = 0;

	if( Wheel.bWheelAlreadyUnloaded && Wheel.bTireOnGround )
	{
		if( Wheel.Chassis.bSparkWheels )
		{
			if( Wheel.GroundSlipVel > 1 )
				WheelScrape(Wheel);
		}
	}

	if( Wheel.bTireOnGround && Wheel.GroundSlipVel > DustSlipThresh )
	{
		SpritePPS = FClamp(DustSlipRate * (Wheel.GroundSlipVel - DustSlipThresh),0,MaxParticles);

		Effect.Emitters[0].ParticlesPerSecond = SpritePPS;
		Effect.Emitters[0].InitialParticlesPerSecond = SpritePPS;
		Effect.Emitters[0].AllParticlesDead = False;
		Effect.SoundVolume = FClamp(SpritePPS * 3,0,SlipVolume);//SpritePPS * 5.5;
	}
	else
	{
		Effect.Emitters[0].ParticlesPerSecond = 0;
		Effect.Emitters[0].InitialParticlesPerSecond = 0;
		Effect.SoundVolume = 0;
	}
}

simulated function Destroyed()
{
	if( Effect != None )
		Effect.Destroy();

	Super.Destroyed();
}

defaultproperties
{
	 SlipVolume=40.0
	 EffectDefaultClass=Class'TireFXStone'
	 EffectGrassClass=Class'TireFXGrass'
	 EffectDirtClass=Class'TireFXDirt'
	 EffectSnowClass=Class'TireFXSnow'
	 EffectSandClass=Class'TireFXSand'
	 EffectIceClass=Class'TireFXIce'
	 MaxParticles=35.0
     bHidden=true
     bReplicateInstigator=false
     bReplicateMovement=true
	 //RemoteRole=ROLE_None
     bGameRelevant=true
     CollisionRadius=0.0
     CollisionHeight=0.0
	 bCollideActors=false
	 bCollideWorld=false
     bBlockZeroExtentTraces=false
     bBlockNonZeroExtentTraces=false
     bIgnoreOutOfWorld=true
}
